Package: tirt
Title: Testlet Item Response Theory
Version: 0.4.0
Authors@R: c(
    person("Jiawei", "Xiong", email = "jiawei.xiong@uga.edu", role = c("aut", "cre")),
    person("Cheng", "Tang", role = "ctb"),
    person("Qidi", "Liu", role = "ctb")
  )
Description: Implementation of Testlet and Item Response Theory. 
    A light-version yet comprehensive and streamlined framework for psychometric analysis using 
    unidimensional and multidimensional Item Response Theory 
    (IRT; Baker & Kim (2004) <doi:10.1201/9781482276725>) and Testlet Response Theory 
    (TRT; Wainer et al., (2007) <doi:10.1017/CBO9780511618765>). 
    Designed for researchers, this package supports the estimation of item and person 
    parameters for a wide variety of models, including binary (i.e., Rasch, 2-Parameter Logistic, 3-Parameter Logistic) 
    and polytomous (Partial Credit Model, Generalized Partial Credit Model, Graded Response Model) formats. It also supports the estimation of Testlet models (Rasch Testlet, 2-Parameter Logistic Testlet, 3-Parameter Logistic Testlet, Bifactor, Partial Credit Model Testlet, Graded Response), allowing users to account for local item dependence in bundled items. A key feature is the specialized support for combination use and joint estimation of item response model and testlet response model in one calibration.
    Beyond standard estimation via Marginal Maximum Likelihood with Expectation-Maximization (EM) or Joint Maximum Likelihood, the package also offers Bayesian estimation using priors with maximum a posteriori (MAP) method for unidimensional item response theory models.
    It also provides functions for scale linking and equating (Mean-Mean, Mean-Sigma, Stocking-Lord) to ensure comparability across mixed-format test forms. It also facilitates fixed-parameter calibration, enabling users to estimate person abilities with known item parameters or 
    vice versa, which is essential for pre-equating studies and item bank 
    maintenance. Comprehensive data simulation functions are included to generate 
    synthetic datasets with complex structures, including mixed-model blocks and 
    specific testlet effects, aiding in methodological research and study design
    validation. Researchers can try multiple simulation situations.
    A suite of post-estimation tools is also provided, including item and test
    information functions with the conditional standard error of measurement,
    summed-score to scale-score conversion tables (expected a posteriori, weighted
    likelihood, and maximum likelihood), person-fit and item-fit statistics, local
    dependence diagnostics (Yen's Q3), differential item functioning
    (M-H and logistic regression), reliability coefficients, test
    characteristic curves, and mixture (latent-class) item response models.
License: GPL-3
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: dplyr, tidyr, purrr, gtools, stats, utils
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-08-05 15:36:51 UTC; jx
Author: Jiawei Xiong [aut, cre],
  Cheng Tang [ctb],
  Qidi Liu [ctb]
Maintainer: Jiawei Xiong <jiawei.xiong@uga.edu>
Repository: CRAN
Date/Publication: 2026-08-20 08:10:02 UTC

# Generated by roxygen2: do not edit by hand

export(binary_irt)
export(dif)
export(equate_irt)
export(fix_person)
export(fixed_item)
export(irt_trt)
export(item_fit)
export(item_info)
export(ld_stats)
export(mirt_binary)
export(mixed_irt)
export(mixture_irt)
export(person_fit)
export(polytomous_irt)
export(reliability)
export(score_table)
export(sim_irt)
export(sim_mirt)
export(sim_tirt)
export(sim_trt)
export(tcc)
export(test_info)
export(trt_binary)
export(trt_poly)
import(stats)
import(utils)
importFrom(dplyr,across)
importFrom(dplyr,any_of)
importFrom(dplyr,everything)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(gtools,mixedorder)
importFrom(purrr,map_dbl)
importFrom(stats,binomial)
importFrom(stats,cor)
importFrom(stats,dnorm)
importFrom(stats,glm)
importFrom(stats,pnorm)
importFrom(stats,qlogis)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(tidyr,pivot_longer)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(tirt)

## ----sim_irt------------------------------------------------------------------
set.seed(123)
# Define item structure: 1 block of 15 items using 2PL
sim_data <- sim_irt(
  n_people = 500, 
  item_structure = list(
    list(model = "2PL", n_items = 15)
  ),
  theta_mean = 0,
  theta_sd = 1
)

# Access responses and true parameters
head(sim_data$resp)
head(sim_data$true_params)

## ----sim_trt------------------------------------------------------------------
# Define 2 testlets with 5 items each
trt_data <- sim_trt(
  n_people = 500,
  item_structure = list(
    list(model = "2PLT", n_items = 5, testlet_id = "T1"),
    list(model = "2PLT", n_items = 5, testlet_id = "T2")
  )
)

head(trt_data$resp)

## ----binary_irt---------------------------------------------------------------
# Estimate 2PL model
# Note: max_iter set low for vignette speed; use default (100) in practice
fit_2pl <- binary_irt(
  data = sim_data$resp, 
  model = "2PL", 
  method = "EM",
  control = list(max_iter = 20, verbose = FALSE) 
)

# View estimated item parameters
print(head(fit_2pl$item_params))

# View model fit indices
print(fit_2pl$model_fit)

## ----trt_binary---------------------------------------------------------------
# Items 1-5 are Testlet 1, Items 6-10 are Testlet 2
testlet_structure <- list(
  c(1, 2, 3, 4, 5),   # Indices for Testlet 1
  c(6, 7, 8, 9, 10)   # Indices for Testlet 2
)

fit_trt <- trt_binary(
  data = trt_data$resp,
  group = testlet_structure,
  model = "2PLT",
  method = "EM",
  control = list(max_iter = 15, verbose = FALSE)
)

# The output includes item parameters
head(fit_trt$item_params)

# And person parameters including the testlet specific effects (Gamma)
head(fit_trt$person_params)

## ----sim----------------------------------------------------------------------
cat("\n\n--- Generating Simulated Data ---\n")
set.seed(2025)

N <- 1000
# 20 Items Total:
# 1-10: Binary (2PL) - Independent
# 11-15: Polytomous (GRM, 3 cats) - Independent
# 16-17: Binary (2PLT) - Testlet 1
# 18-20: Polytomous (GPCT, 3 cats) - Testlet 2

theta <- rnorm(N, 0, 1)
gamma_1 <- rnorm(N, 0, 0.5) # Testlet 1 effect
gamma_2 <- rnorm(N, 0, 0.6) # Testlet 2 effect

resp_matrix <- matrix(NA, N, 20)
item_names <- paste0("Item_", 1:20)
colnames(resp_matrix) <- item_names

# Define Item Parameters (True Values)
a_true <- runif(20, 0.8, 1.5)
b_true <- seq(-1.5, 1.5, length.out = 20)

# 1. Simulate Binary Independent (1-10)
for(j in 1:10) {
  p <- 1 / (1 + exp(-a_true[j] * (theta - b_true[j])))
  resp_matrix[,j] <- rbinom(N, 1, p)
}

# 2. Simulate Poly Independent (11-15) - GR
for(j in 11:15) {
  b_k <- sort(c(b_true[j] - 0.7, b_true[j] + 0.7))
  p1 <- 1 / (1 + exp(-a_true[j] * (theta - b_k[1])))
  p2 <- 1 / (1 + exp(-a_true[j] * (theta - b_k[2])))
  # GRM Probabilities
  prob_0 <- 1 - p1
  prob_1 <- p1 - p2
  prob_2 <- p2
  # Sample
  r <- runif(N)
  resp_matrix[,j] <- ifelse(r < prob_0, 0, ifelse(r < prob_0 + prob_1, 1, 2))
}

# 3. Simulate Binary Testlet 1 (16-17)
for(j in 16:17) {
  eff_theta <- theta + gamma_1
  p <- 1 / (1 + exp(-a_true[j] * (eff_theta - b_true[j])))
  resp_matrix[,j] <- rbinom(N, 1, p)
}

# 4. Simulate Poly Testlet 2 (18-20) - GPCMT
for(j in 18:20) {
  eff_theta <- theta + gamma_2
  # GPCM steps (centered at b_true)
  b_vec <- c(b_true[j] - 0.5, b_true[j] + 0.5) 
  
  numer <- matrix(0, N, 3)
  numer[,1] <- 0
  numer[,2] <- a_true[j] * (eff_theta - b_vec[1])
  numer[,3] <- a_true[j] * (eff_theta - b_vec[1]) + a_true[j] * (eff_theta - b_vec[2])
  
  max_n <- apply(numer, 1, max)
  exps <- exp(numer - max_n)
  probs <- exps / rowSums(exps)
  
  r <- runif(N)
  resp_matrix[,j] <- apply(cbind(r, probs), 1, function(x) {
    if(x[1] < x[2]) return(0)
    if(x[1] < x[2]+x[3]) return(1)
    return(2)
  })
}

sim_data <- as.data.frame(resp_matrix)
#view the data
print(head(sim_data))

## -----------------------------------------------------------------------------
# Create Item Specification
spec <- data.frame(
  item = item_names,
  model = c(rep("2PL", 10), rep("GRM", 5), rep("2PLT", 2), rep("GPCMT", 3)),
  testlet = c(rep(NA, 15), rep("testlet1", 2), rep("testlet2", 3)),
  stringsAsFactors = FALSE
)
#view the spec
print(spec)

## -----------------------------------------------------------------------------
# Run the Universal Function
results <- irt_trt(sim_data, spec, method = "EM", control = list(max_iter=50, verbose=TRUE))

# Save Results
item_par=results$item_params
fit=results$model_fit
person_par=results$person_params

## ----simulation---------------------------------------------------------------
# --- 1. Simulation Helper Functions ---

# Function to simulate Binary 2PL
sim_2pl <- function(theta, a, b) {
  D <- 1.7
  prob <- 1 / (1 + exp(-D * a * (theta - b)))
  resp <- ifelse(runif(length(theta)) < prob, 1, 0)
  return(resp)
}

# Function to simulate Polytomous (GPCM structure to match fixed_item logic)
sim_poly <- function(theta, a, steps) {
  n_cat <- length(steps) + 1
  n_stud <- length(theta)
  
  # Calculate numerators for categories 0, 1, 2...
  # P(k) proportional to exp( sum(a*(theta - step_j)) )
  
  probs <- matrix(0, nrow = n_stud, ncol = n_cat)
  probs[, 1] <- 1 # Reference category (unnormalized exp is 1 or e^0)
  
  current_sum <- 0
  for (k in 2:n_cat) {
    # Step parameters correspond to boundaries
    current_sum <- current_sum + a * (theta - steps[k-1])
    probs[, k] <- exp(current_sum)
  }
  
  # Normalize
  prob_sum <- rowSums(probs)
  probs_final <- probs / prob_sum
  
  # Random draw
  resp <- numeric(n_stud)
  for (i in 1:n_stud) {
    resp[i] <- sample(0:(n_cat-1), 1, prob = probs_final[i, ])
  }
  return(resp)
}

# --- 2. Generate Parameters for 50 Items ---

n_students <- 5497
true_theta <- rnorm(n_students, mean = 0.4, sd = 1.7)

all_items_meta <- list()
response_matrix <- matrix(NA, nrow = n_students, ncol = 50)
colnames(response_matrix) <- paste0("Item_", 1:50)

# --- Set 1: Known Dichotomous (Items 1-10) ---
# Model: 2PL
for (i in 1:10) {
  a <- runif(1, 0.8, 1.4)
  b <- rnorm(1, 0, 1)
  resp <- sim_2pl(true_theta, a, b)
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Known", model = "2PL", a = a, b = b)
}

# --- Set 2: Known Polytomous (Items 11-18) ---
# Model: Poly (3 categories: 0, 1, 2). Requires 2 steps (d1, d2).
for (i in 11:18) {
  a <- runif(1, 0.7, 1.2)
  d1 <- rnorm(1, -0.5, 0.3)
  d2 <- rnorm(1, 0.5, 0.3) # Steps usually ordered
  resp <- sim_poly(true_theta, a, c(d1, d2))
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Known", model = "Poly", a = a, d1 = d1, d2 = d2)
}

# --- Set 3: Unknown Dichotomous (Items 19-40) ---
# 22 Items
for (i in 19:40) {
  a <- runif(1, 0.8, 1.4)
  b <- rnorm(1, 0, 1)
  resp <- sim_2pl(true_theta, a, b)
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Unknown", model = "2PL", a = a, b = b)
}

# --- Set 4: Unknown Polytomous (Items 41-50) ---
# 10 Items (Varying categories: some 3 cat, some 4 cat)
for (i in 41:50) {
  a <- runif(1, 0.7, 1.2)
  # Mix of 3 categories (2 steps) and 4 categories (3 steps)
  if (i %% 2 == 0) {
    steps <- c(-0.8, 0, 0.8) # 4 cats
  } else {
    steps <- c(-0.5, 0.5)    # 3 cats
  }
  resp <- sim_poly(true_theta, a, steps)
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Unknown", model = "Poly", a = a, steps = steps)
}
#check
response_df <- as.data.frame(response_matrix)
head(response_df[, c(1:3, 11:13, 41:43)]) # Peak at data

## -----------------------------------------------------------------------------
# Create empty dataframe structure
known_params_df <- data.frame(
  item = character(),
  model = character(),
  a = numeric(),
  b = numeric(),
  d1 = numeric(),
  d2 = numeric(),
  stringsAsFactors = FALSE
)

# Populate with Known Items (1-18)
for (i in 1:18) {
  meta <- all_items_meta[[i]]
  
  if (meta$model == "2PL") {
    # Add row for Binary
    known_params_df <- rbind(known_params_df, data.frame(
      item = meta$item,
      model = "2PL",
      a = meta$a,
      b = meta$b,
      d1 = NA, d2 = NA
    ))
  } else {
    # Add row for Poly
    # Note: Using "GPCM" model string as our function uses GPCM math
    known_params_df <- rbind(known_params_df, data.frame(
      item = meta$item,
      model = "GPCM", 
      a = meta$a,
      b = NA,
      d1 = meta$d1,
      d2 = meta$d2
    ))
  }
}

# Display input to verify
print("Known Item Parameters Input:")
print(known_params_df)


## -----------------------------------------------------------------------------
# Run the calibration
# We set model_default = "2PL" (this applies to unknown binary items).
# The function automatically detects poly items in the unknown set based on response categories.
results <- fixed_item(
  response_df = response_df, 
  item_params_df = known_params_df,
  control = list(max_iter = 100)
)

# Check that Items 1-18 are "Fixed" and match inputs.
# Check that Items 19-50 are "Estimated".
# Note how Poly items have step_1, step_2, etc.
item=results$item_params
person=results$person_params
fit=results$model_fit

## ----fix_person---------------------------------------------------------------
# --- Example: With Package Data ---
data("ela1", package = "tirt")

# Select Item Responses (Cols 1-30)
df_real <- ela1[, 1:30]
fixed_theta <- ela1$THETA
fixed_cov <- ela1$COVARIATE
real_res <- fix_person(df = df_real,
                            theta = fixed_theta,
                            model = "2PL",
                            covariate = fixed_cov)
head(real_res)

## ----equate_irt---------------------------------------------------------------
# Create dummy parameters for Form X (Base)
base_params <- data.frame(
  item = c("Item1", "Item2", "Item3", "Item4", "Item5"),
  model = "2PL",
  a = c(1.0, 1.2, 0.9, 1.1, 1.0),
  b = c(-1.0, -0.5, 0.0, 0.5, 1.0),
  stringsAsFactors = FALSE
)

# Create dummy parameters for Form Y (New)
# Suppose Form Y is harder and has different discrimination scaling
new_params <- data.frame(
  item = c("Item1", "Item2", "Item3", "Item6", "Item7"), # Items 1-3 are anchors
  model = "2PL",
  a = c(1.0, 1.2, 0.9, 1.1, 1.0) / 0.9,  # Scale shift
  b = (c(-1.0, -0.5, 0.0, 0.8, 1.2) * 0.9) + 0.2, # Location shift
  stringsAsFactors = FALSE
)

# Perform Equating
linked <- equate_irt(
  base_params = base_params,
  new_params = new_params,
  methods = c("Stocking-Lord", "Mean-Mean")
)

# View Linking Constants (A and B)
print(linked$linking_constants)

# View Transformed Parameters for Form Y (put on Form X scale)
print(linked$transformed_item_params)

#' Item Information Function
#'
#' @description
#' Computes the Fisher item information function for every item at a set of
#' ability (theta) points. The item information function shows how precisely an
#' item measures ability across the latent trait continuum: high information at
#' a given theta means the item discriminates well among examinees located
#' there. It is the building block of the test information function
#' (\code{\link{test_info}}) and of the conditional standard error of
#' measurement.
#'
#' @param item_params A data frame of item parameters, typically the
#'   \code{item_params} element returned by \code{\link{binary_irt}},
#'   \code{\link{polytomous_irt}}, or \code{\link{mixed_irt}}. The function
#'   auto-detects the model of each item from the available columns
#'   (discrimination, difficulty, guessing, thresholds/steps) or from a
#'   \code{model} column if present. All models in the package are supported:
#'   Rasch, 2PL, 3PL, GRM, GPCM, and PCM.
#' @param theta A numeric vector of ability values at which to evaluate the
#'   information (default = \code{seq(-4, 4, by = 0.1)}).
#' @param model Optional character string (applied to all items) or character
#'   vector of length equal to the number of items, forcing the model family
#'   used for each item. Advanced use only; when \code{NULL} (default) the model
#'   is detected automatically.
#' @param D Scaling constant for the dichotomous logistic metric (default
#'   \code{1}, matching the estimation functions in this package). Advanced
#'   users who calibrated on the normal-metric scale may set \code{D = 1.702}.
#'
#' @return A numeric matrix with one row per item and one column per theta point.
#'   Row names are the item names and column names are the theta values.
#'
#' @details
#' For dichotomous models the information is
#' \eqn{I(\theta) = (Da)^2 P(\theta)(1 - P(\theta))} for the Rasch and 2PL
#' models, and the standard Birnbaum form
#' \eqn{I(\theta) = (Da)^2 (1 - c)(1 - P^*)P^{*2} / P} for the 3PL model,
#' where \eqn{P^*} is the 2PL probability and \eqn{P} the full 3PL probability
#' (this reduces to the 2PL expression when the guessing parameter \eqn{c = 0}).
#' For polytomous models the general expected-information formula
#' \eqn{I(\theta) = \sum_k (\partial P_k / \partial \theta)^2 / P_k} is used,
#' which for the GPCM and PCM equals \eqn{(Da)^2 \mathrm{Var}(X \mid \theta)}.
#'
#' @seealso \code{\link{test_info}}, \code{\link{binary_irt}},
#'   \code{\link{polytomous_irt}}
#'
#' @examples
#'   # --- Example 1: dichotomous items ---
#'   set.seed(123)
#'   sim <- sim_irt(n_people = 400,
#'                  item_structure = list(list(model = "2PL", n_items = 6)))
#'   fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
#'                     control = list(max_iter = 15, verbose = FALSE))
#'
#'   # Information for each item at a grid of abilities
#'   info <- item_info(fit$item_params, theta = seq(-3, 3, by = 0.5))
#'   round(info, 3)
#'
#'   \donttest{
#'   # --- Example 2: polytomous (GRM) items ---
#'   simp <- sim_irt(n_people = 400,
#'                   item_structure = list(list(model = "GRM", n_items = 5,
#'                                              categories = 4)))
#'   fitp <- polytomous_irt(simp$resp, model = "GRM", method = "EM",
#'                          control = list(max_iter = 15, verbose = FALSE))
#'   item_info(fitp$item_params, theta = c(-2, -1, 0, 1, 2))
#'   }
#' @export
item_info <- function(item_params,
                      theta = seq(-4, 4, by = 0.1),
                      model = NULL,
                      D = 1) {

  parsed <- .tirt_parse_params(item_params, model = model, D = D)
  theta <- as.numeric(theta)

  info_mat <- matrix(NA_real_, nrow = parsed$n_items, ncol = length(theta))
  for (j in seq_len(parsed$n_items)) {
    info_mat[j, ] <- .tirt_item_info(parsed$spec[[j]], theta)
  }

  rownames(info_mat) <- parsed$item
  colnames(info_mat) <- round(theta, 4)
  info_mat
}


#' Test Information Function and Conditional Standard Error
#'
#' @description
#' Computes the test information function (TIF) as the sum of all item
#' information values at each ability point, together with the conditional
#' standard error of measurement (SEM). The test information function
#' summarizes where along the ability scale the test measures most precisely,
#' and the conditional SEM (\eqn{1 / \sqrt{TIF}}) translates that precision back
#' onto the theta metric. This is a routine part of test evaluation and form
#' assembly in operational testing programs.
#'
#' @param item_params A data frame of item parameters (see \code{\link{item_info}}).
#' @param theta A numeric vector of ability values at which to evaluate the
#'   test information (default = \code{seq(-4, 4, by = 0.1)}).
#' @param model Optional model override (see \code{\link{item_info}}).
#' @param D Scaling constant for the dichotomous logistic metric (default
#'   \code{1}).
#'
#' @return A data frame with one row per theta point and the columns:
#' \itemize{
#'   \item \code{theta}: the ability value.
#'   \item \code{test_info}: the test information (sum of item information).
#'   \item \code{sem}: the conditional standard error of measurement,
#'     \eqn{1 / \sqrt{test\_info}}.
#'   \item \code{reliability}: the marginal-style conditional reliability
#'     \eqn{TIF / (TIF + 1)} for a standard-normal ability scale.
#' }
#'
#' @seealso \code{\link{item_info}}
#'
#' @examples
#'   set.seed(123)
#'   sim <- sim_irt(n_people = 400,
#'                  item_structure = list(list(model = "2PL", n_items = 10)))
#'   fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
#'                     control = list(max_iter = 15, verbose = FALSE))
#'
#'   tif <- test_info(fit$item_params, theta = seq(-3, 3, by = 0.5))
#'   print(tif)
#'
#'   # The ability where the test is most informative
#'   tif$theta[which.max(tif$test_info)]
#' @export
test_info <- function(item_params,
                      theta = seq(-4, 4, by = 0.1),
                      model = NULL,
                      D = 1) {

  theta <- as.numeric(theta)
  info_mat <- item_info(item_params, theta = theta, model = model, D = D)

  tif <- colSums(info_mat, na.rm = TRUE)
  sem <- ifelse(tif > 0, 1 / sqrt(tif), NA_real_)

  data.frame(
    theta       = theta,
    test_info   = tif,
    sem         = sem,
    reliability = tif / (tif + 1),
    row.names   = NULL,
    stringsAsFactors = FALSE
  )
}

#' Differential Item Functioning (Mantel-Haenszel and Logistic Regression)
#'
#' @description
#' Detects Differential Item Functioning (DIF) for dichotomous items, that is,
#' items that behave differently for two groups of examinees (for example, a
#' reference and a focal group) after matching on ability. Two widely used
#' approaches are provided: the Mantel-Haenszel (1959) procedure, with the ETS
#' delta effect size and A/B/C flagging scheme (Holland & Thayer, 1988), and the
#' logistic regression approach (Swaminathan & Rogers, 1990), which separates
#' uniform and non-uniform DIF.
#'
#' @param data A data frame of dichotomous (0/1) item responses (rows = persons,
#'   columns = items).
#' @param group A vector of length equal to the number of rows in \code{data}
#'   giving the group membership of each person. It must have exactly two
#'   distinct non-missing values.
#' @param focal Optional. The value of \code{group} that identifies the focal
#'   group. If \code{NULL} (default), the second value (in sorted order) is used
#'   as the focal group and the first as the reference group.
#' @param match Optional numeric vector of matching scores (length equal to the
#'   number of rows in \code{data}). If \code{NULL} (default), the total number-
#'   correct score across all items is used.
#' @param purify Logical. If \code{TRUE}, the matching total score is computed
#'   as the rest score (total minus the studied item) separately for each item,
#'   which avoids contaminating the match with the item under study (default
#'   \code{FALSE}).
#'
#' @return A data frame with one row per item and the columns:
#' \itemize{
#'   \item \code{item}: the item name.
#'   \item \code{MH_chisq}, \code{MH_p}: the Mantel-Haenszel chi-square (with
#'     continuity correction) and its p-value.
#'   \item \code{MH_OR}: the Mantel-Haenszel common odds ratio.
#'   \item \code{MH_delta}: the ETS delta effect size, \eqn{-2.35 \ln(OR)}.
#'   \item \code{ETS_class}: the ETS DIF classification, \code{"A"} (negligible),
#'     \code{"B"} (moderate), or \code{"C"} (large).
#'   \item \code{LR_chisq}, \code{LR_p}: the 2-degree-of-freedom logistic
#'     regression test for combined (uniform + non-uniform) DIF.
#'   \item \code{LR_uniform_p}: p-value for uniform DIF (group main effect).
#'   \item \code{LR_nonuniform_p}: p-value for non-uniform DIF (group-by-match
#'     interaction).
#' }
#'
#' @references
#' Holland, P. W., & Thayer, D. T. (1988). Differential item performance and the
#' Mantel-Haenszel procedure. In H. Wainer & H. I. Braun (Eds.), \emph{Test
#' validity} (pp. 129-145). Erlbaum.
#'
#' Swaminathan, H., & Rogers, H. J. (1990). Detecting differential item
#' functioning using logistic regression procedures. \emph{Journal of
#' Educational Measurement, 27}(4), 361-370.
#'
#' @examples
#'   set.seed(123)
#'   sim <- sim_irt(n_people = 600,
#'                  item_structure = list(list(model = "2PL", n_items = 10)))
#'   resp <- sim$resp
#'
#'   # Create two groups and plant DIF in item 3 (harder for group "B")
#'   grp <- rep(c("A", "B"), each = 300)
#'   flip <- grp == "B" & resp[[3]] == 1
#'   resp[[3]][flip] <- rbinom(sum(flip), 1, 0.6)
#'
#'   dif_res <- dif(resp, group = grp)
#'   dif_res
#' @export
dif <- function(data,
                group,
                focal = NULL,
                match = NULL,
                purify = FALSE) {

  if (!is.data.frame(data) && !is.matrix(data)) stop("'data' must be a data frame or matrix.")
  resp <- as.matrix(data)
  N <- nrow(resp)
  J <- ncol(resp)
  item_names <- colnames(resp)
  if (is.null(item_names)) item_names <- paste0("Item_", seq_len(J))

  if (length(group) != N) stop("'group' must have the same length as the number of rows in 'data'.")

  # only dichotomous items are handled
  is_dich <- apply(resp, 2, function(x) all(x[!is.na(x)] %in% c(0, 1)))
  if (!all(is_dich)) {
    message(sprintf("Note: %d non-dichotomous item(s) detected and skipped. dif() supports dichotomous items.",
                    sum(!is_dich)))
  }

  grp_vals <- sort(unique(group[!is.na(group)]))
  if (length(grp_vals) != 2) stop("'group' must have exactly two distinct values.")
  if (is.null(focal)) focal <- grp_vals[2]
  if (!focal %in% grp_vals) stop("'focal' must be one of the values in 'group'.")
  ref <- setdiff(grp_vals, focal)

  is_focal <- group == focal
  is_ref   <- group == ref

  total_score <- rowSums(resp, na.rm = TRUE)
  if (is.null(match)) match <- total_score

  out <- vector("list", J)

  for (j in seq_len(J)) {
    y <- resp[, j]

    if (!is_dich[j]) {
      out[[j]] <- data.frame(item = item_names[j], MH_chisq = NA, MH_p = NA,
                             MH_OR = NA, MH_delta = NA, ETS_class = NA_character_,
                             LR_chisq = NA, LR_p = NA, LR_uniform_p = NA,
                             LR_nonuniform_p = NA, stringsAsFactors = FALSE)
      next
    }

    # matching variable (rest score if purify)
    mvar <- if (purify) total_score - y else match

    ok <- !is.na(y) & !is.na(mvar) & !is.na(group)

    # ---- Mantel-Haenszel over strata of the matching score ----
    strata <- split(which(ok), mvar[ok])
    num_or <- 0; den_or <- 0
    sum_a <- 0; sum_Ea <- 0; sum_Va <- 0

    for (idx in strata) {
      gy <- y[idx]; gg <- is_focal[idx]
      a1 <- sum(gy[!gg] == 1)   # reference correct
      b1 <- sum(gy[!gg] == 0)   # reference incorrect
      c1 <- sum(gy[gg]  == 1)   # focal correct
      d1 <- sum(gy[gg]  == 0)   # focal incorrect
      nk <- a1 + b1 + c1 + d1
      if (nk < 2) next
      num_or <- num_or + (a1 * d1) / nk
      den_or <- den_or + (b1 * c1) / nk

      nR <- a1 + b1; nF <- c1 + d1
      n1 <- a1 + c1; n0 <- b1 + d1
      sum_a  <- sum_a  + a1
      sum_Ea <- sum_Ea + nR * n1 / nk
      if (nk > 1) sum_Va <- sum_Va + (nR * nF * n1 * n0) / (nk^2 * (nk - 1))
    }

    mh_or <- if (den_or > 0) num_or / den_or else NA_real_
    mh_chisq <- if (sum_Va > 0) (abs(sum_a - sum_Ea) - 0.5)^2 / sum_Va else NA_real_
    mh_p <- if (is.na(mh_chisq)) NA_real_ else pchisq(mh_chisq, df = 1, lower.tail = FALSE)
    mh_delta <- if (!is.na(mh_or) && mh_or > 0) -2.35 * log(mh_or) else NA_real_

    # ETS A/B/C classification
    ets <- "A"
    if (!is.na(mh_delta) && !is.na(mh_p)) {
      sig <- mh_p < 0.05
      absd <- abs(mh_delta)
      if (sig && absd >= 1.5) ets <- "C"
      else if (sig && absd >= 1.0) ets <- "B"
      else ets <- "A"
    } else {
      ets <- NA_character_
    }

    # ---- Logistic regression DIF ----
    lr_chisq <- NA_real_; lr_p <- NA_real_; unif_p <- NA_real_; nonunif_p <- NA_real_
    dat_lr <- data.frame(y = y[ok], m = mvar[ok], g = factor(is_focal[ok]))
    if (length(unique(dat_lr$y)) == 2 && nrow(dat_lr) > 5) {
      m0 <- tryCatch(glm(y ~ m, data = dat_lr, family = binomial()), error = function(e) NULL)
      m1 <- tryCatch(glm(y ~ m + g, data = dat_lr, family = binomial()), error = function(e) NULL)
      m2 <- tryCatch(glm(y ~ m * g, data = dat_lr, family = binomial()), error = function(e) NULL)
      if (!is.null(m0) && !is.null(m2)) {
        lr_chisq <- as.numeric(m0$deviance - m2$deviance)
        lr_p <- pchisq(lr_chisq, df = 2, lower.tail = FALSE)
      }
      if (!is.null(m0) && !is.null(m1)) {
        unif_p <- pchisq(as.numeric(m0$deviance - m1$deviance), df = 1, lower.tail = FALSE)
      }
      if (!is.null(m1) && !is.null(m2)) {
        nonunif_p <- pchisq(as.numeric(m1$deviance - m2$deviance), df = 1, lower.tail = FALSE)
      }
    }

    out[[j]] <- data.frame(
      item = item_names[j],
      MH_chisq = round(mh_chisq, 3), MH_p = round(mh_p, 4),
      MH_OR = round(mh_or, 3), MH_delta = round(mh_delta, 3),
      ETS_class = ets,
      LR_chisq = round(lr_chisq, 3), LR_p = round(lr_p, 4),
      LR_uniform_p = round(unif_p, 4), LR_nonuniform_p = round(nonunif_p, 4),
      stringsAsFactors = FALSE
    )
  }

  res <- do.call(rbind, out)
  row.names(res) <- NULL
  attr(res, "reference") <- ref
  attr(res, "focal") <- focal
  res
}

# =============================================================================
#  Internal computational engine (not exported)
# -----------------------------------------------------------------------------
#  These helpers translate the item-parameter data frames produced by the
#  estimation functions (binary_irt(), polytomous_irt(), mixed_irt(),
#  mirt_binary(), trt_binary(), trt_poly(), irt_trt(), equate_irt(),
#  fixed_item()) into a single normalized representation, and evaluate
#  category probabilities, item information, and expected item scores from it.
#
#  Keeping one shared engine guarantees that item_info(), test_info(),
#  person_fit(), score_table(), ld_stats(), item_fit(), tcc() and
#  reliability() all speak the same "language" as the estimators.
# =============================================================================

# Numerically safe logistic function.
.tirt_plogis <- function(x) {
  x <- pmin(pmax(x, -30), 30)
  1 / (1 + exp(-x))
}

# --- Identify the column holding item names -------------------------------
.tirt_item_column <- function(params) {
  nm <- names(params)
  for (cand in c("item", "item_id", "Item", "ITEM")) {
    if (cand %in% nm) return(as.character(params[[cand]]))
  }
  if (!is.null(rownames(params)) && !all(rownames(params) == seq_len(nrow(params)))) {
    return(rownames(params))
  }
  paste0("Item_", seq_len(nrow(params)))
}

# --- Locate a single-value column from a set of aliases -------------------
.tirt_pick <- function(params, aliases) {
  nm <- tolower(names(params))
  for (a in aliases) {
    hit <- which(nm == tolower(a))
    if (length(hit) > 0) return(params[[hit[1]]])
  }
  NULL
}

# --- Locate the ordered threshold/step columns ----------------------------
# Returns a character vector of column names (in category order), excluding
# any standard-error, z-value or p-value companion columns.
.tirt_threshold_cols <- function(params) {
  nm <- names(params)
  low <- tolower(nm)

  is_val <- grepl("^(thresh(old)?_[0-9]+|step_[0-9]+|step[/.]threshold_[0-9]+|d[0-9]+)$", low)
  is_se  <- grepl("(_se|_zvalue|_pr_z|_pr|_z)$", low)
  keep   <- is_val & !is_se

  cols <- nm[keep]
  if (length(cols) == 0) return(character(0))

  # order by the trailing integer
  ord <- as.integer(sub(".*?([0-9]+)$", "\\1", tolower(cols)))
  cols[order(ord)]
}

# --- Map any model label (incl. testlet variants) to a base family --------
.tirt_base_model <- function(model_label, n_cat) {
  if (is.null(model_label) || is.na(model_label) || model_label == "") {
    return(if (n_cat > 2) "GRM" else "2PL")
  }
  m <- toupper(as.character(model_label))
  m <- gsub("[^A-Z0-9]", "", m)
  # dichotomous families
  if (m %in% c("RASCH", "1PL", "RASCHT", "1PLT")) return("RASCH")
  if (m %in% c("2PL", "2PLT")) return("2PL")
  if (m %in% c("3PL", "3PLT")) return("3PL")
  # polytomous families
  if (m %in% c("GRM", "GRT")) return("GRM")
  if (m %in% c("GPCM", "GPCMT", "GPCT")) return("GPCM")
  if (m %in% c("PCM", "PCMT", "PCT")) return("PCM")
  # bifactor / unknown: decide from category count
  if (n_cat > 2) "GRM" else "2PL"
}

# =============================================================================
#  .tirt_parse_params()
#  Convert an item-parameter data frame into a list of per-item "specs".
#  Each spec is a list: list(model, a, b, c, n_cat) where
#    - b is the difficulty (binary) or an ordered vector of thresholds/steps.
#    - For GRM, b holds category thresholds; for GPCM/PCM, b holds the
#      cumulative category intercepts (the parameterization used internally
#      by polytomous_irt()/mixed_irt()).
# =============================================================================
.tirt_parse_params <- function(params, model = NULL, D = 1) {

  if (!is.data.frame(params)) {
    stop("'item_params' must be a data frame of item parameters ",
         "(for example, the $item_params element returned by binary_irt(), ",
         "polytomous_irt(), or mixed_irt()).")
  }

  item_names <- .tirt_item_column(params)
  J <- nrow(params)

  a_col <- .tirt_pick(params, c("discrimination", "a", "slope"))
  c_col <- .tirt_pick(params, c("guess", "guessing", "c"))
  b_col <- .tirt_pick(params, c("difficulty", "b", "location"))
  model_col <- .tirt_pick(params, c("model"))
  thr_cols  <- .tirt_threshold_cols(params)

  # User-supplied model override (single value or length-J vector)
  if (!is.null(model)) {
    if (length(model) == 1) model <- rep(model, J)
    if (length(model) != J) stop("'model' must have length 1 or match the number of items.")
  }

  specs <- vector("list", J)

  for (j in seq_len(J)) {
    a_j <- if (!is.null(a_col)) suppressWarnings(as.numeric(a_col[j])) else NA_real_
    c_j <- if (!is.null(c_col)) suppressWarnings(as.numeric(c_col[j])) else 0
    if (is.na(c_j)) c_j <- 0

    # gather non-missing threshold/step values for this item
    thr_j <- numeric(0)
    if (length(thr_cols) > 0) {
      raw <- suppressWarnings(as.numeric(unlist(params[j, thr_cols])))
      thr_j <- raw[!is.na(raw)]
    }

    if (length(thr_j) >= 1) {
      # --- polytomous item ---
      n_cat <- length(thr_j) + 1L
      lab <- if (!is.null(model))       model[j]
             else if (!is.null(model_col)) model_col[j]
             else NULL

      if (is.null(lab)) {
        # No explicit model label (e.g. polytomous_irt() output has no 'model'
        # column): infer the polytomous family from the column naming
        # convention -- 'step_' means GPCM/PCM, 'thresh_' means GRM.
        fam <- if (any(grepl("step", tolower(thr_cols)))) "GPCM" else "GRM"
      } else {
        fam <- .tirt_base_model(lab, n_cat)
        if (fam %in% c("RASCH", "2PL", "3PL")) {
          # label says binary but thresholds are present -> infer from names
          fam <- if (any(grepl("step", tolower(thr_cols)))) "GPCM" else "GRM"
        }
      }
      if (fam == "PCM") { fam <- "GPCM"; a_j <- 1 }
      if (is.na(a_j)) a_j <- 1

      specs[[j]] <- list(model = fam, a = a_j, b = thr_j, c = 0, n_cat = n_cat, D = D)

    } else {
      # --- dichotomous item ---
      b_j <- if (!is.null(b_col)) suppressWarnings(as.numeric(b_col[j])) else NA_real_
      lab <- if (!is.null(model))       model[j]
             else if (!is.null(model_col)) model_col[j]
             else NULL

      fam <- .tirt_base_model(lab, 2)
      if (fam %in% c("GRM", "GPCM", "PCM")) fam <- "2PL"  # label mismatch safety

      if (is.null(lab)) {
        # infer family from available columns
        if (!is.null(c_col) && !is.na(suppressWarnings(as.numeric(c_col[j])))) fam <- "3PL"
        else if (!is.null(a_col)) fam <- "2PL"
        else fam <- "RASCH"
      }
      if (fam == "RASCH") a_j <- 1
      if (is.na(a_j)) a_j <- 1
      if (fam != "3PL") c_j <- 0

      specs[[j]] <- list(model = fam, a = a_j, b = b_j, c = c_j, n_cat = 2L, D = D)
    }
  }

  list(item = item_names, spec = specs, n_items = J)
}

# =============================================================================
#  Category probabilities for one item spec at a vector of theta values.
#  Returns a matrix with length(theta) rows and n_cat columns.
# =============================================================================
.tirt_cat_probs <- function(spec, theta) {
  D <- spec$D
  a <- spec$a
  nq <- length(theta)

  if (spec$model %in% c("RASCH", "2PL", "3PL")) {
    b <- spec$b
    if (is.na(a) || is.na(b)) return(matrix(NA_real_, nq, 2))
    g <- .tirt_plogis(D * a * (theta - b))
    P <- spec$c + (1 - spec$c) * g
    return(cbind(1 - P, P))
  }

  if (spec$model == "GRM") {
    b <- spec$b
    K <- spec$n_cat
    Pstar <- matrix(0, nq, K + 1)
    Pstar[, 1] <- 1
    Pstar[, K + 1] <- 0
    bb <- sort(b)
    for (k in seq_len(K - 1)) {
      Pstar[, k + 1] <- .tirt_plogis(D * a * (theta - bb[k]))
    }
    probs <- Pstar[, seq_len(K), drop = FALSE] - Pstar[, seq_len(K) + 1, drop = FALSE]
    return(pmax(probs, 1e-12))
  }

  # GPCM / PCM (cumulative-intercept parameterization; step_0 = 0)
  K <- spec$n_cat
  steps <- c(0, spec$b)              # length K
  num <- matrix(0, nq, K)
  for (k in seq_len(K)) {
    num[, k] <- D * a * (k - 1) * theta - steps[k]
  }
  num <- num - apply(num, 1, max)   # stabilize
  ex <- exp(num)
  ex / rowSums(ex)
}

# =============================================================================
#  Fisher item information at a vector of theta values (returns a vector).
# =============================================================================
.tirt_item_info <- function(spec, theta) {
  D <- spec$D
  a <- spec$a

  if (is.na(a)) return(rep(NA_real_, length(theta)))

  if (spec$model %in% c("RASCH", "2PL")) {
    b <- spec$b
    if (is.na(b)) return(rep(NA_real_, length(theta)))
    g <- .tirt_plogis(D * a * (theta - b))
    return((D * a)^2 * g * (1 - g))
  }

  if (spec$model == "3PL") {
    b <- spec$b
    if (is.na(b)) return(rep(NA_real_, length(theta)))
    cc <- spec$c
    g <- .tirt_plogis(D * a * (theta - b))
    P <- cc + (1 - cc) * g
    P <- pmin(pmax(P, 1e-10), 1 - 1e-10)
    # Standard Birnbaum 3PL information; reduces to 2PL when c = 0.
    return((D * a)^2 * (1 - cc) * (1 - g) * g^2 / P)
  }

  if (spec$model == "GRM") {
    b <- sort(spec$b)
    K <- spec$n_cat
    nq <- length(theta)
    Pstar <- matrix(0, nq, K + 1)
    Pstar[, 1] <- 1
    Pstar[, K + 1] <- 0
    for (k in seq_len(K - 1)) Pstar[, k + 1] <- .tirt_plogis(D * a * (theta - b[k]))
    dPstar <- Pstar * (1 - Pstar) * (D * a)   # derivative of each cumulative prob
    dPstar[, 1] <- 0
    dPstar[, K + 1] <- 0
    probs <- pmax(Pstar[, seq_len(K), drop = FALSE] - Pstar[, seq_len(K) + 1, drop = FALSE], 1e-12)
    dprob <- dPstar[, seq_len(K), drop = FALSE] - dPstar[, seq_len(K) + 1, drop = FALSE]
    return(rowSums(dprob^2 / probs))
  }

  # GPCM / PCM : information = (D a)^2 * Var(score | theta)
  probs <- .tirt_cat_probs(spec, theta)
  scores <- 0:(spec$n_cat - 1)
  ex <- as.vector(probs %*% scores)
  ex2 <- as.vector(probs %*% (scores^2))
  (D * a)^2 * (ex2 - ex^2)
}

# =============================================================================
#  Expected item score at a vector of theta values (returns a vector).
# =============================================================================
.tirt_expected_score <- function(spec, theta) {
  probs <- .tirt_cat_probs(spec, theta)
  scores <- 0:(spec$n_cat - 1)
  as.vector(probs %*% scores)
}

# =============================================================================
#  Maximum possible score for an item (n_cat - 1).
# =============================================================================
.tirt_max_score <- function(spec) spec$n_cat - 1L

# =============================================================================
#  Normalize a supplied theta argument (vector, or a person-parameter data
#  frame with an 'ability'/'theta' column) into a plain numeric vector.
# =============================================================================
.tirt_extract_theta <- function(theta) {
  if (is.data.frame(theta)) {
    for (cand in c("ability", "theta", "Theta", "EAP")) {
      if (cand %in% names(theta)) return(as.numeric(theta[[cand]]))
    }
    stop("Could not find an 'ability' or 'theta' column in the supplied person parameters.")
  }
  as.numeric(theta)
}

# Internal: Lord-Wingersky recursion.
# Given a list of item category-probability matrices (each nodes x n_cat),
# returns a matrix (nodes x (max_total_score + 1)) of the probability of every
# possible summed score at each ability node.
.tirt_lw_recursion <- function(prob_list) {
  n_nodes <- nrow(prob_list[[1]])
  # start: one item with a single "score 0" mass of 1
  dist <- matrix(1, n_nodes, 1)          # P(total = 0) = 1 before any item
  for (p in prob_list) {
    n_cat <- ncol(p)
    cur_max <- ncol(dist) - 1
    new_max <- cur_max + (n_cat - 1)
    new_dist <- matrix(0, n_nodes, new_max + 1)
    for (k in seq_len(n_cat)) {          # category score = k - 1
      s <- k - 1
      new_dist[, (1 + s):(cur_max + 1 + s)] <-
        new_dist[, (1 + s):(cur_max + 1 + s)] + dist * p[, k]
    }
    dist <- new_dist
  }
  dist
}

#' Summed-Score to Theta Conversion Table
#'
#' @description
#' Generates a raw-score (summed-score) to scale-score conversion table, mapping
#' every possible integer summed score to its corresponding ability estimate and
#' standard error. Such tables are standard in operational testing programs
#' because they let each examinee's number-correct (or number-of-points) score
#' be translated directly into a theta estimate without needing the full
#' response pattern. Three scoring methods are supported: expected a posteriori
#' (\code{"EAP"}), weighted likelihood (\code{"WLE"}), and maximum likelihood
#' (\code{"MLE"}).
#'
#' @param item_params A data frame of item parameters (the \code{item_params}
#'   element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
#'   \code{\link{mixed_irt}}). All models in the package are supported.
#' @param method Character. Scoring method: \code{"EAP"} (default),
#'   \code{"WLE"}, or \code{"MLE"}.
#' @param prior_mean,prior_sd Mean and standard deviation of the normal
#'   population prior used for the EAP method and for the quadrature weighting
#'   (defaults \code{0} and \code{1}).
#' @param theta_range Numeric vector of length 2 giving the ability grid bounds
#'   (default \code{c(-4, 4)}).
#' @param n_points Integer. Number of ability grid points used to evaluate the
#'   summed-score distributions (default \code{81}).
#' @param model Optional model override (see \code{\link{item_info}}).
#' @param D Scaling constant for the dichotomous logistic metric (default
#'   \code{1}).
#'
#' @return A data frame with one row per possible summed score and the columns:
#' \itemize{
#'   \item \code{summed_score}: the integer summed score (0 to the maximum
#'     possible score).
#'   \item \code{theta}: the ability estimate for that summed score.
#'   \item \code{se}: the standard error of the ability estimate.
#' }
#'
#' @details
#' The distribution of the summed score at each ability grid point is obtained
#' with the Lord-Wingersky (1984) recursion, which works for any mixture of
#' dichotomous and polytomous items. The \code{"EAP"} estimate is the mean of
#' the posterior of theta given the summed score, with \code{se} equal to the
#' posterior standard deviation. The \code{"MLE"} estimate maximizes the
#' summed-score likelihood; the \code{"WLE"} estimate (Warm, 1989) maximizes the
#' summed-score likelihood weighted by the square root of the test information,
#' which reduces the bias of the MLE. At the two extreme summed scores the MLE is
#' not finite and is reported at the boundary of \code{theta_range} with a missing
#' standard error; the WLE remains finite there and is estimated normally.
#'
#' @references
#' Lord, F. M., & Wingersky, M. S. (1984). Comparison of IRT true-score and
#' equipercentile observed-score equatings. \emph{Applied Psychological
#' Measurement, 8}(4), 453-461.
#'
#' Warm, T. A. (1989). Weighted likelihood estimation of ability in item
#' response theory. \emph{Psychometrika, 54}(3), 427-450.
#'
#' @examples
#'   set.seed(123)
#'   sim <- sim_irt(n_people = 400,
#'                  item_structure = list(list(model = "2PL", n_items = 10)))
#'   fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
#'                     control = list(max_iter = 15, verbose = FALSE))
#'
#'   # EAP conversion table (0 to 10 correct)
#'   score_table(fit$item_params, method = "EAP")
#'
#'   \donttest{
#'   # Maximum-likelihood conversion table
#'   score_table(fit$item_params, method = "MLE")
#'   }
#' @export
score_table <- function(item_params,
                        method = c("EAP", "WLE", "MLE"),
                        prior_mean = 0,
                        prior_sd = 1,
                        theta_range = c(-4, 4),
                        n_points = 81,
                        model = NULL,
                        D = 1) {

  method <- match.arg(method)
  parsed <- .tirt_parse_params(item_params, model = model, D = D)

  # Drop items whose parameters are missing (0-variance items etc.)
  keep <- vapply(parsed$spec, function(s) {
    !is.na(s$a) && !(s$model %in% c("RASCH", "2PL", "3PL") && is.na(s$b))
  }, logical(1))
  specs <- parsed$spec[keep]
  if (length(specs) == 0) stop("No usable items found in 'item_params'.")

  max_score <- sum(vapply(specs, .tirt_max_score, integer(1)))

  nodes <- seq(theta_range[1], theta_range[2], length.out = n_points)
  prior <- dnorm(nodes, prior_mean, prior_sd)
  prior <- prior / sum(prior)

  # Summed-score distributions at each node (nodes x (max_score + 1))
  prob_list <- lapply(specs, .tirt_cat_probs, theta = nodes)
  lw <- .tirt_lw_recursion(prob_list)

  # Test information at each node (for the WLE weight)
  info_mat <- vapply(specs, .tirt_item_info, numeric(length(nodes)), theta = nodes)
  tif <- rowSums(matrix(info_mat, nrow = length(nodes)))

  scores <- 0:max_score
  theta_hat <- rep(NA_real_, length(scores))
  se_hat    <- rep(NA_real_, length(scores))

  for (r in scores) {
    lik <- lw[, r + 1]                                  # P(sum = r | node)

    if (method == "EAP") {
      post <- lik * prior
      denom <- sum(post)
      if (denom <= 0) next
      post <- post / denom
      m  <- sum(post * nodes)
      v  <- sum(post * (nodes - m)^2)
      theta_hat[r + 1] <- m
      se_hat[r + 1]    <- sqrt(v)

    } else {
      # MLE / WLE: maximize a (weighted) log-likelihood over the grid
      logL <- log(pmax(lik, 1e-300))
      if (method == "WLE") logL <- logL + 0.5 * log(pmax(tif, 1e-300))

      # The MLE diverges at the two extreme scores (monotone likelihood); Warm's
      # WLE stays finite there, so only the MLE is pinned to the boundary.
      if (method == "MLE" && (r == 0 || r == max_score)) {
        theta_hat[r + 1] <- if (r == 0) theta_range[1] else theta_range[2]
        se_hat[r + 1]    <- NA_real_
        next
      }

      best <- which.max(logL)
      # parabolic refinement around the grid maximum
      if (best > 1 && best < length(nodes)) {
        y1 <- logL[best - 1]; y2 <- logL[best]; y3 <- logL[best + 1]
        denom <- (y1 - 2 * y2 + y3)
        h <- nodes[2] - nodes[1]
        shift <- if (denom != 0) 0.5 * (y1 - y3) / denom else 0
        shift <- max(min(shift, 1), -1)
        theta_hat[r + 1] <- nodes[best] + shift * h
        # curvature -> standard error
        curv <- -denom / (h^2)
        se_hat[r + 1] <- if (curv > 0) 1 / sqrt(curv) else NA_real_
      } else {
        theta_hat[r + 1] <- nodes[best]
        se_hat[r + 1]    <- NA_real_
      }
    }
  }

  data.frame(
    summed_score = scores,
    theta        = round(theta_hat, 3),
    se           = round(se_hat, 3),
    row.names    = NULL,
    stringsAsFactors = FALSE
  )
}

#' Simulate Multidimensional Item Response Theory Data
#'
#' @description
#' Simulates responses from compensatory multidimensional item response models,
#' the multidimensional counterpart of \code{\link{sim_irt}}. It supports the
#' multidimensional binary models estimated by \code{\link{mirt_binary}}
#' (M-Rasch, M-2PL, M-3PL) as well as multidimensional polytomous models
#' (multidimensional GPCM and GRM). Each block of items may load on any subset of
#' the latent dimensions, so both simple-structure and complex-structure designs
#' are easy to generate.
#'
#' @param n_people Integer. Number of examinees.
#' @param item_structure List of lists defining item blocks (see Details).
#' @param dimension Integer. Number of latent dimensions (D).
#' @param theta Numeric matrix (Optional). An \code{n_people} x \code{dimension}
#'   matrix of abilities. If supplied it is used directly.
#' @param theta_mean Numeric. Mean of the latent traits, recycled across
#'   dimensions (used when \code{theta} is \code{NULL}).
#' @param Sigma Numeric matrix (Optional). A \code{dimension} x \code{dimension}
#'   covariance matrix for the latent traits (used when \code{theta} is
#'   \code{NULL}). Defaults to the identity matrix (uncorrelated dimensions).
#'
#' @details
#' Each element of \code{item_structure} is a list describing one block:
#' \itemize{
#'   \item \code{model}: one of \code{"M2PL"}, \code{"M3PL"}, \code{"MRasch"}
#'     (dichotomous) or \code{"MGPCM"}, \code{"MGRM"} (polytomous). The
#'     non-prefixed names (\code{"2PL"}, \code{"GRM"}, ...) are also accepted.
#'   \item \code{n_items}: number of items in the block.
#'   \item \code{dims}: integer vector of the dimensions the block loads on
#'     (default: all dimensions). Slopes on the remaining dimensions are 0.
#'   \item \code{a}: discrimination/slope, given as a range \code{c(lo, hi)} to
#'     sample from, or a single fixed value (default \code{c(0.8, 1.8)}).
#'   \item \code{d}: intercept, given as a range or a fixed value (default
#'     \code{c(-1.5, 1.5)}). Note the compensatory model uses the intercept
#'     metric \eqn{z = a'\theta + d}, matching \code{mirt_binary()}.
#'   \item \code{c}: lower asymptote for \code{"M3PL"} (default \code{0.2}).
#'   \item \code{categories}: number of categories for polytomous blocks
#'     (default \code{3}).
#' }
#'
#' @return A list containing:
#'   \item{resp}{data.frame of responses (rows = people, cols = items).}
#'   \item{true_params}{data.frame of true item parameters, including one
#'     \code{a_Dim} column per dimension, the intercept \code{d}, guessing, and
#'     step/threshold columns for polytomous items.}
#'   \item{theta}{the \code{n_people} x \code{dimension} matrix of true abilities.}
#'
#' @seealso \code{\link{mirt_binary}}, \code{\link{sim_irt}}
#'
#' @examples
#'   # Two correlated dimensions, simple structure
#'   set.seed(2025)
#'   Sigma <- matrix(c(1, 0.4, 0.4, 1), 2, 2)
#'
#'   design <- list(
#'     list(model = "M2PL", n_items = 8, dims = 1),          # loads on Dim 1
#'     list(model = "M2PL", n_items = 8, dims = 2),          # loads on Dim 2
#'     list(model = "M3PL", n_items = 4, dims = c(1, 2), c = 0.2)  # both dims
#'   )
#'
#'   sim <- sim_mirt(n_people = 500, item_structure = design,
#'                   dimension = 2, Sigma = Sigma)
#'
#'   head(sim$resp)
#'   head(sim$true_params)
#'
#'   \donttest{
#'   # Recover with the multidimensional estimator
#'   Q <- as.matrix(sim$true_params[, c("a_Dim1", "a_Dim2")] != 0) * 1
#'   fit <- mirt_binary(sim$resp, model = "2PL", dimension = 2,
#'                      control = list(Q_matrix = Q, max_iter = 10, verbose = FALSE))
#'   head(fit$item_params)
#'   }
#' @export
sim_mirt <- function(n_people = 1000,
                     item_structure = list(),
                     dimension = 2,
                     theta = NULL,
                     theta_mean = 0,
                     Sigma = NULL) {

  # --- 1. Validation ---
  if (!is.numeric(n_people) || n_people < 1) stop("Error: 'n_people' must be a positive integer.")
  if (length(item_structure) == 0) stop("Error: 'item_structure' cannot be empty.")
  D <- as.integer(dimension)
  if (D < 1) stop("Error: 'dimension' must be a positive integer.")

  message("----------------------------------------------------------------")
  message(sprintf("Starting Multidimensional Simulation (N = %d, D = %d)...", n_people, D))

  # --- 2. Generate Abilities ---
  if (!is.null(theta)) {
    theta <- as.matrix(theta)
    if (nrow(theta) != n_people || ncol(theta) != D) {
      stop(sprintf("Error: 'theta' must be a %d x %d matrix.", n_people, D))
    }
    true_theta <- theta
    message(">> Ability (Theta): Using user-supplied matrix.")
  } else {
    if (is.null(Sigma)) Sigma <- diag(D)
    if (!all(dim(Sigma) == c(D, D))) stop(sprintf("Error: 'Sigma' must be a %d x %d matrix.", D, D))
    mu <- rep(theta_mean, length.out = D)
    Z <- matrix(rnorm(n_people * D), n_people, D)
    true_theta <- Z %*% chol(Sigma)
    true_theta <- sweep(true_theta, 2, mu, "+")
    message(sprintf(">> Ability (Theta): Generated from MVN with mean %.2f and supplied Sigma.", theta_mean))
  }

  valid_dich <- c("MRASCH", "M2PL", "M3PL")
  valid_poly <- c("MGPCM", "MGRM")

  # normalize a model label to its multidimensional form
  norm_model <- function(m) {
    mm <- toupper(m)
    switch(mm,
           "RASCH" = , "MRASCH" = "MRASCH",
           "2PL" = , "M2PL" = "M2PL",
           "3PL" = , "M3PL" = "M3PL",
           "GPCM" = , "MGPCM" = "MGPCM",
           "PCM" = "MGPCM",
           "GRM" = , "MGRM" = "MGRM",
           mm)
  }

  response_list <- list()
  param_list <- list()
  current_item_idx <- 1

  # smart parameter parser (mirrors sim_irt)
  get_param <- function(block, name, default, n_req, friendly) {
    val <- block[[name]]
    if (is.null(val)) {
      if (length(default) == 2) return(runif(n_req, default[1], default[2]))
      return(rep(default, n_req))
    }
    if (length(val) == n_req) return(val)
    if (length(val) == 2 && val[1] < val[2]) return(runif(n_req, val[1], val[2]))
    if (length(val) == 1) return(rep(val, n_req))
    stop(sprintf("Parameter '%s' length mismatch (expected %d, or a range).", name, n_req))
  }

  for (i in seq_along(item_structure)) {
    block <- item_structure[[i]]
    if (is.null(block$model)) stop(sprintf("Block %d Error: Missing 'model'.", i))
    if (is.null(block$n_items)) stop(sprintf("Block %d Error: Missing 'n_items'.", i))

    model <- norm_model(block$model)
    n_items <- block$n_items
    if (!model %in% c(valid_dich, valid_poly)) {
      stop(sprintf("Block %d Error: Invalid model '%s'.", i, block$model))
    }

    # dimensions this block loads on
    dims <- block$dims
    if (is.null(dims)) dims <- seq_len(D)
    if (any(dims < 1 | dims > D)) stop(sprintf("Block %d Error: 'dims' out of range.", i))

    # categories
    if (model %in% valid_dich) {
      cats <- 2
    } else {
      cats <- if (is.null(block$categories)) 3 else block$categories
      if (cats < 2) stop(sprintf("Block %d Error: 'categories' must be >= 2.", i))
    }

    message(sprintf(">> Block %d: %d items (%s), loading on dim(s) %s",
                    i, n_items, model, paste(dims, collapse = ",")))

    # slopes: one value per loaded dimension per item
    a_mat <- matrix(0, n_items, D)
    if (model == "MRASCH") {
      a_mat[, dims] <- 1
    } else {
      for (dd in dims) a_mat[, dd] <- get_param(block, "a", c(0.8, 1.8), n_items, "Slope a")
    }

    d_int <- get_param(block, "d", c(-1.5, 1.5), n_items, "Intercept d")
    c_p <- if (model == "M3PL") get_param(block, "c", 0.2, n_items, "Guessing c") else rep(0, n_items)

    block_params <- data.frame(
      item_id = paste0("item_", current_item_idx:(current_item_idx + n_items - 1)),
      block = i,
      model = model,
      categories = cats,
      stringsAsFactors = FALSE
    )
    for (dd in seq_len(D)) block_params[[paste0("a_Dim", dd)]] <- a_mat[, dd]
    block_params$d <- d_int
    block_params$guessing <- c_p

    block_resp <- matrix(NA, n_people, n_items)
    theta_a <- true_theta   # N x D

    if (cats == 2) {
      for (j in seq_len(n_items)) {
        z <- as.vector(theta_a %*% a_mat[j, ]) + d_int[j]
        prob <- c_p[j] + (1 - c_p[j]) / (1 + exp(-z))
        block_resp[, j] <- ifelse(runif(n_people) < prob, 1, 0)
      }
    } else {
      for (j in seq_len(n_items)) {
        comp <- as.vector(theta_a %*% a_mat[j, ])         # composite ability
        d_k <- sort(rnorm(cats - 1, 0, 1), decreasing = TRUE) + d_int[j]
        for (k in seq_len(cats - 1)) block_params[j, paste0("step_", k)] <- d_k[k]

        if (model == "MGRM") {
          prob_cum <- matrix(0, n_people, cats + 1)
          prob_cum[, 1] <- 1
          for (k in seq_len(cats - 1)) prob_cum[, k + 1] <- 1 / (1 + exp(-(comp + d_k[k])))
          probs <- prob_cum[, seq_len(cats)] - prob_cum[, seq_len(cats) + 1]
        } else {                                          # MGPCM
          numer <- matrix(0, n_people, cats)
          running <- 0
          for (k in seq_len(cats - 1)) {
            running <- running + (comp + d_k[k])
            numer[, k + 1] <- running
          }
          numer <- numer - apply(numer, 1, max)
          probs <- exp(numer); probs <- probs / rowSums(probs)
        }
        cum_probs <- t(apply(probs, 1, cumsum))
        rand_vals <- runif(n_people)
        block_resp[, j] <- apply(cum_probs >= rand_vals, 1, function(x) match(TRUE, x)) - 1
      }
    }

    response_list[[i]] <- block_resp
    param_list[[i]] <- block_params
    current_item_idx <- current_item_idx + n_items
  }

  # --- Final Assembly ---
  message("----------------------------------------------------------------")
  message("Constructing final data frames...")

  all_resp <- as.data.frame(do.call(cbind, response_list))
  all_cols <- unique(unlist(lapply(param_list, names)))
  all_params <- do.call(rbind, lapply(param_list, function(df) {
    missing_cols <- setdiff(all_cols, names(df))
    if (length(missing_cols) > 0) df[missing_cols] <- NA
    df[, all_cols]
  }))
  row.names(all_resp) <- NULL
  row.names(all_params) <- NULL
  colnames(all_resp) <- all_params$item_id
  colnames(true_theta) <- paste0("theta_Dim", seq_len(D))

  message("Simulation Complete.")
  message(sprintf("Summary: %d items, %d examinees, %d dimensions.", ncol(all_resp), n_people, D))
  message("----------------------------------------------------------------")

  list(resp = all_resp, true_params = all_params, theta = true_theta)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(tirt)

## ----calibrate----------------------------------------------------------------
set.seed(2025)
sim <- sim_irt(
  n_people = 600,
  item_structure = list(list(model = "2PL", n_items = 12))
)

fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                  control = list(max_iter = 20, verbose = FALSE))
head(fit$item_params)

## ----information--------------------------------------------------------------
theta_grid <- seq(-3, 3, by = 0.5)

# Item information (items in rows, theta in columns)
info <- item_info(fit$item_params, theta = theta_grid)
round(info[1:3, ], 3)

# Test information function and conditional SEM
tif <- test_info(fit$item_params, theta = theta_grid)
tif

# Where does the test measure most precisely?
tif$theta[which.max(tif$test_info)]

## ----score_table--------------------------------------------------------------
# EAP conversion table (0 to 12 correct)
score_table(fit$item_params, method = "EAP")

# Maximum-likelihood conversion
score_table(fit$item_params, method = "MLE")

## ----person_fit---------------------------------------------------------------
pf <- person_fit(sim$resp, fit$item_params, fit$person_params)
head(pf)

# Number of examinees flagged as potentially misfitting
sum(pf$flag, na.rm = TRUE)

## ----item_fit-----------------------------------------------------------------
item_fit(sim$resp, fit$item_params)

## ----ld_stats-----------------------------------------------------------------
q3 <- ld_stats(sim$resp, fit$item_params)
round(q3[1:5, 1:5], 3)

# Largest absolute residual correlation
attr(q3, "max_abs_q3")

## ----dif----------------------------------------------------------------------
resp_dif <- sim$resp
grp <- rep(c("Reference", "Focal"), each = 300)
flip <- grp == "Focal" & resp_dif[[3]] == 1
resp_dif[[3]][flip] <- rbinom(sum(flip), 1, 0.55)

dif(resp_dif, group = grp)[, c("item", "MH_delta", "ETS_class", "LR_p")]

## ----reliability--------------------------------------------------------------
reliability(person_params = fit$person_params,
            data = sim$resp,
            item_params = fit$item_params)

## ----tcc----------------------------------------------------------------------
curves <- tcc(fit$item_params, theta = seq(-3, 3, by = 1))
curves$test_curve

## ----sim_mirt-----------------------------------------------------------------
# Two correlated dimensions, simple structure
mdat <- sim_mirt(
  n_people = 400,
  dimension = 2,
  Sigma = matrix(c(1, 0.4, 0.4, 1), 2, 2),
  item_structure = list(
    list(model = "M2PL", n_items = 6, dims = 1),
    list(model = "M2PL", n_items = 6, dims = 2)
  )
)
head(mdat$true_params)

## ----sim_tirt-----------------------------------------------------------------
# Independent items plus two testlets
tdat <- sim_tirt(
  n_people = 400,
  item_structure = list(
    list(model = "2PL",  n_items = 6),
    list(model = "2PLT", n_items = 4, testlet_id = "P1", testlet_var = 0.6),
    list(model = "GPCT", n_items = 3, categories = 3, testlet_id = "P2")
  )
)
tdat$true_item_params[, c("item_id", "model", "testlet")]

## ----mixture------------------------------------------------------------------
set.seed(11)
N <- 300; J <- 8
b1 <- seq(-1.5, 1.5, length.out = J); b2 <- rev(b1)
theta <- rnorm(N); cls <- rep(1:2, each = N / 2)
rmat <- matrix(0, N, J)
for (i in 1:N) {
  b <- if (cls[i] == 1) b1 else b2
  rmat[i, ] <- rbinom(J, 1, 1 / (1 + exp(-(theta[i] - b))))
}
mdf <- as.data.frame(rmat); names(mdf) <- paste0("I", 1:J)

mix <- mixture_irt(mdf, n_class = 2, model = "Rasch",
                   control = list(max_iter = 40, verbose = FALSE))
mix$class_params
mix$model_fit

#' Joint Item Response Theory and Testlet Response Theory Estimation (Dichotomous & Polytomous)
#'
#' @description
#' Provides a unified marginal maximum likelihood estimation framework for
#' a broad class of item response theory and testlet response theory models. The function automatically detects data
#' structures to apply appropriate models, along with their testlet-effect extensions (Bradlow et al., 1999).
#'
#' @param data A \code{data.frame} or \code{matrix} containing item responses.
#'   Responses should be 0-indexed integers. Missing values should be coded as \code{NA}.
#' @param item_spec A \code{data.frame} providing item metadata. Must include
#'   columns \code{"item"} (matching \code{colnames(data)}) and \code{"model"}.
#'   Optionally includes a \code{"testlet"} column for TRT specifications.
#' @param method A character string specifying the estimation method. Currently
#'   supports \code{"EM"} (Expectation-Maximization). Defaults to \code{"EM"}.
#' @param control A \code{list} of control parameters for the estimation algorithm:
#'   \itemize{
#'     \item \code{max_iter}: Maximum number of EM iterations (default = 100).
#'     \item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
#'     \item \code{theta_range}: Numeric vector of length 2 specifying the integration
#'           grid bounds (default = c(-4, 4)).
#'     \item \code{quad_points}: Number of quadrature points (default = 21).
#'     \item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
#'     \item \code{fix_discrimination}: Logical; default=FALSE
#'   }
#'
#' @details
#' The estimation utilizes a robust Newton-Raphson update within the M-step. For
#' testlet models, dimension reduction is achieved through the integration of
#' the nuisance testlet effect (Li et al., 2006). The function automatically
#' corrects model specifications if the data levels (binary vs. polytomous)
#' do not align with the requested model string.
#'
#' @return A \code{list} containing three components:
#' \item{item_params}{A data frame of estimated item slopes (discrimination),
#'   difficulties/thresholds, and guessing parameters with associated standard errors.}
#' \item{person_params}{A data frame of EAP-based ability estimates (\eqn{\theta})
#'   and testlet effect estimates (\eqn{\gamma}).}
#' \item{model_fit}{A data frame containing Log-Likelihood, AIC, and BIC indices.}
#'
#' @references
#' Bradlow, E. T., Wainer, H., & Wang, X. (1999). A testlet response model for
#' multidimensionality in item response theory. \emph{Psychometrika, 64}(2), 147-168.
#'
#' Li, Y., Bolt, D. M., & Fu, J. (2006). A comparison of methods for estimating
#' secondary dimensions in testlet-based data. \emph{Applied Psychological
#' Measurement, 30}(3), 203-223.
#'
#' @importFrom stats dnorm qlogis var setNames
#' @examples
#'   # --- Quick Example (small data) ---
#'   set.seed(1)
#'   n <- 50
#'   resp <- data.frame(
#'     I1 = rbinom(n, 1, 0.6),
#'     I2 = rbinom(n, 1, 0.5),
#'     I3 = rbinom(n, 1, 0.4),
#'     I4 = sample(0:2, n, replace = TRUE),
#'     I5 = rbinom(n, 1, 0.5),
#'     I6 = rbinom(n, 1, 0.6)
#'   )
#'   spec <- data.frame(
#'     item = colnames(resp),
#'     model = c("2PL", "2PL", "2PL", "GRM", "2PLT", "2PLT"),
#'     testlet = c(NA, NA, NA, NA, "T1", "T1"),
#'     stringsAsFactors = FALSE
#'   )
#'   res <- irt_trt(resp, spec, method = "EM",
#'                  control = list(max_iter = 5, verbose = FALSE))
#'   head(res$item_params)
#'
#'   \donttest{
#'   # --- Full Example: Simulation (Binary + Poly + Testlets) ---
#'   set.seed(2025)
#'   N <- 100; J <- 20
#'
#'   # 1. Generate Parameters
#'   theta <- rnorm(N, 0, 1)
#'   gamma_1 <- rnorm(N, 0, 0.5) # Testlet 1 effect
#'   gamma_2 <- rnorm(N, 0, 0.6) # Testlet 2 effect
#'
#'   a_true <- runif(J, 0.8, 1.5)
#'   b_true <- seq(-1.5, 1.5, length.out = J)
#'
#'   resp_matrix <- matrix(NA, N, J)
#'   colnames(resp_matrix) <- paste0("Item_", 1:J)
#'
#'   # 2. Simulate Responses
#'   # Items 1-10: Binary Independent (Model: 2PL)
#'   for(j in 1:10) {
#'     p <- 1 / (1 + exp(-a_true[j] * (theta - b_true[j])))
#'     resp_matrix[,j] <- rbinom(N, 1, p)
#'   }
#'
#'   # Items 11-15: Poly Independent (Model: GRM)
#'   for(j in 11:15) {
#'     thresh <- sort(c(b_true[j] - 0.7, b_true[j] + 0.7))
#'     p1 <- 1 / (1 + exp(-a_true[j] * (theta - thresh[1])))
#'     p2 <- 1 / (1 + exp(-a_true[j] * (theta - thresh[2])))
#'     probs <- cbind(1-p1, p1-p2, p2)
#'     resp_matrix[,j] <- apply(probs, 1, function(p) sample(0:2, 1, prob=p))
#'   }
#'
#'   # Items 16-17: Binary Testlet 1 (Model: 2PLT)
#'   for(j in 16:17) {
#'     eff_theta <- theta + gamma_1
#'     p <- 1 / (1 + exp(-a_true[j] * (eff_theta - b_true[j])))
#'     resp_matrix[,j] <- rbinom(N, 1, p)
#'   }
#'
#'   # Items 18-20: Poly Testlet 2 (Model: GRT)
#'   for(j in 18:20) {
#'     eff_theta <- theta + gamma_2
#'     thresh <- sort(c(b_true[j] - 0.5, b_true[j] + 0.5))
#'     p1 <- 1 / (1 + exp(-a_true[j] * (eff_theta - thresh[1])))
#'     p2 <- 1 / (1 + exp(-a_true[j] * (eff_theta - thresh[2])))
#'     probs <- cbind(1-p1, p1-p2, p2)
#'     resp_matrix[,j] <- apply(probs, 1, function(p) sample(0:2, 1, prob=p))
#'   }
#'
#'   df_sim <- as.data.frame(resp_matrix)
#'
#'   # 3. Create Item Specification
#'   # STRICT naming: Independent=2PL/GRM, Testlet=2PLT/GRT
#'   spec <- data.frame(
#'     item = colnames(df_sim),
#'     model = c(rep("2PL", 10), rep("GRM", 5), rep("2PLT", 2), rep("GRT", 3)),
#'     testlet = c(rep(NA, 15), rep("T1", 2), rep("T2", 3)),
#'     stringsAsFactors = FALSE
#'   )
#'
#'   # 4. Run Estimation
#'   res <- irt_trt(df_sim, spec, method = "EM",
#'                  control = list(max_iter = 20, verbose = FALSE))
#'
#'   head(res$item_params)
#'   head(res$person_params)
#'   }
#' @export
irt_trt <- function(data,
                          item_spec,
                          method = "EM",
                          control = list()) {
  # --- 1. Setup and Defaults ---
  con <- list(
    max_iter = 100,
    converge_tol = 1e-4,
    theta_range = c(-4, 4),
    quad_points = 21,
    nr_max_iter = 20,
    nr_damp = 0.8,
    verbose = TRUE,
    fix_discrimination = FALSE
  )
  con[names(control)] <- control

  # --- Input Validation ---
  if(!is.data.frame(data)) stop("Input 'data' must be a data frame.")
  raw_mat <- as.matrix(data)
  if(!is.numeric(raw_mat)) stop("Input data must be numeric.")

  N <- nrow(raw_mat)
  J <- ncol(raw_mat)
  item_names <- colnames(data)
  if(is.null(item_names)) item_names <- paste0("Item_", 1:J)
  colnames(raw_mat) <- item_names

  # Validate spec
  if(!is.data.frame(item_spec)) stop("'item_spec' must be a data.frame.")
  spec_indices <- match(item_names, item_spec$item)
  if(any(is.na(spec_indices))) stop("Error: Item names in data do not match 'item_spec'.")

  item_spec <- item_spec[spec_indices, ]
  mod_vec <- as.character(item_spec$model)

  # Testlet setup
  testlet_vec <- rep(NA, J)
  t_names_map <- NULL
  if("testlet" %in% names(item_spec)) {
    raw_t <- as.character(item_spec$testlet)
    raw_t[raw_t == "" | is.na(raw_t)] <- NA
    unique_t <- unique(raw_t[!is.na(raw_t)])
    if(length(unique_t) > 0) {
      t_names_map <- unique_t
      t_map <- setNames(1:length(unique_t), unique_t)
      testlet_vec <- as.integer(t_map[raw_t])
    }
  }
  n_testlets <- if(all(is.na(testlet_vec))) 0 else max(testlet_vec, na.rm=TRUE)

  # Shift Polytomous Data
  min_val <- min(raw_mat, na.rm=TRUE)
  if(is.finite(min_val) && min_val > 0) {
    if(con$verbose) cat("Note: Input responses shifted to start at 0 (internal requirement).\n")
    raw_mat <- raw_mat - min_val
  }
  n_cats <- apply(raw_mat, 2, function(x) max(x, na.rm=TRUE) + 1)

  valid_items_idx <- which(apply(raw_mat, 2, var, na.rm=TRUE) > 0)


  mismatched_items <- c()

  for(j in 1:J) {
    old_mod <- mod_vec[j]
    new_mod <- old_mod # default assumption
    cats <- n_cats[j]

    # -----------------------------------------------------
    # CASE 1: Data is Polytomous (cats > 2), but Model is Binary
    # -----------------------------------------------------
    if(cats > 2) {
      # Standard Binary -> Poly Equivalent
      if(old_mod %in% c("2PL", "3PL", "1PL")) new_mod <- "GPCM"
      else if(old_mod == "Rasch") new_mod <- "PCM"

      # Testlet Binary -> Poly Testlet Equivalent
      else if(old_mod %in% c("2PLT", "3PLT", "1PLT")) new_mod <- "GPCMT"
      else if(old_mod == "RaschT") new_mod <- "PCMT"
    }

    # -----------------------------------------------------
    # CASE 2: Data is Binary (cats == 2), but Model is Poly
    # -----------------------------------------------------
    else if(cats == 2) {
      # Poly -> Binary Equivalent
      if(old_mod %in% c("GPCM", "GRM")) new_mod <- "2PL"
      else if(old_mod == "PCM") new_mod <- "Rasch"

      # Poly Testlet -> Binary Testlet Equivalent
      else if(old_mod %in% c("GPCMT", "GRT")) new_mod <- "2PLT"
      else if(old_mod == "PCMT") new_mod <- "RaschT"
    }

    # Apply Correction
    if(new_mod != old_mod) {
      mod_vec[j] <- new_mod
      mismatched_items <- c(mismatched_items, sprintf("%s (Data: %d cats) | Spec: %s -> Corrected: %s",
                                                      item_names[j], cats, old_mod, new_mod))
    }
  }


  # --- User-Friendly Start Message ---
  if(con$verbose) {
    cat("\n================================================\n")
    cat("        UNIVERSAL IRT/TRT ESTIMATION            \n")
    cat("================================================\n")

    # Summary of configurations
    n_binary <- sum(n_cats == 2)
    n_poly   <- sum(n_cats > 2)
    n_indep  <- sum(is.na(testlet_vec))
    n_testlet_items <- sum(!is.na(testlet_vec))

    cat("Data Summary:\n")
    cat(sprintf("  - Total Items: %d\n", J))
    cat(sprintf("  - Binary Items: %d\n", n_binary))
    cat(sprintf("  - Polytomous Items: %d\n", n_poly))
    cat("\nStructure:\n")
    cat(sprintf("  - Independent Items: %d\n", n_indep))
    cat(sprintf("  - Testlet Items: %d (grouped into %d testlets)\n", n_testlet_items, n_testlets))

    if(n_testlets > 0) {
      cat("\nNOTE on Testlets:\n")
      cat("  Any item with a value in the 'testlet' string within the item_spec statement will be estimated\n")
      cat("  as a Testlet item (Theta + Gamma), regardless of the model string.\n")
      cat("  (e.g., model='GPCM' + testlet='T1' -> Testlet GPCT).\n")
    }

    # --- Print Warnings for Mismatches ---
    if(length(mismatched_items) > 0) {
      cat("\n[!] WARNING: MODEL SPECIFICATION MISMATCH\n")
      cat("    The following items had models inconsistent with their data structure.\n")
      cat("    They have been automatically corrected to the appropriate model:\n")
      cat(paste("    -", mismatched_items, collapse="\n"))
      cat("\n")
    }

    cat("------------------------------------------------\n")
  }

  # --- Initialize Parameters ---
  a <- rep(1.0, J)
  b_list <- vector("list", J)
  g <- rep(0.0, J)

  # Initial Estimates (CTT)
  p_vals <- colMeans(raw_mat, na.rm=TRUE) / (n_cats - 1)
  p_vals <- pmin(pmax(p_vals, 0.01), 0.99)

  for(j in 1:J) {
    mod <- mod_vec[j]
    K <- n_cats[j]
    if(mod %in% c("Rasch", "RaschT", "PCM", "PCMT")) a[j] <- 1.0 else a[j] <- 1.0

    y <- raw_mat[,j]; y <- y[!is.na(y)]
    if(length(y) < 2) { b_list[[j]] <- rep(0, max(1, K-1)); next }

    if(K == 2) {
      b_list[[j]] <- -log(p_vals[j] / (1 - p_vals[j]))
      if(mod %in% c("3PL", "3PLT")) g[j] <- 0.2
    } else {
      props <- table(factor(y, levels=0:(K-1))) / length(y)
      cum <- cumsum(props)[1:(K-1)]; cum <- pmin(pmax(cum, 0.01), 0.99)
      thresh <- -qlogis(cum)
      if(mod %in% c("GRM", "GRT")) b_list[[j]] <- sort(thresh)
      else b_list[[j]] <- seq(-0.5, 0.5, length.out=K-1)
    }
  }

  sigma_gamma <- rep(0.5, max(1, n_testlets))

  # MLE storage (used if method=MLE)
  theta_mle <- as.vector(scale(rowSums(raw_mat, na.rm=TRUE)))
  theta_mle[is.na(theta_mle)] <- 0
  gamma_mle <- matrix(0, N, max(1, n_testlets))

  # --- Helper: Universal Probability ---
  plogis_c <- function(x) 1 / (1 + exp(-x))

  get_P_univ <- function(th_eff, a_val, b_vec, g_val, mod, n_cat) {
    n_q <- length(th_eff)
    if(n_cat == 2) {
      z <- a_val * (th_eff - b_vec[1])
      z <- pmin(pmax(z, -30), 30)
      p <- g_val + (1 - g_val) * plogis_c(z)
      return(cbind(1-p, p))
    }
    if(mod %in% c("GRM", "GRT")) {
      probs <- matrix(0, n_q, n_cat)
      P_s <- matrix(0, n_q, n_cat+1); P_s[,1] <- 1
      for(k in 1:(n_cat-1)) {
        z <- a_val * (th_eff - b_vec[k]); z <- pmin(pmax(z, -30), 30)
        P_s[, k+1] <- plogis_c(z)
      }
      for(k in 1:n_cat) probs[,k] <- P_s[,k] - P_s[,k+1]
      return(pmax(probs, 1e-15))
    }
    # GPCM/PCM
    numer <- matrix(0, n_q, n_cat)
    curr <- 0
    for(k in 1:(n_cat-1)) {
      curr <- curr + a_val * (th_eff - b_vec[k])
      numer[, k+1] <- curr
    }
    mx <- apply(numer, 1, max)
    exps <- exp(numer - mx)
    return(exps / rowSums(exps))
  }

  # --- Helper: Robust Newton-Raphson Item Update ---
  solve_item <- function(counts, nodes, ca, cb, cg, mod, n_cat, fix_d) {
    start_p <- c()
    if(!fix_d) start_p <- c(start_p, ca)
    start_p <- c(start_p, cb)
    if(mod %in% c("3PL", "3PLT")) start_p <- c(start_p, cg)

    n_par <- length(start_p)

    # Initialize Hessian to avoid scoping crash
    he <- matrix(0, n_par, n_par)
    diag(he) <- -1

    ll_fn <- function(p) {
      idx <- 1
      la <- if(fix_d) 1.0 else { v<-p[idx]; idx<-idx+1; v }
      lb <- p[idx:(idx+length(cb)-1)]; idx<-idx+length(cb)
      lg <- if(mod %in% c("3PL", "3PLT")) p[idx] else 0

      if(la < 0.01 || la > 6.0) return(-1e20)
      if(mod %in% c("3PL", "3PLT") && (lg < 0 || lg > 0.45)) return(-1e20)
      if(mod %in% c("GRM", "GRT") && is.unsorted(lb)) return(-1e20)

      pr <- get_P_univ(nodes, la, lb, lg, mod, n_cat)
      # pmax to prevent log(0)
      sum(counts * log(pmax(pr, 1e-100)))
    }

    curr <- start_p
    opt_fail <- FALSE

    for(iter in 1:con$nr_max_iter) {
      f0 <- ll_fn(curr)
      if(f0 <= -1e19) { opt_fail <- TRUE; break }

      gr <- numeric(n_par)
      he <- matrix(0, n_par, n_par)
      h <- 1e-4

      for(i in 1:n_par) {
        p1 <- curr; p1[i] <- p1[i] + h
        gr[i] <- (ll_fn(p1) - f0)/h
      }
      for(i in 1:n_par) {
        p1 <- curr; p1[i] <- p1[i] + h; p2 <- curr; p2[i] <- p2[i] - h
        he[i,i] <- (ll_fn(p1) - 2*f0 + ll_fn(p2))/h^2
      }
      diag(he) <- diag(he) - 1e-3

      step <- try(solve(he, gr), silent=TRUE)
      if(inherits(step, "try-error")) step <- gr * 0.01
      step <- pmax(pmin(step, 1.0), -1.0)

      curr <- curr - step * con$nr_damp
      if(max(abs(step)) < 1e-4) break
    }

    se <- tryCatch({ sqrt(diag(solve(-he + diag(1e-5, n_par)))) }, error=function(e) rep(NA, n_par))

    if(opt_fail) {
      return(list(se=rep(NA, n_par), a=ca, b=cb, g=cg))
    }

    res <- list(se=se); idx <- 1
    if(!fix_d) { res$a <- curr[idx]; idx<-idx+1 } else res$a <- 1.0
    res$b <- curr[idx:(idx+length(cb)-1)]; idx<-idx+length(cb)
    if(mod %in% c("3PL","3PLT")) res$g <- curr[idx] else res$g <- 0
    return(res)
  }

  # --- Main Estimation Loop ---
  if(con$verbose) {
    cat(sprintf("Data: %d Persons, %d Items, %d Testlets\n", N, J, n_testlets))
    cat("------------------------------------------------\n")
    cat("Starting Estimation Loop...\n")
  }

  nodes <- seq(con$theta_range[1], con$theta_range[2], length.out=con$quad_points)
  wts <- dnorm(nodes); wts <- wts/sum(wts)

  log_lik <- 0
  se_list <- vector("list", J)
  Post <- matrix(1/length(nodes), N, length(nodes)) # Init posterior

  is_converged <- FALSE

  for(iter in 1:con$max_iter) {

    # E-STEP
    L_tot <- matrix(1, N, length(nodes))

    # Independent Items
    for(j in valid_items_idx) {
      if(!is.na(testlet_vec[j])) next
      P <- get_P_univ(nodes, a[j], b_list[[j]], g[j], mod_vec[j], n_cats[j])
      y <- raw_mat[,j]; valid <- !is.na(y)
      if(any(valid)) {
        for(q in 1:length(nodes)) L_tot[valid, q] <- L_tot[valid, q] * P[q, y[valid]+1]
      }
    }

    # Testlet Items (Dim Reduction for EM)
    if(n_testlets > 0) {
      g_grid <- seq(-3, 3, length.out=9)
      gw <- dnorm(g_grid); gw <- gw/sum(gw)

      for(t in 1:n_testlets) {
        t_idx <- which(testlet_vec == t)
        L_t <- matrix(0, N, length(nodes))

        for(q in 1:length(nodes)) {
          eff_t <- nodes[q] + sqrt(sigma_gamma[t]) * g_grid
          L_sub <- matrix(1, N, length(g_grid))
          for(j in t_idx) {
            P <- get_P_univ(eff_t, a[j], b_list[[j]], g[j], mod_vec[j], n_cats[j])
            y <- raw_mat[,j]; valid <- !is.na(y)
            if(any(valid)) {
              for(k in 1:length(g_grid)) L_sub[valid, k] <- L_sub[valid, k] * P[k, y[valid]+1]
            }
          }
          L_t[, q] <- as.vector(L_sub %*% gw)
        }
        L_tot <- L_tot * L_t
      }
    }

    Ly <- as.vector(L_tot %*% wts)
    log_lik <- sum(log(pmax(Ly, 1e-100)))
    Post <- (L_tot * matrix(wts, N, length(nodes), byrow=TRUE)) / Ly

    # M-STEP
    max_diff <- 0
    for(j in valid_items_idx) {
      K <- n_cats[j]
      counts <- matrix(0, length(nodes), K)
      y <- raw_mat[,j]
      valid <- !is.na(y)
      for(k in 1:K) {
        idx <- which(valid & y == (k-1))
        if(length(idx)>0) counts[,k] <- colSums(Post[idx, , drop=FALSE])
      }

      old <- c(a[j], b_list[[j]], g[j])
      res <- solve_item(counts, nodes, a[j], b_list[[j]], g[j], mod_vec[j], K, con$fix_discrimination)
      a[j] <- res$a; b_list[[j]] <- res$b; g[j] <- res$g; se_list[[j]] <- res$se

      max_diff <- max(max_diff, abs(c(a[j], b_list[[j]], g[j]) - old))
    }

    if(con$verbose) cat(sprintf("\rIter %3d | LogLik: %10.2f | Max Change: %.5f    ", iter, log_lik, max_diff))

    if(max_diff < con$converge_tol) {
      if(con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
      is_converged <- TRUE
      break
    }
  }

  if(!is_converged && con$verbose) {
    cat(sprintf("\n\n>>> NOTE: Stopped because max Iteration Reached at %d.\n", con$max_iter))
    cat("    You can increase 'max_iter' in control=list(max_iter=...) if needed.\n")
  }

  # --- Post-Hoc: Person Parameter Estimation ---
  if(con$verbose) {
    cat("------------------------------------------------\n")
    cat("Estimating Final Person Parameters (Theta)...\n")
    cat(sprintf("Using Theta range: [%.1f, %.1f]. Adjust via control=list(theta_range=c(...)).\n", con$theta_range[1], con$theta_range[2]))
  }

  theta_final <- as.vector(Post %*% nodes)
  se_final <- sqrt(abs(as.vector(Post %*% nodes^2) - theta_final^2))

  # --- Post-Hoc: Testlet Effect Estimation (Gamma) ---
  gamma_final <- matrix(NA, N, max(1, n_testlets))
  gamma_se_final <- matrix(NA, N, max(1, n_testlets))

  if(n_testlets > 0) {
    if(con$verbose) cat("Estimating Testlet Effects (Gamma) per person...\n")

    g_grid <- seq(-3, 3, length.out=15) # Finer grid for EAP
    gw <- dnorm(g_grid); gw <- gw/sum(gw)

    for(t in 1:n_testlets) {
      # For EAP(Gamma): We integrate Gamma posterior.
      # P(gamma | y) \propto P(y_t | theta, gamma) * P(theta | y_others) * P(gamma)
      # Approx: use P(theta | all_y) as weight, though slightly circular, is standard approx in unidi code.
      # Better: Joint EAP over Theta/Gamma grid weighted by Posterior Theta.

      t_idx <- which(testlet_vec == t)

      # For each person, we construct a marginal distribution over Gamma
      # Joint(q, k) = Post_Theta[i, q] * P(y_t | theta_q, gamma_k) * P(gamma_k)
      # Note: We divide by L_t(theta_q) to remove double counting if strictly needed,
      # but standard scoring often just uses the joint expectation.

      for(i in 1:N) {
        valid_resp <- !is.na(raw_mat[i, t_idx])
        if(!any(valid_resp)) { gamma_final[i, t] <- 0; gamma_se_final[i, t] <- NA; next }

        # Calculate Likelihood of testlet items on the 2D grid (Theta x Gamma)
        # We perform summation over Theta index (q) weighted by Post[i, q]

        gam_post_num <- numeric(length(g_grid))

        # Pre-calc item probs for this person's responses
        # To speed up: assume Theta is fixed at EAP Theta? (Faster)
        # Or Integrate? (Better). Let's Integrate.

        for(k in 1:length(g_grid)) {
          g_val <- sqrt(sigma_gamma[t]) * g_grid[k]

          # Likelihood of this gamma given theta_q
          L_g_given_th <- numeric(length(nodes))

          # Vectorized P calc
          # Eff Theta Matrix [Nodes x 1]
          eff_mat <- nodes + g_val

          # Accumulate log-lik for items
          log_l <- rep(0, length(nodes))
          for(j_sub in t_idx) {
            if(is.na(raw_mat[i, j_sub])) next
            p_vec <- get_P_univ(eff_mat, a[j_sub], b_list[[j_sub]], g[j_sub], mod_vec[j_sub], n_cats[j_sub])
            log_l <- log_l + log(p_vec[, raw_mat[i, j_sub]+1])
          }
          lik_vec <- exp(log_l)

          # Weighted sum over Theta Posterior
          # P(gamma_k) ~ Sum_q ( P(y_t | th_q, gam_k) * Post(th_q) ) * Prior(gam_k)
          gam_post_num[k] <- sum(lik_vec * Post[i, ]) * gw[k]
        }

        denom <- sum(gam_post_num)
        if(denom == 0) denom <- 1e-10
        gam_dist <- gam_post_num / denom

        # EAP
        real_g_vals <- sqrt(sigma_gamma[t]) * g_grid
        gamma_final[i, t] <- sum(real_g_vals * gam_dist)
        gamma_se_final[i, t] <- sqrt(sum((real_g_vals - gamma_final[i, t])^2 * gam_dist))
      }
    }
  }

  if(con$verbose) cat("All parameters estimated. Formatting results...\n")

  # --- Output Construction ---

  # 1. Item Params
  get_se <- function(j, type, k=1) {
    v <- se_list[[j]]
    if(is.null(v) || any(is.na(v))) return(NA)
    has_a <- !con$fix_discrimination
    has_g <- mod_vec[j] %in% c("3PL", "3PLT")
    n_b <- length(b_list[[j]])
    idx_a <- if(has_a) 1 else 0
    idx_b_start <- idx_a + 1
    idx_g <- if(has_g) idx_b_start + n_b else 0

    if(type == "a") return(if(has_a) v[1] else NA)
    if(type == "g") return(if(has_g) v[idx_g] else NA)
    if(type == "b") {
      if(k > n_b) return(NA)
      return(v[idx_b_start + k - 1])
    }
    return(NA)
  }

  df_items <- data.frame(
    item = item_names,
    model = mod_vec,
    testlet = ifelse(is.na(testlet_vec), NA,
                     if(!is.null(t_names_map)) t_names_map[testlet_vec] else paste0("T", testlet_vec)),
    discrimination = round(a, 3),
    discrimination_se = round(sapply(1:J, get_se, type="a"), 3),

    # --- Binary Items use Difficulty, Poly uses NA ---
    difficulty = round(sapply(1:J, function(j) if(n_cats[j] == 2) b_list[[j]][1] else NA), 3),
    difficulty_se = round(sapply(1:J, function(j) if(n_cats[j] == 2) get_se(j, type="b", k=1) else NA), 3),
    # -----------------------------------------------------------------

    guessing = round(g, 3),
    guessing_se = round(sapply(1:J, get_se, type="g"), 3),
    stringsAsFactors = FALSE
  )

  # Add thresholds (ONLY for Polytomous, to keep clean separation)
  max_thresh <- max(sapply(b_list, length))
  for(k in 1:max_thresh) {
    # Extract value only if item is Polytomous
    val <- sapply(1:J, function(j) {
      if(n_cats[j] > 2 && length(b_list[[j]]) >= k) b_list[[j]][k] else NA
    })
    se_val <- sapply(1:J, function(j) {
      if(n_cats[j] > 2 && length(b_list[[j]]) >= k) get_se(j, type="b", k=k) else NA
    })

    df_items[[paste0("step/threshold_", k)]] <- round(val, 3)
    df_items[[paste0("step/threshold_", k, "_se")]] <- round(se_val, 3)
  }

  # Add number and pvalue at the end
  df_items$number <- colSums(!is.na(raw_mat))
  df_items$pvalue <- round(colMeans(raw_mat, na.rm=TRUE)/(n_cats-1), 3)

  # Remove row names
  row.names(df_items) <- NULL

  # 2. Person Params
  df_persons <- data.frame(
    n_response = rowSums(!is.na(raw_mat)),
    ability = round(theta_final, 3),
    ability_se = round(se_final, 3)
  )

  if(n_testlets > 0) {
    for(t in 1:n_testlets) {
      t_name <- if(!is.null(t_names_map)) t_names_map[t] else paste0("testlet_", t)
      df_persons[[t_name]] <- round(gamma_final[, t], 3)
      df_persons[[paste0(t_name, "_se")]] <- round(gamma_se_final[, t], 3)
    }
  }
  row.names(df_persons) <- NULL

  # 3. Model Fit
  n_par <- length(valid_items_idx) * 2 # Approximation
  df_fit <- data.frame(
    Index=c("LogLik","AIC","BIC", "Iterations"),
    Value=c(round(log_lik,3), round(2*n_par - 2*log_lik,3), round(n_par*log(N) - 2*log_lik,3), iter)
  )

  if(con$verbose) {
    cat("All completed. You can view the results now.\n")
    cat("================================================\n")
  }

  return(list(item_params=df_items, person_params=df_persons, model_fit=df_fit))
}

#' Unidimensional Binary (Dichotomous) Testlet Response Theory Estimation
#'
#' @description
#' Estimates item and person parameters for Unidimensional Binary (Dichotomous) Testlet response models
#' using Penalized Expectation-Maximization or
#' Joint Maximum Likelihood Estimation with stabilization.
#'
#' @param data A data.frame of binary responses (0/1). Rows=persons, Cols=items in testlets.
#' @param group A list defining testlet structures. Example: `list(c(1,2,3), c(4,5,6))`.
#' @param model Character. One of "RaschT" (Rasch Testlet), "2PLT" (2-Parameter Logistic Testlet), "3PLT" (3-Parameter Logistic Testlet), "BiFT" (Bifactor).
#' @param method Character. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood).
#' @param control A \code{list} of control parameters for the estimation algorithm:
#'   \itemize{
#'     \item \code{max_iter}: Maximum number of EM iterations (default = 100).
#'     \item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
#'     \item \code{theta_range}: Numeric vector of length 2 specifying the integration
#'           grid bounds (default = c(-4, 4)).
#'     \item \code{quad_points}: Number of quadrature points (default = 21).
#'     \item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
#'   }
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{item_params}: Estimated item parameters.
#'   \item \code{person_params}: Estimated person abilities and testlet effects.
#'   \item \code{model_fit}: A data frame containing iterations and fit statistics such as Akaike’s Information Criterion (AIC), the Bayesian Information Criterion (BIC), and Log-Likelihood.
#' }
#'
#' @examples
#'   # --- Quick Example (small data) ---
#'   set.seed(1)
#'   n <- 50
#'   J <- 4
#'   theta <- rnorm(n)
#'   gamma <- rnorm(n, 0, 0.5)
#'   resp <- matrix(NA, n, J)
#'   colnames(resp) <- paste0("Item_", 1:J)
#'   b <- c(-0.5, 0, 0.5, 1)
#'   for(j in 1:J) {
#'     p <- 1 / (1 + exp(-(theta + gamma - b[j])))
#'     resp[, j] <- rbinom(n, 1, p)
#'   }
#'   res <- trt_binary(
#'     data = as.data.frame(resp),
#'     group = list(1:2, 3:4),
#'     model = "2PLT",
#'     method = "EM",
#'     control = list(max_iter = 5, verbose = FALSE)
#'   )
#'   head(res$item_params)
#'
#'  \donttest{
#'   # --- Full Example: Simulation (2PLT) ---
#'   set.seed(2025)
#'   n_persons <- 500
#'   n_testlets <- 3
#'   items_per_testlet <- 3
#'   n_items <- n_testlets * items_per_testlet
#'
#'   # 1. Generate Parameters
#'   # Discrimination (a): Varying -> 2PLT
#'   a_true <- runif(n_items, 0.8, 1.5)
#'   # Difficulty (b)
#'   b_true <- seq(-1, 1, length.out = n_items)
#'   # Testlet Variances (Sigma)
#'   sigma_true <- c(1.0, 1.5, 2.0)
#'
#'   # 2. Generate Person Params
#'   theta_true <- rnorm(n_persons, 0, 1)
#'   gamma_matrix <- matrix(0, nrow = n_persons, ncol = n_testlets)
#'   for(d in 1:n_testlets) {
#'     gamma_matrix[, d] <- rnorm(n_persons, 0, sigma_true[d])
#'   }
#'
#'   # 3. Generate Responses
#'   resp_matrix <- matrix(0, nrow = n_persons, ncol = n_items)
#'   colnames(resp_matrix) <- paste0("Item_", 1:n_items)
#'   group_list <- list()
#'
#'   idx_counter <- 1
#'   for(d in 1:n_testlets) {
#'     indices <- idx_counter:(idx_counter + items_per_testlet - 1)
#'     group_list[[d]] <- indices
#'
#'     for(i in indices) {
#'       # 2PLT Model: a * (theta + gamma - b)
#'       lin <- a_true[i] * (theta_true + gamma_matrix[, d] - b_true[i])
#'       prob <- 1 / (1 + exp(-lin))
#'       resp_matrix[, i] <- rbinom(n_persons, 1, prob)
#'     }
#'     idx_counter <- idx_counter + items_per_testlet
#'   }
#'   df_sim <- as.data.frame(resp_matrix)
#'
#'   # 4. Run Estimation
#'   # We use "2PLT" because data was generated with varying 'a'
#'   res <- trt_binary(
#'     data = df_sim,
#'     group = group_list,
#'     model = "2PLT",
#'     method = "EM",
#'     control = list(max_iter = 20, verbose = FALSE)
#'   )
#'
#'   head(res$item_params)
#'   head(res$person_params)
#'   }
#' @export
trt_binary <- function(
    data,
    group,
    model = c("RaschT", "2PLT", "3PLT", "BiFT"),
    method = c("EM", "MLE"),
    control = list()
) {

  # ===========================================================================
  # 1. SETUP & VALIDATION
  # ===========================================================================
  model  <- match.arg(model)
  method <- match.arg(method)

  # Default controls with added regularization parameters
  con <- list(
    max_iter       = 100,
    converge_tol   = 1e-4,
    theta_range    = c(-4, 4),
    quad_points    = 21,
    nr_max_iter    = 20,
    nr_damp        = 1.0,
    verbose        = TRUE,
    # Priors for MAP estimation (Critical for stability)
    prior_a_mu     = 0.0,  # Log-mean (centered at a=1)
    prior_a_sd     = 0.5,  # Moderate constraint
    prior_b_mu     = 0.0,
    prior_b_sd     = 2.0,
    prior_c_alpha  = 5,    # Beta prior for guessing
    prior_c_beta   = 17,   # Mean approx 0.2
    use_priors     = TRUE
  )
  con[names(control)] <- control

  if(!is.data.frame(data)) stop("Input must be a data frame.")
  raw_data <- as.matrix(data)
  if(!is.numeric(raw_data)) stop("Input data must contain only numeric values.")

  n_persons  <- nrow(raw_data)
  n_items    <- ncol(raw_data)
  item_names <- colnames(data)
  if(is.null(item_names)) item_names <- paste0("Item_", 1:n_items)

  # Map Items to Testlets
  item_to_testlet <- rep(NA, n_items)
  for(g_idx in seq_along(group)) {
    g_items <- group[[g_idx]]
    if(is.character(g_items)) idx <- match(g_items, item_names) else idx <- g_items
    if(any(is.na(idx)) || any(idx > n_items)) stop("Invalid item indices.")
    if(any(!is.na(item_to_testlet[idx]))) stop("Items cannot overlap testlets.")
    item_to_testlet[idx] <- g_idx
  }
  n_testlets <- length(group)

  # --- USER-FRIENDLY START MESSAGE ---
  fix_discrimination <- (model == "RaschT")
  if(con$verbose) {
    cat(sprintf("\nStarting %s Estimation using %s algorithm...\n", model, method))
    if(fix_discrimination) cat("Note: Discrimination fixed to 1.0 (RaschT specification).\n")
    cat("------------------------------------------------\n")
    cat(sprintf("Data: %d Persons, %d Items, %d Testlets\n", n_persons, n_items, n_testlets))
  }

  # Helpers
  plogis_c  <- function(x) 1 / (1 + exp(-x))
  safe_prob <- function(p) {
    p[p < 1e-9] <- 1e-9;
    p[p > (1 - 1e-9)] <- 1 - 1e-9;
    p
  }

  # Initialization
  p_vals  <- colMeans(raw_data, na.rm=TRUE)
  p_vals[p_vals < 0.01] <- 0.01; p_vals[p_vals > 0.99] <- 0.99
  logit_p <- log(p_vals / (1 - p_vals))

  a_est <- rep(1.0, n_items)
  b_est <- -logit_p
  c_est <- rep(0.0, n_items)

  if(model == "RaschT") { a_est[] <- 1.0; c_est[] <- 0.0 }
  if(model == "2PLT")   { c_est[] <- 0.0 }
  if(model == "3PLT")   { c_est[] <- 0.1 }

  p_theta <- rep(0, n_persons)
  p_gamma <- matrix(0, n_persons, n_testlets)
  p_se    <- rep(NA, n_persons)
  p_gamma_se <- matrix(NA, n_persons, n_testlets)

  bad_items_idx <- c()
  is_converged  <- FALSE
  log_L_data <- NA
  iter <- 0

  # ===========================================================================
  # STRATEGY 1: EM ALGORITHM (Marginal Likelihood via Quadrature)
  # ===========================================================================
  if(method == "EM") {

    get_gh_points <- function(n) {
      if(n < 2) n <- 2
      i  <- 1:(n - 1); a <- sqrt(i / 2)
      CM <- diag(0, n); CM[cbind(1:(n-1), 2:n)] <- a; CM[cbind(2:n, 1:(n-1))] <- a
      ev <- eigen(CM, symmetric = TRUE)
      x  <- ev$values * sqrt(2); w <- (ev$vectors[1, ]^2) * sqrt(pi) / sqrt(pi)
      list(x = x[order(x)], w = w[order(x)])
    }
    gh  <- get_gh_points(con$quad_points)
    X_q <- gh$x; W_q <- gh$w; n_q <- length(X_q)

    Theta_grid <- matrix(X_q, nrow=n_q, ncol=n_q, byrow=FALSE)
    Gamma_grid <- matrix(X_q, nrow=n_q, ncol=n_q, byrow=TRUE)
    sigma_testlet <- rep(0.5, n_testlets)

    item_expected_n <- array(0, dim=c(n_items, n_q, n_q))
    item_expected_r <- array(0, dim=c(n_items, n_q, n_q))
    post_theta <- matrix(0, n_persons, n_q)

    for(it in 1:con$max_iter) {
      iter <- it

      # --- E-STEP ---
      item_expected_n[] <- 0; item_expected_r[] <- 0
      sum_E_gamma2 <- rep(0, n_testlets)
      L_testlet_theta <- array(1, dim=c(n_persons, n_testlets, n_q))

      for(t_idx in 1:n_testlets) {
        t_items <- which(item_to_testlet == t_idx)
        if(length(t_items) == 0) next
        sig_g <- sigma_testlet[t_idx]

        probs_grid <- array(0, dim=c(length(t_items), n_q, n_q))
        for(ii in seq_along(t_items)) {
          itm <- t_items[ii]
          z   <- a_est[itm] * (Theta_grid + sig_g * Gamma_grid - b_est[itm])
          probs_grid[ii,,] <- safe_prob(c_est[itm] + (1 - c_est[itm]) * plogis_c(z))
        }
        log_P <- log(probs_grid); log_Q <- log(1 - probs_grid)

        for(q_t in 1:n_q) {
          log_L_gamma <- matrix(0, nrow=n_persons, ncol=n_q)
          for(ii in seq_along(t_items)) {
            itm <- t_items[ii]; y <- raw_data[, itm]; obs <- !is.na(y)
            term <- outer(y, log_P[ii, q_t, ], "*") + outer(1-y, log_Q[ii, q_t, ], "*")
            term[!obs, ] <- 0
            log_L_gamma <- log_L_gamma + term
          }
          max_log <- apply(log_L_gamma, 1, max)
          L_gamma_scaled <- exp(log_L_gamma - max_log)
          integral <- as.vector(L_gamma_scaled %*% W_q)
          L_testlet_theta[, t_idx, q_t] <- integral * exp(max_log)
        }
      }

      L_total_theta <- apply(L_testlet_theta, c(1,3), prod)
      L_y <- as.vector(L_total_theta %*% W_q)
      log_L_data <- sum(log(L_y[L_y > 0]))

      inv_L_y <- 1/L_y; inv_L_y[L_y < 1e-300] <- 0
      post_theta <- (L_total_theta * matrix(W_q, n_persons, n_q, byrow=TRUE)) * inv_L_y

      # Accumulate Counts
      for(t_idx in 1:n_testlets) {
        t_items <- which(item_to_testlet == t_idx); if(length(t_items) == 0) next
        sig_g <- sigma_testlet[t_idx]

        probs_grid <- array(0, dim=c(length(t_items), n_q, n_q))
        for(ii in seq_along(t_items)) {
          itm <- t_items[ii]
          z   <- a_est[itm] * (Theta_grid + sig_g * Gamma_grid - b_est[itm])
          probs_grid[ii,,] <- safe_prob(c_est[itm] + (1 - c_est[itm]) * plogis_c(z))
        }
        log_P <- log(probs_grid); log_Q <- log(1 - probs_grid)

        E_g2_accum <- 0
        for(q_t in 1:n_q) {
          log_L_gamma <- matrix(0, nrow=n_persons, ncol=n_q)
          for(ii in seq_along(t_items)) {
            itm <- t_items[ii]; y <- raw_data[, itm]; obs <- !is.na(y)
            term <- outer(y, log_P[ii, q_t, ], "*") + outer(1-y, log_Q[ii, q_t, ], "*")
            term[!obs, ] <- 0
            log_L_gamma <- log_L_gamma + term
          }
          denom <- L_testlet_theta[, t_idx, q_t]; denom[denom < 1e-300] <- 1e-300
          post_g_given_t <- (exp(log_L_gamma) * matrix(W_q, n_persons, n_q, byrow=TRUE)) / denom
          joint_w <- post_g_given_t * post_theta[, q_t]
          E_g2_accum <- E_g2_accum + sum(t(joint_w) * (X_q^2))

          for(ii in seq_along(t_items)) {
            itm <- t_items[ii]; y <- raw_data[, itm]; obs <- !is.na(y)
            w_obs <- joint_w; w_obs[!obs, ] <- 0
            item_expected_n[itm, q_t, ] <- item_expected_n[itm, q_t, ] + colSums(w_obs)
            item_expected_r[itm, q_t, ] <- item_expected_r[itm, q_t, ] + colSums(w_obs * y)
          }
        }
        sum_E_gamma2[t_idx] <- E_g2_accum / n_persons
      }

      # --- M-STEP ---
      max_change <- 0
      for(i in 1:n_items) {
        if(i %in% bad_items_idx) next
        tid <- item_to_testlet[i]; sig <- if(!is.na(tid)) sigma_testlet[tid] else 0
        X_eff <- as.vector(Theta_grid + sig * Gamma_grid)
        E_n <- as.vector(item_expected_n[i,,]); E_r <- as.vector(item_expected_r[i,,])

        if(sum(E_n) < 1e-9) { bad_items_idx <- c(bad_items_idx, i); next }
        old_p <- c(a_est[i], b_est[i], c_est[i])

        for(nr in 1:con$nr_max_iter) {
          a <- a_est[i]; b <- b_est[i]; c <- c_est[i]
          lin <- a * (X_eff - b); psi <- plogis_c(lin); P <- safe_prob(c + (1-c) * psi)
          diff <- (E_r - E_n * P); w <- diff / (P * (1-P)); dP <- (1-c) * psi * (1-psi)

          ga <- sum(w * dP * (X_eff - b)); gb <- sum(w * dP * (-a)); gc <- sum(w * (1 - psi))
          w_h <- E_n / (P * (1-P))
          haa <- -sum(w_h * (dP * (X_eff - b))^2); hbb <- -sum(w_h * (dP * (-a))^2); hcc <- -sum(w_h * (1 - psi)^2)
          hab <- -sum(w_h * (dP * (X_eff - b)) * (dP * (-a))); hac <- -sum(w_h * (dP * (X_eff - b)) * (1 - psi))
          hbc <- -sum(w_h * (dP * (-a)) * (1 - psi))

          if(con$use_priors) {
            la <- log(a); pa_mu <- con$prior_a_mu; pa_sd <- con$prior_a_sd
            ga <- ga - (la - pa_mu)/(pa_sd^2 * a) - 1/a
            haa <- haa - (1 - (la - pa_mu))/(pa_sd^2 * a^2) + 1/a^2
            pb_mu <- con$prior_b_mu; pb_sd <- con$prior_b_sd
            gb <- gb - (b - pb_mu)/(pb_sd^2); hbb <- hbb - 1/(pb_sd^2)
          }

          if(model == "RaschT") { H <- matrix(hbb,1,1); G <- c(gb); idx_p <- 2 }
          else if(model %in% c("2PLT", "BiFT")) { H <- matrix(c(haa,hab,hab,hbb),2,2); G <- c(ga,gb); idx_p <- c(1,2) }
          else { H <- matrix(c(haa,hab,hac,hab,hbb,hbc,hac,hbc,hcc),3,3); G <- c(ga,gb,gc); idx_p <- 1:3 }

          diag(H) <- diag(H) - 1e-3
          delta <- tryCatch(solve(H, G), error=function(e) rep(0, length(G)))
          delta <- pmax(-0.5, pmin(0.5, delta))
          new_v <- old_p[idx_p] - con$nr_damp * delta

          if(1 %in% idx_p) new_v[which(idx_p==1)] <- max(0.1, min(4.0, new_v[which(idx_p==1)]))
          if(3 %in% idx_p) new_v[which(idx_p==3)] <- max(0.0, min(0.4, new_v[which(idx_p==3)]))

          if(model == "RaschT") b_est[i] <- new_v[1]
          else if(model %in% c("2PLT","BiFT")) { a_est[i] <- new_v[1]; b_est[i] <- new_v[2] }
          else { a_est[i] <- new_v[1]; b_est[i] <- new_v[2]; c_est[i] <- new_v[3] }
          if(max(abs(delta)) < 1e-3) break
        }
        max_change <- max(max_change, max(abs(c(a_est[i], b_est[i], c_est[i]) - old_p)))
      }

      old_sig <- sigma_testlet; sigma_testlet <- sqrt(sum_E_gamma2); sigma_testlet <- pmax(0.05, pmin(2.0, sigma_testlet))
      max_change <- max(max_change, max(abs(sigma_testlet - old_sig)))

      if(con$verbose) {
        cat(sprintf("\rIter %3d | LogLik: %10.2f | Max Change: %.6f    ", iter, log_L_data, max_change))
        flush.console()
      }

      # --- CONVERGENCE CHECK (USER MESSAGE) ---
      if(max_change < con$converge_tol) {
        if(con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
        is_converged <- TRUE
        break
      }
    }

    # --- MAX ITER MESSAGE ---
    if(!is_converged && con$verbose) {
      cat(sprintf("\n\n>>> NOTE: Stopped because max Iteration Reached at %d.\n", con$max_iter))
      cat("    Adjust it in control=list(max_iter = ...) to have higher iteration if needed.\n")
    }

    # --- FINAL PARAMETERS MESSAGE ---
    if(con$verbose) {
      cat("\n------------------------------------------------\n")
      cat("Estimating Final Person Parameters...\n")
      cat(sprintf("Personal Parameter estimated with range [%s, %s].\n",
                  con$theta_range[1], con$theta_range[2]))
      cat("Adjust it in control=list(theta_range=c(..)) if you want a different range.\n")
    }

    p_theta <- as.vector(post_theta %*% X_q)
    p_se    <- sqrt(abs(as.vector(post_theta %*% (X_q^2)) - p_theta^2))

    # --- GAMMA POST-HOC (USER MESSAGE) ---
    if(con$verbose) cat("\nCalculating Testlet Effects (EAP) for EM...\n")

    for(t_idx in 1:n_testlets) {
      t_items <- which(item_to_testlet == t_idx); if(length(t_items) == 0) next
      sig_g <- sigma_testlet[t_idx]
      probs_grid <- array(0, dim=c(length(t_items), n_q, n_q))
      for(ii in seq_along(t_items)) {
        itm <- t_items[ii]
        z <- a_est[itm] * (Theta_grid + sig_g * Gamma_grid - b_est[itm])
        probs_grid[ii,,] <- safe_prob(c_est[itm] + (1 - c_est[itm]) * plogis_c(z))
      }
      log_P <- log(probs_grid); log_Q <- log(1 - probs_grid)
      post_gamma_agg <- matrix(0, n_persons, n_q)
      for(q_t in 1:n_q) {
        log_L <- matrix(0, n_persons, n_q)
        for(ii in seq_along(t_items)) {
          y <- raw_data[, t_items[ii]]; obs <- !is.na(y)
          term <- outer(y, log_P[ii, q_t, ], "*") + outer(1-y, log_Q[ii, q_t, ], "*")
          term[!obs, ] <- 0
          log_L <- log_L + term
        }
        denom <- L_testlet_theta[, t_idx, q_t]; denom[denom < 1e-300] <- 1e-300
        pg <- (exp(log_L) * matrix(W_q, n_persons, n_q, byrow=TRUE)) / denom
        post_gamma_agg <- post_gamma_agg + pg * post_theta[, q_t]
      }
      eg  <- as.vector(post_gamma_agg %*% X_q)
      eg2 <- as.vector(post_gamma_agg %*% (X_q^2))
      p_gamma[, t_idx] <- eg * sig_g
      p_gamma_se[, t_idx] <- sqrt(abs(eg2 - eg^2)) * sig_g
    }
  }

  # ===========================================================================
  # STRATEGY 2: MLE ALGORITHM (Joint Maximum Likelihood)
  # ===========================================================================
  else if(method == "MLE") {

    p_theta <- rowSums(raw_data, na.rm=TRUE)
    p_theta <- scale(p_theta); p_theta[is.na(p_theta)] <- 0
    p_gamma <- matrix(0, n_persons, n_testlets)

    for(it in 1:con$max_iter) {
      iter <- it

      # Update Persons
      for(p_iter in 1:2) {
        # Theta
        for(j in 1:n_persons) {
          t_val <- p_theta[j]
          eff <- rep(0, n_items)
          for(k in 1:n_testlets) {
            idx <- which(item_to_testlet == k)
            if(length(idx)>0) eff[idx] <- p_gamma[j, k]
          }
          y <- raw_data[j, ]; obs <- !is.na(y)
          if(sum(obs)==0) next
          for(nr in 1:5) {
            z <- a_est * (t_val + eff - b_est)
            p <- c_est + (1 - c_est) * plogis_c(z); p <- safe_prob(p)
            psi <- plogis_c(z); dP_dZ <- (1 - c_est) * psi * (1 - psi)
            grad <- sum((y[obs] - p[obs]) / (p[obs]*(1-p[obs])) * dP_dZ[obs] * a_est[obs])
            hess <- -sum( (dP_dZ[obs] * a_est[obs])^2 / (p[obs]*(1-p[obs])) )
            grad <- grad - t_val; hess <- hess - 1
            delta <- grad / (hess - 1e-3)
            t_val <- t_val - delta
            if(abs(delta) < 0.01) break
          }
          # Use theta_range for clamping
          p_theta[j] <- max(con$theta_range[1], min(con$theta_range[2], t_val))
        }

        # Gamma
        for(k in 1:n_testlets) {
          t_items <- which(item_to_testlet == k)
          if(length(t_items) == 0) next
          for(j in 1:n_persons) {
            g_val <- p_gamma[j, k]; t_val <- p_theta[j]
            y <- raw_data[j, t_items]; obs <- !is.na(y)
            if(sum(obs)==0) next
            for(nr in 1:5) {
              z <- a_est[t_items] * (t_val + g_val - b_est[t_items])
              psi <- plogis_c(z); p <- safe_prob(c_est[t_items] + (1 - c_est[t_items]) * psi)
              dP_dZ <- (1 - c_est[t_items]) * psi * (1 - psi)
              w <- (y[obs] - p[obs]) / (p[obs]*(1-p[obs]))
              grad <- sum(w * dP_dZ[obs] * a_est[t_items])
              hess <- -sum( (dP_dZ[obs] * a_est[t_items])^2 / (p[obs]*(1-p[obs])) )
              grad <- grad - g_val/0.25; hess <- hess - 1/0.25
              delta <- grad / (hess - 1e-3)
              g_val <- g_val - delta
              if(abs(delta) < 0.01) break
            }
            p_gamma[j, k] <- max(-2, min(2, g_val))
          }
        }
      }
      p_theta <- as.vector(scale(p_theta))

      # Update Items
      max_change <- 0
      for(i in 1:n_items) {
        if(i %in% bad_items_idx) next
        tid <- item_to_testlet[i]; eff <- if(!is.na(tid)) p_gamma[, tid] else rep(0, n_persons)
        X_eff <- p_theta + eff
        y_vec <- raw_data[, i]; obs <- !is.na(y_vec)
        old_p <- c(a_est[i], b_est[i], c_est[i])

        for(nr in 1:con$nr_max_iter) {
          a <- a_est[i]; b <- b_est[i]; c <- c_est[i]
          X_o <- X_eff[obs]; y_o <- y_vec[obs]
          lin <- a * (X_o - b); psi <- plogis_c(lin); P <- safe_prob(c + (1-c) * psi)
          diff <- (y_o - P); w <- diff / (P * (1-P)); dP <- (1-c) * psi * (1-psi)
          ga <- sum(w * dP * (X_o - b)); gb <- sum(w * dP * (-a)); gc <- sum(w * (1 - psi))
          w_h <- 1 / (P * (1-P))
          haa <- -sum(w_h * (dP * (X_o - b))^2); hbb <- -sum(w_h * (dP * (-a))^2); hcc <- -sum(w_h * (1 - psi)^2)
          hab <- -sum(w_h * (dP * (X_o - b)) * (dP * (-a))); hac <- -sum(w_h * (dP * (X_o - b)) * (1 - psi))
          hbc <- -sum(w_h * (dP * (-a)) * (1 - psi))

          if(con$use_priors) {
            la <- log(a); pa_mu <- con$prior_a_mu; pa_sd <- con$prior_a_sd
            ga <- ga - (la - pa_mu)/(pa_sd^2 * a) - 1/a
            haa <- haa - (1 - (la - pa_mu))/(pa_sd^2 * a^2) + 1/a^2
            pb_mu <- con$prior_b_mu; pb_sd <- con$prior_b_sd
            gb <- gb - (b - pb_mu)/(pb_sd^2); hbb <- hbb - 1/(pb_sd^2)
          }

          if(model == "RaschT") { H <- matrix(hbb,1,1); G <- c(gb); idx_p <- 2 }
          else if(model %in% c("2PLT", "BiFT")) { H <- matrix(c(haa,hab,hab,hbb),2,2); G <- c(ga,gb); idx_p <- c(1,2) }
          else { H <- matrix(c(haa,hab,hac,hab,hbb,hbc,hac,hbc,hcc),3,3); G <- c(ga,gb,gc); idx_p <- 1:3 }

          diag(H) <- diag(H) - 1e-3
          delta <- tryCatch(solve(H, G), error=function(e) rep(0, length(G)))
          delta <- pmax(-0.5, pmin(0.5, delta))

          new_v <- old_p[idx_p] - con$nr_damp * delta
          if(1 %in% idx_p) new_v[which(idx_p==1)] <- max(0.1, min(4.0, new_v[which(idx_p==1)]))
          if(3 %in% idx_p) new_v[which(idx_p==3)] <- max(0.0, min(0.4, new_v[which(idx_p==3)]))

          if(model == "RaschT") b_est[i] <- new_v[1]
          else if(model %in% c("2PLT","BiFT")) { a_est[i] <- new_v[1]; b_est[i] <- new_v[2] }
          else { a_est[i] <- new_v[1]; b_est[i] <- new_v[2]; c_est[i] <- new_v[3] }
          if(max(abs(delta)) < 1e-3) break
        }
        max_change <- max(max_change, max(abs(c(a_est[i], b_est[i], c_est[i]) - old_p)))
      }

      curr_ll <- 0
      for(i in 1:n_items) {
        tid <- item_to_testlet[i]; eff <- if(!is.na(tid)) p_gamma[, tid] else rep(0, n_persons)
        z <- a_est[i] * (p_theta + eff - b_est[i])
        p <- safe_prob(c_est[i] + (1 - c_est[i]) * plogis_c(z))
        y <- raw_data[, i]; obs <- !is.na(y)
        curr_ll <- curr_ll + sum(y[obs]*log(p[obs]) + (1-y[obs])*log(1-p[obs]))
      }
      log_L_data <- curr_ll

      if(con$verbose) {
        cat(sprintf("\rIter %3d | LogLik: %10.2f | Max Change: %.6f    ", iter, log_L_data, max_change))
        flush.console()
      }

      # --- CONVERGENCE CHECK (USER MESSAGE) ---
      if(max_change < con$converge_tol) {
        if(con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
        is_converged <- TRUE
        break
      }
    }

    # --- MAX ITER MESSAGE ---
    if(!is_converged && con$verbose) {
      cat(sprintf("\n\n>>> NOTE: Stopped because max Iteration Reached at %d.\n", con$max_iter))
      cat("    Adjust it in control=list(max_iter = ...) to have higher iteration if needed.\n")
    }
    if(con$verbose) {
      cat("\n------------------------------------------------\n")
      cat("Estimating Final Person Parameters...\n")
      cat(sprintf("Personal Parameter estimated with range [%s, %s].\n",
                  con$theta_range[1], con$theta_range[2]))
      cat("Adjust it in control=list(theta_range=c(..)) if you want a different range.\n")
    }

    # Approx SEs for MLE
    for(j in 1:n_persons) {
      eff <- rep(0, n_items)
      for(k in 1:n_testlets) if(!is.na(match(k, item_to_testlet))) eff[item_to_testlet==k] <- p_gamma[j,k]
      z <- a_est * (p_theta[j] + eff - b_est)
      psi <- plogis_c(z); dP_dZ <- (1 - c_est) * psi * (1 - psi)
      p <- safe_prob(c_est + (1 - c_est) * psi)
      info <- sum((dP_dZ * a_est)^2 / (p*(1-p)), na.rm=TRUE)
      p_se[j] <- 1/sqrt(info + 1)
    }
    for(k in 1:n_testlets) {
      t_items <- which(item_to_testlet == k)
      if(length(t_items)==0) next
      for(j in 1:n_persons) {
        z <- a_est[t_items] * (p_theta[j] + p_gamma[j,k] - b_est[t_items])
        psi <- plogis_c(z); dP_dZ <- (1 - c_est[t_items]) * psi * (1 - psi)
        p <- safe_prob(c_est[t_items] + (1 - c_est[t_items]) * psi)
        info <- sum((dP_dZ * a_est[t_items])^2 / (p*(1-p)), na.rm=TRUE)
        p_gamma_se[j, k] <- 1/sqrt(info + 4)
      }
    }
  }

  # ===========================================================================
  # FINAL SEs (ITEMS) & OUTPUT
  # ===========================================================================
  se_a <- rep(NA, n_items); se_b <- rep(NA, n_items); se_c <- rep(NA, n_items)

  for(i in 1:n_items) {
    if(i %in% bad_items_idx) next

    tid <- item_to_testlet[i];
    if(method=="EM") {
      sig <- if(!is.na(tid)) sigma_testlet[tid] else 0
      X_eff <- as.vector(Theta_grid + sig * Gamma_grid)
      E_n <- as.vector(item_expected_n[i,,])
      weights <- E_n
    } else {
      eff <- if(!is.na(tid)) p_gamma[, tid] else 0
      X_eff <- (p_theta + eff)[!is.na(raw_data[,i])]
      weights <- 1
    }

    a <- a_est[i]; b <- b_est[i]; c <- c_est[i]
    lin <- a * (X_eff - b); psi <- plogis_c(lin); P <- safe_prob(c + (1-c) * psi)
    dP <- (1-c) * psi * (1-psi)
    w_h <- weights / (P * (1-P))

    haa <- -sum(w_h * (dP * (X_eff - b))^2); hbb <- -sum(w_h * (dP * (-a))^2)
    hcc <- -sum(w_h * (1 - psi)^2); hab <- -sum(w_h * (dP * (X_eff - b)) * (dP * (-a)))
    hac <- -sum(w_h * (dP * (X_eff - b)) * (1 - psi)); hbc <- -sum(w_h * (dP * (-a)) * (1 - psi))

    if(con$use_priors) {
      la <- log(a); pa_mu <- con$prior_a_mu; pa_sd <- con$prior_a_sd
      haa <- haa - (1 - (la - pa_mu))/(pa_sd^2 * a^2) + 1/a^2
      pb_mu <- con$prior_b_mu; pb_sd <- con$prior_b_sd
      hbb <- hbb - 1/(pb_sd^2)
    }

    H <- NULL; idx_map <- NULL
    if(model == "RaschT") { H <- matrix(hbb,1,1); idx_map <- 2 }
    else if(model %in% c("2PLT", "BiFT")) { H <- matrix(c(haa,hab,hab,hbb),2,2); idx_map <- c(1,2) }
    else { H <- matrix(c(haa,hab,hac,hab,hbb,hbc,hac,hbc,hcc),3,3); idx_map <- 1:3 }

    cv <- tryCatch(solve(-H), error=function(e) matrix(NA, nrow(H), ncol(H)))
    s <- sqrt(diag(cv))
    if(1 %in% idx_map) se_a[i] <- s[which(idx_map==1)]
    if(2 %in% idx_map) se_b[i] <- s[which(idx_map==2)]
    if(3 %in% idx_map) se_c[i] <- s[which(idx_map==3)]
  }

  if(model == "RaschT") { se_a[] <- 0.0; se_c[] <- 0.0 }
  else if(model %in% c("2PLT", "BiFT")) { se_c[] <- 0.0 }

  out_items <- data.frame(
    item = item_names,
    discrimination = a_est, discrimination_se = se_a,
    difficulty = b_est, difficulty_se = se_b,
    guessing = c_est, guessing_se = se_c
  )
  if(length(bad_items_idx) > 0) out_items[bad_items_idx, -1] <- NA
  rownames(out_items) <- NULL

  out_persons <- data.frame(person=1:n_persons, ability=p_theta, ability_se=p_se)

  t_cols <- data.frame(matrix(NA, nrow=n_persons, ncol=2*n_testlets))
  t_names <- c()
  for(k in 1:n_testlets) {
    t_cols[, (k*2)-1] <- p_gamma[, k]
    t_cols[, (k*2)]   <- p_gamma_se[, k]
    t_names <- c(t_names, paste0("testlet_", k), paste0("testlet_", k, "_se"))
  }
  colnames(t_cols) <- t_names
  out_persons <- cbind(out_persons, t_cols)
  rownames(out_persons) <- NULL

  n_p <- switch(model, "RaschT"=n_items+n_testlets, "2PLT"=2*n_items+n_testlets, "BiFT"=2*n_items+n_testlets, "3PLT"=3*n_items+n_testlets)
  out_fit <- data.frame(LogLikelihood=log_L_data, AIC=2*n_p - 2*log_L_data, BIC=n_p*log(n_persons) - 2*log_L_data, Iterations=iter)
  rownames(out_fit) <- NULL

  se_cols_items <- grep("_se$", names(out_items))
  se_cols_persons <- grep("_se$", names(out_persons))

  na_items <- any(is.na(out_items[, se_cols_items]))
  na_persons <- any(is.na(out_persons[, se_cols_persons]))

  if(na_items || na_persons) {
    parts <- c()
    if(na_items) parts <- c(parts, "Item Parameters")
    if(na_persons) parts <- c(parts, "Person Parameters")
    msg <- paste0("\nWarning: Standard error detected NA in ", paste(parts, collapse=" & "),
                  ", indicating estimation for this is not stable (Hessian Singular).")
    message(msg)
  }

  if(con$verbose) {
    cat("Finished All Estimation.\n")
    cat("------------------------------------------------\n")
  }

  list(item_params=out_items, person_params=out_persons, model_fit=out_fit)
}

#' Item Response Theory Equating / Linking
#'
#' @description
#' Conducts item response theory scale linking using Mean-Mean, Mean-Sigma, and Stocking-Lord methods.
#' Supports mixed formats of both dichotomous and polytomous models.
#' Automatically detects anchor items and validates model consistency.
#'
#' @param base_params Data frame of reference item parameters (Form X).
#' @param new_params Data frame of new item parameters to be transformed (Form Y).
#' @param person_params (Optional) Data frame of person parameters from Form Y.
#' @param methods Character vector. Options: "Mean-Mean", "Mean-Sigma", "Stocking-Lord".
#'   If NULL, defaults to all three.
#'
#' @return A list containing three data frames:
#'   \item{transformed_item_params}{New items transformed to Base scale (with SEs).}
#'   \item{transformed_person_params}{New persons transformed to Base scale (if provided).}
#'   \item{linking_constants}{The A (slope) and B (intercept) constants for each method.}
#' @examples
#'   # ===========================================================================
#'   # Example: Equating Form Y (New) to Form X (Base)
#'   # ===========================================================================
#'   set.seed(123)
#'
#'   # 1. Generate "True" Base Parameters (Form X)
#'   # ---------------------------------------------------------------------------
#'   # 10 Common Items (Anchors) + 10 Unique Items
#'   # 2PL and GRM mixed
#'
#'   gen_item_params <- function(n, type="2PL") {
#'     if(type=="2PL") {
#'       data.frame(
#'         item = paste0("Item_", 1:n),
#'         model = "2PL",
#'         a = round(runif(n, 0.8, 1.5), 2),
#'         b = round(rnorm(n, 0, 1), 2),
#'         stringsAsFactors = FALSE
#'       )
#'     } else {
#'       # GRM with 3 thresholds
#'       d <- t(apply(matrix(rnorm(n*3, 0, 0.5), n, 3), 1, sort))
#'       df <- data.frame(
#'         item = paste0("Poly_", 1:n),
#'         model = "GRM",
#'         a = round(runif(n, 0.8, 1.5), 2),
#'         stringsAsFactors = FALSE
#'       )
#'       df <- cbind(df, setNames(as.data.frame(d), paste0("step_", 1:3)))
#'       df
#'     }
#'   }
#'
#'   # Anchors
#'   anchor_2pl <- gen_item_params(5, "2PL")
#'   anchor_grm <- gen_item_params(3, "GRM")
#'   # Unique Form X
#'   unique_x <- gen_item_params(5, "2PL")
#'   unique_x$item <- paste0("X_", unique_x$item)
#'
#'   base_params <- dplyr::bind_rows(anchor_2pl, anchor_grm, unique_x)
#'
#'   # 2. Generate "New" Form Y Parameters (with Scale Shift)
#'   # ---------------------------------------------------------------------------
#'   # Scale Transformation: Theta_base = 1.2 * Theta_new + 0.5
#'   # True Constants: A = 1.2, B = 0.5
#'   TRUE_A <- 1.2
#'   TRUE_B <- 0.5
#'
#'   # Transform Anchor Parameters to "New" scale (Inverse Logic)
#'   # a_new = a_base * A
#'   # b_new = (b_base - B) / A
#'
#'   anchor_2pl_new <- anchor_2pl
#'   anchor_2pl_new$a <- anchor_2pl$a * TRUE_A
#'   anchor_2pl_new$b <- (anchor_2pl$b - TRUE_B) / TRUE_A
#'
#'   anchor_grm_new <- anchor_grm
#'   anchor_grm_new$a <- anchor_grm$a * TRUE_A
#'   step_cols <- grep("step_", names(anchor_grm_new))
#'   anchor_grm_new[, step_cols] <- (anchor_grm[, step_cols] - TRUE_B) / TRUE_A
#'
#'   # Unique Form Y
#'   unique_y <- gen_item_params(5, "2PL")
#'   unique_y$item <- paste0("Y_", unique_y$item)
#'
#'   new_params <- dplyr::bind_rows(anchor_2pl_new, anchor_grm_new, unique_y)
#'
#'   # 3. Create Dummy Person Parameters for Form Y
#'   # ---------------------------------------------------------------------------
#'   person_params <- data.frame(
#'     id = paste0("P", 1:50),
#'     theta = rnorm(50, 0, 1),
#'     theta_se = runif(50, 0.2, 0.5)
#'   )
#'
#'   # 4. Perform Equating
#'   # ---------------------------------------------------------------------------
#'   # We expect to recover A approx 1.2 and B approx 0.5
#'   results <- equate_irt(
#'     base_params = base_params,
#'     new_params = new_params,
#'     person_params = person_params,
#'     methods = c("Mean-Mean", "Stocking-Lord")
#'   )
#'
#'   # 5. Inspect Results
#'   # ---------------------------------------------------------------------------
#'   # Linking Constants
#'   print(results$linking_constants)
#'
#'   # Transformed Items (Form Y items on Form X scale)
#'   head(results$transformed_item_params)
#'
#'   # Transformed Persons
#'   head(results$transformed_person_params)
#' @export
equate_irt <- function(base_params, new_params, person_params = NULL,
                       methods = NULL) {

  # --- 1. Dependencies: Math Helpers ---

  # 1.1 Expected Score Calculator (for Stocking-Lord)
  get_expected_score <- function(theta, a, b_vec, c_param, model) {
    # Returns Expected Score (TCC contribution) for a single item at a vector of theta
    D <- 1.7

    # Binary
    if (model %in% c("Rasch", "2PL", "3PL")) {
      lin <- D * a * (theta - b_vec[1])
      p <- 1 / (1 + exp(-lin))
      if (model == "3PL") p <- c_param + (1 - c_param) * p
      return(p) # Expected score for binary is P(1)
    }

    # GPCM / PCM (Sum of k * Pk)
    if (model %in% c("GPCM", "PCM")) {
      n_cat <- length(b_vec) + 1
      numerators <- matrix(0, nrow = length(theta), ncol = n_cat)
      curr <- 0
      numerators[,1] <- 0
      for (k in 2:n_cat) {
        curr <- curr + a * (theta - b_vec[k-1])
        numerators[,k] <- curr
      }
      exps <- exp(numerators)
      probs <- exps / rowSums(exps)

      # Expected Score = Sum(k * Pk) where k=0..(m-1)
      # Pk is col k+1. Score is k.
      esc <- numeric(length(theta))
      for (k in 0:(n_cat-1)) {
        esc <- esc + k * probs[, k+1]
      }
      return(esc)
    }

    # GRM (Sum of Cumulative Probs)
    if (model == "GRM") {
      # ES = Sum_{k=1}^{m} P*_k
      esc <- numeric(length(theta))
      for (k in 1:length(b_vec)) {
        lin <- D * a * (theta - b_vec[k])
        p_star <- 1 / (1 + exp(-lin))
        esc <- esc + p_star
      }
      return(esc)
    }
    return(numeric(length(theta)))
  }

  # 1.2 Parameter Extractor
  extract_item_row <- function(row) {
    # Reads a DF row and outputs list(a, b_vec, c)
    mod <- row$model

    a <- if(!is.null(row$a) && !is.na(row$a)) row$a else 1.0
    if (mod %in% c("Rasch", "PCM")) a <- 1.0

    c_p <- if(!is.null(row$c) && !is.na(row$c)) row$c else 0.0
    if (!mod %in% c("3PL")) c_p <- 0.0

    # Find b/steps
    b_cols <- grep("^(b|d\\d+|step_\\d+)$", names(row), value=TRUE)
    b_cols <- b_cols[order(nchar(b_cols), b_cols)]
    b_vec <- as.numeric(row[b_cols])
    b_vec <- b_vec[!is.na(b_vec)]
    if(length(b_vec)==0 && "b" %in% names(row)) b_vec <- row$b

    return(list(a=a, b=b_vec, c=c_p, model=mod))
  }

  # --- 2. Input Validation & Setup ---

  message("---------------------------------------------------------")
  message("Starting IRT Equating / Linking")
  message("---------------------------------------------------------")

  # Default methods
  valid_methods <- c("Mean-Mean", "Mean-Sigma", "Stocking-Lord")
  if (is.null(methods)) methods <- valid_methods
  methods <- intersect(methods, valid_methods)

  if (length(methods) == 0) stop("No valid methods selected.")

  message(paste("Methods selected:", paste(methods, collapse=", ")))

  # Anchor Detection
  anchors <- intersect(base_params$item, new_params$item)
  n_anchors <- length(anchors)

  if (n_anchors == 0) stop("No common items (anchors) found between Base and New parameters.")
  if (n_anchors < 3) warning("Fewer than 3 anchor items detected. Linking may be unstable.")

  # Validate Models match
  base_anchors <- base_params[base_params$item %in% anchors, ]
  new_anchors <- new_params[new_params$item %in% anchors, ]

  # Sort to ensure alignment
  base_anchors <- base_anchors[order(base_anchors$item), ]
  new_anchors <- new_anchors[order(new_anchors$item), ]

  mismatches <- base_anchors$model != new_anchors$model
  if (any(mismatches)) {
    stop(paste("Model mismatch for anchor items:",
               paste(base_anchors$item[mismatches], collapse=", ")))
  }

  models_detected <- unique(c(base_params$model, new_params$model))
  message(paste("Models detected:", paste(models_detected, collapse=", ")))
  message(paste("Items detected: Base =", nrow(base_params), "| New =", nrow(new_params)))
  message(paste("Anchor items detected:", n_anchors))
  message(paste("Anchor IDs:", paste(anchors, collapse = ", ")))
  # ----------------------------
  if (!is.null(person_params)) message(paste("Persons to transform:", nrow(person_params)))

  # --- 3. Compute Linking Constants (A & B) ---

  results_list <- list() # Store A/B for each method

  # Pre-extract anchor stats for MM/MS
  # We need vectors of 'a' and 'b' (centroid)

  # Base stats
  ba_a <- numeric(n_anchors)
  ba_b <- numeric(n_anchors) # Centroid

  # New stats
  na_a <- numeric(n_anchors)
  na_b <- numeric(n_anchors)

  for (i in 1:n_anchors) {
    # Base
    p_b <- extract_item_row(base_anchors[i, ])
    ba_a[i] <- p_b$a
    ba_b[i] <- mean(p_b$b) # Mean of steps/difficulty

    # New
    p_n <- extract_item_row(new_anchors[i, ])
    na_a[i] <- p_n$a
    na_b[i] <- mean(p_n$b)
  }

  # --- Method 1: Mean-Mean ---
  if ("Mean-Mean" %in% methods) {
    message("Estimating constants: Mean-Mean...")
    # A = mean(a_base) / mean(a_new)
    # B = mean(b_base) - A * mean(b_new)

    # Note: For Rasch/PCM, a=1, so A=1.
    mu_a_base <- mean(ba_a)
    mu_a_new  <- mean(na_a)
    mu_b_base <- mean(ba_b)
    mu_b_new  <- mean(na_b)

    A_mm <- mu_a_base / mu_a_new
    B_mm <- mu_b_base - A_mm * mu_b_new

    results_list[["Mean-Mean"]] <- c(A = A_mm, B = B_mm)
  }

  # --- Method 2: Mean-Sigma ---
  if ("Mean-Sigma" %in% methods) {
    message("Estimating constants: Mean-Sigma...")
    # A = sd(b_base) / sd(b_new)
    # B = mean(b_base) - A * mean(b_new)

    sd_b_base <- sd(ba_b)
    sd_b_new  <- sd(na_b)

    # Safety for single item (SD=NA)
    if (is.na(sd_b_base) || is.na(sd_b_new) || sd_b_new == 0) {
      warning("Mean-Sigma requires >1 anchor item with variance. Skipping.")
    } else {
      A_ms <- sd_b_base / sd_b_new
      B_ms <- mean(ba_b) - A_ms * mean(na_b)
      results_list[["Mean-Sigma"]] <- c(A = A_ms, B = B_ms)
    }
  }

  # --- Method 3: Stocking-Lord ---
  if ("Stocking-Lord" %in% methods) {
    message("Estimating constants: Stocking-Lord (TCC Optimization)...")

    # Quadrature points for TCC integration
    nodes <- seq(-4, 4, length.out = 31)

    # Calculate Base TCC (Fixed)
    tcc_base <- numeric(length(nodes))
    for (i in 1:n_anchors) {
      p <- extract_item_row(base_anchors[i,])
      tcc_base <- tcc_base + get_expected_score(nodes, p$a, p$b, p$c, p$model)
    }

    # Optimization Function
    # Find A, B to transform New Params -> New TCC -> Match Base TCC
    loss_function <- function(par) {
      A <- par[1]
      B <- par[2]

      tcc_new_trans <- numeric(length(nodes))

      for (i in 1:n_anchors) {
        p <- extract_item_row(new_anchors[i,])

        # Transform parameters
        # a* = a / A
        # b* = A * b + B
        a_star <- p$a / A
        b_star <- A * p$b + B
        c_star <- p$c # Invariant

        tcc_new_trans <- tcc_new_trans + get_expected_score(nodes, a_star, b_star, c_star, p$model)
      }

      # Sum of squared differences
      return(sum((tcc_base - tcc_new_trans)^2))
    }

    # Optimize using L-BFGS-B (Allows bounds for A > 0)
    # Start A=1, B=0
    opt <- tryCatch({
      stats::optim(c(1, 0), loss_function, method = "L-BFGS-B",
                   lower = c(0.01, -10), upper = c(10, 10))
    }, error = function(e) {
      warning("Stocking-Lord optimization failed. Returning NA.")
      return(NULL)
    })

    if (!is.null(opt)) {
      results_list[["Stocking-Lord"]] <- c(A = opt$par[1], B = opt$par[2])
    }
  }

  # --- 4. Apply Transformations & Format Output ---

  message("Transforming parameters...")

  out_items <- data.frame()
  out_persons <- data.frame()
  out_consts <- data.frame()

  # Loop through calculated constants
  for (m in names(results_list)) {
    consts <- results_list[[m]]
    A <- consts["A"]
    B <- consts["B"]

    # 4.1 Transform Linking Constants DF
    out_consts <- rbind(out_consts, data.frame(
      Method = m, A = A, B = B, stringsAsFactors = FALSE
    ))

    # 4.2 Transform Items (New Form)
    for (i in 1:nrow(new_params)) {
      row <- new_params[i, ]
      p <- extract_item_row(row)

      # Transformation Logic
      # a* = a / A
      # b* = A * b + B
      # SE(a*) = SE(a) / A
      # SE(b*) = SE(b) * A

      # Transform Value
      new_a <- p$a / A
      new_b <- A * p$b + B
      new_c <- p$c

      # Transform SE (if exists)
      se_a <- if(!is.null(row$discrimination_se)) row$discrimination_se / A else NA
      se_c <- if(!is.null(row$guessing_se)) row$guessing_se  else NA #SE(c) remains invariant (unchanged).

      # Handle b SEs (need to match step columns)
      se_b_cols <- grep("(_se)$", names(row), value=TRUE)
      # Filter only difficulty/step SEs
      se_b_cols <- se_b_cols[!grepl("discrimination|guessing", se_b_cols)]
      se_b_vals <- as.numeric(row[se_b_cols])
      new_se_b <- se_b_vals * A

      # Construct Result Row
      res <- list(
        item = row$item,
        anchor_status = ifelse(row$item %in% anchors, "Anchor", "New"),
        method = m,
        model = row$model,
        discrimination = new_a,
        discrimination_se = se_a,
        guessing = new_c,
        guessing_se = se_c
      )

      # Add dynamic steps
      if (length(new_b) == 1 && row$model %in% c("Rasch", "2PL", "3PL")) {
        res$difficulty <- new_b[1]
        res$difficulty_se <- if(length(new_se_b)>0) new_se_b[1] else NA
      } else {
        res$difficulty <- NA
        for (k in 1:length(new_b)) {
          res[[paste0("step_", k)]] <- new_b[k]
          res[[paste0("step_", k, "_se")]] <- if(k <= length(new_se_b)) new_se_b[k] else NA
        }
      }

      # Bind to main DF (inefficient but safe for variable columns)
      # We convert list to single-row DF
      res_df <- as.data.frame(res, stringsAsFactors = FALSE)
      out_items <- dplyr::bind_rows(out_items, res_df) # Using dplyr bind_rows logic if avail, else base rbind
    }

    # 4.3 Transform Persons (if provided)
    if (!is.null(person_params)) {
      # theta* = A * theta + B
      # SE(theta*) = A * SE(theta)
      p_df <- person_params
      p_df$theta_trans <- A * p_df$theta + B
      p_df$theta_se_trans <- A * p_df$theta_se
      p_df$method <- m

      out_persons <- rbind(out_persons, p_df)
    }
  }

  # --- 5. Clean Up Output Columns ---

  # Reorder Item DF
  base_cols <- c("item", "anchor_status", "method", "model",
                 "discrimination", "discrimination_se",
                 "difficulty", "difficulty_se", "guessing", "guessing_se")
  step_cols <- sort(grep("step_", names(out_items), value=TRUE))
  final_cols <- c(base_cols, step_cols)
  final_cols <- intersect(final_cols, names(out_items))

  out_items <- out_items[, final_cols]
  row.names(out_items)<-NULL
  row.names(out_persons)<-NULL
  row.names(out_consts)<-NULL

  message("Equating completed.")
  message("---------------------------------------------------------")

  return(list(
    transformed_item_params = out_items,
    transformed_person_params = out_persons,
    linking_constants = out_consts
  ))
}

#' Mixture Item Response Theory Model (Latent-Class Rasch / 2PL)
#'
#' @description
#' Fits a mixture item response theory model (Rost, 1990) in which the examinee
#' population is assumed to consist of a small number of unobserved latent
#' classes, each with its own set of item parameters. Mixture IRT models are used
#' to detect qualitatively different response strategies, unmodeled
#' subpopulations, or classes for which item difficulty ordering differs. The
#' model is estimated by the Expectation-Maximization algorithm with a fixed
#' standard-normal ability distribution within each class.
#'
#' @param data A data frame of dichotomous (0/1) item responses (rows = persons,
#'   columns = items).
#' @param n_class Integer. Number of latent classes to estimate (default
#'   \code{2}).
#' @param model String. \code{"Rasch"} (default) or \code{"2PL"}; controls
#'   whether class-specific discriminations are estimated.
#' @param control A \code{list} of control parameters for the algorithm:
#'   \itemize{
#'     \item \code{max_iter}: Maximum number of EM iterations (default = 100).
#'     \item \code{converge_tol}: Convergence criterion on the log-likelihood
#'       change (default = 1e-4).
#'     \item \code{quad_points}: Number of quadrature points for the within-class
#'       ability distribution (default = 21).
#'     \item \code{theta_range}: Ability grid bounds (default = c(-4, 4)).
#'     \item \code{verbose}: Logical; print progress (default = TRUE).
#'   }
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{item_params}: A data frame of class-specific item difficulties
#'     (and discriminations for the 2PL model), with one column per class.
#'   \item \code{class_params}: A data frame of estimated mixing proportions for
#'     each latent class.
#'   \item \code{person_params}: A data frame of posterior class-membership
#'     probabilities and the modal (most likely) class for each person.
#'   \item \code{model_fit}: A data frame with the log-likelihood, AIC, BIC,
#'     number of classes, and classification entropy.
#' }
#'
#' @references
#' Rost, J. (1990). Rasch models in latent classes: An integration of two
#' approaches to item analysis. \emph{Applied Psychological Measurement, 14}(3),
#' 271-282.
#'
#' @examples
#'   # Two classes with reversed difficulty ordering
#'   set.seed(2025)
#'   N <- 300; J <- 8
#'   b1 <- seq(-1.5, 1.5, length.out = J)
#'   b2 <- rev(b1)
#'   theta <- rnorm(N)
#'   cls <- rep(1:2, each = N / 2)
#'   resp <- matrix(0, N, J)
#'   for (i in 1:N) {
#'     b <- if (cls[i] == 1) b1 else b2
#'     resp[i, ] <- rbinom(J, 1, 1 / (1 + exp(-(theta[i] - b))))
#'   }
#'   df <- as.data.frame(resp); names(df) <- paste0("I", 1:J)
#'
#'   fit <- mixture_irt(df, n_class = 2, model = "Rasch",
#'                      control = list(max_iter = 30, verbose = FALSE))
#'   fit$class_params
#'   head(fit$item_params)
#' @export
mixture_irt <- function(data,
                        n_class = 2,
                        model = "Rasch",
                        control = list()) {

  con <- list(max_iter = 100, converge_tol = 1e-4, quad_points = 21,
              theta_range = c(-4, 4), verbose = TRUE)
  con[names(control)] <- control

  model <- toupper(model)
  if (!model %in% c("RASCH", "2PL")) stop("'model' must be 'Rasch' or '2PL'.")
  if (!is.data.frame(data) && !is.matrix(data)) stop("'data' must be a data frame or matrix.")

  X <- as.matrix(data)
  if (!all(X[!is.na(X)] %in% c(0, 1))) stop("mixture_irt() currently supports dichotomous (0/1) data.")
  N <- nrow(X); J <- ncol(X)
  C <- as.integer(n_class)
  item_names <- colnames(X); if (is.null(item_names)) item_names <- paste0("Item_", seq_len(J))

  nodes <- seq(con$theta_range[1], con$theta_range[2], length.out = con$quad_points)
  qw <- dnorm(nodes); qw <- qw / sum(qw)
  Q <- length(nodes)

  # --- initialize ---
  pi_c <- rep(1 / C, C)
  p_mean <- pmin(pmax(colMeans(X, na.rm = TRUE), 0.05), 0.95)
  b0 <- -log(p_mean / (1 - p_mean))
  # spread classes apart with small perturbations for identifiability
  b <- matrix(rep(b0, C), J, C)
  for (cc in seq_len(C)) b[, cc] <- b0 + seq(-0.5, 0.5, length.out = J) * (cc - (C + 1) / 2)
  a <- matrix(1, J, C)

  prob_node <- function(theta, aj, bj) {
    z <- aj * (theta - bj); z <- pmin(pmax(z, -30), 30); 1 / (1 + exp(-z))
  }

  if (con$verbose) {
    cat(sprintf("\nStarting Mixture %s Estimation (%d classes)...\n", model, C))
    cat("------------------------------------------------\n")
  }

  old_ll <- -Inf
  is_converged <- FALSE
  Post_class <- matrix(1 / C, N, C)

  for (iter in seq_len(con$max_iter)) {

    # E-step: class x node likelihoods
    # L_node[i, q, c] then marginal over q
    Lc <- matrix(0, N, C)                    # marginal per class
    node_post <- array(0, dim = c(N, Q, C))  # unnormalized person-node-class

    for (cc in seq_len(C)) {
      logLq <- matrix(0, N, Q)
      for (j in seq_len(J)) {
        P <- prob_node(nodes, a[j, cc], b[j, cc])
        P <- pmin(pmax(P, 1e-10), 1 - 1e-10)
        x <- X[, j]; valid <- !is.na(x)
        if (!any(valid)) next
        add <- outer(x[valid], log(P)) + outer(1 - x[valid], log(1 - P))
        logLq[valid, ] <- logLq[valid, ] + add
      }
      m <- apply(logLq, 1, max)
      Fq <- exp(logLq - m) * matrix(qw, N, Q, byrow = TRUE)
      Lc[, cc] <- rowSums(Fq) * exp(m)
      node_post[, , cc] <- Fq * exp(m)       # unnormalized joint of node & data
    }

    joint <- sweep(Lc, 2, pi_c, "*")         # N x C, pi_c * L_ic
    denom <- rowSums(joint)
    denom[denom <= 0] <- 1e-300
    Post_class <- joint / denom
    ll <- sum(log(denom))

    # M-step: mixing proportions
    pi_c <- colMeans(Post_class)
    pi_c <- pmax(pi_c, 1e-6); pi_c <- pi_c / sum(pi_c)

    # M-step: item parameters per class
    for (cc in seq_len(C)) {
      # person-node weights within class cc: r_ic * node_post / Lc
      denom_c <- Lc[, cc]; denom_c[denom_c <= 0] <- 1e-300
      w_node <- (node_post[, , cc] / denom_c) * Post_class[, cc]   # N x Q

      for (j in seq_len(J)) {
        x <- X[, j]; valid <- !is.na(x)
        wj <- w_node[valid, , drop = FALSE]
        r_q <- colSums(wj * x[valid])
        n_q <- colSums(wj)
        if (sum(n_q) < 1e-6) next

        # Weighted (expected-count) log-likelihood for this item in this class.
        ll_item <- function(par) {
          la <- if (model == "2PL") par[1] else 1
          lb <- if (model == "2PL") par[2] else par[1]
          P <- prob_node(nodes, la, lb); P <- pmin(pmax(P, 1e-9), 1 - 1e-9)
          sum(r_q * log(P) + (n_q - r_q) * log(1 - P))
        }

        # Damped Newton-Raphson with finite-difference gradient / diagonal Hessian.
        par <- if (model == "2PL") c(a[j, cc], b[j, cc]) else b[j, cc]
        h <- 1e-4
        for (nr in 1:15) {
          f0 <- ll_item(par)
          grad <- numeric(length(par)); hess <- numeric(length(par))
          for (k in seq_along(par)) {
            pu <- par; pu[k] <- pu[k] + h
            pd <- par; pd[k] <- pd[k] - h
            grad[k] <- (ll_item(pu) - ll_item(pd)) / (2 * h)
            hess[k] <- (ll_item(pu) - 2 * f0 + ll_item(pd)) / (h^2)
          }
          hess <- ifelse(hess > -1e-6, -1e-6, hess)   # keep negative definite
          step <- grad / hess
          step <- pmax(pmin(step, 1), -1)
          par <- par - step
          if (model == "2PL") par[1] <- max(0.05, min(4, par[1]))
          par[length(par)] <- max(-10, min(10, par[length(par)]))
          if (max(abs(step)) < 1e-4) break
        }
        if (model == "2PL") { a[j, cc] <- par[1]; b[j, cc] <- par[2] } else b[j, cc] <- par[1]
      }
    }

    if (con$verbose) cat(sprintf("\rIteration %d: LogLik = %.3f   ", iter, ll))

    if (iter > 2 && abs(ll - old_ll) < con$converge_tol) {
      if (con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
      is_converged <- TRUE
      break
    }
    old_ll <- ll
  }

  if (!is_converged && con$verbose) {
    cat(sprintf("\n\n>>> NOTE: Stopped because max Iteration Reached at %d.\n", con$max_iter))
  }

  # --- Output ---
  item_df <- data.frame(item = item_names, stringsAsFactors = FALSE)
  for (cc in seq_len(C)) {
    if (model == "2PL") item_df[[paste0("discrimination_c", cc)]] <- round(a[, cc], 3)
    item_df[[paste0("difficulty_c", cc)]] <- round(b[, cc], 3)
  }

  class_df <- data.frame(class = seq_len(C), proportion = round(pi_c, 4),
                         stringsAsFactors = FALSE)

  modal <- max.col(Post_class, ties.method = "first")
  person_df <- data.frame(person = seq_len(N))
  for (cc in seq_len(C)) person_df[[paste0("class", cc)]] <- round(Post_class[, cc], 3)
  person_df$modal_class <- modal

  # classification entropy (normalized); undefined for a single class
  ent <- -sum(Post_class * log(pmax(Post_class, 1e-12)))
  entropy <- if (C <= 1) NA_real_ else 1 - ent / (N * log(C))

  n_par <- (C - 1) + C * J * (if (model == "2PL") 2 else 1)
  fit_df <- data.frame(
    Index = c("LogLikelihood", "AIC", "BIC", "n_class", "entropy"),
    Value = c(round(old_ll, 3),
              round(2 * n_par - 2 * old_ll, 3),
              round(n_par * log(N) - 2 * old_ll, 3),
              C, round(entropy, 3)),
    stringsAsFactors = FALSE
  )

  if (con$verbose) {
    cat("Finished All Estimation.\n")
    cat("------------------------------------------------\n")
  }

  list(item_params = item_df,
       class_params = class_df,
       person_params = person_df,
       model_fit = fit_df)
}

#' Polytomous Item Response Theory Estimation Using Likelihood or Bayesian
#'
#' @description
#' Estimates item and person parameters for polytomous item response theory models
#' using either Marginal Maximum Likelihood or Joint Maximum Likelihood.
#' Now supports flexible prior distributions for Bayesian estimation (MAP estimation).
#'
#' @param data A N x J data.frame of polytomous responses (0, 1, 2...).
#'             Missing values should be NA. Categories must be continuous integers.
#' @param model String. "GPCM" (Generalized Partial Credit Model), "PCM" (Partial Credit Model), or "GRM" (Graded Response Model).
#' @param method String. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood). However, using Bayesian will override the likelihood estimation.
#' @param control A \code{list} of control parameters for the estimation algorithm:
#'   \itemize{
#'     \item \code{max_iter}: Maximum number of EM iterations (default = 100).
#'     \item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
#'     \item \code{theta_range}: Numeric vector of length 2 specifying the integration
#'           grid bounds (default = c(-4, 4)).
#'     \item \code{quad_points}: Number of quadrature points (default = 21).
#'     \item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
#'     \item \code{prior}: A list specifying prior distributions for item parameters. Default is NULL (no priors).
#'       For GRM: \code{list(a = function(x) dlnorm(x, 0, 0.5, log=TRUE), d = function(x) dnorm(x, 0, 2, log=TRUE))}.
#'       For GPCM: \code{list(a = function(x) dlnorm(x, 0, 0.5, log=TRUE), d = function(x) dnorm(x, 0, 2, log=TRUE))}.
#'       For PCM: \code{list(d = function(x) dnorm(x, 0, 2, log=TRUE))}.
#'       Each prior is a function returning log-density and applies to ALL items.
#'       Fixed value priors are NOT supported.
#'       Item-specific priors are NOT supported.
#'   }
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{item_params}: Data frame of estimated parameters (a, thresholds).
#'   \item \code{person_params}: A data frame of estimated person abilities (theta) and standard errors.
#'   \item \code{model_fit}: A data frame containing fit statistics such as Akaike’s Information Criterion (AIC) and the Bayesian Information Criterion (BIC).
#'   \item \code{settings}: A list of control parameters used in the estimation.
#' }
#' @examples
#'   # --- Quick Example (small data) ---
#'   set.seed(1)
#'   n <- 50
#'   resp <- data.frame(
#'     I1 = sample(0:2, n, replace = TRUE),
#'     I2 = sample(0:2, n, replace = TRUE),
#'     I3 = sample(0:3, n, replace = TRUE)
#'   )
#'   res <- polytomous_irt(resp, model = "GPCM", method = "EM",
#'                         control = list(max_iter = 5, verbose = FALSE))
#'   head(res$item_params)
#'  \donttest{
#'   # --- Full Example 1: Simulation (GPCM) ---
#'   set.seed(2026)
#'   N <- 500; J <- 5
#'   n_cats <- c(3, 4, 3, 5, 4)
#'
#'   true_theta <- rnorm(N)
#'   true_a <- runif(J, 0.8, 1.2)
#'   true_d <- list()
#'
#'   # Generate Thresholds
#'   for(j in 1:J) {
#'     steps <- sort(rnorm(n_cats[j]-1, mean = 0, sd = 1.0))
#'     true_d[[j]] <- c(0, cumsum(steps))
#'   }
#'
#'   # Simulation Helper (GPCM Logic)
#'   generate_resp <- function(theta, a, d_vec, n_cat) {
#'     probs <- matrix(0, length(theta), n_cat)
#'     for(k in 1:n_cat) {
#'       z <- a * (k-1) * theta - d_vec[k]
#'       probs[,k] <- exp(z)
#'     }
#'     probs <- probs / rowSums(probs)
#'     apply(probs, 1, function(p) sample(0:(n_cat-1), 1, prob=p))
#'   }
#'
#'   # Create Data
#'   sim_data <- matrix(NA, nrow = N, ncol = J)
#'   for(j in 1:J) {
#'     sim_data[,j] <- generate_resp(true_theta, true_a[j], true_d[[j]], n_cats[j])
#'   }
#'   df_sim <- as.data.frame(sim_data)
#'
#'   # Run Estimation (GPCM to match simulation logic without prior)
#'   res <- polytomous_irt(df_sim, model="GPCM", method="EM",
#'                         control=list(max_iter=20, verbose=TRUE))
#'
#'   head(res$item_params)
#'   print(res$model_fit)
#'
#'   # Run Estimation with prior (MAP)
#'   res_prior <- polytomous_irt(df_sim, model="PCM", method="EM",
#'                               control=list(max_iter=20, verbose=FALSE,
#'                                           prior=list(
#'                                             d = function(x) dnorm(x, 0, 2, log=TRUE)
#'                                           )))
#'   head(res$item_params)
#'   print(res$model_fit)
#'
#'   # --- Example 2: With Package Data (GRM) ---
#'   data("ela1", package = "tirt")
#'
#'   # Subset polytomous items (columns 31 to 45)
#'   df_poly <- ela1[, 31:45]
#'
#'   # Run Estimation using GRM
#'   real_res <- polytomous_irt(df_poly, model="GRM", method="EM",
#'                              control = list(max_iter = 1000))
#'
#'   head(real_res$item_params)
#'   head(real_res$person_params)
#'   print(real_res$model_fit)
#'   # Run Estimation using GRM with prior
#'   real_res2 <- polytomous_irt(df_poly, model="GRM", method="EM",
#'                              control = list(max_iter = 1000,
#'                                            prior = list(
#'                                              a = function(x) dlnorm(x, 0, 0.5, log=TRUE),
#'                                              d = function(x) dnorm(x, 0, 2, log=TRUE)
#'                                            )))
#'
#'   head(real_res2$item_params)
#'   head(real_res2$person_params)
#'   print(real_res2$model_fit)
#'   }
#' @export
polytomous_irt <- function(data,
                           model = "GPCM",
                           method = "EM",
                           control = list()) {

  # --- 1. Setup and Defaults ---
  con <- list(
    max_iter = 100,
    converge_tol = 1e-4,
    theta_range = c(-4, 4),
    quad_points = 21,
    nr_max_iter = 20,
    nr_damp = 0.8,
    verbose = TRUE
  )
  con[names(control)] <- control

  # Extract prior from control
  prior <- con$prior

  # Normalize model name to uppercase
  model <- toupper(model)

  # Define the set of supported models
  valid_models <- c("GPCM", "PCM", "GRM")

  # Validate that every element in the input vector is within the allowed set
  if (!all(model %in% valid_models)) {
    stop("Model must be 'GPCM', 'PCM', or 'GRM' (case-insensitive)")
  }

  # --- Input Validation ---
  if(!is.data.frame(data)) stop("Input must be a data frame.")

  # Check data types
  raw_mat <- as.matrix(data)
  if(!is.numeric(raw_mat)) stop("Data must be numeric. Remove ID columns.")

  item_names <- colnames(data)
  if(is.null(item_names)) item_names <- paste0("Item_", 1:ncol(data))

  N <- nrow(raw_mat)
  J <- ncol(raw_mat)

  # --- Data Preprocessing (Shift to 0-based) ---
  min_val <- min(raw_mat, na.rm=TRUE)
  if(is.finite(min_val) && min_val > 0) {
    if(con$verbose) cat("Note: Responses shifted to start at 0.\n")
    raw_mat <- raw_mat - min_val
  }

  # Determine max categories per item
  n_cats <- apply(raw_mat, 2, function(x) {
    if(all(is.na(x))) return(0)
    max(x, na.rm=TRUE) + 1
  })
  max_K <- max(n_cats)
  if(max_K < 2) stop("Data must contain at least 2 categories.")

  # Identify valid items (Variance > 0)
  item_vars <- apply(raw_mat, 2, var, na.rm=TRUE)
  valid_items_idx <- which(item_vars > 0)

  if(length(valid_items_idx) == 0) stop("No valid items found.")

  # --- Prior Validation and Processing ---
  # This function validates priors for polytomous models
  validate_and_expand_prior <- function(prior, model, verbose) {
    # If no prior, return NULL
    if(is.null(prior)) {
      if(verbose) cat("No prior specified. Using Maximum Likelihood Estimation as Specified.\n")
      return(NULL)
    }

    # Check that prior is a list
    if(!is.list(prior)) {
      stop("Prior must be a list. See documentation for proper format.")
    }

    # 1. Define allowed parameters based on model
    allowed_params <- switch(model,
                             "PCM"  = "d",
                             "GPCM" = c("a", "d"),
                             "GRM"  = c("a", "d"),
                             stop("Unknown model type specified. Support only PCM, GPCM, and GRM now..."))

    # 2. Check for extra/unexpected parameters
    extra_params <- setdiff(names(prior), allowed_params)
    if(length(extra_params) > 0) {
      warning(sprintf("Prior specification contains parameter(s) [%s] which are not used in a %s model. These will be ignored.",
                      paste(extra_params, collapse=", "), model))
    }

    # 3. Filter to only allowed and provided parameters
    relevant_params <- intersect(names(prior), allowed_params)
    if(length(relevant_params) == 0) return(NULL)

    if(verbose) {
      cat(sprintf("Using MAP (Bayesian) estimation for parameters: %s\n", paste(relevant_params, collapse=", ")))
    }

    # 4. Validate each prior is a function (no fixed values, no item-specific)
    validated_prior <- list()
    for(param in relevant_params) {
      param_prior <- prior[[param]]

      # Check if it's a function
      if(!is.function(param_prior)) {
        stop(sprintf("Prior for parameter '%s' must be a function returning log-density. Fixed value priors and item-specific priors are not supported for polytomous models.",
                     param))
      }

      # Test the function
      test_val <- tryCatch({
        param_prior(0.5)
      }, error = function(e) {
        stop(sprintf("Prior function for parameter '%s' failed when tested: %s", param, e$message))
      })

      if(!is.numeric(test_val) || length(test_val) != 1) {
        stop(sprintf("Prior function for parameter '%s' must return a single numeric value (log-density).", param))
      }

      validated_prior[[param]] <- param_prior

      if(verbose) {
        cat(sprintf("  - Parameter '%s': Prior function applied to ALL items/parameters\n", param))
      }
    }

    return(validated_prior)
  }
  # Validate and process priors
  prior_expanded <- validate_and_expand_prior(prior, model, con$verbose)

  # Helper function to compute log-prior density
  log_prior_density <- function(value, prior_func) {
    if(is.null(prior_func)) return(0)
    if(!is.function(prior_func)) return(0)
    if(is.na(value)) return(0)
    tryCatch(prior_func(value), error = function(e) 0)
  }

  # --- Initialize Parameters ---
  a <- rep(1, J)
  d <- matrix(0, J, max_K)

  # --- UPDATED: Initialize Standard Error Storage ---
  se_a <- rep(NA, J)
  se_d <- matrix(NA, J, max_K)

  for(j in valid_items_idx) {
    valid_resp <- raw_mat[!is.na(raw_mat[,j]), j]
    if(length(valid_resp) == 0) next

    props <- table(factor(valid_resp, levels=0:(n_cats[j]-1))) / length(valid_resp)
    cum_props <- cumsum(props)
    cum_props <- pmin(pmax(cum_props, 0.01), 0.99)
    thresh <- -qlogis(cum_props[-length(cum_props)])

    if(model == "GRM") {
      d[j, 1:(n_cats[j]-1)] <- sort(thresh)
    } else {
      K <- n_cats[j]
      steps <- seq(-0.5, 0.5, length.out = K-1)
      d[j, 2:K] <- steps
    }
  }

  if (model == "PCM") a[] <- 1

  # Initialize Theta
  raw_scores <- rowSums(raw_mat, na.rm=TRUE)
  theta <- as.vector(scale(raw_scores))
  theta[is.nan(theta) | is.na(theta)] <- 0

  # Quadrature setup (Only used for EM)
  nodes <- seq(con$theta_range[1], con$theta_range[2], length.out = con$quad_points)
  weights <- dnorm(nodes)
  weights <- weights / sum(weights)


  # --- 2. Helper Functions ---

  get_P_poly <- function(th_vec, a_val, d_vec, mod_type, n_cat) {
    n_q <- length(th_vec)
    probs <- matrix(0, n_q, n_cat)

    if (mod_type %in% c("GPCM", "PCM")) {
      for(k in 1:n_cat) {
        z <- a_val * (k-1) * th_vec - d_vec[k]
        z <- pmin(pmax(z, -50), 50)
        probs[,k] <- exp(z)
      }
      row_sums <- rowSums(probs)
      row_sums[row_sums == 0] <- 1
      probs <- probs / row_sums

    } else if (mod_type == "GRM") {
      P_star <- matrix(0, n_q, n_cat + 1)
      P_star[,1] <- 1
      P_star[,n_cat + 1] <- 0

      eff_d <- d_vec[1:(n_cat-1)]
      if(is.unsorted(eff_d)) eff_d <- sort(eff_d)

      for(k in 1:(n_cat-1)) {
        z <- a_val * (th_vec - eff_d[k])
        z <- pmin(pmax(z, -30), 30)
        P_star[, k+1] <- 1 / (1 + exp(-z))
      }
      for(k in 1:n_cat) {
        probs[,k] <- P_star[,k] - P_star[,k+1]
      }
      probs <- pmax(probs, 1e-10)
    }
    return(probs)
  }

  solve_item_poly <- function(r_mat, n_vec, th_vec, cur_a, cur_d, mod_type, n_cat) {
    if(mod_type == "PCM") {
      param_vec <- cur_d[2:n_cat]
    } else if(mod_type == "GRM") {
      param_vec <- c(cur_a, cur_d[1:(n_cat-1)])
    } else {
      param_vec <- c(cur_a, cur_d[2:n_cat])
    }

    n_params <- length(param_vec)

    ll_func <- function(p_v) {
      if(mod_type == "PCM") {
        l_a <- 1; l_d <- numeric(max_K); l_d[2:n_cat] <- p_v
      } else {
        l_a <- p_v[1]; l_d <- numeric(max_K)
        if(mod_type == "GRM") l_d[1:(n_cat-1)] <- p_v[-1]
        else l_d[2:n_cat] <- p_v[-1]
      }
      probs <- get_P_poly(th_vec, l_a, l_d, mod_type, n_cat)
      probs <- pmin(pmax(probs, 1e-10), 1-1e-10)

      log_lik_data <- sum(r_mat * log(probs))

      # Add priors if specified
      if(!is.null(prior_expanded)) {
        if(mod_type != "PCM" && !is.null(prior_expanded$a)) {
          log_lik_data <- log_lik_data + log_prior_density(l_a, prior_expanded$a)
        }
        if(!is.null(prior_expanded$d)) {
          if(mod_type == "GRM") {
            for(k in 1:(n_cat-1)) {
              log_lik_data <- log_lik_data + log_prior_density(l_d[k], prior_expanded$d)
            }
          } else {
            for(k in 2:n_cat) {
              log_lik_data <- log_lik_data + log_prior_density(l_d[k], prior_expanded$d)
            }
          }
        }
      }

      return(log_lik_data)
    }

    for(iter in 1:con$nr_max_iter) {
      f0 <- ll_func(param_vec)
      h <- 1e-4
      grad <- numeric(n_params)
      hess <- matrix(0, n_params, n_params)

      for(k in 1:n_params) {
        p_up <- param_vec; p_up[k] <- p_up[k] + h
        grad[k] <- (ll_func(p_up) - f0) / h
      }

      for(k in 1:n_params) {
        p_up <- param_vec; p_up[k] <- p_up[k] + h
        p_dn <- param_vec; p_dn[k] <- p_dn[k] - h
        hess[k,k] <- (ll_func(p_up) - 2*f0 + ll_func(p_dn)) / (h^2)
      }
      diag(hess) <- diag(hess) - 1e-4

      try_step <- try(solve(hess, grad), silent = TRUE)
      if(inherits(try_step, "try-error")) break

      step <- try_step * con$nr_damp
      step <- pmax(pmin(step, 1.0), -1.0)
      param_vec <- param_vec - step

      if(mod_type != "PCM") param_vec[1] <- max(param_vec[1], 0.05)
      if(max(abs(step)) < 1e-4) break
    }

    se_vec <- tryCatch({
      suppressWarnings(sqrt(diag(solve(-hess))))
    }, error = function(e) rep(NA, n_params))

    ret_a <- if(mod_type=="PCM") 1 else param_vec[1]
    ret_d <- cur_d
    ret_se_a <- NA
    ret_se_d <- rep(NA, max_K)

    if(mod_type == "PCM") {
      ret_d[2:n_cat] <- param_vec; ret_se_d[2:n_cat] <- se_vec
    } else if (mod_type == "GRM") {
      raw_thresh <- param_vec[-1]
      ret_d[1:(n_cat-1)] <- sort(raw_thresh)
      ret_se_a <- se_vec[1]
      if(is.unsorted(raw_thresh)) {
        ret_se_d[1:(n_cat-1)] <- NA
      } else {
        ret_se_d[1:(n_cat-1)] <- se_vec[-1]
      }
    } else {
      ret_d[2:n_cat] <- param_vec[-1]; ret_se_a <- se_vec[1]; ret_se_d[2:n_cat] <- se_vec[-1]
    }
    return(list(a=ret_a, d=ret_d, se_a=ret_se_a, se_d=ret_se_d))
  }

  # --- 3. Main Estimation Loop ---

  if(con$verbose) cat(sprintf("\nStarting Polytomous Estimation (%s) using %s...\n", model, method))

  converged <- FALSE

  for (iter in 1:con$max_iter) {

    # --- E-Step (EM) or Person Step (MLE) ---
    if (method == "EM") {
      # Calculate Posterior probabilities for Quadrature nodes
      L_mat <- matrix(0, N, con$quad_points)
      for(j in valid_items_idx) {
        P_jq <- get_P_poly(nodes, a[j], d[j,], model, n_cats[j])
        resp <- raw_mat[, j]
        valid_rows <- !is.na(resp)
        p_indices <- resp[valid_rows] + 1
        item_loglik <- matrix(0, sum(valid_rows), con$quad_points)
        for(k in 1:n_cats[j]) {
          is_cat <- which(p_indices == k)
          if(length(is_cat) > 0) {
            val <- log(pmax(P_jq[,k], 1e-50))
            item_loglik[is_cat, ] <- matrix(val, length(is_cat), con$quad_points, byrow=TRUE)
          }
        }
        L_mat[valid_rows, ] <- L_mat[valid_rows, ] + item_loglik
      }
      F_iq <- exp(L_mat) * matrix(weights, N, con$quad_points, byrow=TRUE)
      sum_Fi <- rowSums(F_iq)
      sum_Fi[sum_Fi == 0] <- 1e-10
      Posterior_iq <- F_iq / sum_Fi

    } else {
      # Method == "MLE": Estimate Theta for every person given current items
      for(i in 1:N) {
        valid <- !is.na(raw_mat[i,]) & (1:J %in% valid_items_idx)
        if(sum(valid)==0) next
        ti <- theta[i]
        for(k in 1:5) { # Few NR steps for efficiency
          num <- 0; den <- 0
          for(j in which(valid)) {
            probs <- get_P_poly(ti, a[j], d[j,], model, n_cats[j])[1,]
            x <- raw_mat[i,j]
            h <- 1e-4
            # Numerical derivatives
            p0 <- probs[x+1]
            pp <- get_P_poly(ti+h, a[j], d[j,], model, n_cats[j])[1, x+1]
            pm <- get_P_poly(ti-h, a[j], d[j,], model, n_cats[j])[1, x+1]
            d1 <- (log(pp)-log(pm))/(2*h)
            d2 <- (log(pp)-2*log(p0)+log(pm))/(h^2)
            num <- num + d1; den <- den + d2
          }
          if(is.na(den) || abs(den) < 1e-5) break
          step <- num/den
          ti <- ti - step
          ti <- max(min(ti, con$theta_range[2]), con$theta_range[1])
          if(abs(step) < 1e-3) break
        }
        theta[i] <- ti
      }
      # Re-center Theta to fix scale indeterminacy in JML
      theta <- theta - mean(theta, na.rm=TRUE)
    }

    # --- M-Step (EM) or Item Step (MLE) ---
    max_change <- 0

    for(j in valid_items_idx) {
      old_par <- c(a[j], d[j,])

      if(method == "EM") {
        # --- EM Preparation ---
        # R_qk: Expected number of people in category k at quadrature node q
        R_qk <- matrix(0, con$quad_points, n_cats[j])
        resp <- raw_mat[, j]
        valid <- !is.na(resp)
        for(k in 1:n_cats[j]) {
          idx <- which(valid & resp == (k-1))
          if(length(idx) > 0) R_qk[,k] <- colSums(Posterior_iq[idx, , drop=FALSE])
        }
        N_q <- rowSums(R_qk)
        use_nodes <- nodes # Use global quadrature nodes

      } else {
        # --- MLE (JML) Preparation ---
        valid_idx <- which(!is.na(raw_mat[, j]))
        if(length(valid_idx) == 0) next

        y <- raw_mat[valid_idx, j]
        use_nodes <- theta[valid_idx]

        n_resp <- length(y)
        R_qk <- matrix(0, n_resp, n_cats[j])

        idx_mat <- cbind(1:n_resp, y + 1)
        R_qk[idx_mat] <- 1

        N_q <- rep(1, n_resp)
      }

      # Solve Item Parameters
      res <- solve_item_poly(R_qk, N_q, use_nodes, a[j], d[j,], model, n_cats[j])
      a[j] <- res$a
      d[j,] <- res$d

      # --- UPDATED: Store Standard Errors ---
      se_a[j] <- res$se_a
      se_d[j,] <- res$se_d

      max_change <- max(max_change, abs(c(a[j], d[j,]) - old_par), na.rm=TRUE)
    }

    if(con$verbose) cat(paste0("\rIteration ", iter, ": Max Param Change = ", round(max_change, 5), "   "))

    if(max_change < con$converge_tol) {
      if(con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
      converged <- TRUE
      break
    }
  }

  if(!converged && con$verbose) {
    cat(sprintf("\nNOTE: Stopped because max Iteration Reached at %d.\n", con$max_iter))
    cat("    Adjust it in control=list(max_iter = ...) to have higher iteration if needed.\n")
  }

  # --- 4. Post-Hoc Estimation & Output ---
  if(con$verbose) {
    cat("\n------------------------------------------------\n")
    cat("Estimating Final Person Parameters...\n")
  }

  final_theta <- numeric(N)
  final_se <- numeric(N)

  for(i in 1:N) {
    valid <- !is.na(raw_mat[i,]) & (1:J %in% valid_items_idx)
    if(sum(valid)==0) { final_theta[i] <- NA; final_se[i] <- NA; next }

    # Use existing estimates as starting values for speed
    ti <- if(method=="EM") 0 else theta[i]

    for(k in 1:20) {
      score_num <- 0; info_den <- 0
      for(j in which(valid)) {
        probs <- get_P_poly(ti, a[j], d[j,], model, n_cats[j])[1,]
        x <- raw_mat[i, j]
        h <- 1e-4
        probs_p <- get_P_poly(ti+h, a[j], d[j,], model, n_cats[j])[1,]
        probs_m <- get_P_poly(ti-h, a[j], d[j,], model, n_cats[j])[1,]

        lp0 <- log(max(probs[x+1], 1e-50))
        lpp <- log(max(probs_p[x+1], 1e-50))
        lpm <- log(max(probs_m[x+1], 1e-50))

        d1 <- (lpp - lpm) / (2*h)
        d2 <- (lpp - 2*lp0 + lpm) / (h^2)
        score_num <- score_num + d1; info_den <- info_den - d2
      }
      if(is.na(info_den) || abs(info_den) < 1e-6) break
      step <- score_num / info_den
      step <- max(min(step, 1.0), -1.0)
      ti <- ti + step
      if(abs(step) < 1e-4) break
    }
    final_theta[i] <- max(min(ti, con$theta_range[2]), con$theta_range[1])
    if(!is.na(info_den) && info_den > 1e-6) final_se[i] <- 1/sqrt(info_den) else final_se[i] <- NA
  }

  if(con$verbose) cat("Person Parameter Estimation Finished.\n")

  # --- Output Construction (UPDATED) ---
  out_list <- list(
    item = as.character(item_names),
    discrimination = as.numeric(round(a, 3)),
    discrimination_se = as.numeric(round(se_a, 3)), # <--- _se suffix
    n_cat = as.integer(n_cats)
  )

  n_thresh_cols <- max_K - 1
  if(n_thresh_cols > 0) {
    for(k in 1:n_thresh_cols) {
      col_vals <- rep(NA_real_, J)
      se_col_vals <- rep(NA_real_, J)

      for(j in valid_items_idx) {
        if(model == "GRM") {
          if(k < n_cats[j]) {
            col_vals[j] <- d[j, k]
            se_col_vals[j] <- se_d[j, k]
          }
        } else {
          if(k < n_cats[j]) {
            col_vals[j] <- d[j, k+1]
            se_col_vals[j] <- se_d[j, k+1]
          }
        }
      }
      col_name <- paste0(if(model=="GRM") "thresh_" else "step_", k)
      out_list[[col_name]] <- round(col_vals, 3)

      se_col_name <- paste0(col_name, "_se") # <--- _se suffix
      out_list[[se_col_name]] <- round(se_col_vals, 3)
    }
  }

  # Ensure no rownames in output data.frames
  df_items <- data.frame(out_list, stringsAsFactors = FALSE, row.names = NULL)

  df_persons <- data.frame(
    ability = as.numeric(round(final_theta, 3)),
    ability_se = as.numeric(round(final_se, 3)), # <--- _se suffix
    n_resp = as.integer(rowSums(!is.na(raw_mat))),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  log_lik <- 0
  for(i in 1:N) {
    valid <- !is.na(raw_mat[i,]) & !is.na(final_theta[i])
    if(!any(valid)) next
    for(j in which(valid)) {
      probs <- get_P_poly(final_theta[i], a[j], d[j,], model, n_cats[j])[1,]
      log_lik <- log_lik + log(max(probs[raw_mat[i,j]+1], 1e-50))
    }
  }

  n_par <- length(valid_items_idx) + sum(n_cats[valid_items_idx]-1)
  aic <- 2*n_par - 2*log_lik
  bic <- n_par*log(N) - 2*log_lik

  df_fit <- data.frame(
    Index = c("LogLikelihood", "AIC", "BIC"),
    Value = as.numeric(c(round(log_lik,3), round(aic,3), round(bic,3))),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  if(con$verbose) {
    cat("Finished All Estimation.\n")
    cat("------------------------------------------------\n")
  }

  return(list(
    item_params = df_items,
    person_params = df_persons,
    model_fit = df_fit,
    settings = list(model=model, method=method)
  ))
}

#' Unidimensional Polytomous Testlet Response Theory Estimation
#'
#' @description
#' Estimates item and person parameters for Polytomous Testlet models using Robust Newton-Raphson optimization.
#'
#' @param data A data.frame of polytomous responses. Rows=persons, Cols=items in testlets.
#' @param group A list defining testlet structures. Example: `list(c(1,2,3), c(4,5,6))`.
#' @param model Character. "GRT" (Graded Response Model), "PCMT" (Partial Credit Model for Testlet), or "BiFT" (Biffactor).
#' @param method Character. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood).
#' @param control A \code{list} of control parameters for the estimation algorithm:
#'   \itemize{
#'     \item \code{max_iter}: Maximum number of EM iterations (default = 100).
#'     \item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
#'     \item \code{theta_range}: Numeric vector of length 2 specifying the integration
#'           grid bounds (default = c(-4, 4)).
#'     \item \code{quad_points}: Number of quadrature points (default = 21).
#'     \item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
#'   }
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{item_params}: A data frame of estimated item parameters.
#'   \item \code{person_params}: A data frame of estimated person abilities and testlet effects .
#'   \item \code{model_fit}: A data frame containing iterations and fit statistics such as Akaike’s Information Criterion (AIC), the Bayesian Information Criterion (BIC), and Log-Likelihood.
#' }
#' @examples
#' \donttest{
#'   # --- Example: Simulation (Mixed Categories GRT) ---
#'   set.seed(42)
#'   N <- 500; J <- 16
#'
#'   # Define Groups (4 Testlets)
#'   groups <- list(c(1:4), c(5:8), c(9:12), c(13:16))
#'
#'   # Define Categories (Binary, 3-cat, 4-cat, Mixed)
#'   # Items 1-4: 2 cats; 5-8: 3 cats; 9-12: 4 cats; 13-16: mixed
#'   cats <- c(rep(2, 4), rep(3, 4), rep(4, 4), 3, 5, 3, 5)
#'
#'   # 1. Generate Parameters
#'   theta <- rnorm(N)
#'   # Gamma for 4 testlets (SD = 0.8)
#'   gamma <- matrix(rnorm(N * 4, 0, 0.8), N, 4)
#'
#'   a <- rlnorm(J, 0, 0.2)
#'   b_list <- vector("list", J)
#'
#'   # Generate Thresholds based on category count
#'   for(j in 1:J) {
#'     n_thresh <- cats[j] - 1
#'     if(n_thresh == 1) {
#'       b_list[[j]] <- rnorm(1)
#'     } else {
#'       # Spread thresholds
#'       b_list[[j]] <- sort(rnorm(1) + seq(-1, 1, length.out=n_thresh))
#'     }
#'   }
#'
#'   # 2. Generate Responses (GRT Logic)
#'   resp <- matrix(NA, N, J)
#'   colnames(resp) <- paste0("Item_", 1:J)
#'
#'   for(i in 1:N) {
#'     for(j in 1:J) {
#'       # Identify Testlet ID
#'       tid <- which(sapply(groups, function(x) j %in% x))
#'       eff <- theta[i] + gamma[i, tid]
#'
#'       # Calculate Probabilities (Graded Response)
#'       K <- cats[j]
#'       probs <- numeric(K)
#'       P_prev <- 1
#'       for(k in 1:(K-1)) {
#'         term <- a[j] * (eff - b_list[[j]][k])
#'         P_star <- 1 / (1 + exp(-term))
#'         probs[k] <- P_prev - P_star
#'         P_prev <- P_star
#'       }
#'       probs[K] <- P_prev
#'
#'       # Sample Response
#'       resp[i, j] <- sample(0:(K-1), 1, prob = probs)
#'     }
#'   }
#'   df_sim <- as.data.frame(resp)
#'
#'   # 3. Run Estimation
#'   fit <- trt_poly(
#'     data = df_sim,
#'     group = groups,
#'     model = "GRT",
#'     method = "EM",
#'     control = list(max_iter = 20, verbose = FALSE)
#'   )
#'
#'   head(fit$item_params)
#'   head(fit$person_params)
#' }
#' @export
trt_poly <- function(
    data,
    group,
    model = c("GRT", "PCMT", "BiFT"),
    method = c("MLE", "EM"),
    control = list()
) {

  # ===========================================================================
  # 1. SETUP & VALIDATION
  # ===========================================================================
  model  <- match.arg(model)
  method <- match.arg(method)

  fix_discrimination <- (model == "PCMT")

  con <- list(
    max_iter       = 100,
    converge_tol   = 1e-4,
    theta_range    = c(-4, 4),
    quad_points    = 21,
    nr_max_iter    = 10,
    nr_damp        = 1.0,
    verbose        = TRUE
  )
  con[names(control)] <- control

  if(!is.data.frame(data)) stop("Input must be a data frame.")
  raw_data <- as.matrix(data)
  if(!is.numeric(raw_data)) stop("Input data must contain only numeric values.")

  n_persons  <- nrow(raw_data)
  n_items    <- ncol(raw_data)
  item_names <- colnames(data)
  if(is.null(item_names)) item_names <- paste0("Item_", 1:n_items)

  min_val <- min(raw_data, na.rm=TRUE)
  if(min_val > 0) raw_data <- raw_data - min_val

  max_cats <- apply(raw_data, 2, max, na.rm=TRUE)
  global_max_K <- max(max_cats)

  item_to_testlet <- rep(NA, n_items)
  for(g_idx in seq_along(group)) {
    g_items <- group[[g_idx]]
    if(is.character(g_items)) idx <- match(g_items, item_names) else idx <- g_items
    item_to_testlet[idx] <- g_idx
  }
  n_testlets <- length(group)

  if(con$verbose) {
    cat(sprintf("\nStarting %s Estimation using %s algorithm...\n", model, method))
    if(fix_discrimination) cat("Note: Discrimination fixed to 1.0 (Rasch/PCMT specification).\n")
    cat("------------------------------------------------\n")
    cat(sprintf("Data: %d Persons, %d Items, %d Testlets\n", n_persons, n_items, n_testlets))
  }

  # ===========================================================================
  # 2. HELPER FUNCTIONS
  # ===========================================================================

  plogis_c  <- function(x) 1 / (1 + exp(-x))

  get_probs <- function(theta_vec, a, b_vec, mod) {
    n <- length(theta_vec)
    K <- length(b_vec)
    if(mod == "GRT") {
      p_star <- matrix(0, n, K + 2)
      p_star[, 1] <- 1; p_star[, K + 2] <- 0
      for(k in 1:K) {
        z <- a * (theta_vec - b_vec[k])
        z[z > 30] <- 30; z[z < -30] <- -30
        p_star[, k+1] <- plogis_c(z)
      }
      probs <- p_star[, 1:(K+1)] - p_star[, 2:(K+2)]
    } else {
      numer <- matrix(0, n, K + 1)
      current_sum <- rep(0, n)
      for(k in 1:K) {
        current_sum <- current_sum + a * (theta_vec - b_vec[k])
        numer[, k+1] <- current_sum
      }
      max_val <- apply(numer, 1, max)
      exps <- exp(numer - max_val)
      probs <- exps / rowSums(exps)
    }
    probs[probs < 1e-15] <- 1e-15; probs[probs > 1-1e-15] <- 1-1e-15
    probs
  }

  get_derivs_1d <- function(y, theta, a, b, mod) {
    eps <- 1e-4
    f <- function(t) {
      probs <- get_probs(t, a, b, mod)
      log(probs[y+1])
    }
    v0 <- f(theta)
    g <- (f(theta+eps) - f(theta-eps))/(2*eps)
    h <- (f(theta+eps) - 2*v0 + f(theta-eps))/(eps^2)
    list(g=g, h=h)
  }

  manual_hessian <- function(fn, par) {
    eps <- 1e-4
    n <- length(par)
    H <- matrix(0, n, n); G <- numeric(n); val0 <- fn(par)
    for(i in 1:n) {
      p_up <- par; p_up[i] <- p_up[i] + eps
      p_dn <- par; p_dn[i] <- p_dn[i] - eps
      G[i] <- (fn(p_up) - fn(p_dn)) / (2*eps)
    }
    for(i in 1:n) {
      for(j in i:n) {
        if(i == j) {
          p_up <- par; p_up[i] <- p_up[i] + eps
          p_dn <- par; p_dn[i] <- p_dn[i] - eps
          H[i,j] <- (fn(p_up) - 2*val0 + fn(p_dn)) / (eps^2)
        } else {
          p_uu <- par; p_uu[i] <- p_uu[i] + eps; p_uu[j] <- p_uu[j] + eps
          p_ud <- par; p_ud[i] <- p_ud[i] + eps; p_ud[j] <- p_ud[j] - eps
          p_du <- par; p_du[i] <- p_du[i] - eps; p_du[j] <- p_du[j] + eps
          p_dd <- par; p_dd[i] <- p_dd[i] - eps; p_dd[j] <- p_dd[j] - eps
          H[i,j] <- (fn(p_uu) - fn(p_ud) - fn(p_du) + fn(p_dd)) / (4*eps^2)
          H[j,i] <- H[i,j]
        }
      }
    }
    list(G=G, H=H)
  }

  nr_item_update_robust <- function(start_par, W, Nodes, mod, fixed_a, item_idx) {
    curr_par <- start_par
    ll_fn <- function(p) {
      if(fixed_a) { a <- 1.0; b <- p } else { a <- p[1]; b <- p[-1] }
      if(!fixed_a && (a < 0.05 || a > 5.0)) return(-1e20)
      if(any(abs(b) > 10)) return(-1e20)
      if(mod == "GRT" && is.unsorted(b)) return(-1e20)
      probs <- get_probs(Nodes, a, b, mod)
      sum(W * log(probs))
    }
    current_ll <- ll_fn(curr_par)
    for(iter in 1:con$nr_max_iter) {
      derivs <- manual_hessian(ll_fn, curr_par)
      H <- derivs$H; diag(H) <- diag(H) - 1e-3
      delta <- try(solve(H, derivs$G), silent=TRUE)
      if(inherits(delta, "try-error")) delta <- -derivs$G * 0.1
      step_size <- 1.0; improved <- FALSE
      for(attempt in 1:5) {
        candidate <- curr_par - step_size * delta
        if(!fixed_a) candidate[1] <- max(0.1, min(4.0, candidate[1]))
        n_b <- if(fixed_a) length(candidate) else length(candidate)-1
        b_indices <- if(fixed_a) 1:n_b else 2:(n_b+1)
        candidate[b_indices] <- pmax(-6, pmin(6, candidate[b_indices]))
        if(mod == "GRT") candidate[b_indices] <- sort(candidate[b_indices])
        new_ll <- ll_fn(candidate)
        if(new_ll > current_ll + 1e-5) {
          curr_par <- candidate; current_ll <- new_ll; improved <- TRUE; break
        } else { step_size <- step_size * 0.5 }
      }
      if(!improved || max(abs(delta * step_size)) < 1e-4) break
    }
    return(curr_par)
  }

  # ===========================================================================
  # 3. INITIALIZATION
  # ===========================================================================

  a_est <- rep(1.0, n_items)
  b_est <- vector("list", n_items)
  bad_items_idx <- c()

  for(i in 1:n_items) {
    y <- raw_data[,i]; y <- y[!is.na(y)]
    if(length(unique(y)) < 2) {
      bad_items_idx <- c(bad_items_idx, i)
      b_est[[i]] <- rep(0, max_cats[i])
      next
    }
    K <- max_cats[i]
    props <- table(y) / length(y)
    cum_props <- cumsum(props)[1:K]
    cum_props[cum_props < 0.01] <- 0.01; cum_props[cum_props > 0.99] <- 0.99
    b_est[[i]] <- sort(-qlogis(cum_props))
  }

  p_theta <- rep(0, n_persons)
  p_gamma <- matrix(0, n_persons, n_testlets)
  gamma_se <- matrix(NA, n_persons, n_testlets)
  sigma_gamma <- rep(0.5, n_testlets)

  log_L_data <- -Inf
  iter <- 0
  is_converged <- FALSE

  # ===========================================================================
  # 4. ALGORITHMS (EM or MLE)
  # ===========================================================================

  # --- 4A. EM ALGORITHM ---
  if(method == "EM") {

    get_gh <- function(n) {
      m <- n-1; i <- 1:m; b <- sqrt(i/2)
      CM <- diag(0,n); CM[cbind(1:m, 2:n)] <- b; CM[cbind(2:n, 1:m)] <- b
      ev <- eigen(CM, symmetric=TRUE)
      list(x = ev$values*sqrt(2), w = (ev$vectors[1,]^2)*sqrt(pi))
    }
    gh <- get_gh(con$quad_points)
    X_q <- gh$x; W_q <- gh$w / sqrt(pi)
    n_q <- length(X_q)

    for(it in 1:con$max_iter) {
      iter <- it

      # E-STEP
      L_indep <- matrix(1, n_persons, n_q)
      for(i in which(is.na(item_to_testlet))) {
        if(i %in% bad_items_idx) next
        probs <- get_probs(X_q, a_est[i], b_est[[i]], model)
        y <- raw_data[, i]; obs <- !is.na(y)
        if(any(obs)) {
          for(q in 1:n_q) L_indep[obs, q] <- L_indep[obs, q] * probs[q, y[obs]+1]
        }
      }

      L_testlets_marg <- array(1, dim=c(n_persons, n_testlets, n_q))

      # We need L_block stored for later M-step approximation in a real efficient code,
      # but here we recalculate or use simplified.

      for(t in 1:n_testlets) {
        t_items <- which(item_to_testlet == t)
        t_items <- setdiff(t_items, bad_items_idx)
        if(length(t_items)==0) next

        sig <- sqrt(sigma_gamma[t]); if(sig < 1e-4) sig <- 1e-4
        G_nodes <- X_q * sig
        L_block <- array(1, dim=c(n_persons, n_q, n_q)) # [Person, Theta, Gamma]

        for(i in t_items) {
          grid_vals <- as.vector(outer(X_q, G_nodes, "+"))
          probs_grid <- get_probs(grid_vals, a_est[i], b_est[[i]], model)
          y <- raw_data[, i]; obs <- !is.na(y)
          if(sum(obs)==0) next
          for(u in 1:n_q) {
            for(v in 1:n_q) {
              idx <- (v-1)*n_q + u
              p_vec <- probs_grid[idx, ]
              L_block[obs, u, v] <- L_block[obs, u, v] * p_vec[y[obs]+1]
            }
          }
        }
        L_testlets_marg[, t, ] <- apply(L_block, c(1,2), function(x) sum(x * W_q))
      }

      L_total <- L_indep
      for(t in 1:n_testlets) L_total <- L_total * L_testlets_marg[, t, ]
      L_y <- as.vector(L_total %*% W_q)
      L_y[L_y < 1e-100] <- 1e-100
      log_L_curr <- sum(log(L_y))
      Post_Theta <- (L_total * matrix(W_q, n_persons, n_q, byrow=TRUE)) / L_y

      # M-STEP
      max_change <- 0
      for(i in 1:n_items) {
        if(i %in% bad_items_idx) next
        y <- raw_data[, i]; obs <- !is.na(y)
        K <- max_cats[i]
        W_counts <- matrix(0, n_q, K+1)
        for(k in 0:K) {
          idx_p <- which(obs & y == k)
          if(length(idx_p) > 0) W_counts[, k+1] <- colSums(Post_Theta[idx_p, , drop=FALSE])
        }

        old_p <- if(fix_discrimination) b_est[[i]] else c(a_est[i], b_est[[i]])
        new_p <- nr_item_update_robust(old_p, W_counts, X_q, model, fix_discrimination, i)

        if(fix_discrimination) b_est[[i]] <- new_p else { a_est[i] <- new_p[1]; b_est[[i]] <- new_p[-1] }
        max_change <- max(max_change, max(abs(new_p - old_p)))
      }

      # Update Sigma Gamma (Approx)
      # Simpler approach: Keep sigma fixed or simple update.
      # Detailed update omitted for brevity/stability, assuming 0.5 start or user input.

      if(con$verbose) cat(sprintf("\rIter %3d | LogLik: %10.2f | Max Change: %.4f   ", iter, log_L_curr, max_change))

      if(max_change < con$converge_tol) {
        if(con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
        is_converged <- TRUE
        break
      }
      log_L_data <- log_L_curr
    }

    # --- EM POST-HOC: EAP SCORING FOR GAMMA ---
    if(con$verbose) cat("\nCalculating Testlet Effects (EAP) for EM...\n")

    # We need to compute E[Gamma | y] for each person/testlet
    # P(Gamma | y) ~ Integral L(y_t | Theta, Gamma) * Post(Theta | y_rest) * P(Gamma)
    # Approx: Use Post_Theta (which includes y_t) but divide out y_t marginal contribution?
    # Simpler Robust Approx: Joint Expectation using final parameters

    for(t in 1:n_testlets) {
      t_items <- which(item_to_testlet == t)
      t_items <- setdiff(t_items, bad_items_idx)
      if(length(t_items)==0) next

      sig <- sqrt(sigma_gamma[t]); if(sig < 1e-4) sig <- 1e-4
      G_nodes <- X_q * sig

      # Re-calculate L_block and L_marg for this testlet
      L_block <- array(1, dim=c(n_persons, n_q, n_q))
      for(i in t_items) {
        grid_vals <- as.vector(outer(X_q, G_nodes, "+"))
        probs_grid <- get_probs(grid_vals, a_est[i], b_est[[i]], model)
        y <- raw_data[, i]; obs <- !is.na(y)
        if(sum(obs)==0) next
        for(u in 1:n_q) {
          for(v in 1:n_q) {
            idx <- (v-1)*n_q + u
            L_block[obs, u, v] <- L_block[obs, u, v] * probs_grid[idx, y[obs]+1]
          }
        }
      }
      L_marg_t <- apply(L_block, c(1,2), function(x) sum(x * W_q))
      L_marg_t[L_marg_t < 1e-100] <- 1e-100

      # Weight: Post_Theta[p, u] removes the marginalized contribution of t
      # approx by dividing Post_Theta by L_marg_t *prior(theta)?
      # Better: Joint(u, v) = Post_Theta(u) * [ L_block(u,v)*W(v) / L_marg_t(u) ]

      Gamma_EAP <- rep(0, n_persons)
      Gamma_Var <- rep(0, n_persons)

      # Loop persons (vectorized over nodes)
      for(p in 1:n_persons) {
        # J[u, v]
        # ratio[u] = Post_Theta[p, u] / L_marg_t[p, u]
        ratio <- Post_Theta[p, ] / L_marg_t[p, ]

        # Joint Distribution P(u, v)
        # J[u, v] = ratio[u] * L_block[p, u, v] * W_q[v]
        J <- (ratio %o% W_q) * L_block[p, , ]

        # Normalize
        total_mass <- sum(J)
        if(total_mass > 0) J <- J / total_mass

        # Marginalize over Theta (sum over u) -> P(v) for Gamma
        P_gamma_v <- colSums(J)

        # Expectation
        val <- sum(P_gamma_v * G_nodes)
        var_val <- sum(P_gamma_v * G_nodes^2) - val^2

        Gamma_EAP[p] <- val
        Gamma_Var[p] <- var_val
      }
      p_gamma[, t] <- Gamma_EAP
      gamma_se[, t] <- sqrt(pmax(0, Gamma_Var))
    }

  }

  # --- 4B. MLE ALGORITHM ---
  else if(method == "MLE") {

    p_theta <- rowSums(raw_data, na.rm=TRUE)
    p_theta <- as.vector(scale(p_theta))

    for(it in 1:con$max_iter) {
      iter <- it

      # Persons
      for(p in 1:n_persons) {
        y <- raw_data[p, ]; obs <- !is.na(y)
        if(!any(obs)) next
        curr_t <- p_theta[p]
        for(k in 1:5) {
          eff <- curr_t + ifelse(is.na(item_to_testlet), 0, p_gamma[p, item_to_testlet])
          d <- list(g=0, h=0)
          for(i in which(obs)) {
            if(i %in% bad_items_idx) next
            res <- get_derivs_1d(y[i], eff[i], a_est[i], b_est[[i]], model)
            d$g <- d$g + res$g; d$h <- d$h + res$h
          }
          d$g <- d$g - curr_t; d$h <- d$h - 1
          delta <- d$g / (d$h - 1e-5)
          curr_t <- curr_t - delta
          if(abs(delta) < 0.01) break
        }
        p_theta[p] <- max(min(curr_t, 4), -4)

        # Gamma
        for(t in 1:n_testlets) {
          t_items <- which(item_to_testlet == t)
          t_items <- setdiff(t_items, bad_items_idx)
          t_obs <- t_items[ !is.na(y[t_items]) ]
          if(length(t_obs) == 0) { p_gamma[p, t] <- 0; next }
          curr_g <- p_gamma[p, t]
          for(k in 1:5) {
            eff <- p_theta[p] + curr_g
            d <- list(g=0, h=0)
            for(i in t_obs) {
              res <- get_derivs_1d(y[i], eff, a_est[i], b_est[[i]], model)
              d$g <- d$g + res$g; d$h <- d$h + res$h
            }
            d$g <- d$g - curr_g/0.5; d$h <- d$h - 1/0.5
            delta <- d$g / (d$h - 1e-5)
            curr_g <- curr_g - delta
            if(abs(delta) < 0.01) break
          }
          p_gamma[p, t] <- max(min(curr_g, 3), -3)
        }
      }
      p_theta <- as.vector(scale(p_theta))

      # Items
      max_change <- 0
      curr_ll <- 0
      for(i in 1:n_items) {
        if(i %in% bad_items_idx) next
        y <- raw_data[, i]; obs <- !is.na(y)
        if(sum(obs) < 5) { bad_items_idx <- unique(c(bad_items_idx, i)); next }

        tid <- item_to_testlet[i]
        eff_theta <- p_theta[obs]
        if(!is.na(tid)) eff_theta <- eff_theta + p_gamma[obs, tid]
        y_obs <- y[obs]

        old_par <- if(fix_discrimination) b_est[[i]] else c(a_est[i], b_est[[i]])

        ll_fn_mle <- function(par) {
          a <- if(fix_discrimination) 1.0 else par[1]
          b <- if(fix_discrimination) par else par[-1]
          if(!fix_discrimination && (a<0.05 || a>5)) return(-1e10)
          if(model=="GRT" && is.unsorted(b)) return(-1e10)
          probs <- get_probs(eff_theta, a, b, model)
          sum(log(probs[cbind(1:length(y_obs), y_obs+1)]))
        }

        for(nr in 1:con$nr_max_iter) {
          derivs <- manual_hessian(ll_fn_mle, curr_par)
          H <- derivs$H; diag(H) <- diag(H) - 1e-3
          delta <- try(solve(H, derivs$G), silent=TRUE)
          if(inherits(delta, "try-error")) delta <- -derivs$G * 0.1

          step <- 1.0; improved <- FALSE; base_ll <- ll_fn_mle(curr_par)
          for(att in 1:5) {
            cand <- curr_par - step*delta
            if(!fix_discrimination) cand[1] <- max(0.1, min(4, cand[1]))
            if(model=="GRT") {
              b_idx <- if(fix_discrimination) 1:length(cand) else 2:length(cand)
              cand[b_idx] <- sort(cand[b_idx])
            }
            if(ll_fn_mle(cand) > base_ll + 1e-5) { curr_par <- cand; improved <- TRUE; break }
            step <- step*0.5
          }
          if(!improved) break
        }

        if(fix_discrimination) b_est[[i]] <- curr_par else { a_est[i] <- curr_par[1]; b_est[[i]] <- curr_par[-1] }
        max_change <- max(max_change, max(abs(curr_par - old_par)))
        curr_ll <- curr_ll + ll_fn_mle(curr_par)
      }

      if(con$verbose) cat(sprintf("\rIter %3d | LogLik: %10.2f | Max Change: %.4f   ", iter, curr_ll, max_change))

      if(max_change < con$converge_tol) {
        if(con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
        is_converged <- TRUE
        break
      }
      log_L_data <- curr_ll
    }
  }

  if(!is_converged && con$verbose) {
    cat(sprintf("\n\n>>> NOTE: Stopped because max Iteration Reached at %d.\n", con$max_iter))
    cat("    Adjust it in control=list(max_iter = ...) to have higher iteration if needed.\n")
  }

  # ===========================================================================
  # 5. FINAL SEs & OUTPUT
  # ===========================================================================

  if(con$verbose) {
    cat("\n------------------------------------------------\n")
    cat("Estimating Final Person Parameters...\n")
    cat(sprintf("Personal Parameter estimated with range [%s, %s].\n",
                con$theta_range[1], con$theta_range[2]))
    cat("Adjust it in control=list(theta_range=c(..)) if you want a different range.\n")
  }

  if(method == "EM") p_theta <- as.vector(Post_Theta %*% X_q)

  se_a <- rep(NA, n_items)
  se_b <- vector("list", n_items)

  for(i in 1:n_items) {
    if(i %in% bad_items_idx) { se_b[[i]] <- rep(NA, length(b_est[[i]])); next }

    tid <- item_to_testlet[i]; eff_theta <- p_theta
    if(!is.na(tid)) eff_theta <- eff_theta + p_gamma[, tid]

    obs_idx <- which(!is.na(raw_data[,i])); eff <- eff_theta[obs_idx]
    if(length(eff) == 0) next

    par_now <- if(fix_discrimination) b_est[[i]] else c(a_est[i], b_est[[i]])
    n_par <- length(par_now)
    a_val <- if(fix_discrimination) 1.0 else par_now[1]
    b_val <- if(fix_discrimination) par_now else par_now[-1]

    P_base <- get_probs(eff, a_val, b_val, model)
    J_list <- vector("list", n_par)
    eps <- 1e-4

    for(k in 1:n_par) {
      p_up <- par_now; p_up[k] <- p_up[k] + eps
      a_u <- if(fix_discrimination) 1.0 else p_up[1]
      b_u <- if(fix_discrimination) p_up else p_up[-1]
      if(model == "GRT" && is.unsorted(b_u)) b_u <- sort(b_u)
      P_up <- get_probs(eff, a_u, b_u, model)
      J_list[[k]] <- (P_up - P_base)/eps
    }

    Info <- matrix(0, n_par, n_par)
    W <- 1/P_base; W[P_base < 1e-10] <- 0
    for(r in 1:n_par) { for(c in r:n_par) { val <- sum(J_list[[r]]*J_list[[c]]*W); Info[r,c] <- val; Info[c,r] <- val } }

    cov_mat <- try(solve(Info + diag(1e-5, n_par)), silent=TRUE)
    if(!inherits(cov_mat, "try-error") && all(diag(cov_mat)>0)) {
      ses <- sqrt(diag(cov_mat))
      if(fix_discrimination) se_b[[i]] <- ses else { se_a[i] <- ses[1]; se_b[[i]] <- ses[-1] }
    } else { se_b[[i]] <- rep(NA, length(b_est[[i]])) }
  }

  # Person SEs for MLE
  if(method == "MLE") {
    # Approx Gamma SE calc similar to EM not done for brevity, just Theta SE
    # Recalculating Theta SE approx
    p_se <- rep(NA, n_persons)
    for(p in 1:n_persons) {
      eff <- p_theta[p] + ifelse(is.na(item_to_testlet), 0, p_gamma[p, item_to_testlet])
      info_t <- 1
      for(i in which(!is.na(raw_data[p,]))) {
        res <- get_derivs_1d(raw_data[p,i], eff[i], a_est[i], b_est[[i]], model)
        info_t <- info_t - res$h
      }
      if(info_t > 0) p_se[p] <- 1/sqrt(info_t)

      # Gamma SE for MLE
      for(t in 1:n_testlets) {
        info_g <- 2; curr_g <- p_gamma[p, t]; eff_g <- p_theta[p] + curr_g
        t_items <- which(item_to_testlet == t)
        for(i in t_items) {
          if(!is.na(raw_data[p,i])) {
            res <- get_derivs_1d(raw_data[p,i], eff_g, a_est[i], b_est[[i]], model)
            info_g <- info_g - res$h
          }
        }
        if(info_g>0) gamma_se[p,t] <- 1/sqrt(info_g)
      }
    }
  } else {
    # EM Person SEs
    p_se <- sqrt(as.vector(Post_Theta %*% X_q^2) - p_theta^2)
  }

  out_items <- data.frame(item=item_names, discrimination=a_est, discrimination_se=se_a, stringsAsFactors=FALSE)
  diff_mean <- sapply(b_est, mean)
  diff_se   <- sapply(se_b, function(x) if(all(is.na(x))) NA else mean(x, na.rm=TRUE))
  out_items$difficulty <- diff_mean; out_items$difficulty_se <- diff_se

  for(k in 1:global_max_K) {
    col <- if(model=="GRT"||model=="BiFT") "threshold" else "step"
    out_items[[paste0(col, "_", k)]] <- sapply(b_est, function(x) if(length(x)>=k) x[k] else NA)
    out_items[[paste0(col, "_", k, "_se")]] <- sapply(se_b, function(x) if(length(x)>=k) x[k] else NA)
  }

  if(length(bad_items_idx) > 0) {
    cols_to_NA <- which(!names(out_items) %in% c("item"))
    out_items[bad_items_idx, cols_to_NA] <- NA
    if(con$verbose)
      message("\nWarning: Some items have zero variance or degenerate categories. Parameters set to NA.")
  }

  out_persons <- data.frame(person=1:n_persons, ability=p_theta, ability_se=p_se)
  for(t in 1:n_testlets) { out_persons[[paste0("testlet_", t)]] <- p_gamma[,t]; out_persons[[paste0("testlet_", t, "_se")]] <- gamma_se[,t] }

  n_p <- sum(sapply(b_est, length)) + (if(fix_discrimination) 0 else n_items)
  if(method=="MLE") n_p <- n_p + n_persons*(1+n_testlets)
  aic <- 2*n_p - 2*log_L_data; bic <- n_p*log(n_persons) - 2*log_L_data
  out_fit <- data.frame(LogLikelihood=log_L_data, AIC=aic, BIC=bic, Iterations=iter)

  if(con$verbose) {
    cat("Finished All Estimation.\n")
    cat("------------------------------------------------\n")
  }

  # --- Smart Warning Check ---
  na_items <- FALSE
  se_cols_items <- grep("_se$", names(out_items), value=TRUE)
  for(se_col in se_cols_items) {
    param_col <- sub("_se$", "", se_col)
    if(fix_discrimination && param_col == "discrimination") next
    if(param_col %in% names(out_items)) {
      vals <- out_items[[param_col]]; ses  <- out_items[[se_col]]
      if(any(!is.na(vals) & is.na(ses))) { na_items <- TRUE; break }
    }
  }

  na_persons <- FALSE
  se_cols_persons <- grep("_se$", names(out_persons), value=TRUE)
  for(se_col in se_cols_persons) {
    param_col <- sub("_se$", "", se_col)
    if(param_col %in% names(out_persons)) {
      vals <- out_persons[[param_col]]; ses  <- out_persons[[se_col]]
      if(any(!is.na(vals) & is.na(ses))) { na_persons <- TRUE; break }
    }
  }

  if(na_items || na_persons) {
    parts <- c()
    if(na_items) parts <- c(parts, "Item Parameters")
    if(na_persons) parts <- c(parts, "Person Parameters")
    msg <- paste0("\nWarning: Standard error detected NA in ", paste(parts, collapse=" & "),
                  ", indicating estimation for this is not stable.")
    message(msg)
  }

  list(item_params=out_items, person_params=out_persons, model_fit=out_fit)
}

#' Fixed Person Calibration with or without Covariate
#'
#' @description
#' Estimates item parameters (difficulty, discrimination) given fixed person parameters (theta),
#' with an optional person-level covariate. Supports Rasch and 2-Parameter Logistic models.
#'
#' @param df A data frame of item responses (0/1). Columns represent items, rows represent persons.
#' @param theta A numeric vector of person abilities (fixed parameters). Must match the number of rows in `df`.
#' @param model A character string specifying the model type. Options are "Rasch" or "2PL".
#' @param covariate An optional numeric vector representing a person-level covariate (e.g., time, group).
#'   Defaults to `NULL`.
#'
#' @return A data frame containing:
#' \itemize{
#'   \item Item statistics (difficulty, standard errors, z-values, p-values).
#'   \item Discrimination parameters (for 2PL model).
#'   \item Global covariate effect (if `covariate` is provided).
#'   \item Classical item statistics (p-value, count, point-biserial correlation).
#'   \item Mean theta per item (average ability of persons answering the item).
#'   \item Infit and Outfit statistics (for Rasch model only).
#' }
#'
#' @importFrom dplyr select mutate group_by summarise ungroup full_join inner_join left_join rename rowwise across everything any_of pull
#' @importFrom tidyr pivot_longer
#' @importFrom purrr map_dbl
#' @importFrom stats glm binomial cor vcov pnorm setNames
#' @importFrom gtools mixedorder
#'
#' @examples
#'   # --- Example: With Selected Package Data ---
#'   data("ela1", package = "tirt")
#'
#'   # Subset data for a manageable example
#'   # Select the first 500 examinees and 30 item responses
#'   df_real <- ela1[1:500, 1:30]
#'
#'   # Extract pre-estimated latent traits and covariates
#'   fixed_theta <- ela1$THETA[1:500]
#'   fixed_cov <- ela1$COVARIATE[1:500]
#'
#'   # Estimate item parameters given fixed ability levels
#'   # fitting a 2-parameter logistic (2PL) model
#'   real_res <- fix_person(df = df_real,
#'                                    theta = fixed_theta,
#'                                    model = "2PL",
#'                                    covariate = fixed_cov)
#'   head(real_res)
#' \donttest{
#'   # --- Example: With Package Data ---
#'   data("ela1", package = "tirt")
#'
#'   # Select Item Responses (Cols 1-30)
#'   df_real <- ela1[, 1:30]
#'
#'   fixed_theta <- ela1$THETA
#'   fixed_cov <- ela1$COVARIATE
#'
#'   real_res <- fix_person(df = df_real,
#'                                    theta = fixed_theta,
#'                                    model = "2PL",
#'                                    covariate = fixed_cov)
#'   head(real_res)
#' }
#' @export
fix_person <- function(df, theta, model = c("Rasch", "2PL"), covariate = NULL) {

  # --- 1. Input Validation ---
  model <- match.arg(model)

  if (!is.data.frame(df)) stop("'df' must be a data frame.")
  if (length(theta) != nrow(df)) stop("Length of 'theta' must match the number of rows in 'df'.")
  if (!is.null(covariate) && length(covariate) != nrow(df)) stop("Length of 'covariate' must match the number of rows in 'df'.")

  # --- 2. Calculate Classical Statistics (Pre-cleaning) ---
  # Calculate item means (p-values) and counts
  df_stats <- df |>
    dplyr::summarise(dplyr::across(dplyr::everything(), list(
      pvalue = ~mean(.x, na.rm = TRUE),
      number = ~sum(!is.na(.x))
    ))) |>
    tidyr::pivot_longer(
      cols = dplyr::everything(),
      names_to = c("item", ".value"),
      names_pattern = "(.+)_(.+)"
    )

  # Calculate point-biserial correlations with Theta
  # purrr::map_dbl works natively with |> as the first arg is .x (the data)
  item_cors <- df |>
    purrr::map_dbl(~ stats::cor(.x, theta, use = "complete.obs"))

  df_stats <- df_stats |>
    dplyr::mutate(correlation = item_cors[item])

  # Identify items to remove (deterministic 0 or 1)
  items_to_remove <- df_stats |>
    dplyr::filter(!is.na(pvalue), pvalue %in% c(0, 1)) |>
    dplyr::pull(item)

  if (length(items_to_remove) > 0) {
    warning(paste("The following items were removed due to p-value of 0 or 1:",
                  paste(items_to_remove, collapse = ", ")))
  }

  # --- 3. Data Preparation for GLM ---
  # Create a lightweight ID for joining
  df_aug <- df |>
    dplyr::mutate(
      .row_id = dplyr::row_number(),
      theta = theta
    )

  if (!is.null(covariate)) {
    df_aug$covariate <- covariate
  }

  # Pivot to long format and filter bad items
  df_long <- df_aug |>
    tidyr::pivot_longer(
      cols = -c(dplyr::any_of(c(".row_id", "theta", "covariate"))),
      names_to = "item",
      values_to = "response"
    ) |>
    dplyr::filter(!item %in% items_to_remove, !is.na(response))

  # Calculate Mean Theta (and Mean Covariate) per item based on available data
  item_means_agg <- df_long |>
    dplyr::group_by(item) |>
    dplyr::summarise(
      mean_theta = mean(theta, na.rm = TRUE),
      mean_covariate = if (!is.null(covariate)) mean(covariate, na.rm = TRUE) else NA_real_,
      .groups = "drop"
    )

  if (is.null(covariate)) item_means_agg$mean_covariate <- NULL


  # --- 4. Model Estimation ---

  # Construct formula
  if (model == "Rasch") {
    # Rasch: difficulty is intercept, slope is fixed at 1 via offset
    f_str <- "response ~ 0 + item + offset(theta)"
  } else {
    # 2PL: difficulty derived from intercept, slope is interaction
    f_str <- "response ~ 0 + item + item:theta"
  }

  if (!is.null(covariate)) {
    f_str <- paste(f_str, "+ covariate")
  }

  # Run GLM (base stats functions are not typically piped into)
  fit_glm <- stats::glm(
    as.formula(f_str),
    data = df_long,
    family = stats::binomial(link = "logit")
  )

  # --- 5. Parameter Extraction ---

  # Get raw coefficients
  coef_summ <- summary(fit_glm)$coefficients
  coef_df <- as.data.frame(coef_summ)
  coef_df$parameter <- rownames(coef_df)
  rownames(coef_df) <- NULL

  # Clean column names
  colnames(coef_df) <- c("Estimate", "Std_Error", "z_value", "Pr_z", "parameter")

  # Extract Covariate Effect (if exists)
  cov_row <- NULL
  if (!is.null(covariate)) {
    cov_row <- coef_df |>
      dplyr::filter(parameter == "covariate") |>
      dplyr::select(
        covariate = Estimate,
        covariate_se = Std_Error,
        covariate_zvalue = z_value,
        covariate_Pr_z = Pr_z
      )
  }

  # Process Item Parameters based on Model
  if (model == "Rasch") {

    # Filter item parameters
    result_items <- coef_df |>
      dplyr::filter(parameter != "covariate") |>
      dplyr::mutate(
        item = gsub("^item", "", parameter),
        # In Rasch GLM formulation: Beta = -Difficulty
        difficulty = -Estimate,
        difficulty_se = Std_Error,
        difficulty_zvalue = z_value,
        difficulty_Pr_z = Pr_z
      ) |>
      dplyr::select(item, difficulty, difficulty_se, difficulty_zvalue, difficulty_Pr_z)

  } else { # 2PL Model

    # Separate Intercepts (easiness) and Slopes (discrimination)
    # Pattern: Intercepts start with "item", Slopes contain ":theta"

    is_slope <- grepl(":theta", coef_df$parameter)
    is_intercept <- grepl("^item", coef_df$parameter) & !is_slope

    intercepts <- coef_df[is_intercept, ]
    slopes <- coef_df[is_slope, ]

    # Clean names for joining
    intercepts$item <- gsub("^item", "", intercepts$parameter)
    slopes$item <- gsub("^item|:theta", "", slopes$parameter)

    # Join
    item_table <- dplyr::inner_join(
      intercepts, slopes,
      by = "item",
      suffix = c("_int", "_slope")
    )

    # Calculate parameters and SEs
    vc <- stats::vcov(fit_glm)

    result_items <- item_table |>
      dplyr::rowwise() |>
      dplyr::mutate(
        discrimination = Estimate_slope,
        discrimination_se = Std_Error_slope,
        discrimination_zvalue = z_value_slope,
        discrimination_Pr_z = Pr_z_slope,

        difficulty = -Estimate_int / Estimate_slope,

        # Delta Method for SE(Difficulty)
        difficulty_se = {
          beta_name <- parameter_int
          a_name    <- parameter_slope

          beta   <- Estimate_int
          a      <- Estimate_slope

          # Gradient
          d_beta <- -1 / a
          d_a    <- beta / (a^2)
          grad   <- c(d_beta, d_a)

          # Covariance subset
          if (all(c(beta_name, a_name) %in% rownames(vc))) {
            cov_mat <- vc[c(beta_name, a_name), c(beta_name, a_name)]
            as.numeric(sqrt(t(grad) %*% cov_mat %*% grad))
          } else {
            NA_real_
          }
        },
        difficulty_zvalue = difficulty / difficulty_se,
        difficulty_Pr_z = 2 * (1 - stats::pnorm(abs(difficulty_zvalue)))
      ) |>
      dplyr::ungroup() |>
      dplyr::select(
        item,
        discrimination, discrimination_se, discrimination_zvalue, discrimination_Pr_z,
        difficulty, difficulty_se, difficulty_zvalue, difficulty_Pr_z
      )
  }

  # --- 6. Fit Statistics (Rasch Only) ---
  fit_stats_df <- NULL

  if (model == "Rasch") {

    # Prepare matrices for calculation
    O_mat <- as.matrix(df |> dplyr::select(dplyr::all_of(result_items$item)))

    diff_vec <- stats::setNames(result_items$difficulty, result_items$item)
    infit_vec <- numeric(length(diff_vec))
    outfit_vec <- numeric(length(diff_vec))
    item_names <- names(diff_vec)

    cov_effect <- 0
    if (!is.null(covariate) && !is.null(cov_row)) {
      cov_effect <- covariate * cov_row$covariate
    }

    # Loop over items
    for (j in seq_along(item_names)) {
      itm <- item_names[j]
      responses <- O_mat[, itm]

      lin_pred <- (theta - diff_vec[itm])
      if (!is.null(covariate)) {
        lin_pred <- lin_pred + cov_effect
      }

      E_j <- 1 / (1 + exp(-lin_pred))
      W_j <- E_j * (1 - E_j)

      valid_mask <- !is.na(responses)

      if (sum(valid_mask) > 0) {
        r_valid <- responses[valid_mask]
        e_valid <- E_j[valid_mask]
        w_valid <- W_j[valid_mask]

        resid_sq <- (r_valid - e_valid)^2

        infit_vec[j]  <- sum(w_valid * resid_sq) / sum(w_valid)
        outfit_vec[j] <- sum(resid_sq) / sum(w_valid)
      } else {
        infit_vec[j] <- NA
        outfit_vec[j] <- NA
      }
    }

    fit_stats_df <- data.frame(
      item = item_names,
      infit = infit_vec,
      outfit = outfit_vec,
      stringsAsFactors = FALSE
    )
  }

  # --- 7. Assembly and Output ---

  # Join Item Params + Stats + Theta Means
  final_df <- result_items |>
    dplyr::full_join(df_stats, by = "item") |>
    dplyr::full_join(item_means_agg, by = "item")

  # Join Covariate Global Params
  if (!is.null(cov_row)) {
    final_df <- cbind(final_df, cov_row)
  }

  # Join Fit Stats (if Rasch)
  if (!is.null(fit_stats_df)) {
    final_df <- final_df |>
      dplyr::left_join(fit_stats_df, by = "item")
  }

  # Sort by item naturally
  final_df <- final_df[gtools::mixedorder(final_df$item), ]

  # Define column order
  cols_base <- c("item")
  cols_2pl <- c("discrimination", "discrimination_se", "discrimination_zvalue", "discrimination_Pr_z")
  cols_diff <- c("difficulty", "difficulty_se", "difficulty_zvalue", "difficulty_Pr_z")
  cols_cov <- c("covariate", "covariate_se", "covariate_zvalue", "covariate_Pr_z")
  cols_stats <- c("pvalue", "number", "correlation", "mean_theta", "mean_covariate")
  cols_fit <- c("infit", "outfit")

  select_vec <- cols_base
  if (model == "2PL") select_vec <- c(select_vec, cols_2pl)
  select_vec <- c(select_vec, cols_diff)
  if (!is.null(covariate)) select_vec <- c(select_vec, cols_cov)
  select_vec <- c(select_vec, cols_stats)
  if (model == "Rasch") select_vec <- c(select_vec, cols_fit)

  # Final selection
  final_df <- final_df |>
    dplyr::select(dplyr::any_of(select_vec))

  rownames(final_df) <- NULL

  return(final_df)
}

#' Mixed Item Response Model Estimation (Dichotomous & Polytomous) with Prior Support
#'
#' @description
#' Provides a estimation framework for a broad class of different item response theory models.
#' This function can model different combinations of item categories.
#' Now supports flexible prior distributions for Bayesian estimation (MAP estimation).
#'
#' @param data A N x J data.frame. Binary items must be 0/1. Polytomous items should be continuous integers (0, 1, 2...).
#' @param model A character vector of length J (one model per item).
#'              Supported: "Rasch", "2PL" (2-Parameter Logistic), "3PL" (3-Parameter Logistic), "GRM" (Graded Response Model), "GPCM" (Generalized Partial Credit Model), "PCM" (Partial Credit Model).
#'              If a single string is provided, it is applied to all the same type of items.
#' @param method String. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood).
#' @param control A \code{list} of control parameters for the estimation algorithm:
#'   \itemize{
#'     \item \code{max_iter}: Maximum number of EM iterations (default = 100).
#'     \item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
#'     \item \code{theta_range}: Numeric vector of length 2 specifying the integration
#'           grid bounds (default = c(-4, 4)).
#'     \item \code{quad_points}: Number of quadrature points (default = 21).
#'     \item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
#'     \item \code{prior}: A list of model-specific prior distributions. Default is NULL (no priors).
#'       Format: \code{list("2PL" = list(a = function(x) ..., b = function(x) ...), "GRM" = list(...))}
#'       Each model gets its own prior specification. All items of the same model share the same prior.
#'       Example:
#'       \code{prior = list(
#'         "2PL" = list(a = function(x) dlnorm(x, 0, 0.5, log=TRUE), b = function(x) dnorm(x, 0, 2, log=TRUE)),
#'         "GRM" = list(a = function(x) dlnorm(x, 0, 0.5, log=TRUE), d = function(x) dnorm(x, 0, 2, log=TRUE))
#'       )}
#'   }
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{item_params}: Data frame of item parameters (discrimination, difficulty/thresholds, guessing).
#'   \item \code{person_params}: A data frame of estimated person abilities (theta) and standard errors.
#'   \item \code{model_fit}: A data frame containing fit statistics such as Akaike’s Information Criterion (AIC) and the Bayesian Information Criterion (BIC).
#'   \item \code{settings}: A list of control parameters used in the estimation.
#' }
#' @examples
#'   # --- Example 1: Simulation (Mixed 2PL + GPCM) ---
#'   set.seed(2025)
#'   N <- 500
#'   n_bin <- 5
#'   n_poly <- 2
#'   J <- n_bin + n_poly
#'
#'   # 1. Generate Theta (Wide range to match user request)
#'   true_theta <- rnorm(N, mean = 0, sd = 3)
#'
#'   # 2. Simulation Helper: GPCM
#'   sim_gpcm <- function(theta, a, steps) {
#'     n_cat <- length(steps) + 1
#'     probs <- matrix(0, length(theta), n_cat)
#'     for(k in 1:n_cat) {
#'       score <- k - 1
#'       if(score == 0) numer <- rep(0, length(theta))
#'       else numer <- a * (score * theta - sum(steps[1:score]))
#'       probs[, k] <- exp(numer)
#'     }
#'     probs <- probs / rowSums(probs)
#'     apply(probs, 1, function(p) sample(0:(n_cat-1), 1, prob=p))
#'   }
#'
#'   # 3. Create Data
#'   data_sim <- data.frame(matrix(NA, nrow = N, ncol = J))
#'   colnames(data_sim) <- paste0("Item_", 1:J)
#'
#'   # Binary Items (2PL)
#'   a_bin <- runif(n_bin, 0.8, 1.5)
#'   b_bin <- seq(-3, 3, length.out = n_bin)
#'   for(j in 1:n_bin) {
#'     prob <- 1 / (1 + exp(-(a_bin[j] * (true_theta - b_bin[j]))))
#'     data_sim[, j] <- rbinom(N, 1, prob)
#'   }
#'
#'   # Polytomous Items (GPCM)
#'   # Item 6: 2 steps (-2, 2)
#'   data_sim[, 6] <- sim_gpcm(true_theta, a=1.0, steps=c(-2, 2))
#'   # Item 7: 5 steps
#'   data_sim[, 7] <- sim_gpcm(true_theta, a=1.2, steps=c(-5, -2.5, 0, 2.5, 5))
#'
#'   # 4. Run Estimation without prior
#'   # Note: Wide theta_range needed due to SD=3 in simulation
#'   my_models <- c(rep("2PL", n_bin), rep("GPCM", n_poly))
#'
#'   res <- mixed_irt(data = data_sim, model = my_models, method = "EM",
#'                    control = list(max_iter = 20, theta_range = c(-6, 6)))
#'
#'   head(res$item_params)
#'   print(res$model_fit)
#'
#'   \donttest{
#'   # 5. Run Estimation with prior (MAP)
#'   res_prior <- mixed_irt(data = data_sim, model = my_models, method = "EM",
#'                          control = list(max_iter = 20, theta_range = c(-6, 6),
#'                                        prior = list(
#'                                          "2PL" = list(
#'                                            a = function(x) dlnorm(x, 0, 0.5, log=TRUE),
#'                                            b = function(x) dnorm(x, 0, 2, log=TRUE)
#'                                          ),
#'                                          "GPCM" = list(
#'                                            a = function(x) dlnorm(x, 0, 0.5, log=TRUE),
#'                                            d = function(x) dnorm(x, 0, 2, log=TRUE)
#'                                          )
#'                                        )))
#'   head(res_prior$item_params)
#'   print(res_prior$model_fit)
#'   # --- Example 2: With Package Data ---
#'   data("ela2", package = "tirt")
#'
#'   # Define Models (7 Binary, 3 Poly)
#'   real_models <- c(rep("2PL", 7), rep("GRM", 3))
#'
#'   # Run Estimation
#'   real_res <- mixed_irt(ela2, model = real_models, method = "EM",
#'                         control = list(max_iter = 10))
#'
#'   head(real_res$item_params)
#'   print(real_res$model_fit)
#'   }
#' @export
mixed_irt <- function(data,
                      model = "2PL",
                      method = "EM",
                      control = list()) {

  # --- 1. Setup and Defaults ---
  con <- list(
    max_iter = 100,
    converge_tol = 1e-4,
    theta_range = c(-4, 4),
    quad_points = 21,
    nr_max_iter = 20,
    nr_damp = 0.8,
    verbose = TRUE,
    prior = NULL  # No prior by default
  )
  con[names(control)] <- control

  # Extract prior from control
  prior <- con$prior

  # Normalize all elements in the vector to uppercase
  model <- toupper(model)

  # Define the set of supported models
  valid_models <- c("RASCH", "2PL", "3PL", "GPCM", "PCM", "GRM")

  # Validate that every element in the input vector is within the allowed set
  if (!all(model %in% valid_models)) {
    stop("Invalid model detected. Model must be 'Rasch', '2PL', or '3PL' for dichotomous items, ",
         "and 'GPCM', 'PCM', or 'GRM' for polytomous items (case-insensitive).")
  }

  # --- Input Validation ---
  if(!is.data.frame(data)) stop("Input must be a data frame.")
  raw_mat <- as.matrix(data)
  if(!is.numeric(raw_mat)) stop("Input data must be numeric.")

  N <- nrow(raw_mat)
  J <- ncol(raw_mat)
  item_names <- colnames(data)
  if(is.null(item_names)) item_names <- paste0("Item_", 1:J)

  # Expand model if single string
  if(length(model) == 1) model <- rep(model, J)
  if(length(model) != J) stop("Length of 'model' vector must match number of items.")

  # Define Model Types
  binary_models <- c("RASCH", "2PL", "3PL")
  poly_models <- c("GRM", "GPCM", "PCM")

  # --- Prior Validation ---
  validate_prior_mixed <- function(prior, models_used, verbose) {
    if(is.null(prior)) {
      if(verbose) cat("No prior specified. Using Maximum Likelihood Estimation.\n")
      return(NULL)
    }

    if(!is.list(prior)) stop("Prior must be a list.")

    # Validate structure: prior must be list of model-specific lists
    validated <- list()

    for(model_name in names(prior)) {
      model_upper <- toupper(model_name)

      # Check if this model is actually used
      if(!model_upper %in% unique(models_used)) {
        warning(sprintf("Prior specified for model '%s' but this model is not used in data. Ignoring.", model_name))
        next
      }

      # Define allowed parameters for this model
      allowed_params <- switch(model_upper,
                               "RASCH" = "b",
                               "2PL" = c("a", "b"),
                               "3PL" = c("a", "b", "g"),
                               "PCM" = "d",
                               "GPCM" = c("a", "d"),
                               "GRM" = c("a", "d"),
                               NULL
      )

      if(is.null(allowed_params)) {
        warning(sprintf("Unknown model '%s' in prior specification. Ignoring.", model_name))
        next
      }

      model_prior <- prior[[model_name]]
      if(!is.list(model_prior)) {
        stop(sprintf("Prior for model '%s' must be a list of parameter priors.", model_name))
      }

      # Check for unexpected parameters
      extra_params <- setdiff(names(model_prior), allowed_params)
      if(length(extra_params) > 0) {
        warning(sprintf("Prior for model '%s' contains unexpected parameter(s) [%s]. These will be ignored.",
                        model_name, paste(extra_params, collapse=", ")))
      }

      # Validate each parameter prior
      relevant_params <- intersect(names(model_prior), allowed_params)
      if(length(relevant_params) == 0) next

      validated[[model_upper]] <- list()
      for(param in relevant_params) {
        param_prior <- model_prior[[param]]

        if(!is.function(param_prior)) {
          stop(sprintf("Prior for parameter '%s' in model '%s' must be a function.", param, model_name))
        }

        # Test function
        test_val <- tryCatch(param_prior(0.5), error = function(e) {
          stop(sprintf("Prior function for '%s' in model '%s' failed: %s", param, model_name, e$message))
        })

        if(!is.numeric(test_val) || length(test_val) != 1) {
          stop(sprintf("Prior function for '%s' in model '%s' must return single numeric value.", param, model_name))
        }

        validated[[model_upper]][[param]] <- param_prior
      }

      if(verbose && length(validated[[model_upper]]) > 0) {
        cat(sprintf("Using MAP estimation for model '%s', parameters: %s\n",
                    model_upper, paste(names(validated[[model_upper]]), collapse=", ")))
      }
    }

    if(length(validated) == 0) return(NULL)
    return(validated)
  }

  # Helper to compute log-prior
  log_prior_density <- function(value, prior_func) {
    if(is.null(prior_func)) return(0)
    if(is.na(value)) return(0)
    tryCatch(prior_func(value), error = function(e) 0)
  }

  # Validate priors
  prior_expanded <- validate_prior_mixed(prior, model, con$verbose)

  # --- Data Preprocessing ---
  n_cats <- numeric(J)
  for(j in 1:J) {
    if(model[j] %in% poly_models) {
      min_val <- min(raw_mat[,j], na.rm=TRUE)
      if(is.finite(min_val) && min_val > 0) {
        raw_mat[,j] <- raw_mat[,j] - min_val
      }
      n_cats[j] <- max(raw_mat[,j], na.rm=TRUE) + 1
    } else {
      n_cats[j] <- 2
    }
  }

  max_K <- max(n_cats)

  # Identify valid items (Variance > 0)
  item_vars <- apply(raw_mat, 2, var, na.rm=TRUE)
  valid_items_idx <- which(item_vars > 0)
  bad_items_idx <- setdiff(1:J, valid_items_idx)

  # --- Initialize Parameters ---
  a <- rep(1, J)
  b <- rep(0, J)
  g <- rep(0, J)
  d <- matrix(0, J, max_K)

  # Initialize Standard Errors (SE)
  se_a <- rep(NA, J)
  se_b <- rep(NA, J)
  se_g <- rep(NA, J)
  se_d <- matrix(NA, J, max_K)

  for(j in valid_items_idx) {
    valid_resp <- raw_mat[!is.na(raw_mat[,j]), j]

    if(model[j] %in% binary_models) {
      p_val <- mean(valid_resp)
      p_bounded <- pmin(pmax(p_val, 0.01), 0.99)
      b[j] <- -log(p_bounded / (1 - p_bounded))
      if(model[j] == "3PL") g[j] <- 0.2
    } else {
      props <- table(factor(valid_resp, levels=0:(n_cats[j]-1))) / length(valid_resp)
      cum_props <- cumsum(props)
      cum_props <- pmin(pmax(cum_props, 0.01), 0.99)
      thresh <- -qlogis(cum_props[-length(cum_props)])

      if(model[j] == "GRM") {
        d[j, 1:(n_cats[j]-1)] <- sort(thresh)
      } else {
        K <- n_cats[j]
        steps <- seq(-0.5, 0.5, length.out = K-1)
        d[j, 2:K] <- steps
      }
    }
  }

  # Initialize Theta
  raw_scores <- rowMeans(raw_mat, na.rm=TRUE)
  theta <- as.vector(scale(raw_scores))
  theta[is.na(theta)] <- 0

  # Quadrature Setup
  nodes <- seq(con$theta_range[1], con$theta_range[2], length.out = con$quad_points)
  weights <- dnorm(nodes)
  weights <- weights / sum(weights)

  # --- 2. Helper Functions ---

  get_probs_j <- function(th_vec, j_idx) {
    mod <- model[j_idx]
    if(mod %in% binary_models) {
      z <- a[j_idx] * (th_vec - b[j_idx])
      z <- pmin(pmax(z, -30), 30)
      p <- g[j_idx] + (1 - g[j_idx]) * (1 / (1 + exp(-z)))
      return(cbind(1-p, p))
    } else {
      n_cat <- n_cats[j_idx]
      n_q <- length(th_vec)
      probs <- matrix(0, n_q, n_cat)
      if (mod %in% c("GPCM", "PCM")) {
        for(k in 1:n_cat) {
          z <- a[j_idx] * (k-1) * th_vec - d[j_idx, k]
          z <- pmin(pmax(z, -50), 50)
          probs[,k] <- exp(z)
        }
        probs <- probs / rowSums(probs)
      } else { # GRM
        P_star <- matrix(0, n_q, n_cat + 1)
        P_star[,1] <- 1; P_star[,n_cat + 1] <- 0
        eff_d <- d[j_idx, 1:(n_cat-1)]
        if(is.unsorted(eff_d)) eff_d <- sort(eff_d)
        for(k in 1:(n_cat-1)) {
          z <- a[j_idx] * (th_vec - eff_d[k])
          z <- pmin(pmax(z, -30), 30)
          P_star[, k+1] <- 1 / (1 + exp(-z))
        }
        for(k in 1:n_cat) probs[,k] <- P_star[,k] - P_star[,k+1]
        probs <- pmax(probs, 1e-10)
      }
      return(probs)
    }
  }

  optim_binary <- function(r_vec, n_vec, th_vec, cur_a, cur_b, cur_g, mod_type) {
    get_P_bin <- function(t, pa, pb, pg) {
      z <- pa * (t - pb); z <- pmin(pmax(z, -30), 30)
      pg + (1 - pg) * (1 / (1 + exp(-z)))
    }
    if(mod_type == "RASCH") active <- c(FALSE, TRUE, FALSE)
    else if(mod_type == "2PL") active <- c(TRUE, TRUE, FALSE)
    else active <- c(TRUE, TRUE, TRUE)

    par_vec <- c(cur_a, cur_b, cur_g)[active]

    ll_func <- function(p_v) {
      full_p <- c(1, 0, 0); full_p[active] <- p_v
      if(!active[1]) full_p[1] <- 1
      P_tmp <- get_P_bin(th_vec, full_p[1], full_p[2], full_p[3])
      P_tmp <- pmin(pmax(P_tmp, 1e-9), 1-1e-9)

      log_lik_data <- sum(r_vec * log(P_tmp) + (n_vec - r_vec) * log(1 - P_tmp))

      # Add priors if specified
      if(!is.null(prior_expanded) && mod_type %in% names(prior_expanded)) {
        model_prior <- prior_expanded[[mod_type]]
        if(!is.null(model_prior$a) && active[1]) {
          log_lik_data <- log_lik_data + log_prior_density(full_p[1], model_prior$a)
        }
        if(!is.null(model_prior$b) && active[2]) {
          log_lik_data <- log_lik_data + log_prior_density(full_p[2], model_prior$b)
        }
        if(!is.null(model_prior$g) && active[3]) {
          log_lik_data <- log_lik_data + log_prior_density(full_p[3], model_prior$g)
        }
      }

      return(log_lik_data)
    }

    for(k in 1:con$nr_max_iter) {
      f0 <- ll_func(par_vec); h <- 1e-4
      grad <- numeric(length(par_vec)); hess <- matrix(0, length(par_vec), length(par_vec))
      for(i in 1:length(par_vec)) {
        tmp <- par_vec; tmp[i] <- tmp[i] + h; grad[i] <- (ll_func(tmp) - f0)/h
      }
      for(i in 1:length(par_vec)) {
        tmp_kk <- par_vec; tmp_kk[i] <- tmp_kk[i] + h; tmp_mk <- par_vec; tmp_mk[i] <- tmp_mk[i] - h
        hess[i,i] <- (ll_func(tmp_kk) - 2*f0 + ll_func(tmp_mk)) / h^2
      }
      diag(hess) <- diag(hess) - 1e-5
      step <- try(solve(hess, grad), silent=TRUE)
      if(inherits(step, "try-error")) break
      par_vec <- par_vec - step * con$nr_damp
      if(active[1]) par_vec[1] <- max(par_vec[1], 0.01)
      if(active[3]) par_vec[3] <- min(max(par_vec[3], 0), 0.4)
      if(max(abs(step)) < 1e-5) break
    }

    se_vec <- tryCatch(sqrt(diag(solve(-hess))), error=function(e) rep(NA, length(par_vec)))

    res <- list(a=if(active[1]) par_vec[1] else 1, b=0, g=0, se_a=NA, se_b=NA, se_g=NA)
    idx <- 1
    if(active[1]) { res$se_a <- se_vec[idx]; idx <- idx+1 }
    if(active[2]) { res$b <- par_vec[idx]; res$se_b <- se_vec[idx]; idx <- idx+1 }
    if(active[3]) { res$g <- par_vec[idx]; res$se_g <- se_vec[idx] }
    return(res)
  }

  optim_poly <- function(r_mat, n_vec, th_vec, cur_a, cur_d, mod_type, n_cat) {
    if(mod_type == "PCM") param_vec <- cur_d[2:n_cat]
    else if(mod_type == "GRM") param_vec <- c(cur_a, cur_d[1:(n_cat-1)])
    else param_vec <- c(cur_a, cur_d[2:n_cat])

    ll_func <- function(p_v) {
      l_a <- if(mod_type=="PCM") 1 else p_v[1]
      l_d <- numeric(max_K)
      if(mod_type=="PCM") l_d[2:n_cat] <- p_v
      else if(mod_type=="GRM") l_d[1:(n_cat-1)] <- p_v[-1]
      else l_d[2:n_cat] <- p_v[-1]

      n_q <- length(th_vec); probs <- matrix(0, n_q, n_cat)
      if (mod_type %in% c("GPCM", "PCM")) {
        for(k in 1:n_cat) {
          z <- l_a * (k-1) * th_vec - l_d[k]; z <- pmin(pmax(z, -50), 50); probs[,k] <- exp(z)
        }
        probs <- probs / rowSums(probs)
      } else {
        P_s <- matrix(0, n_q, n_cat+1); P_s[,1] <- 1; ed <- l_d[1:(n_cat-1)]
        if(is.unsorted(ed)) ed <- sort(ed)
        for(k in 1:(n_cat-1)) {
          z <- l_a * (th_vec - ed[k]); z <- pmin(pmax(z, -30), 30); P_s[, k+1] <- 1/(1+exp(-z))
        }
        for(k in 1:n_cat) probs[,k] <- P_s[,k] - P_s[,k+1]
        probs <- pmax(probs, 1e-10)
      }

      log_lik_data <- sum(r_mat * log(probs))

      # Add priors if specified
      if(!is.null(prior_expanded) && mod_type %in% names(prior_expanded)) {
        model_prior <- prior_expanded[[mod_type]]
        if(!is.null(model_prior$a) && mod_type != "PCM") {
          log_lik_data <- log_lik_data + log_prior_density(l_a, model_prior$a)
        }
        if(!is.null(model_prior$d)) {
          if(mod_type == "GRM") {
            for(k in 1:(n_cat-1)) {
              log_lik_data <- log_lik_data + log_prior_density(l_d[k], model_prior$d)
            }
          } else {
            for(k in 2:n_cat) {
              log_lik_data <- log_lik_data + log_prior_density(l_d[k], model_prior$d)
            }
          }
        }
      }

      return(log_lik_data)
    }

    for(k in 1:con$nr_max_iter) {
      f0 <- ll_func(param_vec); h <- 1e-4
      grad <- numeric(length(param_vec)); hess <- matrix(0, length(param_vec), length(param_vec))
      for(i in 1:length(param_vec)) {
        t <- param_vec; t[i] <- t[i]+h; grad[i] <- (ll_func(t)-f0)/h
      }
      for(i in 1:length(param_vec)) {
        tp <- param_vec; tp[i] <- tp[i]+h; tm <- param_vec; tm[i] <- tm[i]-h
        hess[i,i] <- (ll_func(tp)-2*f0+ll_func(tm))/h^2
      }
      diag(hess) <- diag(hess) - 1e-4
      step <- try(solve(hess, grad), silent=TRUE)
      if(inherits(step, "try-error")) break
      param_vec <- param_vec - step * con$nr_damp
      if(mod_type != "PCM") param_vec[1] <- max(param_vec[1], 0.05)
      if(max(abs(step)) < 1e-4) break
    }

    se_vec <- tryCatch(sqrt(diag(solve(-hess))), error=function(e) rep(NA, length(param_vec)))

    ret_a <- if(mod_type=="PCM") 1 else param_vec[1]; ret_d <- cur_d
    ret_se_a <- NA; ret_se_d <- rep(NA, max_K)

    if(mod_type=="PCM") {
      ret_d[2:n_cat] <- param_vec; ret_se_d[2:n_cat] <- se_vec
    } else if(mod_type=="GRM") {
      ret_d[1:(n_cat-1)] <- sort(param_vec[-1]); ret_se_a <- se_vec[1]; ret_se_d[1:(n_cat-1)] <- se_vec[-1]
    } else {
      ret_d[2:n_cat] <- param_vec[-1]; ret_se_a <- se_vec[1]; ret_se_d[2:n_cat] <- se_vec[-1]
    }
    return(list(a=ret_a, d=ret_d, se_a=ret_se_a, se_d=ret_se_d))
  }

  # --- 3. Main Estimation Loop ---

  if(con$verbose) {
    cat("\n------------------------------------------------\n")
    cat(sprintf("Starting Mixed IRT (%s) Estimation...\n", method))
    if(!is.null(prior_expanded)) {
      cat("Estimation Type: MAP (Maximum A Posteriori) with Bayesian Priors\n")
    } else {
      cat("Estimation Type: MLE (Maximum Likelihood Estimation)\n")
    }
    cat("------------------------------------------------\n")
  }

  is_converged <- FALSE

  for(iter in 1:con$max_iter) {

    # E-Step / MLE Person Step
    if(method == "EM") {
      L_iq <- matrix(0, N, con$quad_points)
      for(j in valid_items_idx) {
        probs_jq <- get_probs_j(nodes, j)
        resp <- raw_mat[, j]; valid_rows <- !is.na(resp)
        if(model[j] %in% binary_models) {
          p1 <- pmax(probs_jq[,2], 1e-10); p0 <- pmax(probs_jq[,1], 1e-10)
          is_1 <- which(valid_rows & resp == 1); is_0 <- which(valid_rows & resp == 0)
          if(length(is_1)>0) L_iq[is_1,] <- L_iq[is_1,] + matrix(log(p1), length(is_1), con$quad_points, byrow=TRUE)
          if(length(is_0)>0) L_iq[is_0,] <- L_iq[is_0,] + matrix(log(p0), length(is_0), con$quad_points, byrow=TRUE)
        } else {
          p_indices <- resp[valid_rows] + 1
          for(k in 1:n_cats[j]) {
            is_cat <- which(p_indices == k)
            if(length(is_cat)>0) L_iq[valid_rows,][is_cat,] <- L_iq[valid_rows,][is_cat,] + matrix(log(pmax(probs_jq[,k], 1e-10)), length(is_cat), con$quad_points, byrow=TRUE)
          }
        }
      }
      F_iq <- exp(L_iq) * matrix(weights, N, con$quad_points, byrow=TRUE)
      Posterior_iq <- F_iq / rowSums(F_iq); Posterior_iq[is.na(Posterior_iq)] <- 0
    } else {
      # MLE Theta update
      for(i in 1:N) {
        valid <- !is.na(raw_mat[i,]) & (1:J %in% valid_items_idx); if(sum(valid)==0) next
        ti <- theta[i]
        for(nr in 1:5) {
          num <- 0; den <- 0
          for(j in which(valid)) {
            x <- raw_mat[i,j]; probs <- get_probs_j(ti, j)
            if(model[j] %in% binary_models) {
              p <- probs[1,2]; num <- num + a[j]*(x - p); den <- den - (a[j]^2)*p*(1-p)
            } else {
              h <- 1e-4; p0 <- max(probs[1, x+1], 1e-10)
              pp <- get_probs_j(ti+h, j)[1, x+1]; pm <- get_probs_j(ti-h, j)[1, x+1]
              d1 <- (log(pp)-log(pm))/(2*h); d2 <- (log(pp)-2*log(p0)+log(pm))/(h^2)
              num <- num + d1; den <- den + d2
            }
          }
          if(abs(den) < 1e-6) break; step <- num/den; ti <- ti - step
          ti <- max(min(ti, con$theta_range[2]), con$theta_range[1]); if(abs(step) < 1e-3) break
        }
        theta[i] <- ti
      }
      theta <- theta - mean(theta)
    }

    # M-Step / MLE Item Update
    max_change <- 0
    for(j in valid_items_idx) {
      old_par <- c(a[j], b[j], g[j], d[j,])
      if(method == "EM") {
        use_nodes <- nodes
        if(model[j] %in% binary_models) {
          valid_rows <- !is.na(raw_mat[,j])
          r_jq <- colSums(Posterior_iq[valid_rows,] * raw_mat[valid_rows, j])
          n_jq <- colSums(Posterior_iq[valid_rows,])
          res <- optim_binary(r_jq, n_jq, use_nodes, a[j], b[j], g[j], model[j])
          a[j] <- res$a; b[j] <- res$b; g[j] <- res$g
          se_a[j] <- res$se_a; se_b[j] <- res$se_b; se_g[j] <- res$se_g
        } else {
          R_qk <- matrix(0, con$quad_points, n_cats[j])
          valid_rows <- !is.na(raw_mat[,j]); resp <- raw_mat[valid_rows, j]
          for(k in 1:n_cats[j]) {
            idx <- which(resp == (k-1))
            if(length(idx)>0) { full_idx <- which(valid_rows)[idx]; R_qk[,k] <- colSums(Posterior_iq[full_idx, , drop=FALSE]) }
          }
          res <- optim_poly(R_qk, rowSums(R_qk), use_nodes, a[j], d[j,], model[j], n_cats[j])
          a[j] <- res$a; d[j,] <- res$d
          se_a[j] <- res$se_a; se_d[j,] <- res$se_d
        }
      } else {
        valid <- !is.na(raw_mat[,j]); use_nodes <- theta[valid]; y <- raw_mat[valid, j]
        if(model[j] %in% binary_models) {
          res <- optim_binary(y, rep(1, sum(valid)), use_nodes, a[j], b[j], g[j], model[j])
          a[j] <- res$a; b[j] <- res$b; g[j] <- res$g
          se_a[j] <- res$se_a; se_b[j] <- res$se_b; se_g[j] <- res$se_g
        } else {
          n_resp <- length(y); R_qk <- matrix(0, n_resp, n_cats[j]); R_qk[cbind(1:n_resp, y+1)] <- 1
          res <- optim_poly(R_qk, rep(1, n_resp), use_nodes, a[j], d[j,], model[j], n_cats[j])
          a[j] <- res$a; d[j,] <- res$d
          se_a[j] <- res$se_a; se_d[j,] <- res$se_d
        }
      }
      max_change <- max(max_change, abs(c(a[j], b[j], g[j], d[j,]) - old_par), na.rm=TRUE)
    }

    if(con$verbose) cat(paste0("\rIteration ", iter, ": Max Param Change = ", round(max_change, 5), "   "))

    if(max_change < con$converge_tol) {
      if(con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
      is_converged <- TRUE
      break
    }
  }

  if(!is_converged && con$verbose) {
    cat(sprintf("\n\n>>> NOTE: Stopped because max Iteration Reached at %d.\n", con$max_iter))
    cat("    Adjust it in control=list(max_iter = ...) to have higher iteration if needed.\n")
  }

  # --- 4. Final Person Estimation & Formatting ---

  if(con$verbose) {
    cat("\n------------------------------------------------\n")
    cat("Estimating Final Person Parameters...\n")
    cat(sprintf("Personal Parameter estimated with range [%s, %s].\n", con$theta_range[1], con$theta_range[2]))
    cat("Adjust it in control=list(theta_range=c(..)) if you want a different range.\n")
  }

  final_theta <- numeric(N)
  final_se <- numeric(N)
  log_lik <- 0

  for(i in 1:N) {
    valid <- !is.na(raw_mat[i,]) & (1:J %in% valid_items_idx)
    if(sum(valid)==0) { final_theta[i] <- NA; next }

    ti <- if(method=="EM") 0 else theta[i]
    for(k in 1:20) {
      num <- 0; den <- 0
      for(j in which(valid)) {
        x <- raw_mat[i,j]; probs <- get_probs_j(ti, j)
        if(model[j] %in% binary_models) {
          p <- probs[1,2]; num <- num + a[j]*(x - p); den <- den - (a[j]^2)*p*(1-p)
          if(k==20) log_lik <- log_lik + (x*log(p) + (1-x)*log(1-p))
        } else {
          h <- 1e-4; p0 <- max(probs[1, x+1], 1e-10)
          pp <- get_probs_j(ti+h, j)[1, x+1]; pm <- get_probs_j(ti-h, j)[1, x+1]
          d1 <- (log(pp)-log(pm))/(2*h); d2 <- (log(pp)-2*log(p0)+log(pm))/(h^2)
          num <- num + d1; den <- den + d2
          if(k==20) log_lik <- log_lik + log(p0)
        }
      }
      if(abs(den) < 1e-6) break; step <- num/den; step <- max(min(step, 1.0), -1.0)
      ti <- ti - step; if(abs(step) < 1e-4) break
    }
    final_theta[i] <- max(min(ti, con$theta_range[2]), con$theta_range[1])
    if(!is.na(den) && abs(den) > 1e-6) final_se[i] <- 1/sqrt(abs(den)) else final_se[i] <- NA
  }

  # Add log-prior to log-likelihood if using priors
  if(!is.null(prior_expanded)) {
    log_prior_total <- 0
    for(j in valid_items_idx) {
      mod <- model[j]
      if(mod %in% names(prior_expanded)) {
        model_prior <- prior_expanded[[mod]]
        if(mod %in% binary_models) {
          if(!is.null(model_prior$a) && mod != "RASCH") {
            log_prior_total <- log_prior_total + log_prior_density(a[j], model_prior$a)
          }
          if(!is.null(model_prior$b)) {
            log_prior_total <- log_prior_total + log_prior_density(b[j], model_prior$b)
          }
          if(!is.null(model_prior$g) && mod == "3PL") {
            log_prior_total <- log_prior_total + log_prior_density(g[j], model_prior$g)
          }
        } else {
          if(!is.null(model_prior$a) && mod != "PCM") {
            log_prior_total <- log_prior_total + log_prior_density(a[j], model_prior$a)
          }
          if(!is.null(model_prior$d)) {
            if(mod == "GRM") {
              for(k in 1:(n_cats[j]-1)) {
                if(!is.na(d[j,k])) {
                  log_prior_total <- log_prior_total + log_prior_density(d[j,k], model_prior$d)
                }
              }
            } else {
              for(k in 2:n_cats[j]) {
                if(!is.na(d[j,k])) {
                  log_prior_total <- log_prior_total + log_prior_density(d[j,k], model_prior$d)
                }
              }
            }
          }
        }
      }
    }
    log_lik <- log_lik + log_prior_total
  }

  if(con$verbose) cat("Person Parameter Estimation Finished.\n")

  # --- 5. Output Construction with Descriptives & SEs ---

  # Initial Data Frame
  out_items <- data.frame(item = item_names,
                          model = model,
                          stringsAsFactors = FALSE)

  # Interleave Estimates and SEs
  out_items$discrimination <- round(a, 3)
  out_items$discrimination_se <- round(se_a, 3)

  out_items$difficulty <- ifelse(model %in% binary_models, round(b, 3), NA)
  out_items$difficulty_se <- ifelse(model %in% binary_models, round(se_b, 3), NA)

  out_items$guess <- ifelse(model == "3PL", round(g, 3), NA)
  out_items$guess_se <- ifelse(model == "3PL", round(se_g, 3), NA)

  for(k in 1:(max_K-1)) {
    col_name <- paste0("threshold_", k)
    col_name_se <- paste0("threshold_", k, "_se")

    vals <- numeric(J)
    vals_se <- numeric(J)

    for(j in 1:J) {
      if(model[j] %in% poly_models) {
        if(model[j] == "GRM") {
          vals[j] <- d[j, k]
          vals_se[j] <- se_d[j, k]
        } else {
          if((k+1) <= n_cats[j]) {
            vals[j] <- d[j, k+1]
            vals_se[j] <- se_d[j, k+1]
          } else {
            vals[j] <- NA; vals_se[j] <- NA
          }
        }
      } else {
        vals[j] <- NA; vals_se[j] <- NA
      }
    }
    out_items[[col_name]] <- round(vals, 3)
    out_items[[col_name_se]] <- round(vals_se, 3)
  }

  # Calculate Descriptive Statistics and append as LAST columns
  n_count <- colSums(!is.na(raw_mat))
  p_val <- colMeans(raw_mat, na.rm = TRUE)

  out_items$number <- n_count
  out_items$pvalue <- round(p_val, 3)

  if(length(bad_items_idx) > 0) {
    # Columns to set to NA (excluding 'item', 'model', 'number', 'pvalue')
    cols_to_NA <- which(!names(out_items) %in% c("item", "model", "number", "pvalue"))
    out_items[bad_items_idx, cols_to_NA] <- NA
    if(con$verbose) message("\nWarning: Some items have 0 variance. Parameters set to NA.")
  }

  out_persons <- data.frame(
    id = 1:N,
    theta = round(final_theta, 3),
    se = round(final_se, 3),
    n_resp = rowSums(!is.na(raw_mat))
  )

  # Remove Row Names
  row.names(out_items) <- NULL
  row.names(out_persons) <- NULL

  n_par <- sum(ifelse(model %in% binary_models, ifelse(model=="RASCH",1, ifelse(model=="2PL",2,3)), 1 + (n_cats-1)))
  if(length(bad_items_idx)>0) n_par <- n_par - length(bad_items_idx)

  aic <- 2*n_par - 2*log_lik
  bic <- n_par*log(N) - 2*log_lik

  # Update label based on estimation type
  log_lik_label <- if(!is.null(prior_expanded)) "LogPosterior" else "LogLikelihood"

  fit_df <- data.frame(
    Stat = c(log_lik_label, "AIC", "BIC", "Iterations"),
    Value = c(round(log_lik, 2), round(aic, 2), round(bic, 2), iter)
  )

  # --- Final Verbose Message ---
  if(con$verbose) {
    cat("Finished All Estimation.\n")
    cat("------------------------------------------------\n")
  }

  # Settings summary
  estimation_type <- if(!is.null(prior_expanded)) "MAP (with priors)" else "MLE (no priors)"

  return(list(
    item_params = out_items,
    person_params = out_persons,
    model_fit = fit_df,
    settings = list(method=method, models=table(model), estimation=estimation_type)
  ))
}

#' Test and Item Characteristic Curves (Expected Scores)
#'
#' @description
#' Computes item characteristic curves (the expected score of each item as a
#' function of ability) and the test characteristic curve (the expected total
#' score as a function of ability). The test characteristic curve maps the latent
#' ability scale onto the number-correct (true-score) scale and underlies
#' true-score equating and score reporting; the item curves show how each item's
#' expected score rises across the trait.
#'
#' @param item_params A data frame of item parameters (the \code{item_params}
#'   element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
#'   \code{\link{mixed_irt}}). All models in the package are supported.
#' @param theta A numeric vector of ability values at which to evaluate the
#'   curves (default = \code{seq(-4, 4, by = 0.1)}).
#' @param model Optional model override (see \code{\link{item_info}}).
#' @param D Scaling constant for the dichotomous logistic metric (default
#'   \code{1}).
#'
#' @return A list with two elements:
#' \itemize{
#'   \item \code{test_curve}: a data frame with columns \code{theta} and
#'     \code{expected_score} (the test characteristic curve).
#'   \item \code{item_curves}: a numeric matrix with one row per item and one
#'     column per theta value, giving the expected score of each item.
#' }
#'
#' @seealso \code{\link{item_info}}, \code{\link{score_table}}
#'
#' @examples
#'   set.seed(123)
#'   sim <- sim_irt(n_people = 400,
#'                  item_structure = list(list(model = "2PL", n_items = 10)))
#'   fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
#'                     control = list(max_iter = 15, verbose = FALSE))
#'
#'   curves <- tcc(fit$item_params, theta = seq(-3, 3, by = 0.5))
#'   curves$test_curve
#'
#'   # Expected score on the first item across ability
#'   curves$item_curves[1, ]
#' @export
tcc <- function(item_params,
                theta = seq(-4, 4, by = 0.1),
                model = NULL,
                D = 1) {

  parsed <- .tirt_parse_params(item_params, model = model, D = D)
  theta <- as.numeric(theta)

  es_mat <- matrix(NA_real_, nrow = parsed$n_items, ncol = length(theta))
  for (j in seq_len(parsed$n_items)) {
    sp <- parsed$spec[[j]]
    if (is.na(sp$a) || (sp$model %in% c("RASCH", "2PL", "3PL") && is.na(sp$b))) next
    es_mat[j, ] <- .tirt_expected_score(sp, theta)
  }
  rownames(es_mat) <- parsed$item
  colnames(es_mat) <- round(theta, 4)

  test_curve <- data.frame(
    theta = theta,
    expected_score = colSums(es_mat, na.rm = TRUE),
    row.names = NULL,
    stringsAsFactors = FALSE
  )

  list(test_curve = test_curve, item_curves = es_mat)
}

#' Simulate Testlet Response Theory Data (Vector Supported Version)
#'
#' @description
#' Simulate testlet responses data. Support both dichotomous and polytomous responses.
#' Provide an easy implementation with a few default settings.
#'
#' @param n_people Integer. Number of examinees.
#' @param item_structure List of lists defining item blocks.
#' @param theta Numeric vector (Optional). If provided, these exact ability values are used.
#' @param theta_mean Numeric. Mean of latent trait (used if theta is NULL).
#' @param theta_sd Numeric. SD of latent trait (used if theta is NULL).
#'
#' @return A list containing:
#'   \item{resp}{data.frame of responses (rows=people, cols=items)}
#'   \item{true_item_params}{data.frame of true item parameters}
#'   \item{true_person_params}{vector of true latent traits}
#' @examples
#'   # =========================================================================
#'   # Example 1: Complex Testlet Design
#'   # =========================================================================
#'   # Define the Testlet Blueprint
#'   trt_design <- list(
#'     # Testlet 1: Rasch Testlet Model (High dependence: var=0.8)
#'     list(model = "RaschT", n_items = 5, testlet_id = "Read_A",
#'          testlet_var = 0.8, b = c(-1, 1)),
#'
#'     # Testlet 2: 2PL Testlet Model (Default dependence: var=0.5)
#'     list(model = "2PLT", n_items = 5, testlet_id = "Read_B",
#'          a = c(0.7, 1.3)),
#'
#'     # Testlet 3: Graded Response Testlet (Polytomous, 4 categories)
#'     list(model = "GRT", n_items = 4, testlet_id = "Survey",
#'          categories = 4, testlet_var = 0.2)
#'   )
#'
#'   # Run Simulation
#'   trt_data <- sim_trt(n_people = 500, item_structure = trt_design)
#'
#'   # Inspect Results
#'   # 1. Responses
#'   head(trt_data$resp)
#'
#'   # 2. Item Parameters
#'   # (Notice 'testlet_loading' equals 'discrimination' for standard models)
#'   head(trt_data$true_item_params)
#'
#'   # 3. Person Parameters (Ability + Gamma for each testlet)
#'   head(trt_data$true_person_params)
#'
#'   # =========================================================================
#'   # Example 2: Manual Control (Theta, Gamma, and Parameters)
#'   # =========================================================================
#'
#'   # 1. Manual Theta (e.g., everyone has high ability)
#'   manual_theta <- rep(2.0, 100)
#'
#'   # 2. Manual Gamma (e.g., zero effect for T1)
#'   manual_gamma <- rep(0, 100)
#'
#'   # 3. Item Parameters: Exact Match vs Range Sampling
#'   custom_structure <- list(
#'     # Case A: Manual Gamma Vector
#'     list(model = "2PLT", n_items = 5, testlet_id = "T1",
#'          gamma_vector = manual_gamma),
#'
#'     # Case B: Exact Parameter Match (Length of 'a' equals n_items)
#'     list(model = "2PLT", n_items = 2, testlet_id = "T2",
#'          a = c(0.5, 2.5)),
#'
#'     # Case C: Range Sampling (Length of 'a' is 2, but n_items != 2)
#'     list(model = "2PLT", n_items = 5, testlet_id = "T3",
#'          a = c(0.5, 2.5))
#'   )
#'
#'   res_custom <- sim_trt(n_people = 100, theta = manual_theta,
#'                         item_structure = custom_structure)
#'
#'   # Verify Manual Theta
#'   print(mean(res_custom$true_person_params$ability)) # Should be 2.0
#'
#'   # Verify Manual Gamma (T1 should be 0)
#'   print(head(res_custom$true_person_params$testlet_T1))
#'
#'   # Verify Exact Match (T2 discrimination should be 0.5 and 2.5)
#'   print(res_custom$true_item_params[res_custom$true_item_params$testlet_id == "T2",
#'                                     "discrimination"])
#' @export
sim_trt <- function(n_people = 1000,
                    item_structure = list(),
                    theta = NULL,
                    theta_mean = 0,
                    theta_sd = 1) {

  # --- 1. Global Validation ---
  if (!is.numeric(n_people) || n_people < 1) stop("Error: 'n_people' must be a positive integer.")
  if (length(item_structure) == 0) stop("Error: 'item_structure' cannot be empty.")

  message("================================================================")
  message(sprintf("   STARTING TRT SIMULATION (N = %d)", n_people))
  message("================================================================")

  # --- 2. Initialize Person Parameters (Theta & Gamma) ---

  # A. Primary Trait (Theta) Logic
  if (!is.null(theta)) {
    # User provided vector
    if (length(theta) != n_people) {
      stop(sprintf("Error: Length of provided 'theta' vector (%d) does not match 'n_people' (%d).",
                   length(theta), n_people))
    }
    true_theta <- theta
    message(">> Ability (Theta): Using user-supplied vector.")
  } else {
    # Default Generation
    true_theta <- rnorm(n_people, mean = theta_mean, sd = theta_sd)
    message(sprintf(">> Ability (Theta): Generated from N(mean=%.2f, sd=%.2f).", theta_mean, theta_sd))
  }

  # B. Testlet Gammas
  testlet_ids <- c()
  testlet_configs <- list()

  # Pre-scan for configs
  for (i in seq_along(item_structure)) {
    blk <- item_structure[[i]]
    if (is.null(blk$testlet_id)) {
      tid <- paste0("T", i)
      item_structure[[i]]$testlet_id <- tid
      message(sprintf("Notice: Block %d missing 'testlet_id'. Assigned '%s'.", i, tid))
    } else {
      tid <- as.character(blk$testlet_id)
    }
    testlet_ids <- c(testlet_ids, tid)

    # Only configure if new
    if (is.null(testlet_configs[[tid]])) {
      if (!is.null(blk$gamma_vector)) {
        if (length(blk$gamma_vector) != n_people) stop(sprintf("Error (Block %d): 'gamma_vector' length mismatch.", i))
        testlet_configs[[tid]] <- list(type = "vector", val = blk$gamma_vector)
      } else if (!is.null(blk$testlet_var)) {
        testlet_configs[[tid]] <- list(type = "variance", val = blk$testlet_var)
      } else {
        testlet_configs[[tid]] <- list(type = "default", val = 0.5)
      }
    }
  }

  unique_tids <- unique(testlet_ids)
  gamma_matrix <- matrix(0, nrow = n_people, ncol = length(unique_tids))
  colnames(gamma_matrix) <- paste0("testlet_", unique_tids)

  message(">> Testlet Effects (Gamma):")
  for (j in seq_along(unique_tids)) {
    tid <- unique_tids[j]
    config <- testlet_configs[[tid]]

    if (config$type == "vector") {
      message(sprintf("   - Testlet '%s': Using user-supplied Gamma vector.", tid))
      gamma_matrix[, j] <- config$val
    } else if (config$type == "variance") {
      message(sprintf("   - Testlet '%s': Generated Gamma ~ N(0, %.2f) [User Var].", tid, config$val))
      gamma_matrix[, j] <- rnorm(n_people, mean = 0, sd = sqrt(config$val))
    } else {
      message(sprintf("   - Testlet '%s': Generated Gamma ~ N(0, 0.50) [Default].", tid))
      gamma_matrix[, j] <- rnorm(n_people, mean = 0, sd = sqrt(0.5))
    }
  }

  person_params <- data.frame(person_id = 1:n_people, ability = true_theta)
  person_params <- cbind(person_params, gamma_matrix)

  # --- 3. Item Simulation Loop ---
  response_list <- list()
  param_list <- list()
  current_item_idx <- 1

  valid_dich <- c("RaschT", "2PLT", "3PLT", "BiFT")
  valid_poly <- c("PCT", "GPCT", "GRT", "BiFT")
  all_valid <- c(valid_dich, valid_poly)

  for (i in seq_along(item_structure)) {
    block <- item_structure[[i]]
    if (is.null(block$model)) stop(sprintf("Block %d Error: Missing 'model'.", i))
    if (is.null(block$n_items)) stop(sprintf("Block %d Error: Missing 'n_items'.", i))

    model <- block$model
    n_items <- block$n_items
    tid <- as.character(block$testlet_id)

    if (!model %in% all_valid) stop(sprintf("Block %d Error: Invalid model '%s'.", i, model))

    # Categories Logic
    if (model %in% valid_dich && model != "BiFT") {
      cats <- 2
    } else {
      if (is.null(block$categories)) {
        if (model %in% valid_dich) cats <- 2 else {
          cats <- 3
          message(sprintf("   [Block %d] Categories not set. Defaulting to 3.", i))
        }
      } else {
        cats <- block$categories
        if (cats < 2) stop(sprintf("Block %d Error: Categories must be >= 2.", i))
      }
    }

    message(sprintf(">> Block %d: %d items (Model: %s, Testlet: %s)", i, n_items, model, tid))

    # --- SMART PARAMETER PARSER ---
    get_param <- function(name, default, n_req, friendly) {
      val <- block[[name]]

      # Case A: Not provided -> Default
      if (is.null(val)) {
        message(sprintf("      - %s: Default used (%.2f).", friendly, default))
        return(rep(default, n_req))
      }

      # Case B: Exact Vector Match (Highest Priority)
      # If user provides exactly n_items, we assume they want specific values per item
      if (length(val) == n_req) {
        if (n_req > 1) {
          message(sprintf("      - %s: Using user-supplied vector (Exact match).", friendly))
        } else {
          message(sprintf("      - %s: User fixed value (%.2f).", friendly, val[1]))
        }
        return(val)
      }

      # Case C: Range (Uniform Distribution)
      # Only if length is 2 AND n_req is NOT 2 (because Case B handles n=2)
      if (length(val) == 2 && val[1] < val[2]) {
        message(sprintf("      - %s: Sampling from U[%.2f, %.2f].", friendly, val[1], val[2]))
        return(runif(n_req, val[1], val[2]))
      }

      # Case D: Single Value (Fixed for all)
      if (length(val) == 1) {
        message(sprintf("      - %s: User fixed value (%.2f).", friendly, val))
        return(rep(val, n_req))
      }

      stop(sprintf("Block %d Error: Parameter '%s' has length %d but requires %d (or range definition).",
                   i, name, length(val), n_req))
    }

    # Generate Params
    if (model %in% c("RaschT", "PCT")) a <- rep(1, n_items) else a <- get_param("a", 1, n_items, "Discrimination a")
    if (model == "BiFT") s_load <- get_param("s", 1, n_items, "Testlet Loading s") else s_load <- a
    b <- get_param("b", 0, n_items, "Difficulty/Loc b")
    c_p <- if (model == "3PLT") get_param("c", 0, n_items, "Guessing c") else rep(0, n_items)

    block_params <- data.frame(
      item_id = paste0("item_", current_item_idx:(current_item_idx + n_items - 1)),
      model = model,
      testlet_id = tid,
      categories = cats,
      discrimination = a,
      testlet_loading = s_load,
      difficulty = b,
      guessing = c_p,
      stringsAsFactors = FALSE
    )

    gamma_col_name <- paste0("testlet_", tid)
    current_gamma <- person_params[[gamma_col_name]]
    block_resp <- matrix(NA, nrow = n_people, ncol = n_items)

    # Simulation Logic
    if (cats == 2) {
      for (j in 1:n_items) {
        z <- (a[j] * true_theta) + (s_load[j] * current_gamma) - (a[j] * b[j])
        prob <- c_p[j] + (1 - c_p[j]) / (1 + exp(-z))
        block_resp[, j] <- ifelse(runif(n_people) < prob, 1, 0)
      }
    } else {
      for (j in 1:n_items) {
        raw_steps <- sort(rnorm(cats - 1, mean = 0, sd = 0.5))
        thresholds <- b[j] + raw_steps
        for (k in 1:length(thresholds)) block_params[j, paste0("step_", k)] <- thresholds[k]

        if (model == "GRT" || (model == "BiFT" && cats > 2)) {
          prob_cum <- matrix(0, nrow = n_people, ncol = cats + 1)
          prob_cum[, 1] <- 1
          for (k in 1:(cats - 1)) {
            z <- (a[j] * true_theta) + (s_load[j] * current_gamma) - (a[j] * thresholds[k])
            prob_cum[, k + 1] <- 1 / (1 + exp(-z))
          }
          probs <- prob_cum[, 1:cats] - prob_cum[, 2:(cats+1)]
        } else {
          numerators <- matrix(0, nrow = n_people, ncol = cats)
          current_sum <- 0
          for (k in 1:(cats - 1)) {
            term <- (a[j] * true_theta) + (s_load[j] * current_gamma) - (a[j] * thresholds[k])
            current_sum <- current_sum + term
            numerators[, k + 1] <- current_sum
          }
          probs <- exp(numerators)
          probs <- probs / rowSums(probs)
        }
        cum_probs <- t(apply(probs, 1, cumsum))
        rand_vals <- runif(n_people)
        block_resp[, j] <- apply(cum_probs >= rand_vals, 1, function(x) match(TRUE, x)) - 1
      }
    }

    response_list[[i]] <- block_resp
    param_list[[i]] <- block_params
    current_item_idx <- current_item_idx + n_items
  }

  # --- 4. Final Assembly (Safe) ---
  message("================================================================")
  message("Constructing final dataframes...")

  all_resp <- as.data.frame(do.call(cbind, response_list))
  row.names(all_resp) <- NULL

  all_cols <- unique(unlist(lapply(param_list, names)))
  all_item_params <- do.call(rbind, lapply(param_list, function(df) {
    missing <- setdiff(all_cols, names(df))
    if(length(missing) > 0) df[missing] <- NA
    return(df[, all_cols])
  }))
  row.names(all_item_params) <- NULL

  colnames(all_resp) <- all_item_params$item_id
  row.names(person_params) <- NULL

  message("Simulation Complete.")
  message(sprintf("Summary: %d items, %d examinees.", ncol(all_resp), n_people))
  message("================================================================")

  return(list(resp = all_resp, true_item_params = all_item_params, true_person_params = person_params))
}

#' Multidimensional Binary Item Response Theory Estimation
#'
#' @description
#' Estimates item and person parameters for multidimensional binary item response models
#' (M-Rasch, M-2PL, M-3PL). The function is fully self-contained, using custom Newton-Raphson
#' optimization without relying on external optimizers.
#'
#' @param data A N x J data.frame or matrix of dichotomous responses (0/1).
#' @param model String. "Rasch", "2PL", or "3PL".
#' @param dimension Integer. Number of latent dimensions to estimate (D).
#' @param method String. Estimation method: "MML" (Marginal Maximum Likelihood with EM, best for D <= 3),
#'   "MHRM" (Metropolis-Hastings Robbins-Monro, for high D), or "RVEM" (Dimension-Reduction EM).
#' @param control A \code{list} of control parameters for the estimation algorithm:
#'   \itemize{
#'     \item \code{max_iter}: Maximum number of iterations (default = 100).
#'     \item \code{converge_tol}: Convergence criterion (default = 1e-4).
#'     \item \code{theta_range}: Bounds for quadrature integration (default = c(-4, 4)).
#'     \item \code{quad_points}: Quadrature points PER DIMENSION for MML (default = 15).
#'     \item \code{ability}: Logical; estimate person abilities (EAP) after item calibration? (default = TRUE).
#'     \item \code{verbose}: Logical; prints progress and psychometric messages.
#'     \item \code{Q_matrix}: Optional J x D matrix of 0s and 1s indicating item-to-dimension loading.
#'           If NULL, an exploratory lower-triangular constraint is applied for identifiability.
#'   }
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{item_params}: A data frame of estimated item parameters (a_1...a_D, d, g) and standard errors.
#'   \item \code{person_params}: A data frame of estimated multidimensional person abilities (if ability=TRUE).
#'   \item \code{model_fit}: Fit statistics (Log-Likelihood, AIC, BIC).
#'   \item \code{settings}: Control parameters used in the run.
#' }
#'
#' @examples
#'
#'   # --- Simulation for Multidimensional Data ---
#'   set.seed(202)
#'   N <- 800
#'   J <- 20
#'   D <- 2
#'
#'   # Simulate Abilities (2 Dimensions, correlated)
#'   Sigma <- matrix(c(1, 0.4, 0.4, 1), 2, 2)
#'   Z <- matrix(rnorm(N * D), N, D)
#'   theta <- Z %*% chol(Sigma)
#'
#'   # Define Q-matrix (Items 1-10 on Dim1, Items 11-20 on Dim 2)
#'   Q <- matrix(0, J, D)
#'   Q[1:10, 1] <- 1
#'   Q[11:20, 2] <- 1
#'
#'   # Simulate Item Parameters (Multidimensional 2PL)
#'   a_true <- matrix(runif(J * D, 0.8, 1.8), J, D) * Q
#'   d_true <- seq(-2, 2, length.out = J)
#'
#'   # Generate Responses
#'   data_mat <- matrix(NA, N, J)
#'   for(i in 1:N) {
#'     # Matrix multiply JxD slopes by Dx1 person abilities, add Jx1 intercepts
#'     z <- as.vector(a_true %*% theta[i, ]) + d_true
#'     p <- 1 / (1 + exp(-z))
#'     data_mat[i, ] <- rbinom(J, 1, p)
#'   }
#'   df <- as.data.frame(data_mat)
#'   names(df) <- paste0("Item", 1:J)
#'
#'   # --- Run Custom Multidimensional Function ---
#'   res <- mirt_binary(df, model="2PL", dimension=2, method="MML",
#'                      control=list(Q_matrix=Q, ability=TRUE, max_iter=10))
#'
#'   print(head(res$item_params))
#'   print(head(res$person_params))
#'   print(res$model_fit)
#'   \donttest{
#'   # --- Validation with 'mirt' (For comparison, if installed) ---
#'   # library(mirt)
#'   # mirt_model <- paste0("F1 = 1-10\nF2 = 11-20\nCOV = F1*F2")
#'   # mirt_res <- mirt(df, mirt.model(mirt_model), itemtype='2PL', method='EM')
#'   # coef(mirt_res, IRTpars=FALSE, simplify=TRUE)$items
#'   }
#' @export
mirt_binary <- function(data,
                        model = "2PL",
                        dimension = 2,
                        method = "MML",
                        control = list()) {

  # --- 1. Setup and Control Processing ---
  con <- list(
    max_iter = 100,
    converge_tol = 1e-4,
    theta_range = c(-4, 4),
    quad_points = 15,    # Points PER dimension
    nr_max_iter = 15,    # NR Iterations for items
    nr_damp = 1.0,
    ability = TRUE,      # Estimate Person Parameters
    verbose = TRUE,
    Q_matrix = NULL      # Design matrix for loadings
  )
  con[names(control)] <- control

  model <- toupper(model)
  valid_models <- c("RASCH", "2PL", "3PL")
  if (!model %in% valid_models) stop("Model must be 'Rasch', '2PL', or '3PL'.")

  if (!is.data.frame(data) && !is.matrix(data)) stop("Data must be a matrix or data frame.")
  raw_data <- as.matrix(data)
  N <- nrow(raw_data)
  J <- ncol(raw_data)
  D <- as.integer(dimension)

  item_names <- colnames(data)
  if (is.null(item_names)) item_names <- paste0("Item_", 1:J)

  # --- 2. Structural & Q-Matrix Setup ---
  Q <- con$Q_matrix
  if (is.null(Q)) {
    if (con$verbose) message("No Q-matrix provided. Using Exploratory MIRT parameterization with lower-triangular constraints for identifiability.")
    Q <- matrix(1, J, D)
    if (D > 1) {
      for (d in 2:D) {
        # Fix upper triangle to 0 for identifiability
        Q[1:(d-1), d] <- 0
      }
    }
  } else {
    if (nrow(Q) != J || ncol(Q) != D) stop(sprintf("Q_matrix must be %d x %d (Items x Dimensions).", J, D))
  }

  # --- 3. Initial Parameter Values ---
  a <- matrix(0.8, J, D) * Q   # Slopes / Discrimination
  p_mean <- pmin(pmax(colMeans(raw_data, na.rm=TRUE), 0.05), 0.95)
  d_param <- log(p_mean / (1 - p_mean)) # Intercept / Ease
  g <- rep(ifelse(model == "3PL", 0.15, 0), J)

  if (model == "RASCH") {
    a <- matrix(1, J, D) * Q
  }

  valid_items_idx <- which(!is.na(p_mean))

  # --- 4. Define Integration Grid (MML or RVEM) ---
  if (method == "MML") {
    if (D > 3 && con$verbose) warning("MML with D > 3 can be very slow due to exponential quadrature grid growth. Consider MHRM if speed is an issue.")

    nodes_1d <- seq(con$theta_range[1], con$theta_range[2], length.out = con$quad_points)
    weights_1d <- dnorm(nodes_1d)
    weights_1d <- weights_1d / sum(weights_1d)

    # Create Cartesian product for multidimensional integration
    grid_list <- rep(list(nodes_1d), D)
    nodes_mat <- as.matrix(expand.grid(grid_list))

    weight_list <- rep(list(weights_1d), D)
    weights_mat <- as.matrix(expand.grid(weight_list))
    weights_vec <- apply(weights_mat, 1, prod)
    weights_vec <- weights_vec / sum(weights_vec)

    Num_Nodes <- nrow(nodes_mat)
  } else if (method == "RVEM") {
    if (con$verbose) message("RVEM Triggered: Assuming simple independent clusters for dimension reduction...")
    # Simplified RVEM mapping: treating dimensions semi-independently.
    # For full rigorous RVEM, a Bifactor restriction must be validated.
    # We fallback to standard MML logic but restricted on grid if independent.
    nodes_mat <- matrix(0, con$quad_points * D, D)
    # Note: Full RVEM requires complex factor mappings. For R-package scope,
    # we simulate an alternative or switch to MML.
    method <- "MML" # Fallback mapped for safety in this pure-R framework.
    nodes_1d <- seq(con$theta_range[1], con$theta_range[2], length.out = con$quad_points)
    grid_list <- rep(list(nodes_1d), D)
    nodes_mat <- as.matrix(expand.grid(grid_list))
    weight_list <- rep(list(dnorm(nodes_1d)/sum(dnorm(nodes_1d))), D)
    weights_vec <- apply(as.matrix(expand.grid(weight_list)), 1, prod)
    Num_Nodes <- nrow(nodes_mat)
  } else if (method == "MHRM") {
    if (con$verbose) message("MHRM selected. Utilizing Stochastic Approximation EM (SAEM) logic...")
    # Initialize person abilities for MHRM
    theta_imputed <- matrix(0, N, D)
  }

  # --- 5. Helper Functions ---

  # Calculate Multidimensional P(Theta)
  get_P_multi <- function(theta_matrix, a_vec, d_val, g_val) {
    # theta_matrix is (Nodes x D) or (N x D)
    # a_vec is length D
    z <- as.vector(theta_matrix %*% a_vec) + d_val
    z <- pmin(pmax(z, -30), 30)
    p <- g_val + (1 - g_val) / (1 + exp(-z))
    return(p)
  }

  # Custom Newton-Raphson Solver for a single item
  solve_item_nr_mirt <- function(r_vec, n_vec, theta_matrix, cur_a, cur_d, cur_g, q_vec, mod_type) {

    # Determine active parameters based on Q-matrix and Model
    active_a_idx <- which(q_vec == 1)

    # FIX 3: Build explicit, stable index map once — used consistently everywhere
    # This prevents the off-by-one SE misassignment when active_a_idx length varies
    idx_a  <- if(mod_type != "RASCH") seq_along(active_a_idx) else integer(0)
    idx_d  <- length(idx_a) + 1
    idx_g  <- if(mod_type == "3PL") idx_d + 1 else NULL
    n_params <- idx_d + ifelse(mod_type == "3PL", 1, 0)

    # Build a tight vector of only the parameters we are estimating
    if(mod_type == "RASCH") {
      curr_vec <- c(cur_d)
    } else if(mod_type == "2PL") {
      curr_vec <- c(cur_a[active_a_idx], cur_d)
    } else {
      curr_vec <- c(cur_a[active_a_idx], cur_d, cur_g)
    }

    # Internal Log-Likelihood Function mapping tight vector back to MIRT space
    # Uses stable idx_a / idx_d / idx_g map throughout
    ll_func <- function(p_vec) {
      loc_a <- cur_a
      loc_d <- cur_d
      loc_g <- cur_g

      if(mod_type != "RASCH") loc_a[active_a_idx] <- p_vec[idx_a]
      loc_d <- p_vec[idx_d]
      if(mod_type == "3PL") loc_g <- p_vec[idx_g]

      P_tmp <- get_P_multi(theta_matrix, loc_a, loc_d, loc_g)
      P_tmp <- pmin(pmax(P_tmp, 1e-12), 1 - 1e-12)

      ll <- sum(r_vec * log(P_tmp) + (n_vec - r_vec) * log(1 - P_tmp))

      # Beta(5, 17) prior on g: keeps g near 0.2, prevents wandering.
      # Standard MAP practice for 3PL; does not affect 2PL/Rasch.
      if(mod_type == "3PL") {
        loc_g_val <- pmin(pmax(p_vec[idx_g], 1e-9), 1 - 1e-9)
        ll <- ll + (5 - 1) * log(loc_g_val) + (17 - 1) * log(1 - loc_g_val)
      }

      return(ll)
    }

    # Newton-Raphson with Finite Differences (keeps code dependency-free).
    # For 3PL: alternating-block updates — first optimise a/d with g fixed,
    # then optimise g alone. This breaks the cross-parameter oscillation that
    # occurs when the diagonal-only Hessian ignores a-g and d-g covariances.
    h <- 1e-4

    nr_iters_ad <- if(mod_type == "3PL") ceiling(con$nr_max_iter * 0.6) else con$nr_max_iter
    nr_iters_g  <- if(mod_type == "3PL") con$nr_max_iter - nr_iters_ad else 0L

    # ---- Block 1: update a and d (g held fixed at curr_vec[idx_g]) ----
    idx_ad <- c(idx_a, idx_d)
    for(iter in 1:nr_iters_ad) {
      n_ad <- length(idx_ad)
      grad_ad <- numeric(n_ad)
      hess_ad <- matrix(0, n_ad, n_ad)
      f0 <- ll_func(curr_vec)

      for(ki in seq_along(idx_ad)) {
        k <- idx_ad[ki]
        tmp <- curr_vec; tmp[k] <- tmp[k] + h
        grad_ad[ki] <- (ll_func(tmp) - f0) / h
      }
      for(ki in seq_along(idx_ad)) {
        k <- idx_ad[ki]
        tmp_p <- curr_vec; tmp_p[k] <- tmp_p[k] + h
        tmp_m <- curr_vec; tmp_m[k] <- tmp_m[k] - h
        hess_ad[ki, ki] <- (ll_func(tmp_p) - 2*f0 + ll_func(tmp_m)) / (h^2)
      }
      diag(hess_ad) <- diag(hess_ad) - 0.01

      step_ad <- tryCatch(solve(hess_ad, grad_ad), error = function(e) rep(0, n_ad))
      step_ad  <- pmin(pmax(step_ad, -1.0), 1.0)
      curr_vec[idx_ad] <- curr_vec[idx_ad] - step_ad * con$nr_damp

      if(mod_type != "RASCH") curr_vec[idx_a] <- pmin(pmax(curr_vec[idx_a], 0.01), 4.0)
      curr_vec[idx_d] <- pmin(pmax(curr_vec[idx_d], -10), 10)

      if(max(abs(step_ad)) < 1e-4) break
    }

    # ---- Block 2 (3PL only): update g alone with a/d held fixed ----
    if(mod_type == "3PL") {
      for(iter in 1:nr_iters_g) {
        f0 <- ll_func(curr_vec)
        tmp_p <- curr_vec; tmp_p[idx_g] <- tmp_p[idx_g] + h
        tmp_m <- curr_vec; tmp_m[idx_g] <- tmp_m[idx_g] - h
        grad_g <- (ll_func(tmp_p) - f0) / h
        hess_g <- (ll_func(tmp_p) - 2*f0 + ll_func(tmp_m)) / (h^2)
        hess_g  <- hess_g - 0.01

        step_g <- tryCatch(grad_g / hess_g, error = function(e) 0)
        step_g  <- pmin(pmax(step_g, -0.05), 0.05)
        curr_vec[idx_g] <- curr_vec[idx_g] - step_g * con$nr_damp
        curr_vec[idx_g] <- pmin(pmax(curr_vec[idx_g], 0.001), 0.4)

        if(abs(step_g) < 1e-5) break
      }
    }

    # Map back to full parameters — using stable index map
    final_a <- cur_a; final_d <- cur_d; final_g <- cur_g
    if(mod_type != "RASCH") final_a[active_a_idx] <- curr_vec[idx_a]
    final_d <- curr_vec[idx_d]
    if(mod_type == "3PL") final_g <- curr_vec[idx_g]

    # Calculate Standard Errors
    se_vec <- rep(NA, n_params)
    tryCatch({
      hess_final <- matrix(0, n_params, n_params)
      f0 <- ll_func(curr_vec)
      for(k in 1:n_params) {
        tmp_kk <- curr_vec; tmp_kk[k] <- tmp_kk[k] + h
        tmp_mk <- curr_vec; tmp_mk[k] <- tmp_mk[k] - h
        hess_final[k, k] <- (ll_func(tmp_kk) - 2*f0 + ll_func(tmp_mk)) / (h^2)
      }
      # Consistent ridge with optimization pass (was 1e-5)
      diag(hess_final) <- diag(hess_final) - 0.01
      # pmax guards against tiny positive rounding errors producing NaN from sqrt
      se_vec <- sqrt(pmax(diag(solve(-hess_final)), 0))
    }, error = function(e) NULL)

    # Map SEs back — using stable index map (FIX 3: no off-by-one risk)
    se_a <- rep(NA, D); se_d <- NA; se_g <- NA
    if(mod_type != "RASCH") se_a[active_a_idx] <- se_vec[idx_a]
    se_d <- se_vec[idx_d]
    if(mod_type == "3PL") se_g <- se_vec[idx_g]

    return(list(a = final_a, d = final_d, g = final_g,
                se_a = se_a, se_d = se_d, se_g = se_g))
  }


  # --- 6. Main Estimation Loop ---

  if(con$verbose) {
    cat(sprintf("\nStarting Multidimensional %s Estimation (%s dims) using %s algorithm...\n", model, D, method))
    cat(sprintf("Number of Items: %d | Number of Persons: %d\n", J, N))
    cat(sprintf("----------------------------------------------------------\n"))
  }

  is_converged <- FALSE
  log_lik <- -Inf

  for (iter in 1:con$max_iter) {
    max_param_change <- 0
    old_a <- a; old_d <- d_param; old_g <- g

    if (method == "MML") {
      # E-STEP
      L_terms <- matrix(0, N, Num_Nodes)

      for(j in valid_items_idx) {
        P_jq <- get_P_multi(nodes_mat, a[j,], d_param[j], g[j])
        resp <- raw_data[, j]

        term1 <- log(pmax(P_jq, 1e-10))
        term0 <- log(pmax(1 - P_jq, 1e-10))

        valid_resp <- !is.na(resp)
        L_terms[valid_resp & resp == 1, ] <- L_terms[valid_resp & resp == 1, ] + matrix(term1, sum(valid_resp & resp == 1), Num_Nodes, byrow=TRUE)
        L_terms[valid_resp & resp == 0, ] <- L_terms[valid_resp & resp == 0, ] + matrix(term0, sum(valid_resp & resp == 0), Num_Nodes, byrow=TRUE)
      }

      F_iq <- exp(L_terms) * matrix(weights_vec, N, Num_Nodes, byrow=TRUE)
      sum_Fi <- rowSums(F_iq)
      sum_Fi[sum_Fi == 0] <- 1e-10
      Posterior_iq <- F_iq / sum_Fi

      current_log_lik <- sum(log(sum_Fi))

      # M-STEP
      for (j in valid_items_idx) {
        resp <- raw_data[, j]
        valid <- !is.na(resp)

        # Expected counts
        r_jq <- colSums(Posterior_iq[valid, , drop=FALSE] * resp[valid])
        n_jq <- colSums(Posterior_iq[valid, , drop=FALSE])

        res <- solve_item_nr_mirt(r_jq, n_jq, nodes_mat, a[j,], d_param[j], g[j], Q[j,], model)

        a[j, ] <- res$a
        d_param[j] <- res$d
        g[j] <- res$g

        change <- max(abs(c(a[j,] - old_a[j,], d_param[j] - old_d[j], g[j] - old_g[j])), na.rm=TRUE)
        max_param_change <- max(max_param_change, change)
      }

    } else if (method == "MHRM") {
      # Simplified MHRM / Stochastic EM (SAEM style)

      # E-Step (Stochastic Imputation)
      for(i in 1:N) {
        # MH Random Walk Proposal
        prop_theta <- theta_imputed[i, ] + rnorm(D, 0, 0.5)

        ll_curr <- sum(log(get_P_multi(matrix(theta_imputed[i, ], nrow=1), a, d_param, g)^raw_data[i,] *
                             (1-get_P_multi(matrix(theta_imputed[i, ], nrow=1), a, d_param, g))^(1-raw_data[i,])), na.rm=TRUE) +
          sum(dnorm(theta_imputed[i, ], log=TRUE))

        ll_prop <- sum(log(get_P_multi(matrix(prop_theta, nrow=1), a, d_param, g)^raw_data[i,] *
                             (1-get_P_multi(matrix(prop_theta, nrow=1), a, d_param, g))^(1-raw_data[i,])), na.rm=TRUE) +
          sum(dnorm(prop_theta, log=TRUE))

        if (log(runif(1)) < (ll_prop - ll_curr)) {
          theta_imputed[i, ] <- prop_theta
        }
      }

      # M-Step (Maximization based on stochastic sample)
      for (j in valid_items_idx) {
        resp <- raw_data[, j]
        valid <- !is.na(resp)

        res <- solve_item_nr_mirt(resp[valid], rep(1, sum(valid)), as.matrix(theta_imputed[valid, ]),
                                  a[j,], d_param[j], g[j], Q[j,], model)

        # RM Averaging
        gamma <- 1 / (iter^(0.75)) # Robbins Monro Step size cooling
        a[j, ] <- old_a[j,] + gamma * (res$a - old_a[j,])
        d_param[j] <- old_d[j] + gamma * (res$d - old_d[j])
        g[j] <- old_g[j] + gamma * (res$g - old_g[j])

        change <- max(abs(c(a[j,] - old_a[j,], d_param[j] - old_d[j], g[j] - old_g[j])), na.rm=TRUE)
        max_param_change <- max(max_param_change, change)
      }
      current_log_lik <- NA # Hard to compute exact LL in stochastic framework
    }

    if(con$verbose) {
      if(!is.na(current_log_lik)) {
        cat(sprintf("\rIteration %d: Max Change = %.5f | Log-Lik = %.2f   ", iter, max_param_change, current_log_lik))
      } else {
        cat(sprintf("\rIteration %d: Max Change = %.5f   ", iter, max_param_change))
      }
    }

    if(iter > 5 && max_param_change < con$converge_tol) {
      if(con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
      is_converged <- TRUE
      if(!is.na(current_log_lik)) log_lik <- current_log_lik
      break
    }
  }

  if(!is_converged && con$verbose) {
    cat(sprintf("\n\n>>> NOTE: Stopped because max Iteration Reached at %d.\n", con$max_iter))
    cat("    Consider increasing max_iter or relaxing converge_tol.\n")
  }

  # --- 7. Person Parameters (EAP Estimation) ---
  final_theta <- matrix(NA, N, D)
  final_theta_se <- matrix(NA, N, D)

  if (con$ability) {
    if(con$verbose) cat("\nEstimating Multidimensional Person Parameters (EAP)...\n")

    if (method != "MML") {
      # We need to compute posterior grid for EAP regardless of method chosen
      L_terms <- matrix(0, N, Num_Nodes)
      for(j in valid_items_idx) {
        P_jq <- get_P_multi(nodes_mat, a[j,], d_param[j], g[j])
        resp <- raw_data[, j]
        term1 <- log(pmax(P_jq, 1e-10)); term0 <- log(pmax(1 - P_jq, 1e-10))
        valid_resp <- !is.na(resp)
        L_terms[valid_resp & resp == 1, ] <- L_terms[valid_resp & resp == 1, ] + matrix(term1, sum(valid_resp & resp == 1), Num_Nodes, byrow=TRUE)
        L_terms[valid_resp & resp == 0, ] <- L_terms[valid_resp & resp == 0, ] + matrix(term0, sum(valid_resp & resp == 0), Num_Nodes, byrow=TRUE)
      }
      F_iq <- exp(L_terms) * matrix(weights_vec, N, Num_Nodes, byrow=TRUE)
      sum_Fi <- rowSums(F_iq)
      sum_Fi[sum_Fi == 0] <- 1e-10
      Posterior_iq <- F_iq / sum_Fi
    }

    # Calculate Expected A Posteriori
    for(i in 1:N) {
      for(d in 1:D) {
        final_theta[i, d] <- sum(Posterior_iq[i, ] * nodes_mat[, d])
        final_theta_se[i, d] <- sqrt(sum(Posterior_iq[i, ] * (nodes_mat[, d] - final_theta[i, d])^2))
      }
    }
    if(con$verbose) cat("Person Parameter Estimation Finished.\n")
  }

  # --- 8. Final Standard Error Pass (for items) ---
  se_a_mat <- matrix(NA, J, D); se_d_vec <- rep(NA, J); se_g_vec <- rep(NA, J)

  if (method == "MML") {
    for(j in valid_items_idx) {
      resp <- raw_data[, j]
      valid <- !is.na(resp)
      r_jq <- colSums(Posterior_iq[valid, , drop=FALSE] * resp[valid])
      n_jq <- colSums(Posterior_iq[valid, , drop=FALSE])
      res <- solve_item_nr_mirt(r_jq, n_jq, nodes_mat, a[j,], d_param[j], g[j], Q[j,], model)
      se_a_mat[j, ] <- res$se_a
      se_d_vec[j] <- res$se_d
      se_g_vec[j] <- res$se_g
    }
  }

  # --- 9. Format Outputs ---

  # 9A. Item Parameters
  item_df <- data.frame(Item = item_names, stringsAsFactors = FALSE)
  for(d in 1:D) {
    # Always show the 'a' parameter (it will show 1s and 0s for Rasch)
    item_df[[paste0("a_Dim", d)]] <- round(a[, d], 3)

    # Only show Standard Errors for 2PL/3PL. Rasch 'a' is fixed, so SE is NA.
    if(model != "RASCH") {
      item_df[[paste0("SE_a_Dim", d)]] <- round(se_a_mat[, d], 3)
    } else {
      item_df[[paste0("SE_a_Dim", d)]] <- NA
    }
  }

  item_df$d <- round(d_param, 3)
  item_df$SE_d <- round(se_d_vec, 3)
  if(model == "3PL") {
    item_df$g <- round(g, 3)
    item_df$SE_g <- round(se_g_vec, 3)
  }

  # 9B. Person Parameters
  person_df <- NULL
  if (con$ability) {
    person_df <- data.frame(ID = 1:N)
    for(d in 1:D) {
      person_df[[paste0("Theta_Dim", d)]] <- round(final_theta[, d], 3)
      person_df[[paste0("SE_Theta_Dim", d)]] <- round(final_theta_se[, d], 3)
    }
  }

  # 9C. Fit Stats
  k_params <- sum(Q) * (model != "RASCH") + J + (model == "3PL") * J
  aic <- 2 * k_params - 2 * log_lik
  bic <- k_params * log(N) - 2 * log_lik

  fit_df <- data.frame(
    Statistic = c("LogLikelihood", "AIC", "BIC", "Estimated_Parameters", "Iterations"),
    Value = c(round(log_lik, 3), round(aic, 3), round(bic, 3), k_params, iter),
    stringsAsFactors = FALSE
  )

  # 9D. Settings
  settings_df <- data.frame(
    Parameter = c("Model", "Dimensions", "Method", "Max_Iter", "Tolerance", "Ability_Estimates"),
    Value = c(model, D, method, con$max_iter, con$converge_tol, con$ability),
    stringsAsFactors = FALSE
  )

  if(con$verbose) cat("----------------------------------------------------------\nFinished All Estimation.\n")

  return(list(
    item_params = item_df,
    person_params = person_df,
    model_fit = fit_df,
    settings = settings_df
  ))
}

#' Binary (Dichotomous) Item Response Theory Estimation Using Likelihood or Bayesian
#'
#' @description
#' Estimates item and person parameters for binary item response models
#' using either Marginal Maximum Likelihood or Joint Maximum Likelihood.
#' Now supports flexible prior distributions for Bayesian estimation (MAP estimation).
#'
#' @param data A N x J data.frame of dichotomous responses (0/1).
#' @param model String. "Rasch", "2PL" (2-Parameter Logistic), or "3PL" (3-Parameter Logistic).
#' @param method String. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood). However, using Bayesian will override the likelihood estimation.
#' @param control A \code{list} of control parameters for the estimation algorithm:
#'   \itemize{
#'     \item \code{max_iter}: Maximum number of EM iterations (default = 100).
#'     \item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
#'     \item \code{theta_range}: Numeric vector of length 2 specifying the integration
#'           grid bounds (default = c(-4, 4)).
#'     \item \code{quad_points}: Number of quadrature points (default = 21).
#'     \item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
#'     \item \code{prior}: A list specifying prior distributions or fixed values for item parameters. Default is NULL (no priors).
#'       For Rasch: \code{list(b = value)} or \code{list(b = function(x) dnorm(x, 0, 1, log=TRUE))}.
#'       For 2PL: \code{list(a = ..., b = ...)}.
#'       For 3PL: \code{list(a = ..., b = ..., g = ...)}.
#'       Each parameter can be:
#'       - A single numeric value (applied to all items)
#'       - A numeric vector of length k (item-specific fixed values)
#'       - A function returning log-density (applied to all items)
#'       - A list of length k with functions or values (item-specific)
#'   }
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{item_params}: A data frame of estimated item parameters (discrimination, difficulty, guessing) and their standard errors.
#'   \item \code{person_params}: A data frame of estimated person abilities (theta) and standard errors.
#'   \item \code{model_fit}: A data frame containing fit statistics such as Akaike's Information Criterion (AIC), the Bayesian Information Criterion (BIC), and Log-Likelihood.
#'   \item \code{settings}: A list of control parameters used in the estimation.
#' }
#' @examples
#'   # # Simulate data
#'   set.seed(123)
#'   N <- 500; J <- 10
#'   true_theta <- rnorm(N)
#'   true_b <- seq(-2, 2, length.out=J)
#'   true_a <- runif(J, 0.8, 1.2)
#'   data_mat <- matrix(NA, N, J)
#'   for(i in 1:N) {
#'     p <- 1 / (1 + exp(-true_a * (true_theta[i] - true_b)))
#'     data_mat[i,] <- rbinom(J, 1, p)
#'   }
#'   df <- as.data.frame(data_mat)
#'   names(df) <- paste0("Q", 1:J)
#'
#'   # # Run Function without prior
#'   res <- binary_irt(df, model="2PL", method="EM")
#'
#' \donttest{
#'   # # Run Function with prior (function-based)
#'   res_prior <- binary_irt(df, model="2PL", method="EM",
#'                           control=list(prior=list(
#'                             a = function(x) dlnorm(x, 0, 0.5, log=TRUE),
#'                             b = function(x) dnorm(x, 0, 2, log=TRUE)
#'                           )))
#'
#'   # # Run Function with fixed value prior
#'   res_fixed <- binary_irt(df, model="2PL", method="EM",
#'                           control=list(prior=list(
#'                             a = 1,  # Fix all discrimination prior to 1
#'                             b = function(x) dnorm(x, 0, 2, log=TRUE)
#'                           )))
#'
#'   # View Results
#'   head(res$item_params)
#'   head(res$person_params)
#'   print(res$model_fit)
#'   # --- Example 2: With Package Data ---
#'   data("ela1", package = "tirt")
#'   # Subset the first 30 columns (must use the object name 'data_binary')
#'   df <- ela1[, 1:30]
#'   # Run Function on package data
#'   real_res <- binary_irt(df, model="2PL", method="EM")
#'   head(real_res$item_params)
#'   }
#' @export
binary_irt <- function(data,
                       model = "2PL",
                       method = "EM",
                       control = list()) {

  # --- 1. Setup and Defaults ---
  con <- list(
    max_iter = 100,
    converge_tol = 1e-4,
    theta_range = c(-4, 4),
    quad_points = 21,
    nr_max_iter = 20,
    nr_damp = 1.0,
    verbose = TRUE,
    prior = NULL  # No prior by default
  )
  con[names(control)] <- control

  # Extract prior from control
  prior <- con$prior

  # Normalize model name to uppercase
  model <- toupper(model)

  # Define the set of supported models
  valid_models <- c("RASCH", "2PL", "3PL")

  # Validate that every element in the input vector is within the allowed set
  if (!all(model %in% valid_models)) {
    stop("Model must be 'Rasch', '2PL', or '3PL' (case-insensitive)")
  }

  # --- Input Validation ---
  if(!is.data.frame(data)) stop("Input must be a data frame.")

  raw_data <- as.matrix(data)
  if(!is.numeric(raw_data)) {
    stop("Input data must contain only numeric values (0/1). Please remove ID columns or convert non-numeric data.")
  }

  item_names <- colnames(data)
  if(is.null(item_names)) item_names <- paste0("Item_", 1:ncol(data))

  N <- nrow(raw_data)
  J <- ncol(raw_data)

  # --- Prior Validation and Processing ---
  # This function validates and expands priors to match the number of items
  validate_and_expand_prior <- function(prior, model, J, verbose) {
    # If no prior, return NULL
    if(is.null(prior)) {
      if(con$verbose) cat("No prior specified. Using Maximum Likelihood Estimation as Specified.\n")
      return(NULL)
    }

    # Check that prior is a list
    if(!is.list(prior)) {
      stop("Prior must be a list. See documentation for proper format.")
    }

    # 1. Define allowed parameters based on model
    allowed_params <- switch(model,
                             "RASCH" = "b",
                             "2PL"   = c("a", "b"),
                             "3PL"   = c("a", "b", "g"),
                             stop("Unknown model type specified. Support only Rasch, 2PL, and 3PL now..."))

    # 2. Check for extra/unexpected parameters
    extra_params <- setdiff(names(prior), allowed_params)
    if(length(extra_params) > 0) {
      warning(sprintf("Prior specification contains parameter(s) [%s] which are not used in a %s model. These will be ignored.",
                      paste(extra_params, collapse=", "), model))
    }

    # 3. Filter to only allowed and provided parameters
    relevant_params <- intersect(names(prior), allowed_params)
    if(length(relevant_params) == 0) return(NULL)

    if(con$verbose) {
      cat(sprintf("Using MAP estimation for parameters: %s\n", paste(relevant_params, collapse=", ")))
    }

    # Internal check for valid types
    is_valid_prior <- function(x) {
      if(is.numeric(x)) return(TRUE)
      if(is.function(x)) return(TRUE)
      if(is.list(x)) return(all(sapply(x, function(elem) is.numeric(elem) || is.function(elem))))
      return(FALSE)
    }

    expanded_prior <- list()
    for(param in relevant_params) {
      param_prior <- prior[[param]]

      # Validate type (numeric, function, or list)
      isValid <- is.numeric(param_prior) || is.function(param_prior) || is.list(param_prior)
      if(!isValid) stop(sprintf("Prior for '%s' is an invalid type.", param))

      if(!is_valid_prior(param_prior)) {
        stop(sprintf("Prior for parameter '%s' must be numeric, a function, or a list.", param))
      }

      # Case 1: Single numeric value - apply to all items
      if(is.numeric(param_prior) && length(param_prior) == 1) {
        expanded_prior[[param]] <- replicate(J, param_prior, simplify=FALSE)
        if(con$verbose) cat(sprintf("  - Parameter '%s': Fixed value %.3f applied to all %d items\n",
                                    param, param_prior, J))

        # Case 2: Numeric vector of length J - item-specific fixed values
      } else if(is.numeric(param_prior) && length(param_prior) == J) {
        expanded_prior[[param]] <- as.list(param_prior)
        if(con$verbose) cat(sprintf("  - Parameter '%s': Item-specific fixed values for %d items\n",
                                    param, J))

        # Case 3: Numeric vector of wrong length
      } else if(is.numeric(param_prior) && length(param_prior) != J && length(param_prior) != 1) {
        stop(sprintf("Prior for parameter '%s' has length %d, but must be either 1 (applied to all items) or %d (one per item)",
                     param, length(param_prior), J))

        # Case 4: Single function - apply to all items
      } else if(is.function(param_prior)) {
        # Test the function to make sure it works
        test_val <- tryCatch({
          param_prior(0.5)
        }, error = function(e) {
          stop(sprintf("Prior function for parameter '%s' failed when tested with value 0.5. Error: %s",
                       param, e$message))
        })
        if(!is.numeric(test_val) || length(test_val) != 1) {
          stop(sprintf("Prior function for parameter '%s' must return a single numeric value (log-density)", param))
        }

        expanded_prior[[param]] <- replicate(J, param_prior, simplify=FALSE)
        if(con$verbose) cat(sprintf("  - Parameter '%s': Prior function applied to all %d items\n",
                                    param, J))

        # Case 5: List of length J - item-specific priors (can be mix of values and functions)
      } else if(is.list(param_prior) && length(param_prior) == J) {
        # Validate each element
        for(j in 1:J) {
          elem <- param_prior[[j]]
          if(is.function(elem)) {
            # Test the function
            test_val <- tryCatch({
              elem(0.5)
            }, error = function(e) {
              stop(sprintf("Prior function for parameter '%s'[Item %d] failed when tested. Error: %s",
                           param, j, e$message))
            })
            if(!is.numeric(test_val) || length(test_val) != 1) {
              stop(sprintf("Prior function for parameter '%s'[Item %d] must return a single numeric value",
                           param, j))
            }
          } else if(!is.numeric(elem) || length(elem) != 1) {
            stop(sprintf("Prior for parameter '%s'[Item %d] must be a single numeric value or function",
                         param, j))
          }
        }
        expanded_prior[[param]] <- param_prior
        if(con$verbose) cat(sprintf("  - Parameter '%s': Item-specific priors for %d items\n",
                                    param, J))

        # Case 6: List of wrong length
      } else if(is.list(param_prior)) {
        stop(sprintf("Prior list for parameter '%s' has length %d, but must be length %d (one per item)",
                     param, length(param_prior), J))
      }
    }

    return(expanded_prior)
  }

  # Validate and expand the prior
  prior_expanded <- validate_and_expand_prior(con$prior, model, J, con$verbose)

  # Decide on the message based on the RESULT of validation
  if(is.null(prior_expanded)) {
    if(con$verbose) cat("No valid priors for this model. Using Maximum Likelihood Estimation (MLE).\n")
  } else {
    if(con$verbose) cat("Valid priors detected. Using MAP (Maximum A Posteriori) Estimation.\n")
  }

  # --- Helper function to compute log prior density ---
  log_prior_density <- function(value, prior_spec) {
    # If no prior, return 0 (no contribution to log posterior)
    if(is.null(prior_spec)) return(0)

    # If prior is a fixed numeric value, use a very strong normal prior centered at that value
    if(is.numeric(prior_spec)) {
      # Very tight prior: essentially fixes the parameter
      return(dnorm(value, mean=prior_spec, sd=0.001, log=TRUE))
    }

    # If prior is a function, evaluate it
    if(is.function(prior_spec)) {
      result <- tryCatch({
        prior_spec(value)
      }, error = function(e) {
        warning(sprintf("Prior function evaluation failed for value %.3f. Returning -Inf. Error: %s",
                        value, e$message))
        return(-Inf)
      })

      # if the NR algorithm probes slightly outside [0, 1]
      if(is.na(result) || is.nan(result)) return(-Inf)

      if(!is.numeric(result) || length(result) != 1) {
        warning(sprintf("Prior function returned invalid value. Expected single numeric, got %s",
                        class(result)))
        return(-Inf)
      }

      return(result)
    }

    return(0)  # Shouldn't reach here
  }

  # --- Helper function to compute log prior gradient (derivative) using numerical differentiation ---
  log_prior_gradient <- function(value, prior_spec) {
    # If no prior, return 0
    if(is.null(prior_spec)) return(0)

    # If prior is a fixed numeric value, gradient pulls strongly toward that value
    if(is.numeric(prior_spec)) {
      return(-(value - prior_spec) / (0.001^2))
    }

    # If prior is a function, use numerical differentiation
    if(is.function(prior_spec)) {
      h <- 1e-5
      f_plus <- log_prior_density(value + h, prior_spec)
      f_minus <- log_prior_density(value - h, prior_spec)
      return((f_plus - f_minus) / (2 * h))
    }

    return(0)
  }

  # --- Helper function to compute log prior hessian (second derivative) using numerical differentiation ---
  log_prior_hessian <- function(value, prior_spec) {
    # If no prior, return 0
    if(is.null(prior_spec)) return(0)

    # If prior is a fixed numeric value, hessian is constant (very negative = strong pull)
    if(is.numeric(prior_spec)) {
      return(-1 / (0.001^2))
    }

    # If prior is a function, use numerical differentiation
    if(is.function(prior_spec)) {
      h <- 1e-5
      f_0 <- log_prior_density(value, prior_spec)
      f_plus <- log_prior_density(value + h, prior_spec)
      f_minus <- log_prior_density(value - h, prior_spec)
      return((f_plus - 2*f_0 + f_minus) / (h^2))
    }

    return(0)
  }

  # Identify valid items
  item_means <- colMeans(raw_data, na.rm = TRUE)
  valid_items_idx <- which(item_means > 0 & item_means < 1)
  bad_items_idx <- setdiff(1:J, valid_items_idx)

  # Initialize Parameters
  a <- rep(1, J)
  # Calculate proportion correct, avoiding 0 and 1
  p_bounded <- pmin(pmax(item_means, 0.01), 0.99)
  b <- -log(p_bounded / (1 - p_bounded))
  g <- rep(0, J)

  if (model == "3PL") {
    # For 3PL, difficulty must be adjusted for guessing
    # Starting g at 0.15 is safer than 0.2 or 0
    g <- rep(0.15, J)
    # Corrected b calculation for 3PL: b = -log((p - g)/(1 - p)) / a
    b <- -log(pmax(p_bounded - 0.15, 0.01) / (1 - p_bounded))
  } else {
    g <- rep(0, J)
    b <- -log(p_bounded / (1 - p_bounded))
  }

  if (model == "RASCH") a[] <- 1

  # Initialize Theta safely
  raw_scores <- rowMeans(raw_data, na.rm=TRUE)
  raw_scores[is.nan(raw_scores)] <- 0.5
  theta <- as.vector(scale(qlogis(pmin(pmax(raw_scores, 0.01), 0.99))))
  theta[is.nan(theta)] <- 0

  # Quadrature setup for EM
  if (method == "EM") {
    nodes <- seq(-4, 4, length.out = con$quad_points)
    weights <- dnorm(nodes)
    weights <- weights / sum(weights)
  }

  # --- 2. Helper Functions ---

  get_P <- function(th, a_val, b_val, g_val) {
    z <- a_val * (th - b_val)
    z <- pmin(pmax(z, -30), 30)
    p <- g_val + (1 - g_val) * (1 / (1 + exp(-z)))
    return(p)
  }

  # Updated solve_item_nr to include prior information
  solve_item_nr <- function(r_vec, n_vec, theta_vec, cur_a, cur_b, cur_g, mod_type, item_idx) {
    pa <- cur_a; pb <- cur_b; pg <- cur_g

    # Determine which parameters are active for this model
    if(mod_type == "RASCH") active_idx <- 2
    else if(mod_type == "2PL") active_idx <- 1:2
    else active_idx <- 1:3

    # Get priors for this item (if they exist)
    prior_a <- if(!is.null(prior_expanded) && "a" %in% names(prior_expanded)) prior_expanded$a[[item_idx]] else NULL
    prior_b <- if(!is.null(prior_expanded) && "b" %in% names(prior_expanded)) prior_expanded$b[[item_idx]] else NULL
    prior_g <- if(!is.null(prior_expanded) && "g" %in% names(prior_expanded)) prior_expanded$g[[item_idx]] else NULL

    # Log-likelihood function (now includes prior - making it log-posterior)
    ll_func <- function(p_vec) {
      loc_a <- if(1 %in% active_idx) p_vec[1] else 1
      loc_b <- if(mod_type == "RASCH") p_vec[1] else p_vec[2]
      loc_g <- if(3 %in% active_idx) p_vec[3] else 0

      P_tmp <- get_P(theta_vec, loc_a, loc_b, loc_g)
      P_tmp <- pmin(pmax(P_tmp, 1e-9), 1-1e-9)

      # Log-likelihood from data
      log_lik <- sum(r_vec * log(P_tmp) + (n_vec - r_vec) * log(1 - P_tmp))

      # Add log-prior contributions
      log_prior <- 0
      if(!is.null(prior_a) && mod_type != "RASCH") log_prior <- log_prior + log_prior_density(loc_a, prior_a)
      if(!is.null(prior_b))                        log_prior <- log_prior + log_prior_density(loc_b, prior_b)
      if(!is.null(prior_g) && mod_type == "3PL")   log_prior <- log_prior + log_prior_density(loc_g, prior_g)

      return(log_lik + log_prior)
    }

    # Newton-Raphson iterations
    for(iter in 1:con$nr_max_iter) {
      if(mod_type == "RASCH") curr_vec <- c(pb)
      else if(mod_type == "2PL") curr_vec <- c(pa, pb)
      else curr_vec <- c(pa, pb, pg)

      h <- 1e-4
      grad <- numeric(length(curr_vec))
      hess <- matrix(0, length(curr_vec), length(curr_vec))
      f0 <- ll_func(curr_vec)

      # Compute gradient using finite differences
      for(k in 1:length(curr_vec)) {
        tmp <- curr_vec; tmp[k] <- tmp[k] + h
        grad[k] <- (ll_func(tmp) - f0) / h
      }

      # Compute hessian (diagonal only) using finite differences
      for(k in 1:length(curr_vec)) {
        tmp_kk <- curr_vec; tmp_kk[k] <- tmp_kk[k] + h
        tmp_mk <- curr_vec; tmp_mk[k] <- tmp_mk[k] - h
        hess[k,k] <- (ll_func(tmp_kk) - 2*f0 + ll_func(tmp_mk)) / (h^2)
      }

      # Add small regularization for numerical stability
      diag(hess) <- diag(hess) - 1e-5

      try_step <- try(solve(hess, grad), silent = TRUE)
      if(inherits(try_step, "try-error")) break

      step <- try_step * con$nr_damp

      # Check if step is finite before the if statement
      if(any(!is.finite(step))) break

      # Limit the step size to prevent "shooting" to infinity in one jump
      step <- pmin(pmax(step, -2), 2)

      new_vec <- curr_vec - step

      # Update parameters with constraints
      if(mod_type == "RASCH") {
        pb <- max(min(new_vec[1], 15), -15) # Allow extremely low difficulty
      } else if (mod_type == "2PL") {
        pa <- max(min(new_vec[1], 5), 0.01)
        pb <- max(min(new_vec[2], 15), -15)
      } else {
        pa <- max(min(new_vec[1], 5), 0.01)
        pb <- max(min(new_vec[2], 15), -15)
        # Stay strictly within (0.001, 0.5) so priors like Beta don't crash
        pg <- max(min(new_vec[3], 0.5), 0.001)
      }

      # Check convergence
      if(max(abs(step)) < 1e-5) break
    }

    # Compute standard errors
    se_vec <- tryCatch({
      if(mod_type == "RASCH") curr_vec <- c(pb)
      else if(mod_type == "2PL") curr_vec <- c(pa, pb)
      else curr_vec <- c(pa, pb, pg)

      hess_final <- matrix(0, length(curr_vec), length(curr_vec))
      f0 <- ll_func(curr_vec)

      for(k in 1:length(curr_vec)) {
        tmp_kk <- curr_vec; tmp_kk[k] <- tmp_kk[k] + h
        tmp_mk <- curr_vec; tmp_mk[k] <- tmp_mk[k] - h
        hess_final[k,k] <- (ll_func(tmp_kk) - 2*f0 + ll_func(tmp_mk)) / (h^2)
      }

      # Stability: Add a tiny penalty to the diagonal to ensure invertibility
      diag(hess_final) <- diag(hess_final) - 1e-6

      # Check if Hessian is still invalid
      if(any(is.na(hess_final)) || any(diag(hess_final) >= 0)) {
        return(rep(NA, length(curr_vec)))
      }

      sqrt(diag(solve(-hess_final)))
    }, error = function(e) rep(NA, length(curr_vec)), warning = function(w) rep(NA, length(curr_vec)))

    return(list(a=pa, b=pb, g=pg, se=se_vec))
  }

  # --- 3. Main Estimation Loop ---

  if(con$verbose) {
    cat(sprintf("\nStarting %s Estimation using %s algorithm...\n", model, method))
    if(!is.null(prior_expanded)) {
      cat("Bayesian estimation with priors enabled (MAP estimation).\n")
    }
    cat(sprintf("------------------------------------------------\n"))
  }

  is_converged <- FALSE

  for (iter in 1:con$max_iter) {

    # E-STEP
    if (method == "EM") {
      L_terms <- matrix(0, N, con$quad_points)

      for(j in valid_items_idx) {
        if(is.na(a[j]) | is.na(b[j])) next
        P_jq <- get_P(nodes, a[j], b[j], g[j])
        resp <- raw_data[, j]

        term1 <- log(pmax(P_jq, 1e-10))
        term0 <- log(pmax(1 - P_jq, 1e-10))

        r1 <- which(resp == 1)
        r0 <- which(resp == 0)

        if(length(r1)>0) L_terms[r1,] <- L_terms[r1,] + matrix(term1, length(r1), con$quad_points, byrow=TRUE)
        if(length(r0)>0) L_terms[r0,] <- L_terms[r0,] + matrix(term0, length(r0), con$quad_points, byrow=TRUE)
      }

      F_iq <- exp(L_terms) * matrix(weights, N, con$quad_points, byrow=TRUE)
      sum_Fi <- rowSums(F_iq)
      sum_Fi[sum_Fi == 0] <- 1e-10
      Posterior_iq <- F_iq / sum_Fi

    } else {
      # MLE Person Update (no change needed - person parameters don't have priors)
      for(i in 1:N) {
        valid <- !is.na(raw_data[i,]) & (1:J %in% valid_items_idx)
        if(sum(valid)==0) next
        ti <- theta[i]
        for(k in 1:5) {
          p_vec <- get_P(ti, a, b, g)
          diff <- raw_data[i, valid] - p_vec[valid]
          d1 <- sum(a[valid] * diff)
          d2 <- -sum( (a[valid]^2) * p_vec[valid] * (1-p_vec[valid]) )
          if(abs(d2) < 1e-6) break
          step <- d1/d2
          ti <- ti - step
          ti <- max(min(ti, con$theta_range[2]), con$theta_range[1])
          if(abs(step) < 1e-3) break
        }
        theta[i] <- ti
      }
    }

    # M-STEP (now with prior information)
    max_param_change <- 0

    for (j in valid_items_idx) {
      old_params <- c(a[j], b[j], g[j])
      if (method == "EM") {
        r_jq <- colSums(Posterior_iq * ifelse(!is.na(raw_data[,j]) & raw_data[,j]==1, 1, 0))
        n_jq <- colSums(Posterior_iq * ifelse(!is.na(raw_data[,j]), 1, 0))
        # Pass item index to get the correct prior
        res <- solve_item_nr(r_jq, n_jq, nodes, a[j], b[j], g[j], model, j)
      } else {
        valid <- !is.na(raw_data[,j])
        # Pass item index to get the correct prior
        res <- solve_item_nr(raw_data[valid,j], rep(1, sum(valid)), theta[valid], a[j], b[j], g[j], model, j)
      }
      a[j] <- res$a; b[j] <- res$b; g[j] <- res$g
      max_param_change <- max(max_param_change, abs(c(a[j], b[j], g[j]) - old_params), na.rm=TRUE)
    }

    if(con$verbose) cat(paste0("\rIteration ", iter, ": Max Param Change = ", round(max_param_change, 5), "   "))

    # Only allow convergence if parameters moved at least once
    # and the change is small but finite.
    if(iter > 10 && max_param_change < con$converge_tol) {
      if(con$verbose) cat("\n\n>>> Convergence Confirmation: Model Converged!\n")
      is_converged <- TRUE
      break
    }
  }

  # --- User Friendly Reminder: Max Iterations ---
  if(!is_converged && con$verbose) {
    cat(sprintf("\n\n>>> NOTE: Stopped because max Iteration Reached at %d.\n", con$max_iter))
    cat("    Adjust it in control=list(max_iter = ...) to have higher iteration if needed.\n")
  }

  # --- 4. Post-Hoc Estimation & formatting ---

  if(con$verbose) {
    cat("\n------------------------------------------------\n")
    cat("Estimating Final Person Parameters...\n")
    cat(sprintf("Personal Parameter estimated with range [%s, %s].\n", con$theta_range[1], con$theta_range[2]))
    cat("Adjust it in control=list(theta_range=c(..)) if you want a different range.\n")
  }

  if(length(bad_items_idx) > 0) {
    a[bad_items_idx] <- NA; b[bad_items_idx] <- NA; g[bad_items_idx] <- NA
  }

  # Final Person Parameter Estimation
  final_theta <- numeric(N)
  final_theta_se <- numeric(N)
  n_resp <- numeric(N)

  for(i in 1:N) {
    valid <- !is.na(raw_data[i,]) & !is.na(a) & !is.na(b)
    n_resp[i] <- sum(valid)

    if(n_resp[i] == 0) {
      final_theta[i] <- NA; final_theta_se[i] <- NA; next
    }
    score <- sum(raw_data[i, valid])
    if(score == 0) {
      final_theta[i] <- con$theta_range[1]; final_theta_se[i] <- NA; next
    }
    if(score == n_resp[i]) {
      final_theta[i] <- con$theta_range[2]; final_theta_se[i] <- NA; next
    }

    ti <- if(method=="EM") 0 else theta[i]
    for(k in 1:20) {
      p_vec <- get_P(ti, a[valid], b[valid], g[valid])
      u <- raw_data[i, valid]
      num <- (1-g[valid]) * exp(a[valid]*(ti-b[valid]))
      den <- (1 + exp(a[valid]*(ti-b[valid])))^2
      d_P <- a[valid] * num / den
      denom_info <- p_vec * (1-p_vec)
      denom_info[denom_info < 1e-10] <- 1e-10

      score_func <- sum( (u - p_vec) * d_P / denom_info )
      info_func  <- sum( (d_P^2) / denom_info )

      if(is.na(info_func) || info_func < 1e-9) break
      step <- score_func / info_func
      step <- max(min(step, 2), -2)
      ti <- ti + step
      if(abs(step) < 1e-4) break
    }
    final_theta[i] <- ti
    final_theta_se[i] <- if(!is.na(info_func) && info_func > 1e-9) 1/sqrt(info_func) else NA
  }

  if(con$verbose) cat("Person Parameter Estimation Finished.\n")

  # Final Item Statistics with standard errors
  se_a <- rep(NA, J); se_b <- rep(NA, J); se_g <- rep(NA, J)

  for(j in valid_items_idx) {
    if (method == "EM") {
      r_jq <- colSums(Posterior_iq * ifelse(!is.na(raw_data[,j]) & raw_data[,j]==1, 1, 0))
      n_jq <- colSums(Posterior_iq * ifelse(!is.na(raw_data[,j]), 1, 0))
      res <- solve_item_nr(r_jq, n_jq, nodes, a[j], b[j], g[j], model, j)
    } else {
      valid <- !is.na(raw_data[,j])
      res <- solve_item_nr(raw_data[valid,j], rep(1, sum(valid)), theta[valid], a[j], b[j], g[j], model, j)
    }
    if(model == "RASCH") { se_b[j] <- res$se[1] }
    else if (model == "2PL") { se_a[j] <- res$se[1]; se_b[j] <- res$se[2] }
    else { se_a[j] <- res$se[1]; se_b[j] <- res$se[2]; se_g[j] <- res$se[3] }
  }

  # Fit Statistics (including log-prior in log-likelihood for MAP estimation)
  infit <- rep(NA, J); outfit <- rep(NA, J); log_lik <- 0

  for(j in valid_items_idx) {
    valid_p <- !is.na(final_theta) & !is.na(raw_data[,j])
    if(sum(valid_p) == 0) next
    t_use <- final_theta[valid_p]
    x_use <- raw_data[valid_p, j]
    P <- get_P(t_use, a[j], b[j], g[j])
    P_safe <- pmin(pmax(P, 1e-10), 1-1e-10)
    W <- P * (1-P)
    Z <- (x_use - P) / sqrt(W + 1e-10)
    outfit[j] <- sum(Z^2) / length(Z)
    infit[j] <- sum(W * Z^2) / (sum(W) + 1e-10)

    # Log-likelihood from data
    log_lik <- log_lik + sum(x_use * log(P_safe) + (1-x_use) * log(1-P_safe))
  }

  # Add log-prior contribution to log-likelihood if priors are specified
  if(!is.null(prior_expanded)) {
    log_prior_total <- 0
    for(j in valid_items_idx) {
      if(model != "RASCH" && !is.na(a[j])) {
        log_prior_total <- log_prior_total + log_prior_density(a[j], prior_expanded$a[[j]])
      }
      if(!is.na(b[j])) {
        log_prior_total <- log_prior_total + log_prior_density(b[j], prior_expanded$b[[j]])
      }
      if(model == "3PL" && !is.na(g[j])) {
        log_prior_total <- log_prior_total + log_prior_density(g[j], prior_expanded$g[[j]])
      }
    }
    log_lik <- log_lik + log_prior_total
  }

  # Construct Output - SAFELY
  n_count <- as.vector(colSums(!is.na(raw_data)))
  p_val <- as.vector(colMeans(raw_data, na.rm=TRUE))

  if(model == "RASCH") {
    out_items <- data.frame(item=as.vector(item_names),
                            difficulty=as.vector(round(b,3)),
                            difficulty_se=as.vector(round(se_b,3)),
                            number=n_count,
                            pvalue=as.vector(round(p_val,3)),
                            infit=as.vector(round(infit,3)),
                            outfit=as.vector(round(outfit,3)),
                            stringsAsFactors = FALSE)
  } else if (model == "2PL") {
    out_items <- data.frame(item=as.vector(item_names),
                            discrimination=as.vector(round(a,3)),
                            discrimination_se=as.vector(round(se_a,3)),
                            difficulty=as.vector(round(b,3)),
                            difficulty_se=as.vector(round(se_b,3)),
                            number=n_count,
                            pvalue=as.vector(round(p_val,3)),
                            stringsAsFactors = FALSE)
  } else {
    out_items <- data.frame(item=as.vector(item_names),
                            discrimination=as.vector(round(a,3)),
                            discrimination_se=as.vector(round(se_a,3)),
                            difficulty=as.vector(round(b,3)),
                            difficulty_se=as.vector(round(se_b,3)),
                            guess=as.vector(round(g,3)),
                            guess_se=as.vector(round(se_g,3)),
                            number=n_count,
                            pvalue=as.vector(round(p_val,3)),
                            stringsAsFactors = FALSE)
  }

  if(length(bad_items_idx) > 0) {
    cols_to_NA <- which(!names(out_items) %in% c("item", "number", "pvalue"))
    out_items[bad_items_idx, cols_to_NA] <- NA
    if(con$verbose) message("\nWarning: Some items have 0 variance. Parameters set to NA.")
  }

  out_persons <- data.frame(ability=as.vector(round(final_theta,3)),
                            ability_se=as.vector(round(final_theta_se,3)),
                            number=as.vector(n_resp),
                            stringsAsFactors = FALSE)

  k_params <- length(valid_items_idx) * (if(model=="RASCH") 1 else if(model=="2PL") 2 else 3)
  aic <- 2*k_params - 2*log_lik
  bic <- k_params*log(N) - 2*log_lik

  # Add note about MAP estimation in fit statistics
  log_lik_label <- if(!is.null(prior_expanded)) "LogPosterior" else "LogLikelihood"

  out_fit <- data.frame(Index=c(log_lik_label, "AIC", "BIC", "Iterations"),
                        Value=as.vector(c(round(log_lik,3), round(aic,3), round(bic,3), iter)),
                        stringsAsFactors = FALSE)

  # Settings summary for user reference
  estimation_type <- if(!is.null(prior_expanded)) "MAP (with priors)" else "MLE (no priors)"
  out_settings <- data.frame(
    Parameter = c("Model", "Method", "Estimation", "Max_Iter", "Converge_Tol", "Theta_Min", "Theta_Max"),
    Value = as.vector(c(model, method, estimation_type, con$max_iter, con$converge_tol,
                        con$theta_range[1], con$theta_range[2])),
    stringsAsFactors = FALSE
  )

  if(con$verbose) {
    cat("Finished All Estimation.\n")
    cat("------------------------------------------------\n")
  }

  return(list(item_params=out_items,
              person_params=out_persons,
              model_fit=out_fit,
              settings=out_settings))
}

#' Fixed Item Calibration
#'
#' @description
#' Estimates unknown item parameters using Marginal Maximum Likelihood via Expectation-Maximization Algorithm.
#' Uses a custom Bounded Newton-Raphson solver.
#' Supports mixed-format data containing dichotomous and polytomous responses
#'
#' @param response_df A data.frame of responses. Rows=Students, Cols=Items.
#'   Data MUST be from 0-indexed (0, 1, 2...).
#' @param item_params_df A data.frame of known parameters.
#'   Required: "item", "model".
#' @param control A \code{list} of control parameters for the estimation algorithm:
#'   \itemize{
#'     \item \code{max_iter}: Maximum number of EM iterations (default = 50).
#'     \item \code{conv_crit}: Convergence criterion for parameter change (default = 0.005).
#'     \item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
#'   }
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{item_params}: Estimated parameters for unknown items.
#'   \item \code{person_params}: Estimated person parameters.
#'   \item \code{model_fit}: A data frame containing number off estimated parameters and fit statistics such as Akaike’s Information Criterion (AIC), the Bayesian Information Criterion (BIC), and Log-Likelihood.
#' }
#'
#' @examples
#' # 1. TOY EXAMPLE
#' # ===========================================================================
#' set.seed(123)
#' # Create a very small dataset (N=50, J=4)
#' N_toy <- 50
#' df_toy <- data.frame(
#'   I1 = rbinom(N_toy, 1, 0.5), I2 = rbinom(N_toy, 1, 0.6), # Known items
#'   U1 = rbinom(N_toy, 1, 0.5), U2 = rbinom(N_toy, 1, 0.4)  # Unknown items
#' )
#'
#' # Define the "Known" parameters for I1 and I2
#' known_params <- data.frame(
#'   item = c("I1", "I2"),
#'   model = c("2PL", "2PL"),
#'   a = c(1.0, 1.2),
#'   b = c(-0.5, 0.5)
#' )
#'
#' # Run Fixed Item Calibration with very low iterations
#' fit_toy <- fixed_item(df_toy, known_params, control=list(max_iter=2, verbose=FALSE))
#' print(head(fit_toy$item_params))
#'   \donttest{
#'   # --- Example 2: Simulation ---
#'   set.seed(123)
#'   N <- 500
#'   true_theta <- rnorm(N, 0, 1)
#'
#'   # 1. Simulation Helpers
#'   sim_2pl <- function(theta, a, b) {
#'     p <- 1 / (1 + exp(-1.7 * a * (theta - b)))
#'     rbinom(N, 1, p)
#'   }
#'   sim_poly <- function(theta, a, steps) {
#'     n_cat <- length(steps) + 1
#'     probs <- matrix(0, length(theta), n_cat)
#'     for(k in 1:n_cat) {
#'       score <- k - 1
#'       if(score == 0) num <- 0
#'       else num <- a * (score * theta - sum(steps[1:score]))
#'       probs[, k] <- exp(num)
#'     }
#'     probs <- probs / rowSums(probs)
#'     apply(probs, 1, function(x) sample(0:(n_cat-1), 1, prob=x))
#'   }
#'
#'   # 2. Generate Data (Mixed Known/Unknown Items)
#'   # Items 1-5: Known Binary (2PL)
#'   # Items 6-10: Unknown Binary (2PL)
#'   # Items 11-12: Known Poly (GPCM)
#'   # Items 13-15: Unknown Poly (GPCM)
#'
#'   resp_mat <- matrix(NA, N, 15)
#'   colnames(resp_mat) <- paste0("Item_", 1:15)
#'
#'   # Known Binary Parameters
#'   a_bin <- c(1.0, 1.2, 0.9, 1.1, 0.8)
#'   b_bin <- c(-1, -0.5, 0, 0.5, 1)
#'
#'   for(i in 1:5) resp_mat[,i] <- sim_2pl(true_theta, a_bin[i], b_bin[i])
#'   for(i in 6:10) resp_mat[,i] <- sim_2pl(true_theta, runif(1,0.8,1.2), rnorm(1))
#'
#'   # Known Poly Parameters
#'   a_poly <- c(1.0, 0.8)
#'   d_poly <- list(c(-1, 1), c(-0.5, 0.5))
#'
#'   resp_mat[,11] <- sim_poly(true_theta, a_poly[1], d_poly[[1]])
#'   resp_mat[,12] <- sim_poly(true_theta, a_poly[2], d_poly[[2]])
#'   for(i in 13:15) resp_mat[,i] <- sim_poly(true_theta, 1.0, c(-0.5, 0.5))
#'
#'   df_resp <- as.data.frame(resp_mat)
#'
#'   # 3. Create 'Known Parameters' Dataframe
#'   # This tells the function: "Fix these, Estimate the rest"
#'   known_df <- data.frame(
#'     item = c(paste0("Item_", 1:5), "Item_11", "Item_12"),
#'     model = c(rep("2PL", 5), rep("GPCM", 2)),
#'     a = c(a_bin, a_poly),
#'     b = c(b_bin, NA, NA),     # Binary difficulty
#'     step_1 = c(rep(NA, 5), -1, -0.5), # Poly steps
#'     step_2 = c(rep(NA, 5),  1,  0.5),
#'     stringsAsFactors = FALSE
#'   )
#'
#'   # 4. Run Estimation
#'   res <- fixed_item(df_resp, known_df, control=list(max_iter=20))
#'
#'   # View Results
#'   # Notice Items 1-5 and 11-12 have Status "Fixed"
#'   head(res$item_params, 12)
#'
#'   # --- Example 2: With Package Data ---
#'   data("ela1", package = "tirt")
#'
#'   # Let's treat the first 5 items as "Known" with arbitrary parameters
#'   # just to demonstrate syntax.
#'   df_real <- ela1[, 1:20]
#'
#'   known_real <- data.frame(
#'     item = paste0("Q", 1:5),
#'     model = "2PL",
#'     a = 1.0,
#'     b = seq(-1, 1, length.out=5)
#'   )
#'
#'   # Ideally, column names in df_real should match 'item' column in known_real
#'   colnames(df_real)[1:5] <- paste0("Q", 1:5)
#'
#'   real_res <- fixed_item(df_real, known_real, control=list(max_iter=10))
#'   head(real_res$item_params)
#'   }
#' @export
fixed_item <- function(response_df, item_params_df, control = list()) {

  # --- 0. Control Parameters ---
  con <- list(max_iter = 50, conv_crit = 0.005, verbose = TRUE)
  con[names(control)] <- control

  # --- 1. INTERNAL MATH FUNCTIONS (The Engine) ---

  # 1.1 Probability Engine
  get_prob_matrix <- function(theta, a, b_vec, c_param, model) {
    n_q <- length(theta)

    # Binary Models
    if (model %in% c("Rasch", "2PL", "3PL")) {
      D <- 1.7
      lin <- D * a * (theta - b_vec[1])
      # Logistic function
      p1 <- 1 / (1 + exp(-lin))
      # 3PL Guessing
      if (model == "3PL") p1 <- c_param + (1 - c_param) * p1
      p0 <- 1 - p1
      return(cbind(p0, p1))
    }

    # GPCM / PCM (Divide by total)
    if (model %in% c("GPCM", "PCM")) {
      n_cat <- length(b_vec) + 1
      numerators <- matrix(0, nrow = n_q, ncol = n_cat)
      current_sum <- 0
      numerators[, 1] <- 0
      for (k in 2:n_cat) {
        term <- a * (theta - b_vec[k-1])
        current_sum <- current_sum + term
        numerators[, k] <- current_sum
      }
      exps <- exp(numerators)
      # Row sums safety to prevent overflow/NaN
      rs <- rowSums(exps)
      rs[rs == 0] <- 1
      probs <- exps / rs
      return(probs)
    }

    # GRM (Difference of cumulative)
    if (model == "GRM") {
      n_cat <- length(b_vec) + 1
      D <- 1.7
      p_star <- matrix(0, nrow = n_q, ncol = n_cat + 1)
      p_star[, 1] <- 1.0; p_star[, n_cat + 1] <- 0.0

      for (k in 1:(n_cat - 1)) {
        lin <- D * a * (theta - b_vec[k])
        p_star[, k+1] <- 1 / (1 + exp(-lin))
      }
      probs <- matrix(0, nrow = n_q, ncol = n_cat)
      for (k in 1:n_cat) probs[, k] <- p_star[, k] - p_star[, k+1]
      probs <- pmax(probs, 1e-10) # Floor small probs
      probs <- probs / rowSums(probs)
      return(probs)
    }
    stop(paste("Unknown Model:", model))
  }

  # 1.2 Custom Bounded Newton-Raphson Solver
  # Used for both M-Step (Item Params) and Person Scoring
  custom_solver <- function(start_params, fn_minimize, lower_b, upper_b) {
    p <- start_params
    n_p <- length(p)

    # Settings for the inner loop
    max_inner <- 10
    tol <- 1e-4
    h <- 1e-4 # Step size for derivatives

    final_hessian <- matrix(NA, n_p, n_p)

    for (iter in 1:max_inner) {
      val <- fn_minimize(p)

      grad <- numeric(n_p)
      hess <- numeric(n_p) # Diagonal approx for speed & stability

      # Compute Numerical Gradient & Diagonal Hessian
      for (k in 1:n_p) {
        p_plus <- p; p_plus[k] <- p_plus[k] + h
        p_minus <- p; p_minus[k] <- p_minus[k] - h

        v_plus <- fn_minimize(p_plus)
        v_minus <- fn_minimize(p_minus)

        # Central Difference Gradient
        grad[k] <- (v_plus - v_minus) / (2 * h)

        # Second Derivative (Hessian diag)
        h_val <- (v_plus - 2 * val + v_minus) / (h^2)
        hess[k] <- h_val
      }

      # Update Rule: p_new = p - LearningRate * (Grad / Hessian)
      # We ensure Hessian is positive (curvature checks)
      hess <- pmax(hess, 1e-6)

      update <- grad / hess

      # Damping: Don't jump too far
      update <- pmax(pmin(update, 1.0), -1.0)

      p_new <- p - update

      # Apply Bounds (Clamping)
      p_new <- pmax(pmin(p_new, upper_b), lower_b)

      # Check convergence
      if (max(abs(p - p_new)) < tol) {
        p <- p_new
        final_hessian <- diag(hess, nrow=n_p) # Store diagonal matrix
        break
      }
      p <- p_new

      # Final Hessian update on last loop
      if (iter == max_inner) final_hessian <- diag(hess, nrow=n_p)
    }

    return(list(par = p, hessian = final_hessian))
  }

  # 1.3 Invert Hessian for SE
  get_se <- function(hess_mat) {
    tryCatch({
      # Solve inversion with ridge for stability
      inv <- solve(hess_mat + diag(1e-8, nrow(hess_mat)))
      se <- sqrt(diag(inv))
      se[is.nan(se)] <- NA
      return(se)
    }, error = function(e) return(rep(NA, nrow(hess_mat))))
  }

  # --- 2. Input Setup ---

  if(con$verbose) {
    message("---------------------------------------------------------")
    message("Starting Fixed Item Calibration...")
    message("---------------------------------------------------------")
  }

  items <- colnames(response_df)
  resp_mat <- as.matrix(response_df)

  min_val <- min(resp_mat, na.rm = TRUE)
  if (min_val < 0) stop("Data must be 0-indexed (non-negative).")
  if (min_val > 0) stop("Data must start at 0 (e.g., 0/1).")

  known_items <- intersect(items, item_params_df$item)
  unknown_items <- setdiff(items, known_items)

  if (length(known_items) == 0) stop("No known items found.")

  # --- Intelligent Model Detection ---
  known_models <- unique(item_params_df$model)

  # Binary Hierarchy: 3PL > 2PL > Rasch
  if ("3PL" %in% known_models) {
    default_binary <- "3PL"
  } else if ("2PL" %in% known_models) {
    default_binary <- "2PL"
  } else {
    default_binary <- "Rasch"
  }

  # Poly Hierarchy: GRM > PCM > GPCM
  if ("GRM" %in% known_models) {
    default_poly <- "GRM"
  } else if ("PCM" %in% known_models) {
    default_poly <- "PCM"
  } else {
    default_poly <- "GPCM"
  }

  if(con$verbose) {
    message(paste("Inferred Unknown Binary:", default_binary))
    message(paste("Inferred Unknown Poly:  ", default_poly))
  }

  # --- 3. Initialize Parameters ---

  item_list <- list()
  total_est_params <- 0

  # 3.1 Load Known
  for (itm in known_items) {
    row <- item_params_df[item_params_df$item == itm, ]
    mod <- row$model[1]

    a <- if(!is.null(row$a) && !is.na(row$a)) row$a else 1.0
    c_p <- if(!is.null(row$c) && !is.na(row$c)) row$c else 0.0

    b_cols <- grep("^(b|d\\d+|step_\\d+)$", names(row), value=TRUE)
    b_cols <- b_cols[order(nchar(b_cols), b_cols)]
    b_vec <- as.numeric(row[b_cols])
    b_vec <- b_vec[!is.na(b_vec)]
    if (length(b_vec) == 0 && "b" %in% names(row)) b_vec <- row$b

    item_list[[itm]] <- list(
      type = "fixed", model = mod,
      a = a, b = b_vec, c = c_p,
      n_cat = length(b_vec) + 1,
      se_a=NA, se_b=rep(NA, length(b_vec)), se_c=NA
    )
    if(mod %in% c("Rasch","2PL","3PL")) item_list[[itm]]$n_cat <- 2
  }

  # 3.2 Initialize Unknown
  for (itm in unknown_items) {
    obs <- na.omit(resp_mat[, itm])
    max_k <- max(obs)
    n_cat <- max_k + 1
    mod_use <- if (max_k == 1) default_binary else default_poly

    # Initial guesses
    prop <- mean(obs)/max_k
    prop <- min(max(prop,0.01),0.99)
    logit_p <- -log(prop/(1-prop))

    a_init <- 1.0
    c_init <- 0.0
    b_init <- logit_p

    if (mod_use == "3PL") c_init <- 0.1
    if (!mod_use %in% c("Rasch","2PL","3PL")) {
      b_init <- seq(logit_p-0.5, logit_p+0.5, length.out=max_k)
    }

    # Count parameters
    n_p <- length(b_init)
    if (mod_use %in% c("2PL","3PL","GPCM","GRM")) n_p <- n_p + 1
    if (mod_use == "3PL") n_p <- n_p + 1
    total_est_params <- total_est_params + n_p

    item_list[[itm]] <- list(
      type = "estimated", model = mod_use,
      a = a_init, b = b_init, c = c_init,
      n_cat = n_cat,
      se_a=NA, se_b=rep(NA, length(b_init)), se_c=NA
    )
  }

  # --- 4. EM Algorithm ---

  nodes <- seq(-6, 6, length.out = 41)
  weights <- dnorm(nodes); weights <- weights / sum(weights)

  cycle <- 0
  converged <- FALSE
  last_ll <- -Inf

  while(!converged && cycle < con$max_iter) {
    cycle <- cycle + 1

    # --- E-Step ---
    posterior <- matrix(rep(weights, nrow(resp_mat)), nrow = nrow(resp_mat), byrow = TRUE)

    for (itm in items) {
      obj <- item_list[[itm]]
      probs <- get_prob_matrix(nodes, obj$a, obj$b, obj$c, obj$model)

      obs <- resp_mat[, itm]
      valid <- which(!is.na(obs))
      if (length(valid) == 0) next

      idx <- obs[valid] + 1
      lik_mat <- t(probs[, idx])
      posterior[valid, ] <- posterior[valid, ] * lik_mat
    }

    row_sums <- rowSums(posterior)
    row_sums[row_sums == 0] <- 1e-10
    posterior <- posterior / row_sums
    curr_ll <- sum(log(row_sums))

    # --- M-Step ---
    for (itm in unknown_items) {
      obj <- item_list[[itm]]
      obs <- resp_mat[, itm]
      valid <- which(!is.na(obs))
      r_qk <- matrix(0, nrow = length(nodes), ncol = obj$n_cat)

      if (length(valid) > 0) {
        clean_resp <- obs[valid]
        for (k in 0:(obj$n_cat - 1)) {
          is_k <- which(clean_resp == k)
          if (length(is_k) > 0) {
            subset_post <- posterior[valid[is_k], , drop=FALSE]
            r_qk[, k+1] <- colSums(subset_post)
          }
        }
      }

      # 1. Define Function to Minimize (Negative Log Likelihood)
      fn_neg_ll <- function(p) {
        t_a <- obj$a; t_b <- obj$b; t_c <- obj$c
        idx <- 1

        if (obj$model %in% c("2PL","3PL","GPCM","GRM")) { t_a <- p[idx]; idx <- idx+1 }
        if (obj$model == "3PL") { t_c <- p[idx]; idx <- idx+1 }
        n_b <- length(obj$b)
        t_b <- p[idx:(idx+n_b-1)]

        pr <- get_prob_matrix(nodes, t_a, t_b, t_c, obj$model)
        pr <- pmax(pr, 1e-10)
        return(-sum(r_qk * log(pr)))
      }

      # 2. Build Params and Bounds
      par_init <- c()
      lower_b <- c()
      upper_b <- c()

      if (obj$model %in% c("2PL","3PL","GPCM","GRM")) {
        par_init <- c(par_init, obj$a)
        lower_b <- c(lower_b, 0.01); upper_b <- c(upper_b, 8.0)
      }
      if (obj$model == "3PL") {
        par_init <- c(par_init, obj$c)
        lower_b <- c(lower_b, 0.0); upper_b <- c(upper_b, 0.45)
      }

      par_init <- c(par_init, obj$b)
      lower_b <- c(lower_b, rep(-8.0, length(obj$b)))
      upper_b <- c(upper_b, rep(8.0, length(obj$b)))

      # 3. Call Custom Solver
      res <- custom_solver(par_init, fn_neg_ll, lower_b, upper_b)

      # 4. Unpack Results
      p_new <- res$par
      ses <- get_se(res$hessian)
      idx <- 1

      if (obj$model %in% c("2PL","3PL","GPCM","GRM")) {
        obj$a <- p_new[idx]; obj$se_a <- ses[idx]; idx <- idx+1
      }
      if (obj$model == "3PL") {
        obj$c <- p_new[idx]; obj$se_c <- ses[idx]; idx <- idx+1
      }
      n_b <- length(obj$b)
      obj$b <- p_new[idx:(idx+n_b-1)]
      obj$se_b <- ses[idx:(idx+n_b-1)]

      item_list[[itm]] <- obj
    }

    diff <- abs(curr_ll - last_ll)
    if(con$verbose) message(sprintf("Cycle %d: LogLik = %.2f, Change = %.4f", cycle, curr_ll, diff))
    if (diff < con$conv_crit) converged <- TRUE
    last_ll <- curr_ll
  }

  # --- 5. Person Estimation (MLE) ---
  if(con$verbose) message("Estimating Person Parameters...")

  theta_est <- numeric(nrow(resp_mat))
  theta_se <- numeric(nrow(resp_mat))

  for (i in 1:nrow(resp_mat)) {
    obs_i <- resp_mat[i,]

    fn_person_neg_ll <- function(p_vec) {
      th <- p_vec[1]
      ll <- 0
      # Use item names directly to avoid index mismatches
      for (itm_name in items) {
        val <- obs_i[itm_name]
        if (!is.na(val)) {
          itm_obj <- item_list[[itm_name]]
          pr <- get_prob_matrix(th, itm_obj$a, itm_obj$b, itm_obj$c, itm_obj$model)
          # Ensure we don't exceed the number of columns in the prob matrix
          idx_val <- min(val + 1, ncol(pr))
          ll <- ll + log(pr[1, idx_val])
        }
      }
      return(-ll)
    }

    res <- custom_solver(c(0), fn_person_neg_ll, c(-6), c(6))
    theta_est[i] <- res$par[1]
    theta_se[i] <- get_se(res$hessian)[1]
  }

  # --- 6. Formatting ---
  out_rows <- list()
  for (itm in items) {
    obj <- item_list[[itm]]
    r <- list(
      item = itm, model = obj$model, status = ifelse(obj$type=="fixed","Fixed","Estimated"),
      discrimination = obj$a, discrimination_se = obj$se_a,
      guessing = if(obj$model=="3PL") obj$c else NA, guessing_se = obj$se_c
    )
    if (obj$model %in% c("Rasch","2PL","3PL")) {
      r$difficulty <- obj$b[1]; r$difficulty_se <- obj$se_b[1]
    } else {
      r$difficulty <- NA; r$difficulty_se <- NA
      for (k in seq_along(obj$b)) {
        r[[paste0("step_", k)]] <- obj$b[k]
        r[[paste0("step_", k, "_se")]] <- obj$se_b[k]
      }
    }
    out_rows[[itm]] <- r
  }

  all_cols <- unique(unlist(lapply(out_rows, names)))
  df_items <- data.frame(item = items)
  for (col in all_cols) if (col != "item") df_items[[col]] <- sapply(out_rows, function(x) if(col %in% names(x)) x[[col]] else NA)

  step_cols <- sort(grep("step_", names(df_items), value=TRUE))
  base_cols <- c("item", "model", "status", "discrimination", "discrimination_se",
                 "difficulty", "difficulty_se", "guessing", "guessing_se")
  df_items <- df_items[, c(base_cols, step_cols)[c(base_cols, step_cols) %in% names(df_items)]]

  df_fit <- data.frame(
    LogLik = last_ll,
    AIC = 2 * total_est_params - 2 * last_ll,
    BIC = total_est_params * log(nrow(resp_mat)) - 2 * last_ll,
    N_Est_Params = total_est_params, N_Students = nrow(resp_mat), Converged = converged
  )
  df_persons=data.frame(theta=theta_est,
                        theta_se=theta_se)
  message("---------------------------------------------------------")
  message("Process Complete. You can view results now.")

  return(list(
    item_params = df_items,
    person_params = df_persons,
    model_fit = df_fit
  ))
}

#' Mixed-Format English Language Arts (ELA) Assessment Data (Form 1)
#'
#' A dataset containing binary and polytomous responses for demonstration.
#'
#' @format A data frame with 52417 rows and 47 columns:
#' \itemize{
#'   \item \code{ITEM1} - \code{ITEM30}: Binary responses (0 = Incorrect, 1 = Correct).
#'   \item \code{ITEM31} - \code{ITEM45}: Polytomous responses (scored 0-5).
#'   \item \code{THETA}: Latent ability estimates.
#'   \item \code{COVARIATE}: Person-level background variable.
#' }
#' @source Tang, C., Xiong, J., & Engelhard, G. (2025). Identification of writing
#' strategies in educational assessments with an unsupervised learning
#' measurement framework. \emph{Education Sciences}, 15(7), 912.
#' \doi{10.3390/educsci15070912}
#' @examples
#' data(ela1)
#' head(ela1)
"ela1"

#' Mixed-Format English Language Arts (ELA) Assessment Data (Form 2)
#'
#' A smaller dataset containing item responses.
#'
#' @format A data frame with columns representing item responses.
#' \itemize{
#'   \item \code{ITEM1} - \code{ITEM7}: Binary responses (0 = Incorrect, 1 = Correct).
#'   \item \code{ITEM8}: Polytomous response (scored 0-2).
#'   \item \code{ITEM9}: Polytomous response (scored 0-5).
#'   \item \code{ITEM10}: Polytomous response (scored 0-5).
#' }
#' @source Tang, C., Xiong, J., & Engelhard, G. (2025). Identification of writing
#' strategies in educational assessments with an unsupervised learning
#' measurement framework. \emph{Education Sciences}, 15(7), 912.
#' \doi{10.3390/educsci15070912}
#' @examples
#' data(ela2)
#' head(ela2)
"ela2"

#' Large-Scale Mixed-Format English Language Arts (ELA) Assessment Data (Form 3)
#'
#' A long format dataset containing binary and polytomous responses for demonstration.
#'
#' @format A data frame with 2,434,185 observations and 5 variables:
#' \itemize{
#'   \item{STUDENTID}: Unique student identifier.
#'   \item{FORMID}: Unique test form identifier.
#'   \item{ITEMID}: Unique item identifier, where _T1 indicates trait 1 and _T2 indicates trait 2.
#'   \item{SEQ}: Position of the item in a form.
#'   \item{SCORE}: Dichotomously and polytomously scored item responses (0,1,2...).
#' }
#' @source Tang, C., Xiong, J., & Engelhard, G. (2025). Identification of writing
#' strategies in educational assessments with an unsupervised learning
#' measurement framework. \emph{Education Sciences}, 15(7), 912.
#' \doi{10.3390/educsci15070912}
#' @examples
#' data(ela3)
#' head(ela3)
"ela3"

#' Large-Scale Mixed-Format English Language Arts (ELA) Assessment Data Testmap (Form 3 Testmap)
#'
#' Test map for the long format ela3 response data
#'
#' @format A data frame with 328 rows and 5 variables:
#' \itemize{
#'   \item{FORMID}: Unique test form identifier.
#'   \item{SEQ}: Position of the item in a form.
#'   \item{ITEMID}: Unique item identifier, where _T1 indicates trait 1 and _T2 indicates trait 2.
#'   \item{TYPE}: Type of an item, where OP indicates operational items and FT indicates field-test items.
#'   \item{MAX_SCORE}: Max score of that item.
#' }
#' @source Tang, C., Xiong, J., & Engelhard, G. (2025). Identification of writing
#' strategies in educational assessments with an unsupervised learning
#' measurement framework. \emph{Education Sciences}, 15(7), 912.
#' \doi{10.3390/educsci15070912}
#' @examples
#' data(ela3_testmap)
#' head(ela3_testmap)
"ela3_testmap"

#' @import stats
#' @import utils
NULL

utils::globalVariables(c(
  "item", "model", "parameter", "response", "pvalue", "number",
  "Estimate", "Std_Error", "z_value", "Pr_z",
  "Estimate_slope", "Std_Error_slope", "z_value_slope", "Pr_z_slope",
  "Estimate_int", "parameter_int", "parameter_slope",
  "difficulty", "difficulty_se", "difficulty_zvalue", "difficulty_Pr_z",
  "discrimination", "discrimination_se", "discrimination_zvalue", "discrimination_Pr_z",
  "covariate", "covariate_se", "covariate_zvalue", "covariate_Pr_z",
  "infit", "outfit", "mean_theta", "mean_covariate",
  ".row_id", "theta", "step_1"
))

#' Local Dependence Statistics (Yen's Q3)
#'
#' @description
#' Computes Yen's (1984) \eqn{Q_3} statistic for every item pair. \eqn{Q_3} is
#' the correlation between the model residuals of two items after the latent
#' trait has been partialled out. Under the local-independence assumption of
#' unidimensional item response theory these residual correlations should be
#' small and negative (around \eqn{-1/(J-1)}); large positive values signal that
#' two items share something beyond the common trait, such as a passage or a
#' scenario. \eqn{Q_3} is therefore used both to decide whether testlet modeling
#' is needed and to check whether a testlet model has adequately absorbed the
#' residual dependence.
#'
#' @param data A data frame of item responses (rows = persons, columns = items).
#'   Dichotomous items are 0/1; polytomous items are consecutive integers
#'   starting at 0.
#' @param item_params A data frame of item parameters (the \code{item_params}
#'   element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
#'   \code{\link{mixed_irt}}). Item order must match the columns of \code{data}.
#' @param theta Optional numeric vector of estimated abilities (length equal to
#'   the number of rows in \code{data}), or a person-parameter data frame with
#'   an \code{ability}/\code{theta} column. If \code{NULL} (default), abilities
#'   are estimated internally by expected a posteriori (EAP) scoring from
#'   \code{item_params}.
#' @param theta_range Numeric vector of length 2 giving the ability grid bounds
#'   used for internal EAP scoring (default \code{c(-4, 4)}).
#' @param n_points Integer. Number of quadrature points for internal EAP scoring
#'   (default \code{41}).
#' @param model Optional model override (see \code{\link{item_info}}).
#' @param D Scaling constant for the dichotomous logistic metric (default
#'   \code{1}).
#'
#' @return A symmetric numeric matrix of \eqn{Q_3} statistics with one row and
#'   one column per item (diagonal set to \code{1}). The off-diagonal entries
#'   are the residual correlations for each item pair. The average off-diagonal
#'   \eqn{Q_3} is attached as the attribute \code{"mean_q3"} and the largest
#'   absolute value as \code{"max_abs_q3"}.
#'
#' @references
#' Yen, W. M. (1984). Effects of local item dependence on the fit and equating
#' performance of the three-parameter logistic model. \emph{Applied
#' Psychological Measurement, 8}(2), 125-145.
#'
#' @examples
#'   set.seed(123)
#'   sim <- sim_irt(n_people = 400,
#'                  item_structure = list(list(model = "2PL", n_items = 8)))
#'   fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
#'                     control = list(max_iter = 15, verbose = FALSE))
#'
#'   q3 <- ld_stats(sim$resp, fit$item_params)
#'   round(q3, 3)
#'   attr(q3, "max_abs_q3")
#' @export
ld_stats <- function(data,
                     item_params,
                     theta = NULL,
                     theta_range = c(-4, 4),
                     n_points = 41,
                     model = NULL,
                     D = 1) {

  if (!is.data.frame(data) && !is.matrix(data)) {
    stop("'data' must be a data frame or matrix of responses.")
  }
  resp <- as.matrix(data)
  N <- nrow(resp)
  J <- ncol(resp)

  parsed <- .tirt_parse_params(item_params, model = model, D = D)
  if (parsed$n_items != J) {
    stop(sprintf("Number of items in 'item_params' (%d) does not match the number of columns in 'data' (%d).",
                 parsed$n_items, J))
  }
  specs <- parsed$spec

  # --- 1. Abilities: user-supplied or estimated by EAP ---
  if (is.null(theta)) {
    nodes <- seq(theta_range[1], theta_range[2], length.out = n_points)
    wts <- dnorm(nodes); wts <- wts / sum(wts)

    # log-likelihood of each person across nodes
    logL <- matrix(0, N, n_points)
    for (j in seq_len(J)) {
      sp <- specs[[j]]
      if (is.na(sp$a) || (sp$model %in% c("RASCH", "2PL", "3PL") && is.na(sp$b))) next
      pr <- pmin(pmax(.tirt_cat_probs(sp, nodes), 1e-10), 1)   # nodes x n_cat
      x <- resp[, j]
      valid <- !is.na(x)
      if (!any(valid)) next
      lp <- log(pr)                                            # nodes x n_cat
      logL[valid, ] <- logL[valid, ] + t(lp[, x[valid] + 1L, drop = FALSE])
    }
    Fpost <- exp(logL - apply(logL, 1, max)) * matrix(wts, N, n_points, byrow = TRUE)
    Fpost <- Fpost / rowSums(Fpost)
    th <- as.vector(Fpost %*% nodes)
  } else {
    th <- .tirt_extract_theta(theta)
    if (length(th) != N) {
      stop(sprintf("Length of 'theta' (%d) must match the number of rows in 'data' (%d).",
                   length(th), N))
    }
  }

  # --- 2. Residuals: observed minus expected item score ---
  resid <- matrix(NA_real_, N, J)
  for (j in seq_len(J)) {
    sp <- specs[[j]]
    if (is.na(sp$a) || (sp$model %in% c("RASCH", "2PL", "3PL") && is.na(sp$b))) next
    expct <- .tirt_expected_score(sp, th)
    resid[, j] <- resp[, j] - expct
  }

  # --- 3. Pairwise correlations of residuals ---
  q3 <- suppressWarnings(cor(resid, use = "pairwise.complete.obs"))
  q3 <- as.matrix(q3)
  rownames(q3) <- colnames(q3) <- parsed$item
  diag(q3) <- 1

  off <- q3[upper.tri(q3)]
  off <- off[is.finite(off)]
  attr(q3, "mean_q3") <- if (length(off)) mean(off) else NA_real_
  attr(q3, "max_abs_q3") <- if (length(off)) max(abs(off)) else NA_real_
  q3
}

#' Simulate Mixed IRT and Testlet (TRT) Data
#'
#' @description
#' Simulates a data set that mixes ordinary independent item response theory
#' items with testlet (locally dependent) items in a single test form, the
#' simulation counterpart of \code{\link{irt_trt}}. Independent blocks depend
#' only on the primary trait (theta); testlet blocks additionally depend on a
#' testlet-specific nuisance effect (gamma). Both dichotomous and polytomous
#' formats are supported for the independent and the testlet parts, so the whole
#' family of models handled by \code{\link{irt_trt}} can be generated at once.
#'
#' @param n_people Integer. Number of examinees.
#' @param item_structure List of lists defining item blocks (see Details).
#' @param theta Numeric vector (Optional). If provided, these exact ability
#'   values are used.
#' @param theta_mean Numeric. Mean of the latent trait (used if \code{theta} is
#'   \code{NULL}).
#' @param theta_sd Numeric. SD of the latent trait (used if \code{theta} is
#'   \code{NULL}).
#'
#' @details
#' Each element of \code{item_structure} is a list describing one block. The
#' \code{model} name determines whether the block is independent or a testlet:
#' \itemize{
#'   \item Independent (theta only): \code{"Rasch"}, \code{"2PL"}, \code{"3PL"},
#'     \code{"PCM"}, \code{"GPCM"}, \code{"GRM"}.
#'   \item Testlet (theta + gamma): \code{"RaschT"}, \code{"2PLT"},
#'     \code{"3PLT"}, \code{"BiFT"}, \code{"PCT"}, \code{"GPCT"}, \code{"GRT"}.
#' }
#' Other recognized keys are \code{n_items}, \code{categories} (polytomous),
#' \code{a}, \code{b}, \code{c} (given as a fixed value or a \code{c(lo, hi)}
#' range to sample from), \code{s} (testlet loading for \code{"BiFT"}),
#' \code{testlet_id} (the testlet label, required for testlet blocks),
#' \code{testlet_var} (variance of the gamma effect), and \code{gamma_vector}
#' (a user-supplied gamma effect of length \code{n_people}).
#'
#' @return A list containing:
#'   \item{resp}{data.frame of responses (rows = people, cols = items).}
#'   \item{true_item_params}{data.frame of true item parameters, including a
#'     \code{model} column and a \code{testlet} column (\code{NA} for
#'     independent items). Its \code{item_id}, \code{model}, and \code{testlet}
#'     columns provide everything needed to build the \code{item_spec} argument
#'     of \code{\link{irt_trt}} (rename \code{item_id} to \code{item}).}
#'   \item{true_person_params}{data.frame of true person parameters: the primary
#'     ability plus one gamma column per testlet.}
#'
#' @seealso \code{\link{irt_trt}}, \code{\link{sim_trt}}, \code{\link{sim_irt}}
#'
#' @examples
#'   # A form with independent items and two testlets
#'   set.seed(2025)
#'   design <- list(
#'     list(model = "2PL",  n_items = 8),                       # independent
#'     list(model = "GRM",  n_items = 4, categories = 3),       # independent poly
#'     list(model = "2PLT", n_items = 4, testlet_id = "P1",
#'          testlet_var = 0.6),                                 # testlet
#'     list(model = "GPCT", n_items = 3, categories = 3,
#'          testlet_id = "P2", testlet_var = 0.5)               # testlet poly
#'   )
#'
#'   sim <- sim_tirt(n_people = 600, item_structure = design)
#'
#'   head(sim$resp)
#'   sim$true_item_params[, c("item_id", "model", "testlet")]
#'   head(sim$true_person_params)
#'
#'   \donttest{
#'   # Feed the true structure straight into the joint estimator
#'   spec <- data.frame(
#'     item = sim$true_item_params$item_id,
#'     model = sim$true_item_params$model,
#'     testlet = sim$true_item_params$testlet,
#'     stringsAsFactors = FALSE
#'   )
#'   fit <- irt_trt(sim$resp, spec, method = "EM",
#'                  control = list(max_iter = 15, verbose = FALSE))
#'   head(fit$item_params)
#'   }
#' @export
sim_tirt <- function(n_people = 1000,
                     item_structure = list(),
                     theta = NULL,
                     theta_mean = 0,
                     theta_sd = 1) {

  # --- 1. Validation ---
  if (!is.numeric(n_people) || n_people < 1) stop("Error: 'n_people' must be a positive integer.")
  if (length(item_structure) == 0) stop("Error: 'item_structure' cannot be empty.")

  message("================================================================")
  message(sprintf("   STARTING MIXED IRT/TRT SIMULATION (N = %d)", n_people))
  message("================================================================")

  # --- 2. Primary Trait (Theta) ---
  if (!is.null(theta)) {
    if (length(theta) != n_people) {
      stop(sprintf("Error: Length of provided 'theta' (%d) does not match 'n_people' (%d).",
                   length(theta), n_people))
    }
    true_theta <- theta
    message(">> Ability (Theta): Using user-supplied vector.")
  } else {
    true_theta <- rnorm(n_people, mean = theta_mean, sd = theta_sd)
    message(sprintf(">> Ability (Theta): Generated from N(mean=%.2f, sd=%.2f).", theta_mean, theta_sd))
  }

  independent_dich <- c("RASCH", "2PL", "3PL")
  independent_poly <- c("PCM", "GPCM", "GRM")
  testlet_dich <- c("RASCHT", "2PLT", "3PLT", "BIFT")
  testlet_poly <- c("PCT", "GPCT", "GPCMT", "GRT", "BIFT")

  is_testlet <- function(m) toupper(m) %in% c(testlet_dich, testlet_poly)

  # --- 3. Configure testlet gamma effects (only for testlet blocks) ---
  testlet_ids <- character(0)
  testlet_configs <- list()

  for (i in seq_along(item_structure)) {
    blk <- item_structure[[i]]
    if (is.null(blk$model)) stop(sprintf("Block %d Error: Missing 'model'.", i))
    if (!is_testlet(blk$model)) next

    if (is.null(blk$testlet_id)) {
      tid <- paste0("T", i)
      item_structure[[i]]$testlet_id <- tid
      message(sprintf("Notice: Testlet block %d missing 'testlet_id'. Assigned '%s'.", i, tid))
    } else {
      tid <- as.character(blk$testlet_id)
    }
    testlet_ids <- c(testlet_ids, tid)

    if (is.null(testlet_configs[[tid]])) {
      if (!is.null(blk$gamma_vector)) {
        if (length(blk$gamma_vector) != n_people) stop(sprintf("Error (Block %d): 'gamma_vector' length mismatch.", i))
        testlet_configs[[tid]] <- list(type = "vector", val = blk$gamma_vector)
      } else if (!is.null(blk$testlet_var)) {
        testlet_configs[[tid]] <- list(type = "variance", val = blk$testlet_var)
      } else {
        testlet_configs[[tid]] <- list(type = "default", val = 0.5)
      }
    }
  }

  unique_tids <- unique(testlet_ids)
  person_params <- data.frame(person_id = seq_len(n_people), ability = true_theta)

  if (length(unique_tids) > 0) {
    gamma_matrix <- matrix(0, n_people, length(unique_tids))
    colnames(gamma_matrix) <- paste0("testlet_", unique_tids)
    message(">> Testlet Effects (Gamma):")
    for (j in seq_along(unique_tids)) {
      tid <- unique_tids[j]
      config <- testlet_configs[[tid]]
      if (config$type == "vector") {
        message(sprintf("   - Testlet '%s': Using user-supplied Gamma vector.", tid))
        gamma_matrix[, j] <- config$val
      } else if (config$type == "variance") {
        message(sprintf("   - Testlet '%s': Generated Gamma ~ N(0, %.2f) [User Var].", tid, config$val))
        gamma_matrix[, j] <- rnorm(n_people, 0, sqrt(config$val))
      } else {
        message(sprintf("   - Testlet '%s': Generated Gamma ~ N(0, 0.50) [Default].", tid))
        gamma_matrix[, j] <- rnorm(n_people, 0, sqrt(0.5))
      }
    }
    person_params <- cbind(person_params, gamma_matrix)
  } else {
    message(">> No testlet blocks detected: all items are independent.")
  }

  # --- 4. Item simulation loop ---
  response_list <- list()
  param_list <- list()
  current_item_idx <- 1

  for (i in seq_along(item_structure)) {
    block <- item_structure[[i]]
    if (is.null(block$n_items)) stop(sprintf("Block %d Error: Missing 'n_items'.", i))

    model <- block$model
    n_items <- block$n_items
    testlet_block <- is_testlet(model)
    tid <- if (testlet_block) as.character(block$testlet_id) else NA_character_
    mU <- toupper(model)

    # categories
    if (mU %in% c(independent_dich, testlet_dich) && mU != "BIFT") {
      cats <- 2
    } else {
      cats <- if (is.null(block$categories)) {
        if (mU %in% c(independent_dich, testlet_dich)) 2 else 3
      } else block$categories
      if (cats < 2) stop(sprintf("Block %d Error: 'categories' must be >= 2.", i))
    }

    message(sprintf(">> Block %d: %d items (Model: %s, %s)",
                    i, n_items, model,
                    if (testlet_block) paste0("Testlet: ", tid) else "Independent"))

    get_param <- function(name, default, n_req, friendly) {
      val <- block[[name]]
      if (is.null(val)) return(rep(default, n_req))
      if (length(val) == n_req) return(val)
      if (length(val) == 2 && val[1] < val[2]) return(runif(n_req, val[1], val[2]))
      if (length(val) == 1) return(rep(val, n_req))
      stop(sprintf("Block %d Error: Parameter '%s' length %d but requires %d (or a range).",
                   i, name, length(val), n_req))
    }

    a <- if (mU %in% c("RASCH", "RASCHT", "PCM", "PCT")) rep(1, n_items) else get_param("a", 1, n_items, "Discrimination a")
    s_load <- if (mU == "BIFT") get_param("s", 1, n_items, "Testlet Loading s") else a
    b <- get_param("b", 0, n_items, "Difficulty/Loc b")
    c_p <- if (mU %in% c("3PL", "3PLT")) get_param("c", 0, n_items, "Guessing c") else rep(0, n_items)

    gamma_vec <- if (testlet_block) person_params[[paste0("testlet_", tid)]] else rep(0, n_people)

    block_params <- data.frame(
      item_id = paste0("item_", current_item_idx:(current_item_idx + n_items - 1)),
      model = model,
      testlet = tid,
      categories = cats,
      discrimination = a,
      testlet_loading = if (testlet_block) s_load else rep(NA_real_, n_items),
      difficulty = b,
      guessing = c_p,
      stringsAsFactors = FALSE
    )

    block_resp <- matrix(NA, n_people, n_items)

    if (cats == 2) {
      for (j in seq_len(n_items)) {
        z <- (a[j] * true_theta) + (s_load[j] * gamma_vec) - (a[j] * b[j])
        prob <- c_p[j] + (1 - c_p[j]) / (1 + exp(-z))
        block_resp[, j] <- ifelse(runif(n_people) < prob, 1, 0)
      }
    } else {
      is_graded <- mU %in% c("GRM", "GRT") || (mU == "BIFT" && cats > 2)
      for (j in seq_len(n_items)) {
        raw_steps <- sort(rnorm(cats - 1, 0, 0.5))
        thresholds <- b[j] + raw_steps
        for (k in seq_along(thresholds)) block_params[j, paste0("step_", k)] <- thresholds[k]

        if (is_graded) {
          prob_cum <- matrix(0, n_people, cats + 1)
          prob_cum[, 1] <- 1
          for (k in seq_len(cats - 1)) {
            z <- (a[j] * true_theta) + (s_load[j] * gamma_vec) - (a[j] * thresholds[k])
            prob_cum[, k + 1] <- 1 / (1 + exp(-z))
          }
          probs <- prob_cum[, seq_len(cats)] - prob_cum[, seq_len(cats) + 1]
        } else {
          numer <- matrix(0, n_people, cats)
          running <- 0
          for (k in seq_len(cats - 1)) {
            running <- running + ((a[j] * true_theta) + (s_load[j] * gamma_vec) - (a[j] * thresholds[k]))
            numer[, k + 1] <- running
          }
          numer <- numer - apply(numer, 1, max)
          probs <- exp(numer); probs <- probs / rowSums(probs)
        }
        cum_probs <- t(apply(probs, 1, cumsum))
        rand_vals <- runif(n_people)
        block_resp[, j] <- apply(cum_probs >= rand_vals, 1, function(x) match(TRUE, x)) - 1
      }
    }

    response_list[[i]] <- block_resp
    param_list[[i]] <- block_params
    current_item_idx <- current_item_idx + n_items
  }

  # --- 5. Final assembly ---
  message("================================================================")
  message("Constructing final data frames...")

  all_resp <- as.data.frame(do.call(cbind, response_list))
  all_cols <- unique(unlist(lapply(param_list, names)))
  all_item_params <- do.call(rbind, lapply(param_list, function(df) {
    missing_cols <- setdiff(all_cols, names(df))
    if (length(missing_cols) > 0) df[missing_cols] <- NA
    df[, all_cols]
  }))
  row.names(all_resp) <- NULL
  row.names(all_item_params) <- NULL
  row.names(person_params) <- NULL
  colnames(all_resp) <- all_item_params$item_id

  message("Simulation Complete.")
  message(sprintf("Summary: %d items (%d testlets), %d examinees.",
                  ncol(all_resp), length(unique_tids), n_people))
  message("================================================================")

  list(resp = all_resp,
       true_item_params = all_item_params,
       true_person_params = person_params)
}

#' Item Fit Statistics (Infit and Outfit)
#'
#' @description
#' Computes item-level fit statistics based on standardized response residuals:
#' the information-weighted mean square (infit) and the unweighted mean square
#' (outfit), together with their standardized (ZSTD) transformations. These
#' statistics flag items whose observed responses depart from the fitted model,
#' either because of noise in the tails of the ability distribution (outfit) or
#' near the item location (infit). The statistics are computed for dichotomous
#' and polytomous items alike.
#'
#' @param data A data frame of item responses (rows = persons, columns = items).
#'   Dichotomous items are 0/1; polytomous items are consecutive integers
#'   starting at 0. Item order must match \code{item_params}.
#' @param item_params A data frame of item parameters (the \code{item_params}
#'   element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
#'   \code{\link{mixed_irt}}).
#' @param theta Optional numeric vector of estimated abilities, or a person-
#'   parameter data frame with an \code{ability}/\code{theta} column. If
#'   \code{NULL} (default), abilities are estimated internally by EAP scoring.
#' @param theta_range Numeric vector of length 2 giving the ability grid bounds
#'   used for internal EAP scoring (default \code{c(-4, 4)}).
#' @param n_points Integer. Number of quadrature points for internal EAP scoring
#'   (default \code{41}).
#' @param model Optional model override (see \code{\link{item_info}}).
#' @param D Scaling constant for the dichotomous logistic metric (default
#'   \code{1}).
#'
#' @return A data frame with one row per item and the columns:
#' \itemize{
#'   \item \code{item}: item name.
#'   \item \code{n}: number of responses used.
#'   \item \code{outfit}: outfit mean square (unweighted).
#'   \item \code{outfit_z}: standardized outfit (ZSTD).
#'   \item \code{infit}: infit mean square (information-weighted).
#'   \item \code{infit_z}: standardized infit (ZSTD).
#' }
#' Mean-square values near 1 indicate good fit; values above about 1.3-1.5
#' suggest underfit (noise), and values below about 0.7 suggest overfit
#' (dependency or redundancy).
#'
#' @references
#' Wright, B. D., & Masters, G. N. (1982). \emph{Rating scale analysis}. MESA
#' Press.
#'
#' @examples
#'   set.seed(123)
#'   sim <- sim_irt(n_people = 400,
#'                  item_structure = list(list(model = "2PL", n_items = 8)))
#'   fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
#'                     control = list(max_iter = 15, verbose = FALSE))
#'
#'   item_fit(sim$resp, fit$item_params)
#' @export
item_fit <- function(data,
                     item_params,
                     theta = NULL,
                     theta_range = c(-4, 4),
                     n_points = 41,
                     model = NULL,
                     D = 1) {

  if (!is.data.frame(data) && !is.matrix(data)) stop("'data' must be a data frame or matrix.")
  resp <- as.matrix(data)
  N <- nrow(resp); J <- ncol(resp)

  parsed <- .tirt_parse_params(item_params, model = model, D = D)
  if (parsed$n_items != J) {
    stop(sprintf("Number of items in 'item_params' (%d) does not match the number of columns in 'data' (%d).",
                 parsed$n_items, J))
  }
  specs <- parsed$spec

  # --- abilities: supplied or EAP ---
  if (is.null(theta)) {
    nodes <- seq(theta_range[1], theta_range[2], length.out = n_points)
    wts <- dnorm(nodes); wts <- wts / sum(wts)
    logL <- matrix(0, N, n_points)
    for (j in seq_len(J)) {
      sp <- specs[[j]]
      if (is.na(sp$a) || (sp$model %in% c("RASCH", "2PL", "3PL") && is.na(sp$b))) next
      pr <- pmin(pmax(.tirt_cat_probs(sp, nodes), 1e-10), 1)
      x <- resp[, j]; valid <- !is.na(x)
      if (!any(valid)) next
      logL[valid, ] <- logL[valid, ] + t(log(pr)[, x[valid] + 1L, drop = FALSE])
    }
    Fpost <- exp(logL - apply(logL, 1, max)) * matrix(wts, N, n_points, byrow = TRUE)
    Fpost <- Fpost / rowSums(Fpost)
    th <- as.vector(Fpost %*% nodes)
  } else {
    th <- .tirt_extract_theta(theta)
    if (length(th) != N) stop("Length of 'theta' must match the number of rows in 'data'.")
  }

  out <- vector("list", J)
  for (j in seq_len(J)) {
    sp <- specs[[j]]
    x <- resp[, j]; valid <- !is.na(x)
    if (is.na(sp$a) || (sp$model %in% c("RASCH", "2PL", "3PL") && is.na(sp$b)) || !any(valid)) {
      out[[j]] <- data.frame(item = parsed$item[j], n = sum(valid),
                             outfit = NA, outfit_z = NA, infit = NA, infit_z = NA,
                             stringsAsFactors = FALSE)
      next
    }

    thj <- th[valid]; xj <- x[valid]
    probs <- .tirt_cat_probs(sp, thj)          # n x n_cat
    scores <- 0:(sp$n_cat - 1)
    E  <- as.vector(probs %*% scores)
    W  <- as.vector(probs %*% (scores^2)) - E^2      # model variance
    W  <- pmax(W, 1e-8)
    # fourth central moment (kurtosis term) C_i = sum_k (k - E)^4 P_k
    cen4 <- outer(E, scores, function(e, s) (s - e)^4)
    C  <- rowSums(probs * cen4)

    resid <- xj - E
    z2 <- resid^2 / W

    outfit <- mean(z2)
    infit  <- sum(resid^2) / sum(W)

    # Wilson-Hilferty standardization
    n <- length(xj)
    q_out2 <- sum(C / W^2) / n^2 - 1 / n
    q_out  <- sqrt(max(q_out2, 1e-8))
    q_in2  <- sum(C - W^2) / (sum(W)^2)
    q_in   <- sqrt(max(q_in2, 1e-8))

    outfit_z <- (outfit^(1/3) - 1) * (3 / q_out) + (q_out / 3)
    infit_z  <- (infit^(1/3)  - 1) * (3 / q_in)  + (q_in  / 3)

    out[[j]] <- data.frame(
      item = parsed$item[j], n = n,
      outfit = round(outfit, 3), outfit_z = round(outfit_z, 3),
      infit = round(infit, 3),  infit_z = round(infit_z, 3),
      stringsAsFactors = FALSE
    )
  }

  res <- do.call(rbind, out)
  row.names(res) <- NULL
  res
}

#' Simulate Item Response Theory Data
#'
#' @description
#' Simulate item responses data. Support both dichotomous and polytomous responses.
#' Provide an easy implementation with a few default settings.
#'
#' @param n_people Integer. Number of students.
#' @param item_structure List of lists defining item blocks.
#' @param theta Numeric vector (Optional). If provided, these exact ability values are used.
#' @param theta_mean Numeric. Mean of latent trait (used if theta is NULL).
#' @param theta_sd Numeric. SD of latent trait (used if theta is NULL).
#'
#' @return A list containing:
#'   \item{resp}{data.frame of responses (rows=people, cols=items)}
#'   \item{true_params}{data.frame of true item parameters}
#'   \item{theta}{vector of true latent traits}
#' @examples
#'   # 1. Define the Test Blueprint
#'   # We want:
#'   # - 10 items using 2PL (medium difficulty)
#'   # - 5 items using 3PL (difficult, with guessing)
#'   # - 5 items using GPCM (4-point Likert scale)
#'   # - 5 items using GRM (5-point Likert scale)
#'
#'   my_test_structure <- list(
#'     # Block 1: 2PL
#'     list(model = "2PL", n_items = 10, a = c(0.8, 1.2), b = c(-1, 1)),
#'
#'     # Block 2: 3PL (Harder items, b from 1 to 2.5, fixing guessing at 0.2)
#'     list(model = "3PL", n_items = 5, a = c(1.0, 1.5), b = c(1.0, 2.5), c = 0.2),
#'
#'     # Block 3: GPCM (Polytomous, 4 categories 0-3)
#'     list(model = "GPCM", n_items = 5, categories = 4, a = c(0.7, 1.3), b = c(-1, 1)),
#'
#'     # Block 4: GRM (Polytomous, 5 categories 0-4)
#'     list(model = "GRM", n_items = 5, categories = 5, a = c(1.0, 2.0))
#'   )
#'
#'   # 2. Run the Simulation
#'   # Define N and a specific Theta vector
#'   N <- 2000
#'   theta_vec <- rnorm(N, 0, 2)
#'
#'   sim_data <- sim_irt(
#'     n_people = N,
#'     theta = theta_vec,
#'     item_structure = my_test_structure
#'   )
#'
#'   # 3. Inspect the Output
#'   # The Response Matrix
#'   head(sim_data$resp)
#'
#'   # The True Parameters (Useful for recovery studies)
#'   # Note how it aligns a, b, and threshold parameters (step_1, step_2...)
#'   head(sim_data$true_params)
#' @export
sim_irt <- function(n_people = 1000,
                    item_structure = list(),
                    theta = NULL,
                    theta_mean = 0,
                    theta_sd = 1) {

  # --- INTERNAL HELPER: SANITIZE OUTPUTS ---
  # Strips hidden attributes to prevent RStudio Viewer crashes
  sanitize_output <- function(df_input) {
    data_list <- as.list(df_input)
    df_clean <- data.frame(data_list, stringsAsFactors = FALSE)
    row.names(df_clean) <- NULL
    return(df_clean)
  }

  # --- 1. Global Validation ---
  if (!is.numeric(n_people) || n_people < 1) {
    stop("Error: 'n_people' must be a positive integer (e.g., 500).")
  }

  message("----------------------------------------------------------------")
  message(sprintf("Starting Simulation for N = %d examinees...", n_people))

  # --- 2. Generate Theta ---
  if (!is.null(theta)) {
    if (length(theta) != n_people) {
      stop(sprintf("Error: Length of provided 'theta' vector (%d) does not match 'n_people' (%d).",
                   length(theta), n_people))
    }
    true_theta <- theta
    message(">> Ability (Theta): Using user-supplied vector.")
  } else {
    true_theta <- rnorm(n_people, mean = theta_mean, sd = theta_sd)
    message(sprintf(">> Ability (Theta): Generated from N(mean=%.2f, sd=%.2f).", theta_mean, theta_sd))
  }

  # Storage
  response_list <- list()
  param_list <- list()
  current_item_idx <- 1

  # Allowed Models Registry
  valid_dich <- c("Rasch", "2PL", "3PL")
  valid_poly <- c("PCM", "GPCM", "GRM")
  all_valid_models <- c(valid_dich, valid_poly)

  # --- 3. Iterate Blocks ---
  for (i in seq_along(item_structure)) {
    block <- item_structure[[i]]

    # Validation A: Missing Critical Keys
    if (is.null(block$model)) stop(sprintf("Block %d Error: You must specify 'model'.", i))
    if (is.null(block$n_items)) stop(sprintf("Block %d Error: You must specify 'n_items'.", i))

    model <- block$model
    n_items <- block$n_items

    # Validation B: Invalid Model Name
    if (!model %in% all_valid_models) {
      stop(sprintf(
        "Block %d Error: Invalid model '%s'.\nAllowed models: %s",
        i, model, paste(all_valid_models, collapse = ", ")
      ))
    }

    # Validation C: Invalid Item Count
    if (!is.numeric(n_items) || n_items < 1) {
      stop(sprintf("Block %d Error: 'n_items' must be a positive integer.", i))
    }

    message(sprintf(">> Block %d: %d items using %s", i, n_items, model))

    # --- Categories Logic ---
    if (model %in% valid_dich) {
      cats <- 2
      if (!is.null(block$categories) && block$categories != 2) {
        warning(sprintf("Block %d Warning: Model '%s' is dichotomous. Ignoring categories=%s.", i, model, block$categories))
      }
    } else {
      # Polytomous Models
      if (is.null(block$categories)) {
        cats <- 3
        message("   - Categories: Not specified. Defaulting to 3.")
      } else {
        cats <- block$categories
        if (!is.numeric(cats) || cats < 2) stop(sprintf("Block %d Error: Categories must be integer >= 2.", i))
        message(sprintf("   - Categories: User specified %d.", cats))
      }
    }

    # Init Block Storage
    block_resp <- matrix(NA, nrow = n_people, ncol = n_items)

    # Initialize Parameter Storage
    block_params <- data.frame(
      item_id = paste0("item_", current_item_idx:(current_item_idx + n_items - 1)),
      block = i,
      model = model,
      categories = cats,
      discrimination = NA,
      guessing = NA,
      stringsAsFactors = FALSE
    )

    # --- Helper: Smart Parameter Parser ---
    get_param <- function(param_name, default_val, n_req, friendly_name) {
      val <- block[[param_name]]

      # Case 1: Default
      if (is.null(val)) {
        message(sprintf("   - %s: Default values used (Fixed at %s).", friendly_name, default_val))
        return(rep(default_val, n_req))
      }

      # Case 2: Exact Vector Match (Highest Priority)
      # If provided vector length equals required items, use it exactly.
      if (length(val) == n_req) {
        if (n_req > 1) message(sprintf("   - %s: Using user-supplied vector (Exact match).", friendly_name))
        else message(sprintf("   - %s: User fixed value (%.2f).", friendly_name, val[1]))
        return(val)
      }

      # Case 3: Range (Uniform Distribution)
      # Only if length is 2 AND n_req is NOT 2 (because Case 2 handles n=2)
      if (length(val) == 2 && val[1] < val[2]) {
        message(sprintf("   - %s: Sampling from U[%.2f, %.2f].", friendly_name, val[1], val[2]))
        return(runif(n_req, val[1], val[2]))
      }

      # Case 4: Fixed Single Value
      if (length(val) == 1) {
        message(sprintf("   - %s: User fixed value (%.2f).", friendly_name, val))
        return(rep(val, n_req))
      }

      stop(sprintf("Block %d Error: Parameter '%s' length mismatch (expected %d or range).", i, param_name, n_req))
    }

    # --- Model Logic ---

    # A. Dichotomous
    if (model %in% valid_dich) {
      a <- if (model == "Rasch") rep(1, n_items) else get_param("a", 1, n_items, "Discrimination (a)")
      b <- get_param("b", 0, n_items, "Difficulty (b)")
      c_p <- if (model == "3PL") get_param("c", 0, n_items, "Guessing (c)") else rep(0, n_items)

      block_params$discrimination <- a
      block_params$difficulty <- b
      block_params$guessing <- c_p

      for (j in 1:n_items) {
        z <- a[j] * (true_theta - b[j])
        prob <- c_p[j] + (1 - c_p[j]) / (1 + exp(-z))
        block_resp[, j] <- ifelse(runif(n_people) < prob, 1, 0)
      }

      # B. Polytomous (PCM/GPCM)
    } else if (model %in% c("PCM", "GPCM")) {
      a <- if (model == "PCM") rep(1, n_items) else get_param("a", 1, n_items, "Discrimination (a)")
      loc <- get_param("b", 0, n_items, "Location (b)")

      block_params$discrimination <- a
      # Note: We do not store single difficulty here, we store steps below

      for (j in 1:n_items) {
        raw_steps <- sort(rnorm(cats - 1, 0, 0.5))
        thresholds <- loc[j] + raw_steps

        for (k in 1:length(thresholds)) {
          block_params[j, paste0("step_", k)] <- thresholds[k]
        }

        numerators <- matrix(0, nrow = n_people, ncol = cats)
        current_sum <- 0
        for (k in 1:(cats - 1)) {
          current_sum <- current_sum + (a[j] * (true_theta - thresholds[k]))
          numerators[, k + 1] <- current_sum
        }
        probs <- exp(numerators)
        probs <- probs / rowSums(probs)

        cum_probs <- t(apply(probs, 1, cumsum))
        rand_vals <- runif(n_people)
        block_resp[, j] <- apply(cum_probs >= rand_vals, 1, function(x) match(TRUE, x)) - 1
      }

      # C. GRM
    } else if (model == "GRM") {
      a <- get_param("a", 1, n_items, "Discrimination (a)")
      block_params$discrimination <- a
      message("   - Thresholds: Generated random ordered thresholds.")

      for (j in 1:n_items) {
        thresholds <- sort(runif(cats - 1, -2, 2))
        for (k in 1:length(thresholds)) {
          block_params[j, paste0("step_", k)] <- thresholds[k]
        }

        prob_cum <- matrix(0, nrow = n_people, ncol = cats + 1)
        prob_cum[, 1] <- 1
        for (k in 1:(cats - 1)) {
          prob_cum[, k + 1] <- 1 / (1 + exp(-(a[j] * (true_theta - thresholds[k]))))
        }
        probs <- prob_cum[, 1:cats] - prob_cum[, 2:(cats+1)]

        cum_probs_final <- t(apply(probs, 1, cumsum))
        rand_vals <- runif(n_people)
        block_resp[, j] <- apply(cum_probs_final >= rand_vals, 1, function(x) match(TRUE, x)) - 1
      }
    }

    response_list[[i]] <- block_resp
    param_list[[i]] <- block_params
    current_item_idx <- current_item_idx + n_items
  }

  # --- 4. Final Assembly (Sanitized) ---
  message("----------------------------------------------------------------")
  message("Constructing final data frames...")

  # A. Responses (Ensure safe conversion from matrix list)
  all_responses_mat <- do.call(cbind, response_list)
  all_responses <- as.data.frame(all_responses_mat, stringsAsFactors = FALSE)

  # B. Parameters (Handle missing columns)
  all_cols <- unique(unlist(lapply(param_list, names)))
  all_params <- do.call(rbind, lapply(param_list, function(df) {
    missing_cols <- setdiff(all_cols, names(df))
    if(length(missing_cols) > 0) df[missing_cols] <- NA
    return(df[, all_cols])
  }))

  # Align Names
  colnames(all_responses) <- all_params$item_id

  # C. Sanitize (Strip Attributes)
  final_resp <- sanitize_output(all_responses)
  final_params <- sanitize_output(all_params)

  message("Simulation Complete.")
  message(sprintf("Summary: %d items, %d examinees.", ncol(final_resp), n_people))
  message("----------------------------------------------------------------")

  return(list(resp = final_resp,
              true_params = final_params,
              theta = true_theta))
}

#' Test Reliability Indices
#'
#' @description
#' Computes commonly reported reliability coefficients for a test. Depending on
#' the information supplied it returns the empirical (marginal) reliability from
#' latent-trait estimates and their standard errors, the model-based marginal
#' reliability from the test information function, and the classical Cronbach's
#' alpha from the raw responses. Reliability summarizes how consistently the test
#' orders examinees and is a standard entry in every technical report.
#'
#' @param person_params Optional. A person-parameter data frame containing an
#'   \code{ability}/\code{theta} column and a standard-error column (such as
#'   \code{ability_se} or \code{se}); typically the \code{person_params} element
#'   returned by the estimation functions. Used for the empirical reliability.
#' @param data Optional. A data frame of item responses used to compute
#'   Cronbach's alpha (rows = persons, columns = items).
#' @param item_params Optional. A data frame of item parameters used to compute
#'   the model-based marginal reliability from the test information function.
#' @param prior_sd Numeric. Standard deviation of the population ability
#'   distribution used for the model-based marginal reliability (default
#'   \code{1}).
#' @param theta_range Numeric vector of length 2 giving the ability grid bounds
#'   for the marginal-reliability integration (default \code{c(-4, 4)}).
#' @param n_points Integer. Number of grid points for the marginal-reliability
#'   integration (default \code{61}).
#' @param model Optional model override (see \code{\link{item_info}}).
#' @param D Scaling constant for the dichotomous logistic metric (default
#'   \code{1}).
#'
#' @return A data frame with the columns \code{Index} and \code{Value},
#'   reporting whichever reliability coefficients could be computed from the
#'   supplied arguments:
#' \itemize{
#'   \item \code{empirical_reliability}: from person estimates and their SEs.
#'   \item \code{marginal_reliability}: from the test information function.
#'   \item \code{cronbach_alpha}: classical internal-consistency reliability.
#'   \item \code{n_items}, \code{n_persons}: sample descriptors when available.
#' }
#'
#' @details
#' The empirical reliability is
#' \eqn{\mathrm{Var}(\hat\theta) / (\mathrm{Var}(\hat\theta) + \overline{SE^2})}.
#' The model-based marginal reliability is
#' \eqn{\sigma^2 / (\sigma^2 + \overline{1/I(\theta)})}, where the average error
#' variance is taken over the population ability distribution. Cronbach's alpha
#' is \eqn{\frac{J}{J-1}\left(1 - \frac{\sum_j s_j^2}{s_T^2}\right)}.
#'
#' @examples
#'   set.seed(123)
#'   sim <- sim_irt(n_people = 500,
#'                  item_structure = list(list(model = "2PL", n_items = 12)))
#'   fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
#'                     control = list(max_iter = 15, verbose = FALSE))
#'
#'   reliability(person_params = fit$person_params,
#'               data = sim$resp,
#'               item_params = fit$item_params)
#' @export
reliability <- function(person_params = NULL,
                        data = NULL,
                        item_params = NULL,
                        prior_sd = 1,
                        theta_range = c(-4, 4),
                        n_points = 61,
                        model = NULL,
                        D = 1) {

  idx <- character(0)
  val <- numeric(0)
  add <- function(name, value) { idx[[length(idx) + 1]] <<- name; val[[length(val) + 1]] <<- value }

  # --- Empirical reliability from person estimates ---
  if (!is.null(person_params)) {
    th <- .tirt_extract_theta(person_params)
    se <- NULL
    if (is.data.frame(person_params)) {
      for (cand in c("ability_se", "se", "theta_se", "SE", "SE_Theta")) {
        if (cand %in% names(person_params)) { se <- as.numeric(person_params[[cand]]); break }
      }
    }
    if (!is.null(se)) {
      ok <- is.finite(th) & is.finite(se)
      if (sum(ok) > 1) {
        v_th <- var(th[ok])
        m_se2 <- mean(se[ok]^2)
        add("empirical_reliability", round(v_th / (v_th + m_se2), 4))
      }
    }
  }

  # --- Marginal reliability from test information ---
  if (!is.null(item_params)) {
    nodes <- seq(theta_range[1], theta_range[2], length.out = n_points)
    w <- dnorm(nodes, 0, prior_sd); w <- w / sum(w)
    tif <- test_info(item_params, theta = nodes, model = model, D = D)$test_info
    err_var <- sum(w * ifelse(tif > 0, 1 / tif, NA_real_), na.rm = TRUE)
    sig2 <- prior_sd^2
    add("marginal_reliability", round(sig2 / (sig2 + err_var), 4))
  }

  # --- Classical Cronbach's alpha ---
  if (!is.null(data)) {
    mat <- as.matrix(data)
    cc <- mat[stats::complete.cases(mat), , drop = FALSE]
    J <- ncol(cc)
    if (nrow(cc) > 1 && J > 1) {
      item_var <- apply(cc, 2, var)
      total_var <- var(rowSums(cc))
      if (total_var > 0) {
        alpha <- (J / (J - 1)) * (1 - sum(item_var) / total_var)
        add("cronbach_alpha", round(alpha, 4))
      }
      add("n_items", J)
      add("n_persons", nrow(cc))
    }
  }

  if (length(idx) == 0) {
    stop("Nothing to compute: supply at least one of 'person_params', 'item_params', or 'data'.")
  }

  data.frame(Index = unlist(idx), Value = unlist(val),
             row.names = NULL, stringsAsFactors = FALSE)
}

#' Person Fit Statistics (Standardized Log-Likelihood, lz)
#'
#' @description
#' Computes person-level fit statistics to detect examinees whose response
#' patterns are unlikely under the fitted item response model (for example,
#' careless responding, cheating, or unusual guessing). The function returns the
#' \eqn{l_z} standardized log-likelihood index of Drasgow, Levine, and Williams
#' (1985), which under model fit is approximately standard normal, and flags
#' persons whose \eqn{l_z} falls below a critical value.
#'
#' @param data A data frame of item responses (rows = persons, columns = items).
#'   Dichotomous items are 0/1; polytomous items are consecutive integers
#'   starting at 0. Missing values (\code{NA}) are allowed and are skipped
#'   person-by-person.
#' @param item_params A data frame of item parameters (the \code{item_params}
#'   element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
#'   \code{\link{mixed_irt}}). Item order must match the columns of \code{data}.
#' @param theta A numeric vector of estimated person abilities (length equal to
#'   the number of rows in \code{data}), or a person-parameter data frame
#'   containing an \code{ability} or \code{theta} column (such as the
#'   \code{person_params} element returned by the estimation functions).
#' @param critical Numeric. Persons with \eqn{l_z} below this value are flagged
#'   as potentially misfitting (default = \code{-1.96}, the lower 2.5\% point of
#'   the standard normal distribution; \code{-2} is another common choice).
#' @param model Optional model override (see \code{\link{item_info}}).
#' @param D Scaling constant for the dichotomous logistic metric (default
#'   \code{1}).
#'
#' @return A data frame with one row per person and the columns:
#' \itemize{
#'   \item \code{person}: person index (row number).
#'   \item \code{n_items}: number of items answered.
#'   \item \code{theta}: the ability estimate used.
#'   \item \code{loglik}: the observed log-likelihood of the response pattern.
#'   \item \code{lz}: the standardized log-likelihood person-fit index.
#'   \item \code{flag}: logical; \code{TRUE} if \code{lz < critical}.
#' }
#'
#' @details
#' For a person with ability \eqn{\theta}, let \eqn{l_0} be the log-likelihood of
#' the observed responses. The index is
#' \eqn{l_z = (l_0 - E[l_0]) / \sqrt{\mathrm{Var}(l_0)}}, where the expectation
#' and variance are taken over the model-implied category probabilities at
#' \eqn{\theta}. Large negative values indicate response patterns that are less
#' likely than the model predicts.
#'
#' @references
#' Drasgow, F., Levine, M. V., & Williams, E. A. (1985). Appropriateness
#' measurement with polychotomous item response models and standardized indices.
#' \emph{British Journal of Mathematical and Statistical Psychology, 38}(1),
#' 67-86.
#'
#' @examples
#'   set.seed(123)
#'   sim <- sim_irt(n_people = 300,
#'                  item_structure = list(list(model = "2PL", n_items = 15)))
#'   fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
#'                     control = list(max_iter = 15, verbose = FALSE))
#'
#'   pf <- person_fit(sim$resp, fit$item_params, fit$person_params)
#'   head(pf)
#'
#'   # How many examinees are flagged as misfitting?
#'   sum(pf$flag, na.rm = TRUE)
#' @export
person_fit <- function(data,
                       item_params,
                       theta,
                       critical = -1.96,
                       model = NULL,
                       D = 1) {

  if (!is.data.frame(data) && !is.matrix(data)) {
    stop("'data' must be a data frame or matrix of responses.")
  }
  resp <- as.matrix(data)
  N <- nrow(resp)
  J <- ncol(resp)

  parsed <- .tirt_parse_params(item_params, model = model, D = D)
  if (parsed$n_items != J) {
    stop(sprintf("Number of items in 'item_params' (%d) does not match the number of columns in 'data' (%d).",
                 parsed$n_items, J))
  }

  th <- .tirt_extract_theta(theta)
  if (length(th) != N) {
    stop(sprintf("Length of 'theta' (%d) must match the number of rows in 'data' (%d).",
                 length(th), N))
  }

  # Evaluate item-by-item, reusing the per-item probability engine.
  # Probabilities are computed at the vector of person thetas once per item.
  obs_ll  <- numeric(N)
  exp_ll  <- numeric(N)
  var_ll  <- numeric(N)
  counts  <- integer(N)

  for (j in seq_len(J)) {
    sp <- parsed$spec[[j]]
    if (is.na(sp$a) || (sp$model %in% c("RASCH", "2PL", "3PL") && is.na(sp$b))) next

    probs <- .tirt_cat_probs(sp, th)          # N x n_cat
    probs <- pmin(pmax(probs, 1e-10), 1)
    logp  <- log(probs)

    x <- resp[, j]
    valid <- !is.na(x)
    if (!any(valid)) next

    # observed log-likelihood contribution
    idx <- cbind(which(valid), x[valid] + 1L)
    obs_ll[valid] <- obs_ll[valid] + logp[idx]

    # expectation and variance of the log-likelihood over categories
    e_term <- rowSums(probs * logp)                 # E[log P]
    v_term <- rowSums(probs * logp^2) - e_term^2    # Var[log P]
    exp_ll[valid] <- exp_ll[valid] + e_term[valid]
    var_ll[valid] <- var_ll[valid] + v_term[valid]
    counts[valid] <- counts[valid] + 1L
  }

  lz <- ifelse(var_ll > 0 & counts > 0,
               (obs_ll - exp_ll) / sqrt(var_ll),
               NA_real_)
  obs_ll[counts == 0] <- NA_real_

  out <- data.frame(
    person  = seq_len(N),
    n_items = counts,
    theta   = round(th, 3),
    loglik  = round(obs_ll, 3),
    lz      = round(lz, 3),
    flag    = lz < critical,
    row.names = NULL,
    stringsAsFactors = FALSE
  )
  out
}

---
title: "Introduction to tirt: Testlet Item Response Theory"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to tirt: Testlet Item Response Theory}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
**1. Overview**

The **tirt** package provides a light-version user-friendly comprehensive toolkit for **Item Response Theory (IRT)** and **Testlet Response Theory (TRT)** analysis. It supports:

1. **Data Simulation**: Generating data for simple IRT and Testlet models.

2. **Estimation**: Marginal Maximum Likelihood (MML) and Joint Maximum Likelihood (JML) estimation.

3. **Fixed Calibration**: Estimating item parameters with fixed persons, or person parameters with fixed items.

4. **Equating**: Linking scales using Mean-Mean, Mean-Sigma, and Stocking-Lord methods.

This vignette walks through the primary workflows.
```{r setup}
library(tirt)
```
**2. Data Simulation**

Before estimating models, let's generate some synthetic data using sim_irt (standard IRT) and sim_trt (testlet structure).

**2.1 Standard IRT Simulation**

We generate responses for **500 examinees** on **15 items** using the *2-Parameter Logistic (2PL)* model.
```{r sim_irt}
set.seed(123)
# Define item structure: 1 block of 15 items using 2PL
sim_data <- sim_irt(
  n_people = 500, 
  item_structure = list(
    list(model = "2PL", n_items = 15)
  ),
  theta_mean = 0,
  theta_sd = 1
)

# Access responses and true parameters
head(sim_data$resp)
head(sim_data$true_params)
```

**2.2 Testlet Simulation**

For **Testlet Response Theory**, we define blocks that share a common local dependency (Gamma).

```{r sim_trt}
# Define 2 testlets with 5 items each
trt_data <- sim_trt(
  n_people = 500,
  item_structure = list(
    list(model = "2PLT", n_items = 5, testlet_id = "T1"),
    list(model = "2PLT", n_items = 5, testlet_id = "T2")
  )
)

head(trt_data$resp)
```

**3. IRT Estimation**

We can estimate parameters using **binary_irt** (for dichotomous data), **polytomous_irt** (for graded data), or **mixed_irt** (for both).

Here, we estimate the parameters for the data we just simulated.
```{r binary_irt}
# Estimate 2PL model
# Note: max_iter set low for vignette speed; use default (100) in practice
fit_2pl <- binary_irt(
  data = sim_data$resp, 
  model = "2PL", 
  method = "EM",
  control = list(max_iter = 20, verbose = FALSE) 
)

# View estimated item parameters
print(head(fit_2pl$item_params))

# View model fit indices
print(fit_2pl$model_fit)
```

**4. Testlet (TRT) Estimation**

When items are grouped into testlets (e.g., reading passages), standard IRT assumptions are violated. tirt allows you to model this dependency using **Bi-factor** or **Testlet** models. This package allows estimations for binary and polytomous testlet models

We use **trt_binary** for this example. We must define the group structure.
```{r trt_binary}
# Items 1-5 are Testlet 1, Items 6-10 are Testlet 2
testlet_structure <- list(
  c(1, 2, 3, 4, 5),   # Indices for Testlet 1
  c(6, 7, 8, 9, 10)   # Indices for Testlet 2
)

fit_trt <- trt_binary(
  data = trt_data$resp,
  group = testlet_structure,
  model = "2PLT",
  method = "EM",
  control = list(max_iter = 15, verbose = FALSE)
)

# The output includes item parameters
head(fit_trt$item_params)

# And person parameters including the testlet specific effects (Gamma)
head(fit_trt$person_params)
```

**5. Universal Estimation (Mixed standaline & testlet items)**

Real-world exams often contain a mix of independent items (i.e., using standard IRT) and item bundles/testlets (i.e., using TRT). The **irt_trt** function is a universal estimator that handles both simultaneously. It can also handle mixed data types (binary and polytomous) in the same run.

To use **irt_trt**, you must provide an *item_spec* data frame that maps each item to its model and testlet ID (if any).

**5.1. Step 1: Simulate Mixed Structure Data**

We will generate data for 1000 examinees with:

1. **10 Independent Binary Items** (2PL Model)

2. **5 Independent Polytomous Items** (GRM Model)

3. **2 Testlet Items** belonging to Testlet "T1" (2PLT Model)

4. **3 Testlet Items** belonging to Testlet "T2" (GPCMT Model)
```{r sim}
cat("\n\n--- Generating Simulated Data ---\n")
set.seed(2025)

N <- 1000
# 20 Items Total:
# 1-10: Binary (2PL) - Independent
# 11-15: Polytomous (GRM, 3 cats) - Independent
# 16-17: Binary (2PLT) - Testlet 1
# 18-20: Polytomous (GPCT, 3 cats) - Testlet 2

theta <- rnorm(N, 0, 1)
gamma_1 <- rnorm(N, 0, 0.5) # Testlet 1 effect
gamma_2 <- rnorm(N, 0, 0.6) # Testlet 2 effect

resp_matrix <- matrix(NA, N, 20)
item_names <- paste0("Item_", 1:20)
colnames(resp_matrix) <- item_names

# Define Item Parameters (True Values)
a_true <- runif(20, 0.8, 1.5)
b_true <- seq(-1.5, 1.5, length.out = 20)

# 1. Simulate Binary Independent (1-10)
for(j in 1:10) {
  p <- 1 / (1 + exp(-a_true[j] * (theta - b_true[j])))
  resp_matrix[,j] <- rbinom(N, 1, p)
}

# 2. Simulate Poly Independent (11-15) - GR
for(j in 11:15) {
  b_k <- sort(c(b_true[j] - 0.7, b_true[j] + 0.7))
  p1 <- 1 / (1 + exp(-a_true[j] * (theta - b_k[1])))
  p2 <- 1 / (1 + exp(-a_true[j] * (theta - b_k[2])))
  # GRM Probabilities
  prob_0 <- 1 - p1
  prob_1 <- p1 - p2
  prob_2 <- p2
  # Sample
  r <- runif(N)
  resp_matrix[,j] <- ifelse(r < prob_0, 0, ifelse(r < prob_0 + prob_1, 1, 2))
}

# 3. Simulate Binary Testlet 1 (16-17)
for(j in 16:17) {
  eff_theta <- theta + gamma_1
  p <- 1 / (1 + exp(-a_true[j] * (eff_theta - b_true[j])))
  resp_matrix[,j] <- rbinom(N, 1, p)
}

# 4. Simulate Poly Testlet 2 (18-20) - GPCMT
for(j in 18:20) {
  eff_theta <- theta + gamma_2
  # GPCM steps (centered at b_true)
  b_vec <- c(b_true[j] - 0.5, b_true[j] + 0.5) 
  
  numer <- matrix(0, N, 3)
  numer[,1] <- 0
  numer[,2] <- a_true[j] * (eff_theta - b_vec[1])
  numer[,3] <- a_true[j] * (eff_theta - b_vec[1]) + a_true[j] * (eff_theta - b_vec[2])
  
  max_n <- apply(numer, 1, max)
  exps <- exp(numer - max_n)
  probs <- exps / rowSums(exps)
  
  r <- runif(N)
  resp_matrix[,j] <- apply(cbind(r, probs), 1, function(x) {
    if(x[1] < x[2]) return(0)
    if(x[1] < x[2]+x[3]) return(1)
    return(2)
  })
}

sim_data <- as.data.frame(resp_matrix)
#view the data
print(head(sim_data))
```

**5.2. Step 2: Create the Item Specification**

The item_spec data frame tells the function how to treat each item.

1. **item**: Must match column names in data.

2. **model**: The model to use (e.g., "2PL", "2PLT", "GPCMT", "RaschT", etc.).

3. **testlet**: The ID of the testlet. Use NA for independent items.
```{r}
# Create Item Specification
spec <- data.frame(
  item = item_names,
  model = c(rep("2PL", 10), rep("GRM", 5), rep("2PLT", 2), rep("GPCMT", 3)),
  testlet = c(rep(NA, 15), rep("testlet1", 2), rep("testlet2", 3)),
  stringsAsFactors = FALSE
)
#view the spec
print(spec)
```

**5.3. Step 3: Run Estimation**

We run the estimation. The function automatically detects that "T1" items share a nuisance dimension (Gamma) while the others depend only on Theta.
```{r}
# Run the Universal Function
results <- irt_trt(sim_data, spec, method = "EM", control = list(max_iter=50, verbose=TRUE))

# Save Results
item_par=results$item_params
fit=results$model_fit
person_par=results$person_params
```

**6. Fixed Parameter Calibration**

In equating scenarios or computer-adaptive testing, we often need to estimate one set of parameters while holding the other fixed.

**6.1 Fixed Item Calibration (Estimating Person Ability)**

If we know the item parameters (e.g., from an item bank), we can estimate person ability. Below we first simulate items, including some items have "known parameters" and some items are "new items".
```{r simulation}
# --- 1. Simulation Helper Functions ---

# Function to simulate Binary 2PL
sim_2pl <- function(theta, a, b) {
  D <- 1.7
  prob <- 1 / (1 + exp(-D * a * (theta - b)))
  resp <- ifelse(runif(length(theta)) < prob, 1, 0)
  return(resp)
}

# Function to simulate Polytomous (GPCM structure to match fixed_item logic)
sim_poly <- function(theta, a, steps) {
  n_cat <- length(steps) + 1
  n_stud <- length(theta)
  
  # Calculate numerators for categories 0, 1, 2...
  # P(k) proportional to exp( sum(a*(theta - step_j)) )
  
  probs <- matrix(0, nrow = n_stud, ncol = n_cat)
  probs[, 1] <- 1 # Reference category (unnormalized exp is 1 or e^0)
  
  current_sum <- 0
  for (k in 2:n_cat) {
    # Step parameters correspond to boundaries
    current_sum <- current_sum + a * (theta - steps[k-1])
    probs[, k] <- exp(current_sum)
  }
  
  # Normalize
  prob_sum <- rowSums(probs)
  probs_final <- probs / prob_sum
  
  # Random draw
  resp <- numeric(n_stud)
  for (i in 1:n_stud) {
    resp[i] <- sample(0:(n_cat-1), 1, prob = probs_final[i, ])
  }
  return(resp)
}

# --- 2. Generate Parameters for 50 Items ---

n_students <- 5497
true_theta <- rnorm(n_students, mean = 0.4, sd = 1.7)

all_items_meta <- list()
response_matrix <- matrix(NA, nrow = n_students, ncol = 50)
colnames(response_matrix) <- paste0("Item_", 1:50)

# --- Set 1: Known Dichotomous (Items 1-10) ---
# Model: 2PL
for (i in 1:10) {
  a <- runif(1, 0.8, 1.4)
  b <- rnorm(1, 0, 1)
  resp <- sim_2pl(true_theta, a, b)
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Known", model = "2PL", a = a, b = b)
}

# --- Set 2: Known Polytomous (Items 11-18) ---
# Model: Poly (3 categories: 0, 1, 2). Requires 2 steps (d1, d2).
for (i in 11:18) {
  a <- runif(1, 0.7, 1.2)
  d1 <- rnorm(1, -0.5, 0.3)
  d2 <- rnorm(1, 0.5, 0.3) # Steps usually ordered
  resp <- sim_poly(true_theta, a, c(d1, d2))
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Known", model = "Poly", a = a, d1 = d1, d2 = d2)
}

# --- Set 3: Unknown Dichotomous (Items 19-40) ---
# 22 Items
for (i in 19:40) {
  a <- runif(1, 0.8, 1.4)
  b <- rnorm(1, 0, 1)
  resp <- sim_2pl(true_theta, a, b)
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Unknown", model = "2PL", a = a, b = b)
}

# --- Set 4: Unknown Polytomous (Items 41-50) ---
# 10 Items (Varying categories: some 3 cat, some 4 cat)
for (i in 41:50) {
  a <- runif(1, 0.7, 1.2)
  # Mix of 3 categories (2 steps) and 4 categories (3 steps)
  if (i %% 2 == 0) {
    steps <- c(-0.8, 0, 0.8) # 4 cats
  } else {
    steps <- c(-0.5, 0.5)    # 3 cats
  }
  resp <- sim_poly(true_theta, a, steps)
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Unknown", model = "Poly", a = a, steps = steps)
}
#check
response_df <- as.data.frame(response_matrix)
head(response_df[, c(1:3, 11:13, 41:43)]) # Peak at data
```

Then we create a necessary data structure for the next calibration step. In this step, we need to specify known item with corresponding parameters.
```{r}
# Create empty dataframe structure
known_params_df <- data.frame(
  item = character(),
  model = character(),
  a = numeric(),
  b = numeric(),
  d1 = numeric(),
  d2 = numeric(),
  stringsAsFactors = FALSE
)

# Populate with Known Items (1-18)
for (i in 1:18) {
  meta <- all_items_meta[[i]]
  
  if (meta$model == "2PL") {
    # Add row for Binary
    known_params_df <- rbind(known_params_df, data.frame(
      item = meta$item,
      model = "2PL",
      a = meta$a,
      b = meta$b,
      d1 = NA, d2 = NA
    ))
  } else {
    # Add row for Poly
    # Note: Using "GPCM" model string as our function uses GPCM math
    known_params_df <- rbind(known_params_df, data.frame(
      item = meta$item,
      model = "GPCM", 
      a = meta$a,
      b = NA,
      d1 = meta$d1,
      d2 = meta$d2
    ))
  }
}

# Display input to verify
print("Known Item Parameters Input:")
print(known_params_df)

```

Finally, we can conduct a fixed-item calibration and check the final results.
```{r}
# Run the calibration
# We set model_default = "2PL" (this applies to unknown binary items).
# The function automatically detects poly items in the unknown set based on response categories.
results <- fixed_item(
  response_df = response_df, 
  item_params_df = known_params_df,
  control = list(max_iter = 100)
)

# Check that Items 1-18 are "Fixed" and match inputs.
# Check that Items 19-50 are "Estimated".
# Note how Poly items have step_1, step_2, etc.
item=results$item_params
person=results$person_params
fit=results$model_fit
```


**6.2 Fixed Person Calibration (Estimating Item Difficulty)**

Conversely, if we have known person abilities, we can calibrate new items. In this estimation, you can also choose to model a covariate if you believe that is releated to your latent variables. If not specified, then there will be no covariate in the modeling process.

The final output also contains some classic item statistics such as p-values and mean of item scores.
```{r fix_person}
# --- Example: With Package Data ---
data("ela1", package = "tirt")

# Select Item Responses (Cols 1-30)
df_real <- ela1[, 1:30]
fixed_theta <- ela1$THETA
fixed_cov <- ela1$COVARIATE
real_res <- fix_person(df = df_real,
                            theta = fixed_theta,
                            model = "2PL",
                            covariate = fixed_cov)
head(real_res)
```

**7. Equating / Linking**

**equate_irt** links two scales (Form X and Form Y) using common anchor items.
```{r equate_irt}
# Create dummy parameters for Form X (Base)
base_params <- data.frame(
  item = c("Item1", "Item2", "Item3", "Item4", "Item5"),
  model = "2PL",
  a = c(1.0, 1.2, 0.9, 1.1, 1.0),
  b = c(-1.0, -0.5, 0.0, 0.5, 1.0),
  stringsAsFactors = FALSE
)

# Create dummy parameters for Form Y (New)
# Suppose Form Y is harder and has different discrimination scaling
new_params <- data.frame(
  item = c("Item1", "Item2", "Item3", "Item6", "Item7"), # Items 1-3 are anchors
  model = "2PL",
  a = c(1.0, 1.2, 0.9, 1.1, 1.0) / 0.9,  # Scale shift
  b = (c(-1.0, -0.5, 0.0, 0.8, 1.2) * 0.9) + 0.2, # Location shift
  stringsAsFactors = FALSE
)

# Perform Equating
linked <- equate_irt(
  base_params = base_params,
  new_params = new_params,
  methods = c("Stocking-Lord", "Mean-Mean")
)

# View Linking Constants (A and B)
print(linked$linking_constants)

# View Transformed Parameters for Form Y (put on Form X scale)
print(linked$transformed_item_params)
```

**8. Final Comment**

The **tirt** package streamlines complex psychometric analyses. For detailed documentation on specific functions, please use the help command, e.g., *?binary_irt*.

---
title: "Psychometric Tools in tirt: Information, Scoring, Fit, DIF, and Mixtures"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Psychometric Tools in tirt: Information, Scoring, Fit, DIF, and Mixtures}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

**1. Overview**

Once item and person parameters have been estimated, a psychometric study usually
continues with a battery of follow-up analyses: evaluating measurement precision,
building score-conversion tables, checking person and item fit, screening for
local dependence and differential item functioning, summarizing reliability, and
sometimes probing for latent subpopulations. This vignette walks through the
post-estimation tools in **tirt**. Every function accepts the item-parameter data
frame produced by `binary_irt()`, `polytomous_irt()`, or `mixed_irt()`, so the
workflow is uniform across models.

```{r setup}
library(tirt)
```

We start from a small simulated 2PL data set and a quick calibration.

```{r calibrate}
set.seed(2025)
sim <- sim_irt(
  n_people = 600,
  item_structure = list(list(model = "2PL", n_items = 12))
)

fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                  control = list(max_iter = 20, verbose = FALSE))
head(fit$item_params)
```

**2. Item and Test Information**

`item_info()` returns the Fisher information of each item across a grid of
ability values, and `test_info()` sums it into the test information function
together with the conditional standard error of measurement (SEM).

```{r information}
theta_grid <- seq(-3, 3, by = 0.5)

# Item information (items in rows, theta in columns)
info <- item_info(fit$item_params, theta = theta_grid)
round(info[1:3, ], 3)

# Test information function and conditional SEM
tif <- test_info(fit$item_params, theta = theta_grid)
tif

# Where does the test measure most precisely?
tif$theta[which.max(tif$test_info)]
```

**3. Summed-Score to Theta Conversion Table**

Operational programs report scale scores from raw (summed) scores. `score_table()`
produces the conversion using the Lord-Wingersky recursion, supporting expected a
posteriori (`"EAP"`), weighted likelihood (`"WLE"`), and maximum likelihood
(`"MLE"`) scoring.

```{r score_table}
# EAP conversion table (0 to 12 correct)
score_table(fit$item_params, method = "EAP")

# Maximum-likelihood conversion
score_table(fit$item_params, method = "MLE")
```

**4. Person Fit**

`person_fit()` computes the `lz` standardized log-likelihood index to identify
examinees whose response patterns are unlikely under the model.

```{r person_fit}
pf <- person_fit(sim$resp, fit$item_params, fit$person_params)
head(pf)

# Number of examinees flagged as potentially misfitting
sum(pf$flag, na.rm = TRUE)
```

**5. Item Fit**

`item_fit()` returns the infit and outfit mean-square statistics and their
standardized versions, for dichotomous and polytomous items alike.

```{r item_fit}
item_fit(sim$resp, fit$item_params)
```

**6. Local Dependence (Yen's Q3)**

`ld_stats()` computes Yen's Q3 residual correlations for every item pair. Large
positive values signal that a pair depends on something beyond the common trait,
which is the situation a testlet model is designed to absorb.

```{r ld_stats}
q3 <- ld_stats(sim$resp, fit$item_params)
round(q3[1:5, 1:5], 3)

# Largest absolute residual correlation
attr(q3, "max_abs_q3")
```

**7. Differential Item Functioning**

`dif()` screens dichotomous items for DIF using the Mantel-Haenszel procedure
(with the ETS delta effect size and A/B/C flags) and logistic regression (which
separates uniform and non-uniform DIF). Here we plant DIF in item 3.

```{r dif}
resp_dif <- sim$resp
grp <- rep(c("Reference", "Focal"), each = 300)
flip <- grp == "Focal" & resp_dif[[3]] == 1
resp_dif[[3]][flip] <- rbinom(sum(flip), 1, 0.55)

dif(resp_dif, group = grp)[, c("item", "MH_delta", "ETS_class", "LR_p")]
```

**8. Reliability**

`reliability()` reports the empirical (marginal) reliability from person estimates
and their standard errors, the model-based marginal reliability from the test
information function, and classical Cronbach's alpha from the raw responses.

```{r reliability}
reliability(person_params = fit$person_params,
            data = sim$resp,
            item_params = fit$item_params)
```

**9. Test Characteristic Curve**

`tcc()` returns the expected score of each item and the test characteristic curve,
which maps ability onto the number-correct metric.

```{r tcc}
curves <- tcc(fit$item_params, theta = seq(-3, 3, by = 1))
curves$test_curve
```

**10. Multidimensional and Mixed Simulation**

Two new simulators extend the data-generation toolkit. `sim_mirt()` produces
compensatory multidimensional data (aligned with `mirt_binary()`), and
`sim_tirt()` produces forms that mix independent items with testlets (aligned with
`irt_trt()`).

```{r sim_mirt}
# Two correlated dimensions, simple structure
mdat <- sim_mirt(
  n_people = 400,
  dimension = 2,
  Sigma = matrix(c(1, 0.4, 0.4, 1), 2, 2),
  item_structure = list(
    list(model = "M2PL", n_items = 6, dims = 1),
    list(model = "M2PL", n_items = 6, dims = 2)
  )
)
head(mdat$true_params)
```

```{r sim_tirt}
# Independent items plus two testlets
tdat <- sim_tirt(
  n_people = 400,
  item_structure = list(
    list(model = "2PL",  n_items = 6),
    list(model = "2PLT", n_items = 4, testlet_id = "P1", testlet_var = 0.6),
    list(model = "GPCT", n_items = 3, categories = 3, testlet_id = "P2")
  )
)
tdat$true_item_params[, c("item_id", "model", "testlet")]
```

**11. Mixture (Latent-Class) IRT**

`mixture_irt()` fits a mixture Rasch or 2PL model, in which the population is a
blend of latent classes with class-specific item parameters.

```{r mixture}
set.seed(11)
N <- 300; J <- 8
b1 <- seq(-1.5, 1.5, length.out = J); b2 <- rev(b1)
theta <- rnorm(N); cls <- rep(1:2, each = N / 2)
rmat <- matrix(0, N, J)
for (i in 1:N) {
  b <- if (cls[i] == 1) b1 else b2
  rmat[i, ] <- rbinom(J, 1, 1 / (1 + exp(-(theta[i] - b))))
}
mdf <- as.data.frame(rmat); names(mdf) <- paste0("I", 1:J)

mix <- mixture_irt(mdf, n_class = 2, model = "Rasch",
                   control = list(max_iter = 40, verbose = FALSE))
mix$class_params
mix$model_fit
```

**12. Final Comment**

Together with the estimation, calibration, and equating functions described in the
companion vignette, these tools cover the routine post-estimation workflow of an
operational testing program. For details on any function, use the help system,
for example `?score_table`.

---
title: "Introduction to tirt: Testlet Item Response Theory"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to tirt: Testlet Item Response Theory}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
**1. Overview**

The **tirt** package provides a light-version user-friendly comprehensive toolkit for **Item Response Theory (IRT)** and **Testlet Response Theory (TRT)** analysis. It supports:

1. **Data Simulation**: Generating data for simple IRT and Testlet models.

2. **Estimation**: Marginal Maximum Likelihood (MML) and Joint Maximum Likelihood (JML) estimation.

3. **Fixed Calibration**: Estimating item parameters with fixed persons, or person parameters with fixed items.

4. **Equating**: Linking scales using Mean-Mean, Mean-Sigma, and Stocking-Lord methods.

5. **Post-estimation tools**: Item and test information, summed-score conversion tables, person- and item-fit statistics, local dependence (Yen's Q3), differential item functioning, reliability, test characteristic curves, and mixture models. These are covered in the companion vignette, `vignette("psychometric_tools", package = "tirt")`.

This vignette walks through the primary workflows.
```{r setup}
library(tirt)
```
**2. Data Simulation**

Before estimating models, let's generate some synthetic data using sim_irt (standard IRT) and sim_trt (testlet structure).

**2.1 Standard IRT Simulation**

We generate responses for **500 examinees** on **15 items** using the *2-Parameter Logistic (2PL)* model.
```{r sim_irt}
set.seed(123)
# Define item structure: 1 block of 15 items using 2PL
sim_data <- sim_irt(
  n_people = 500, 
  item_structure = list(
    list(model = "2PL", n_items = 15)
  ),
  theta_mean = 0,
  theta_sd = 1
)

# Access responses and true parameters
head(sim_data$resp)
head(sim_data$true_params)
```

**2.2 Testlet Simulation**

For **Testlet Response Theory**, we define blocks that share a common local dependency (Gamma).

```{r sim_trt}
# Define 2 testlets with 5 items each
trt_data <- sim_trt(
  n_people = 500,
  item_structure = list(
    list(model = "2PLT", n_items = 5, testlet_id = "T1"),
    list(model = "2PLT", n_items = 5, testlet_id = "T2")
  )
)

head(trt_data$resp)
```

**3. IRT Estimation**

We can estimate parameters using **binary_irt** (for dichotomous data), **polytomous_irt** (for graded data), or **mixed_irt** (for both).

Here, we estimate the parameters for the data we just simulated.
```{r binary_irt}
# Estimate 2PL model
# Note: max_iter set low for vignette speed; use default (100) in practice
fit_2pl <- binary_irt(
  data = sim_data$resp, 
  model = "2PL", 
  method = "EM",
  control = list(max_iter = 20, verbose = FALSE) 
)

# View estimated item parameters
print(head(fit_2pl$item_params))

# View model fit indices
print(fit_2pl$model_fit)
```

**4. Testlet (TRT) Estimation**

When items are grouped into testlets (e.g., reading passages), standard IRT assumptions are violated. tirt allows you to model this dependency using **Bi-factor** or **Testlet** models. This package allows estimations for binary and polytomous testlet models

We use **trt_binary** for this example. We must define the group structure.
```{r trt_binary}
# Items 1-5 are Testlet 1, Items 6-10 are Testlet 2
testlet_structure <- list(
  c(1, 2, 3, 4, 5),   # Indices for Testlet 1
  c(6, 7, 8, 9, 10)   # Indices for Testlet 2
)

fit_trt <- trt_binary(
  data = trt_data$resp,
  group = testlet_structure,
  model = "2PLT",
  method = "EM",
  control = list(max_iter = 15, verbose = FALSE)
)

# The output includes item parameters
head(fit_trt$item_params)

# And person parameters including the testlet specific effects (Gamma)
head(fit_trt$person_params)
```

**5. Universal Estimation (Mixed standaline & testlet items)**

Real-world exams often contain a mix of independent items (i.e., using standard IRT) and item bundles/testlets (i.e., using TRT). The **irt_trt** function is a universal estimator that handles both simultaneously. It can also handle mixed data types (binary and polytomous) in the same run.

To use **irt_trt**, you must provide an *item_spec* data frame that maps each item to its model and testlet ID (if any).

**5.1. Step 1: Simulate Mixed Structure Data**

We will generate data for 1000 examinees with:

1. **10 Independent Binary Items** (2PL Model)

2. **5 Independent Polytomous Items** (GRM Model)

3. **2 Testlet Items** belonging to Testlet "T1" (2PLT Model)

4. **3 Testlet Items** belonging to Testlet "T2" (GPCMT Model)
```{r sim}
cat("\n\n--- Generating Simulated Data ---\n")
set.seed(2025)

N <- 1000
# 20 Items Total:
# 1-10: Binary (2PL) - Independent
# 11-15: Polytomous (GRM, 3 cats) - Independent
# 16-17: Binary (2PLT) - Testlet 1
# 18-20: Polytomous (GPCT, 3 cats) - Testlet 2

theta <- rnorm(N, 0, 1)
gamma_1 <- rnorm(N, 0, 0.5) # Testlet 1 effect
gamma_2 <- rnorm(N, 0, 0.6) # Testlet 2 effect

resp_matrix <- matrix(NA, N, 20)
item_names <- paste0("Item_", 1:20)
colnames(resp_matrix) <- item_names

# Define Item Parameters (True Values)
a_true <- runif(20, 0.8, 1.5)
b_true <- seq(-1.5, 1.5, length.out = 20)

# 1. Simulate Binary Independent (1-10)
for(j in 1:10) {
  p <- 1 / (1 + exp(-a_true[j] * (theta - b_true[j])))
  resp_matrix[,j] <- rbinom(N, 1, p)
}

# 2. Simulate Poly Independent (11-15) - GR
for(j in 11:15) {
  b_k <- sort(c(b_true[j] - 0.7, b_true[j] + 0.7))
  p1 <- 1 / (1 + exp(-a_true[j] * (theta - b_k[1])))
  p2 <- 1 / (1 + exp(-a_true[j] * (theta - b_k[2])))
  # GRM Probabilities
  prob_0 <- 1 - p1
  prob_1 <- p1 - p2
  prob_2 <- p2
  # Sample
  r <- runif(N)
  resp_matrix[,j] <- ifelse(r < prob_0, 0, ifelse(r < prob_0 + prob_1, 1, 2))
}

# 3. Simulate Binary Testlet 1 (16-17)
for(j in 16:17) {
  eff_theta <- theta + gamma_1
  p <- 1 / (1 + exp(-a_true[j] * (eff_theta - b_true[j])))
  resp_matrix[,j] <- rbinom(N, 1, p)
}

# 4. Simulate Poly Testlet 2 (18-20) - GPCMT
for(j in 18:20) {
  eff_theta <- theta + gamma_2
  # GPCM steps (centered at b_true)
  b_vec <- c(b_true[j] - 0.5, b_true[j] + 0.5) 
  
  numer <- matrix(0, N, 3)
  numer[,1] <- 0
  numer[,2] <- a_true[j] * (eff_theta - b_vec[1])
  numer[,3] <- a_true[j] * (eff_theta - b_vec[1]) + a_true[j] * (eff_theta - b_vec[2])
  
  max_n <- apply(numer, 1, max)
  exps <- exp(numer - max_n)
  probs <- exps / rowSums(exps)
  
  r <- runif(N)
  resp_matrix[,j] <- apply(cbind(r, probs), 1, function(x) {
    if(x[1] < x[2]) return(0)
    if(x[1] < x[2]+x[3]) return(1)
    return(2)
  })
}

sim_data <- as.data.frame(resp_matrix)
#view the data
print(head(sim_data))
```

**5.2. Step 2: Create the Item Specification**

The item_spec data frame tells the function how to treat each item.

1. **item**: Must match column names in data.

2. **model**: The model to use (e.g., "2PL", "2PLT", "GPCMT", "RaschT", etc.).

3. **testlet**: The ID of the testlet. Use NA for independent items.
```{r}
# Create Item Specification
spec <- data.frame(
  item = item_names,
  model = c(rep("2PL", 10), rep("GRM", 5), rep("2PLT", 2), rep("GPCMT", 3)),
  testlet = c(rep(NA, 15), rep("testlet1", 2), rep("testlet2", 3)),
  stringsAsFactors = FALSE
)
#view the spec
print(spec)
```

**5.3. Step 3: Run Estimation**

We run the estimation. The function automatically detects that "T1" items share a nuisance dimension (Gamma) while the others depend only on Theta.
```{r}
# Run the Universal Function
results <- irt_trt(sim_data, spec, method = "EM", control = list(max_iter=50, verbose=TRUE))

# Save Results
item_par=results$item_params
fit=results$model_fit
person_par=results$person_params
```

**6. Fixed Parameter Calibration**

In equating scenarios or computer-adaptive testing, we often need to estimate one set of parameters while holding the other fixed.

**6.1 Fixed Item Calibration (Estimating Person Ability)**

If we know the item parameters (e.g., from an item bank), we can estimate person ability. Below we first simulate items, including some items have "known parameters" and some items are "new items".
```{r simulation}
# --- 1. Simulation Helper Functions ---

# Function to simulate Binary 2PL
sim_2pl <- function(theta, a, b) {
  D <- 1.7
  prob <- 1 / (1 + exp(-D * a * (theta - b)))
  resp <- ifelse(runif(length(theta)) < prob, 1, 0)
  return(resp)
}

# Function to simulate Polytomous (GPCM structure to match fixed_item logic)
sim_poly <- function(theta, a, steps) {
  n_cat <- length(steps) + 1
  n_stud <- length(theta)
  
  # Calculate numerators for categories 0, 1, 2...
  # P(k) proportional to exp( sum(a*(theta - step_j)) )
  
  probs <- matrix(0, nrow = n_stud, ncol = n_cat)
  probs[, 1] <- 1 # Reference category (unnormalized exp is 1 or e^0)
  
  current_sum <- 0
  for (k in 2:n_cat) {
    # Step parameters correspond to boundaries
    current_sum <- current_sum + a * (theta - steps[k-1])
    probs[, k] <- exp(current_sum)
  }
  
  # Normalize
  prob_sum <- rowSums(probs)
  probs_final <- probs / prob_sum
  
  # Random draw
  resp <- numeric(n_stud)
  for (i in 1:n_stud) {
    resp[i] <- sample(0:(n_cat-1), 1, prob = probs_final[i, ])
  }
  return(resp)
}

# --- 2. Generate Parameters for 50 Items ---

n_students <- 5497
true_theta <- rnorm(n_students, mean = 0.4, sd = 1.7)

all_items_meta <- list()
response_matrix <- matrix(NA, nrow = n_students, ncol = 50)
colnames(response_matrix) <- paste0("Item_", 1:50)

# --- Set 1: Known Dichotomous (Items 1-10) ---
# Model: 2PL
for (i in 1:10) {
  a <- runif(1, 0.8, 1.4)
  b <- rnorm(1, 0, 1)
  resp <- sim_2pl(true_theta, a, b)
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Known", model = "2PL", a = a, b = b)
}

# --- Set 2: Known Polytomous (Items 11-18) ---
# Model: Poly (3 categories: 0, 1, 2). Requires 2 steps (d1, d2).
for (i in 11:18) {
  a <- runif(1, 0.7, 1.2)
  d1 <- rnorm(1, -0.5, 0.3)
  d2 <- rnorm(1, 0.5, 0.3) # Steps usually ordered
  resp <- sim_poly(true_theta, a, c(d1, d2))
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Known", model = "Poly", a = a, d1 = d1, d2 = d2)
}

# --- Set 3: Unknown Dichotomous (Items 19-40) ---
# 22 Items
for (i in 19:40) {
  a <- runif(1, 0.8, 1.4)
  b <- rnorm(1, 0, 1)
  resp <- sim_2pl(true_theta, a, b)
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Unknown", model = "2PL", a = a, b = b)
}

# --- Set 4: Unknown Polytomous (Items 41-50) ---
# 10 Items (Varying categories: some 3 cat, some 4 cat)
for (i in 41:50) {
  a <- runif(1, 0.7, 1.2)
  # Mix of 3 categories (2 steps) and 4 categories (3 steps)
  if (i %% 2 == 0) {
    steps <- c(-0.8, 0, 0.8) # 4 cats
  } else {
    steps <- c(-0.5, 0.5)    # 3 cats
  }
  resp <- sim_poly(true_theta, a, steps)
  response_matrix[, i] <- resp
  all_items_meta[[i]] <- list(item = colnames(response_matrix)[i], type = "Unknown", model = "Poly", a = a, steps = steps)
}
#check
response_df <- as.data.frame(response_matrix)
head(response_df[, c(1:3, 11:13, 41:43)]) # Peak at data
```

Then we create a necessary data structure for the next calibration step. In this step, we need to specify known item with corresponding parameters.
```{r}
# Create empty dataframe structure
known_params_df <- data.frame(
  item = character(),
  model = character(),
  a = numeric(),
  b = numeric(),
  d1 = numeric(),
  d2 = numeric(),
  stringsAsFactors = FALSE
)

# Populate with Known Items (1-18)
for (i in 1:18) {
  meta <- all_items_meta[[i]]
  
  if (meta$model == "2PL") {
    # Add row for Binary
    known_params_df <- rbind(known_params_df, data.frame(
      item = meta$item,
      model = "2PL",
      a = meta$a,
      b = meta$b,
      d1 = NA, d2 = NA
    ))
  } else {
    # Add row for Poly
    # Note: Using "GPCM" model string as our function uses GPCM math
    known_params_df <- rbind(known_params_df, data.frame(
      item = meta$item,
      model = "GPCM", 
      a = meta$a,
      b = NA,
      d1 = meta$d1,
      d2 = meta$d2
    ))
  }
}

# Display input to verify
print("Known Item Parameters Input:")
print(known_params_df)

```

Finally, we can conduct a fixed-item calibration and check the final results.
```{r}
# Run the calibration
# We set model_default = "2PL" (this applies to unknown binary items).
# The function automatically detects poly items in the unknown set based on response categories.
results <- fixed_item(
  response_df = response_df, 
  item_params_df = known_params_df,
  control = list(max_iter = 100)
)

# Check that Items 1-18 are "Fixed" and match inputs.
# Check that Items 19-50 are "Estimated".
# Note how Poly items have step_1, step_2, etc.
item=results$item_params
person=results$person_params
fit=results$model_fit
```


**6.2 Fixed Person Calibration (Estimating Item Difficulty)**

Conversely, if we have known person abilities, we can calibrate new items. In this estimation, you can also choose to model a covariate if you believe that is releated to your latent variables. If not specified, then there will be no covariate in the modeling process.

The final output also contains some classic item statistics such as p-values and mean of item scores.
```{r fix_person}
# --- Example: With Package Data ---
data("ela1", package = "tirt")

# Select Item Responses (Cols 1-30)
df_real <- ela1[, 1:30]
fixed_theta <- ela1$THETA
fixed_cov <- ela1$COVARIATE
real_res <- fix_person(df = df_real,
                            theta = fixed_theta,
                            model = "2PL",
                            covariate = fixed_cov)
head(real_res)
```

**7. Equating / Linking**

**equate_irt** links two scales (Form X and Form Y) using common anchor items.
```{r equate_irt}
# Create dummy parameters for Form X (Base)
base_params <- data.frame(
  item = c("Item1", "Item2", "Item3", "Item4", "Item5"),
  model = "2PL",
  a = c(1.0, 1.2, 0.9, 1.1, 1.0),
  b = c(-1.0, -0.5, 0.0, 0.5, 1.0),
  stringsAsFactors = FALSE
)

# Create dummy parameters for Form Y (New)
# Suppose Form Y is harder and has different discrimination scaling
new_params <- data.frame(
  item = c("Item1", "Item2", "Item3", "Item6", "Item7"), # Items 1-3 are anchors
  model = "2PL",
  a = c(1.0, 1.2, 0.9, 1.1, 1.0) / 0.9,  # Scale shift
  b = (c(-1.0, -0.5, 0.0, 0.8, 1.2) * 0.9) + 0.2, # Location shift
  stringsAsFactors = FALSE
)

# Perform Equating
linked <- equate_irt(
  base_params = base_params,
  new_params = new_params,
  methods = c("Stocking-Lord", "Mean-Mean")
)

# View Linking Constants (A and B)
print(linked$linking_constants)

# View Transformed Parameters for Form Y (put on Form X scale)
print(linked$transformed_item_params)
```

**8. Final Comment**

The **tirt** package streamlines complex psychometric analyses. For detailed documentation on specific functions, please use the help command, e.g., *?binary_irt*.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_mirt.R
\name{sim_mirt}
\alias{sim_mirt}
\title{Simulate Multidimensional Item Response Theory Data}
\usage{
sim_mirt(
  n_people = 1000,
  item_structure = list(),
  dimension = 2,
  theta = NULL,
  theta_mean = 0,
  Sigma = NULL
)
}
\arguments{
\item{n_people}{Integer. Number of examinees.}

\item{item_structure}{List of lists defining item blocks (see Details).}

\item{dimension}{Integer. Number of latent dimensions (D).}

\item{theta}{Numeric matrix (Optional). An \code{n_people} x \code{dimension}
matrix of abilities. If supplied it is used directly.}

\item{theta_mean}{Numeric. Mean of the latent traits, recycled across
dimensions (used when \code{theta} is \code{NULL}).}

\item{Sigma}{Numeric matrix (Optional). A \code{dimension} x \code{dimension}
covariance matrix for the latent traits (used when \code{theta} is
\code{NULL}). Defaults to the identity matrix (uncorrelated dimensions).}
}
\value{
A list containing:
\item{resp}{data.frame of responses (rows = people, cols = items).}
\item{true_params}{data.frame of true item parameters, including one
\code{a_Dim} column per dimension, the intercept \code{d}, guessing, and
step/threshold columns for polytomous items.}
\item{theta}{the \code{n_people} x \code{dimension} matrix of true abilities.}
}
\description{
Simulates responses from compensatory multidimensional item response models,
the multidimensional counterpart of \code{\link{sim_irt}}. It supports the
multidimensional binary models estimated by \code{\link{mirt_binary}}
(M-Rasch, M-2PL, M-3PL) as well as multidimensional polytomous models
(multidimensional GPCM and GRM). Each block of items may load on any subset of
the latent dimensions, so both simple-structure and complex-structure designs
are easy to generate.
}
\details{
Each element of \code{item_structure} is a list describing one block:
\itemize{
\item \code{model}: one of \code{"M2PL"}, \code{"M3PL"}, \code{"MRasch"}
(dichotomous) or \code{"MGPCM"}, \code{"MGRM"} (polytomous). The
non-prefixed names (\code{"2PL"}, \code{"GRM"}, ...) are also accepted.
\item \code{n_items}: number of items in the block.
\item \code{dims}: integer vector of the dimensions the block loads on
(default: all dimensions). Slopes on the remaining dimensions are 0.
\item \code{a}: discrimination/slope, given as a range \code{c(lo, hi)} to
sample from, or a single fixed value (default \code{c(0.8, 1.8)}).
\item \code{d}: intercept, given as a range or a fixed value (default
\code{c(-1.5, 1.5)}). Note the compensatory model uses the intercept
metric \eqn{z = a'\theta + d}, matching \code{mirt_binary()}.
\item \code{c}: lower asymptote for \code{"M3PL"} (default \code{0.2}).
\item \code{categories}: number of categories for polytomous blocks
(default \code{3}).
}
}
\examples{
  # Two correlated dimensions, simple structure
  set.seed(2025)
  Sigma <- matrix(c(1, 0.4, 0.4, 1), 2, 2)

  design <- list(
    list(model = "M2PL", n_items = 8, dims = 1),          # loads on Dim 1
    list(model = "M2PL", n_items = 8, dims = 2),          # loads on Dim 2
    list(model = "M3PL", n_items = 4, dims = c(1, 2), c = 0.2)  # both dims
  )

  sim <- sim_mirt(n_people = 500, item_structure = design,
                  dimension = 2, Sigma = Sigma)

  head(sim$resp)
  head(sim$true_params)

  \donttest{
  # Recover with the multidimensional estimator
  Q <- as.matrix(sim$true_params[, c("a_Dim1", "a_Dim2")] != 0) * 1
  fit <- mirt_binary(sim$resp, model = "2PL", dimension = 2,
                     control = list(Q_matrix = Q, max_iter = 10, verbose = FALSE))
  head(fit$item_params)
  }
}
\seealso{
\code{\link{mirt_binary}}, \code{\link{sim_irt}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mbinary.R
\name{mirt_binary}
\alias{mirt_binary}
\title{Multidimensional Binary Item Response Theory Estimation}
\usage{
mirt_binary(
  data,
  model = "2PL",
  dimension = 2,
  method = "MML",
  control = list()
)
}
\arguments{
\item{data}{A N x J data.frame or matrix of dichotomous responses (0/1).}

\item{model}{String. "Rasch", "2PL", or "3PL".}

\item{dimension}{Integer. Number of latent dimensions to estimate (D).}

\item{method}{String. Estimation method: "MML" (Marginal Maximum Likelihood with EM, best for D <= 3),
"MHRM" (Metropolis-Hastings Robbins-Monro, for high D), or "RVEM" (Dimension-Reduction EM).}

\item{control}{A \code{list} of control parameters for the estimation algorithm:
\itemize{
\item \code{max_iter}: Maximum number of iterations (default = 100).
\item \code{converge_tol}: Convergence criterion (default = 1e-4).
\item \code{theta_range}: Bounds for quadrature integration (default = c(-4, 4)).
\item \code{quad_points}: Quadrature points PER DIMENSION for MML (default = 15).
\item \code{ability}: Logical; estimate person abilities (EAP) after item calibration? (default = TRUE).
\item \code{verbose}: Logical; prints progress and psychometric messages.
\item \code{Q_matrix}: Optional J x D matrix of 0s and 1s indicating item-to-dimension loading.
If NULL, an exploratory lower-triangular constraint is applied for identifiability.
}}
}
\value{
A list containing:
\itemize{
\item \code{item_params}: A data frame of estimated item parameters (a_1...a_D, d, g) and standard errors.
\item \code{person_params}: A data frame of estimated multidimensional person abilities (if ability=TRUE).
\item \code{model_fit}: Fit statistics (Log-Likelihood, AIC, BIC).
\item \code{settings}: Control parameters used in the run.
}
}
\description{
Estimates item and person parameters for multidimensional binary item response models
(M-Rasch, M-2PL, M-3PL). The function is fully self-contained, using custom Newton-Raphson
optimization without relying on external optimizers.
}
\examples{

  # --- Simulation for Multidimensional Data ---
  set.seed(202)
  N <- 800
  J <- 20
  D <- 2

  # Simulate Abilities (2 Dimensions, correlated)
  Sigma <- matrix(c(1, 0.4, 0.4, 1), 2, 2)
  Z <- matrix(rnorm(N * D), N, D)
  theta <- Z \%*\% chol(Sigma)

  # Define Q-matrix (Items 1-10 on Dim1, Items 11-20 on Dim 2)
  Q <- matrix(0, J, D)
  Q[1:10, 1] <- 1
  Q[11:20, 2] <- 1

  # Simulate Item Parameters (Multidimensional 2PL)
  a_true <- matrix(runif(J * D, 0.8, 1.8), J, D) * Q
  d_true <- seq(-2, 2, length.out = J)

  # Generate Responses
  data_mat <- matrix(NA, N, J)
  for(i in 1:N) {
    # Matrix multiply JxD slopes by Dx1 person abilities, add Jx1 intercepts
    z <- as.vector(a_true \%*\% theta[i, ]) + d_true
    p <- 1 / (1 + exp(-z))
    data_mat[i, ] <- rbinom(J, 1, p)
  }
  df <- as.data.frame(data_mat)
  names(df) <- paste0("Item", 1:J)

  # --- Run Custom Multidimensional Function ---
  res <- mirt_binary(df, model="2PL", dimension=2, method="MML",
                     control=list(Q_matrix=Q, ability=TRUE, max_iter=10))

  print(head(res$item_params))
  print(head(res$person_params))
  print(res$model_fit)
  \donttest{
  # --- Validation with 'mirt' (For comparison, if installed) ---
  # library(mirt)
  # mirt_model <- paste0("F1 = 1-10\nF2 = 11-20\nCOV = F1*F2")
  # mirt_res <- mirt(df, mirt.model(mirt_model), itemtype='2PL', method='EM')
  # coef(mirt_res, IRTpars=FALSE, simplify=TRUE)$items
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fixed_person.R
\name{fix_person}
\alias{fix_person}
\title{Fixed Person Calibration with or without Covariate}
\usage{
fix_person(df, theta, model = c("Rasch", "2PL"), covariate = NULL)
}
\arguments{
\item{df}{A data frame of item responses (0/1). Columns represent items, rows represent persons.}

\item{theta}{A numeric vector of person abilities (fixed parameters). Must match the number of rows in \code{df}.}

\item{model}{A character string specifying the model type. Options are "Rasch" or "2PL".}

\item{covariate}{An optional numeric vector representing a person-level covariate (e.g., time, group).
Defaults to \code{NULL}.}
}
\value{
A data frame containing:
\itemize{
\item Item statistics (difficulty, standard errors, z-values, p-values).
\item Discrimination parameters (for 2PL model).
\item Global covariate effect (if \code{covariate} is provided).
\item Classical item statistics (p-value, count, point-biserial correlation).
\item Mean theta per item (average ability of persons answering the item).
\item Infit and Outfit statistics (for Rasch model only).
}
}
\description{
Estimates item parameters (difficulty, discrimination) given fixed person parameters (theta),
with an optional person-level covariate. Supports Rasch and 2-Parameter Logistic models.
}
\examples{
  # --- Example: With Selected Package Data ---
  data("ela1", package = "tirt")

  # Subset data for a manageable example
  # Select the first 500 examinees and 30 item responses
  df_real <- ela1[1:500, 1:30]

  # Extract pre-estimated latent traits and covariates
  fixed_theta <- ela1$THETA[1:500]
  fixed_cov <- ela1$COVARIATE[1:500]

  # Estimate item parameters given fixed ability levels
  # fitting a 2-parameter logistic (2PL) model
  real_res <- fix_person(df = df_real,
                                   theta = fixed_theta,
                                   model = "2PL",
                                   covariate = fixed_cov)
  head(real_res)
\donttest{
  # --- Example: With Package Data ---
  data("ela1", package = "tirt")

  # Select Item Responses (Cols 1-30)
  df_real <- ela1[, 1:30]

  fixed_theta <- ela1$THETA
  fixed_cov <- ela1$COVARIATE

  real_res <- fix_person(df = df_real,
                                   theta = fixed_theta,
                                   model = "2PL",
                                   covariate = fixed_cov)
  head(real_res)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ld_stats.R
\name{ld_stats}
\alias{ld_stats}
\title{Local Dependence Statistics (Yen's Q3)}
\usage{
ld_stats(
  data,
  item_params,
  theta = NULL,
  theta_range = c(-4, 4),
  n_points = 41,
  model = NULL,
  D = 1
)
}
\arguments{
\item{data}{A data frame of item responses (rows = persons, columns = items).
Dichotomous items are 0/1; polytomous items are consecutive integers
starting at 0.}

\item{item_params}{A data frame of item parameters (the \code{item_params}
element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
\code{\link{mixed_irt}}). Item order must match the columns of \code{data}.}

\item{theta}{Optional numeric vector of estimated abilities (length equal to
the number of rows in \code{data}), or a person-parameter data frame with
an \code{ability}/\code{theta} column. If \code{NULL} (default), abilities
are estimated internally by expected a posteriori (EAP) scoring from
\code{item_params}.}

\item{theta_range}{Numeric vector of length 2 giving the ability grid bounds
used for internal EAP scoring (default \code{c(-4, 4)}).}

\item{n_points}{Integer. Number of quadrature points for internal EAP scoring
(default \code{41}).}

\item{model}{Optional model override (see \code{\link{item_info}}).}

\item{D}{Scaling constant for the dichotomous logistic metric (default
\code{1}).}
}
\value{
A symmetric numeric matrix of \eqn{Q_3} statistics with one row and
one column per item (diagonal set to \code{1}). The off-diagonal entries
are the residual correlations for each item pair. The average off-diagonal
\eqn{Q_3} is attached as the attribute \code{"mean_q3"} and the largest
absolute value as \code{"max_abs_q3"}.
}
\description{
Computes Yen's (1984) \eqn{Q_3} statistic for every item pair. \eqn{Q_3} is
the correlation between the model residuals of two items after the latent
trait has been partialled out. Under the local-independence assumption of
unidimensional item response theory these residual correlations should be
small and negative (around \eqn{-1/(J-1)}); large positive values signal that
two items share something beyond the common trait, such as a passage or a
scenario. \eqn{Q_3} is therefore used both to decide whether testlet modeling
is needed and to check whether a testlet model has adequately absorbed the
residual dependence.
}
\examples{
  set.seed(123)
  sim <- sim_irt(n_people = 400,
                 item_structure = list(list(model = "2PL", n_items = 8)))
  fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                    control = list(max_iter = 15, verbose = FALSE))

  q3 <- ld_stats(sim$resp, fit$item_params)
  round(q3, 3)
  attr(q3, "max_abs_q3")
}
\references{
Yen, W. M. (1984). Effects of local item dependence on the fit and equating
performance of the three-parameter logistic model. \emph{Applied
Psychological Measurement, 8}(2), 125-145.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trt_poly.R
\name{trt_poly}
\alias{trt_poly}
\title{Unidimensional Polytomous Testlet Response Theory Estimation}
\usage{
trt_poly(
  data,
  group,
  model = c("GRT", "PCMT", "BiFT"),
  method = c("MLE", "EM"),
  control = list()
)
}
\arguments{
\item{data}{A data.frame of polytomous responses. Rows=persons, Cols=items in testlets.}

\item{group}{A list defining testlet structures. Example: \code{list(c(1,2,3), c(4,5,6))}.}

\item{model}{Character. "GRT" (Graded Response Model), "PCMT" (Partial Credit Model for Testlet), or "BiFT" (Biffactor).}

\item{method}{Character. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood).}

\item{control}{A \code{list} of control parameters for the estimation algorithm:
\itemize{
\item \code{max_iter}: Maximum number of EM iterations (default = 100).
\item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
\item \code{theta_range}: Numeric vector of length 2 specifying the integration
grid bounds (default = c(-4, 4)).
\item \code{quad_points}: Number of quadrature points (default = 21).
\item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
}}
}
\value{
A list containing:
\itemize{
\item \code{item_params}: A data frame of estimated item parameters.
\item \code{person_params}: A data frame of estimated person abilities and testlet effects .
\item \code{model_fit}: A data frame containing iterations and fit statistics such as Akaike’s Information Criterion (AIC), the Bayesian Information Criterion (BIC), and Log-Likelihood.
}
}
\description{
Estimates item and person parameters for Polytomous Testlet models using Robust Newton-Raphson optimization.
}
\examples{
\donttest{
  # --- Example: Simulation (Mixed Categories GRT) ---
  set.seed(42)
  N <- 500; J <- 16

  # Define Groups (4 Testlets)
  groups <- list(c(1:4), c(5:8), c(9:12), c(13:16))

  # Define Categories (Binary, 3-cat, 4-cat, Mixed)
  # Items 1-4: 2 cats; 5-8: 3 cats; 9-12: 4 cats; 13-16: mixed
  cats <- c(rep(2, 4), rep(3, 4), rep(4, 4), 3, 5, 3, 5)

  # 1. Generate Parameters
  theta <- rnorm(N)
  # Gamma for 4 testlets (SD = 0.8)
  gamma <- matrix(rnorm(N * 4, 0, 0.8), N, 4)

  a <- rlnorm(J, 0, 0.2)
  b_list <- vector("list", J)

  # Generate Thresholds based on category count
  for(j in 1:J) {
    n_thresh <- cats[j] - 1
    if(n_thresh == 1) {
      b_list[[j]] <- rnorm(1)
    } else {
      # Spread thresholds
      b_list[[j]] <- sort(rnorm(1) + seq(-1, 1, length.out=n_thresh))
    }
  }

  # 2. Generate Responses (GRT Logic)
  resp <- matrix(NA, N, J)
  colnames(resp) <- paste0("Item_", 1:J)

  for(i in 1:N) {
    for(j in 1:J) {
      # Identify Testlet ID
      tid <- which(sapply(groups, function(x) j \%in\% x))
      eff <- theta[i] + gamma[i, tid]

      # Calculate Probabilities (Graded Response)
      K <- cats[j]
      probs <- numeric(K)
      P_prev <- 1
      for(k in 1:(K-1)) {
        term <- a[j] * (eff - b_list[[j]][k])
        P_star <- 1 / (1 + exp(-term))
        probs[k] <- P_prev - P_star
        P_prev <- P_star
      }
      probs[K] <- P_prev

      # Sample Response
      resp[i, j] <- sample(0:(K-1), 1, prob = probs)
    }
  }
  df_sim <- as.data.frame(resp)

  # 3. Run Estimation
  fit <- trt_poly(
    data = df_sim,
    group = groups,
    model = "GRT",
    method = "EM",
    control = list(max_iter = 20, verbose = FALSE)
  )

  head(fit$item_params)
  head(fit$person_params)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ela3_testmap}
\alias{ela3_testmap}
\title{Large-Scale Mixed-Format English Language Arts (ELA) Assessment Data Testmap (Form 3 Testmap)}
\format{
A data frame with 328 rows and 5 variables:
\itemize{
\item{FORMID}: Unique test form identifier.
\item{SEQ}: Position of the item in a form.
\item{ITEMID}: Unique item identifier, where _T1 indicates trait 1 and _T2 indicates trait 2.
\item{TYPE}: Type of an item, where OP indicates operational items and FT indicates field-test items.
\item{MAX_SCORE}: Max score of that item.
}
}
\source{
Tang, C., Xiong, J., & Engelhard, G. (2025). Identification of writing
strategies in educational assessments with an unsupervised learning
measurement framework. \emph{Education Sciences}, 15(7), 912.
\doi{10.3390/educsci15070912}
}
\usage{
ela3_testmap
}
\description{
Test map for the long format ela3 response data
}
\examples{
data(ela3_testmap)
head(ela3_testmap)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_trt.R
\name{sim_trt}
\alias{sim_trt}
\title{Simulate Testlet Response Theory Data (Vector Supported Version)}
\usage{
sim_trt(
  n_people = 1000,
  item_structure = list(),
  theta = NULL,
  theta_mean = 0,
  theta_sd = 1
)
}
\arguments{
\item{n_people}{Integer. Number of examinees.}

\item{item_structure}{List of lists defining item blocks.}

\item{theta}{Numeric vector (Optional). If provided, these exact ability values are used.}

\item{theta_mean}{Numeric. Mean of latent trait (used if theta is NULL).}

\item{theta_sd}{Numeric. SD of latent trait (used if theta is NULL).}
}
\value{
A list containing:
\item{resp}{data.frame of responses (rows=people, cols=items)}
\item{true_item_params}{data.frame of true item parameters}
\item{true_person_params}{vector of true latent traits}
}
\description{
Simulate testlet responses data. Support both dichotomous and polytomous responses.
Provide an easy implementation with a few default settings.
}
\examples{
  # =========================================================================
  # Example 1: Complex Testlet Design
  # =========================================================================
  # Define the Testlet Blueprint
  trt_design <- list(
    # Testlet 1: Rasch Testlet Model (High dependence: var=0.8)
    list(model = "RaschT", n_items = 5, testlet_id = "Read_A",
         testlet_var = 0.8, b = c(-1, 1)),

    # Testlet 2: 2PL Testlet Model (Default dependence: var=0.5)
    list(model = "2PLT", n_items = 5, testlet_id = "Read_B",
         a = c(0.7, 1.3)),

    # Testlet 3: Graded Response Testlet (Polytomous, 4 categories)
    list(model = "GRT", n_items = 4, testlet_id = "Survey",
         categories = 4, testlet_var = 0.2)
  )

  # Run Simulation
  trt_data <- sim_trt(n_people = 500, item_structure = trt_design)

  # Inspect Results
  # 1. Responses
  head(trt_data$resp)

  # 2. Item Parameters
  # (Notice 'testlet_loading' equals 'discrimination' for standard models)
  head(trt_data$true_item_params)

  # 3. Person Parameters (Ability + Gamma for each testlet)
  head(trt_data$true_person_params)

  # =========================================================================
  # Example 2: Manual Control (Theta, Gamma, and Parameters)
  # =========================================================================

  # 1. Manual Theta (e.g., everyone has high ability)
  manual_theta <- rep(2.0, 100)

  # 2. Manual Gamma (e.g., zero effect for T1)
  manual_gamma <- rep(0, 100)

  # 3. Item Parameters: Exact Match vs Range Sampling
  custom_structure <- list(
    # Case A: Manual Gamma Vector
    list(model = "2PLT", n_items = 5, testlet_id = "T1",
         gamma_vector = manual_gamma),

    # Case B: Exact Parameter Match (Length of 'a' equals n_items)
    list(model = "2PLT", n_items = 2, testlet_id = "T2",
         a = c(0.5, 2.5)),

    # Case C: Range Sampling (Length of 'a' is 2, but n_items != 2)
    list(model = "2PLT", n_items = 5, testlet_id = "T3",
         a = c(0.5, 2.5))
  )

  res_custom <- sim_trt(n_people = 100, theta = manual_theta,
                        item_structure = custom_structure)

  # Verify Manual Theta
  print(mean(res_custom$true_person_params$ability)) # Should be 2.0

  # Verify Manual Gamma (T1 should be 0)
  print(head(res_custom$true_person_params$testlet_T1))

  # Verify Exact Match (T2 discrimination should be 0.5 and 2.5)
  print(res_custom$true_item_params[res_custom$true_item_params$testlet_id == "T2",
                                    "discrimination"])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polytomousirt.R
\name{polytomous_irt}
\alias{polytomous_irt}
\title{Polytomous Item Response Theory Estimation Using Likelihood or Bayesian}
\usage{
polytomous_irt(data, model = "GPCM", method = "EM", control = list())
}
\arguments{
\item{data}{A N x J data.frame of polytomous responses (0, 1, 2...).
Missing values should be NA. Categories must be continuous integers.}

\item{model}{String. "GPCM" (Generalized Partial Credit Model), "PCM" (Partial Credit Model), or "GRM" (Graded Response Model).}

\item{method}{String. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood). However, using Bayesian will override the likelihood estimation.}

\item{control}{A \code{list} of control parameters for the estimation algorithm:
\itemize{
\item \code{max_iter}: Maximum number of EM iterations (default = 100).
\item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
\item \code{theta_range}: Numeric vector of length 2 specifying the integration
grid bounds (default = c(-4, 4)).
\item \code{quad_points}: Number of quadrature points (default = 21).
\item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
\item \code{prior}: A list specifying prior distributions for item parameters. Default is NULL (no priors).
For GRM: \code{list(a = function(x) dlnorm(x, 0, 0.5, log=TRUE), d = function(x) dnorm(x, 0, 2, log=TRUE))}.
For GPCM: \code{list(a = function(x) dlnorm(x, 0, 0.5, log=TRUE), d = function(x) dnorm(x, 0, 2, log=TRUE))}.
For PCM: \code{list(d = function(x) dnorm(x, 0, 2, log=TRUE))}.
Each prior is a function returning log-density and applies to ALL items.
Fixed value priors are NOT supported.
Item-specific priors are NOT supported.
}}
}
\value{
A list containing:
\itemize{
\item \code{item_params}: Data frame of estimated parameters (a, thresholds).
\item \code{person_params}: A data frame of estimated person abilities (theta) and standard errors.
\item \code{model_fit}: A data frame containing fit statistics such as Akaike’s Information Criterion (AIC) and the Bayesian Information Criterion (BIC).
\item \code{settings}: A list of control parameters used in the estimation.
}
}
\description{
Estimates item and person parameters for polytomous item response theory models
using either Marginal Maximum Likelihood or Joint Maximum Likelihood.
Now supports flexible prior distributions for Bayesian estimation (MAP estimation).
}
\examples{
  # --- Quick Example (small data) ---
  set.seed(1)
  n <- 50
  resp <- data.frame(
    I1 = sample(0:2, n, replace = TRUE),
    I2 = sample(0:2, n, replace = TRUE),
    I3 = sample(0:3, n, replace = TRUE)
  )
  res <- polytomous_irt(resp, model = "GPCM", method = "EM",
                        control = list(max_iter = 5, verbose = FALSE))
  head(res$item_params)
 \donttest{
  # --- Full Example 1: Simulation (GPCM) ---
  set.seed(2026)
  N <- 500; J <- 5
  n_cats <- c(3, 4, 3, 5, 4)

  true_theta <- rnorm(N)
  true_a <- runif(J, 0.8, 1.2)
  true_d <- list()

  # Generate Thresholds
  for(j in 1:J) {
    steps <- sort(rnorm(n_cats[j]-1, mean = 0, sd = 1.0))
    true_d[[j]] <- c(0, cumsum(steps))
  }

  # Simulation Helper (GPCM Logic)
  generate_resp <- function(theta, a, d_vec, n_cat) {
    probs <- matrix(0, length(theta), n_cat)
    for(k in 1:n_cat) {
      z <- a * (k-1) * theta - d_vec[k]
      probs[,k] <- exp(z)
    }
    probs <- probs / rowSums(probs)
    apply(probs, 1, function(p) sample(0:(n_cat-1), 1, prob=p))
  }

  # Create Data
  sim_data <- matrix(NA, nrow = N, ncol = J)
  for(j in 1:J) {
    sim_data[,j] <- generate_resp(true_theta, true_a[j], true_d[[j]], n_cats[j])
  }
  df_sim <- as.data.frame(sim_data)

  # Run Estimation (GPCM to match simulation logic without prior)
  res <- polytomous_irt(df_sim, model="GPCM", method="EM",
                        control=list(max_iter=20, verbose=TRUE))

  head(res$item_params)
  print(res$model_fit)

  # Run Estimation with prior (MAP)
  res_prior <- polytomous_irt(df_sim, model="PCM", method="EM",
                              control=list(max_iter=20, verbose=FALSE,
                                          prior=list(
                                            d = function(x) dnorm(x, 0, 2, log=TRUE)
                                          )))
  head(res$item_params)
  print(res$model_fit)

  # --- Example 2: With Package Data (GRM) ---
  data("ela1", package = "tirt")

  # Subset polytomous items (columns 31 to 45)
  df_poly <- ela1[, 31:45]

  # Run Estimation using GRM
  real_res <- polytomous_irt(df_poly, model="GRM", method="EM",
                             control = list(max_iter = 1000))

  head(real_res$item_params)
  head(real_res$person_params)
  print(real_res$model_fit)
  # Run Estimation using GRM with prior
  real_res2 <- polytomous_irt(df_poly, model="GRM", method="EM",
                             control = list(max_iter = 1000,
                                           prior = list(
                                             a = function(x) dlnorm(x, 0, 0.5, log=TRUE),
                                             d = function(x) dnorm(x, 0, 2, log=TRUE)
                                           )))

  head(real_res2$item_params)
  head(real_res2$person_params)
  print(real_res2$model_fit)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equating.R
\name{equate_irt}
\alias{equate_irt}
\title{Item Response Theory Equating / Linking}
\usage{
equate_irt(base_params, new_params, person_params = NULL, methods = NULL)
}
\arguments{
\item{base_params}{Data frame of reference item parameters (Form X).}

\item{new_params}{Data frame of new item parameters to be transformed (Form Y).}

\item{person_params}{(Optional) Data frame of person parameters from Form Y.}

\item{methods}{Character vector. Options: "Mean-Mean", "Mean-Sigma", "Stocking-Lord".
If NULL, defaults to all three.}
}
\value{
A list containing three data frames:
\item{transformed_item_params}{New items transformed to Base scale (with SEs).}
\item{transformed_person_params}{New persons transformed to Base scale (if provided).}
\item{linking_constants}{The A (slope) and B (intercept) constants for each method.}
}
\description{
Conducts item response theory scale linking using Mean-Mean, Mean-Sigma, and Stocking-Lord methods.
Supports mixed formats of both dichotomous and polytomous models.
Automatically detects anchor items and validates model consistency.
}
\examples{
  # ===========================================================================
  # Example: Equating Form Y (New) to Form X (Base)
  # ===========================================================================
  set.seed(123)

  # 1. Generate "True" Base Parameters (Form X)
  # ---------------------------------------------------------------------------
  # 10 Common Items (Anchors) + 10 Unique Items
  # 2PL and GRM mixed

  gen_item_params <- function(n, type="2PL") {
    if(type=="2PL") {
      data.frame(
        item = paste0("Item_", 1:n),
        model = "2PL",
        a = round(runif(n, 0.8, 1.5), 2),
        b = round(rnorm(n, 0, 1), 2),
        stringsAsFactors = FALSE
      )
    } else {
      # GRM with 3 thresholds
      d <- t(apply(matrix(rnorm(n*3, 0, 0.5), n, 3), 1, sort))
      df <- data.frame(
        item = paste0("Poly_", 1:n),
        model = "GRM",
        a = round(runif(n, 0.8, 1.5), 2),
        stringsAsFactors = FALSE
      )
      df <- cbind(df, setNames(as.data.frame(d), paste0("step_", 1:3)))
      df
    }
  }

  # Anchors
  anchor_2pl <- gen_item_params(5, "2PL")
  anchor_grm <- gen_item_params(3, "GRM")
  # Unique Form X
  unique_x <- gen_item_params(5, "2PL")
  unique_x$item <- paste0("X_", unique_x$item)

  base_params <- dplyr::bind_rows(anchor_2pl, anchor_grm, unique_x)

  # 2. Generate "New" Form Y Parameters (with Scale Shift)
  # ---------------------------------------------------------------------------
  # Scale Transformation: Theta_base = 1.2 * Theta_new + 0.5
  # True Constants: A = 1.2, B = 0.5
  TRUE_A <- 1.2
  TRUE_B <- 0.5

  # Transform Anchor Parameters to "New" scale (Inverse Logic)
  # a_new = a_base * A
  # b_new = (b_base - B) / A

  anchor_2pl_new <- anchor_2pl
  anchor_2pl_new$a <- anchor_2pl$a * TRUE_A
  anchor_2pl_new$b <- (anchor_2pl$b - TRUE_B) / TRUE_A

  anchor_grm_new <- anchor_grm
  anchor_grm_new$a <- anchor_grm$a * TRUE_A
  step_cols <- grep("step_", names(anchor_grm_new))
  anchor_grm_new[, step_cols] <- (anchor_grm[, step_cols] - TRUE_B) / TRUE_A

  # Unique Form Y
  unique_y <- gen_item_params(5, "2PL")
  unique_y$item <- paste0("Y_", unique_y$item)

  new_params <- dplyr::bind_rows(anchor_2pl_new, anchor_grm_new, unique_y)

  # 3. Create Dummy Person Parameters for Form Y
  # ---------------------------------------------------------------------------
  person_params <- data.frame(
    id = paste0("P", 1:50),
    theta = rnorm(50, 0, 1),
    theta_se = runif(50, 0.2, 0.5)
  )

  # 4. Perform Equating
  # ---------------------------------------------------------------------------
  # We expect to recover A approx 1.2 and B approx 0.5
  results <- equate_irt(
    base_params = base_params,
    new_params = new_params,
    person_params = person_params,
    methods = c("Mean-Mean", "Stocking-Lord")
  )

  # 5. Inspect Results
  # ---------------------------------------------------------------------------
  # Linking Constants
  print(results$linking_constants)

  # Transformed Items (Form Y items on Form X scale)
  head(results$transformed_item_params)

  # Transformed Persons
  head(results$transformed_person_params)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trt_binary.R
\name{trt_binary}
\alias{trt_binary}
\title{Unidimensional Binary (Dichotomous) Testlet Response Theory Estimation}
\usage{
trt_binary(
  data,
  group,
  model = c("RaschT", "2PLT", "3PLT", "BiFT"),
  method = c("EM", "MLE"),
  control = list()
)
}
\arguments{
\item{data}{A data.frame of binary responses (0/1). Rows=persons, Cols=items in testlets.}

\item{group}{A list defining testlet structures. Example: \code{list(c(1,2,3), c(4,5,6))}.}

\item{model}{Character. One of "RaschT" (Rasch Testlet), "2PLT" (2-Parameter Logistic Testlet), "3PLT" (3-Parameter Logistic Testlet), "BiFT" (Bifactor).}

\item{method}{Character. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood).}

\item{control}{A \code{list} of control parameters for the estimation algorithm:
\itemize{
\item \code{max_iter}: Maximum number of EM iterations (default = 100).
\item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
\item \code{theta_range}: Numeric vector of length 2 specifying the integration
grid bounds (default = c(-4, 4)).
\item \code{quad_points}: Number of quadrature points (default = 21).
\item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
}}
}
\value{
A list containing:
\itemize{
\item \code{item_params}: Estimated item parameters.
\item \code{person_params}: Estimated person abilities and testlet effects.
\item \code{model_fit}: A data frame containing iterations and fit statistics such as Akaike’s Information Criterion (AIC), the Bayesian Information Criterion (BIC), and Log-Likelihood.
}
}
\description{
Estimates item and person parameters for Unidimensional Binary (Dichotomous) Testlet response models
using Penalized Expectation-Maximization or
Joint Maximum Likelihood Estimation with stabilization.
}
\examples{
  # --- Quick Example (small data) ---
  set.seed(1)
  n <- 50
  J <- 4
  theta <- rnorm(n)
  gamma <- rnorm(n, 0, 0.5)
  resp <- matrix(NA, n, J)
  colnames(resp) <- paste0("Item_", 1:J)
  b <- c(-0.5, 0, 0.5, 1)
  for(j in 1:J) {
    p <- 1 / (1 + exp(-(theta + gamma - b[j])))
    resp[, j] <- rbinom(n, 1, p)
  }
  res <- trt_binary(
    data = as.data.frame(resp),
    group = list(1:2, 3:4),
    model = "2PLT",
    method = "EM",
    control = list(max_iter = 5, verbose = FALSE)
  )
  head(res$item_params)

 \donttest{
  # --- Full Example: Simulation (2PLT) ---
  set.seed(2025)
  n_persons <- 500
  n_testlets <- 3
  items_per_testlet <- 3
  n_items <- n_testlets * items_per_testlet

  # 1. Generate Parameters
  # Discrimination (a): Varying -> 2PLT
  a_true <- runif(n_items, 0.8, 1.5)
  # Difficulty (b)
  b_true <- seq(-1, 1, length.out = n_items)
  # Testlet Variances (Sigma)
  sigma_true <- c(1.0, 1.5, 2.0)

  # 2. Generate Person Params
  theta_true <- rnorm(n_persons, 0, 1)
  gamma_matrix <- matrix(0, nrow = n_persons, ncol = n_testlets)
  for(d in 1:n_testlets) {
    gamma_matrix[, d] <- rnorm(n_persons, 0, sigma_true[d])
  }

  # 3. Generate Responses
  resp_matrix <- matrix(0, nrow = n_persons, ncol = n_items)
  colnames(resp_matrix) <- paste0("Item_", 1:n_items)
  group_list <- list()

  idx_counter <- 1
  for(d in 1:n_testlets) {
    indices <- idx_counter:(idx_counter + items_per_testlet - 1)
    group_list[[d]] <- indices

    for(i in indices) {
      # 2PLT Model: a * (theta + gamma - b)
      lin <- a_true[i] * (theta_true + gamma_matrix[, d] - b_true[i])
      prob <- 1 / (1 + exp(-lin))
      resp_matrix[, i] <- rbinom(n_persons, 1, prob)
    }
    idx_counter <- idx_counter + items_per_testlet
  }
  df_sim <- as.data.frame(resp_matrix)

  # 4. Run Estimation
  # We use "2PLT" because data was generated with varying 'a'
  res <- trt_binary(
    data = df_sim,
    group = group_list,
    model = "2PLT",
    method = "EM",
    control = list(max_iter = 20, verbose = FALSE)
  )

  head(res$item_params)
  head(res$person_params)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binaryirt.R
\name{binary_irt}
\alias{binary_irt}
\title{Binary (Dichotomous) Item Response Theory Estimation Using Likelihood or Bayesian}
\usage{
binary_irt(data, model = "2PL", method = "EM", control = list())
}
\arguments{
\item{data}{A N x J data.frame of dichotomous responses (0/1).}

\item{model}{String. "Rasch", "2PL" (2-Parameter Logistic), or "3PL" (3-Parameter Logistic).}

\item{method}{String. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood). However, using Bayesian will override the likelihood estimation.}

\item{control}{A \code{list} of control parameters for the estimation algorithm:
\itemize{
\item \code{max_iter}: Maximum number of EM iterations (default = 100).
\item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
\item \code{theta_range}: Numeric vector of length 2 specifying the integration
grid bounds (default = c(-4, 4)).
\item \code{quad_points}: Number of quadrature points (default = 21).
\item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
\item \code{prior}: A list specifying prior distributions or fixed values for item parameters. Default is NULL (no priors).
For Rasch: \code{list(b = value)} or \code{list(b = function(x) dnorm(x, 0, 1, log=TRUE))}.
For 2PL: \code{list(a = ..., b = ...)}.
For 3PL: \code{list(a = ..., b = ..., g = ...)}.
Each parameter can be:
- A single numeric value (applied to all items)
- A numeric vector of length k (item-specific fixed values)
- A function returning log-density (applied to all items)
- A list of length k with functions or values (item-specific)
}}
}
\value{
A list containing:
\itemize{
\item \code{item_params}: A data frame of estimated item parameters (discrimination, difficulty, guessing) and their standard errors.
\item \code{person_params}: A data frame of estimated person abilities (theta) and standard errors.
\item \code{model_fit}: A data frame containing fit statistics such as Akaike's Information Criterion (AIC), the Bayesian Information Criterion (BIC), and Log-Likelihood.
\item \code{settings}: A list of control parameters used in the estimation.
}
}
\description{
Estimates item and person parameters for binary item response models
using either Marginal Maximum Likelihood or Joint Maximum Likelihood.
Now supports flexible prior distributions for Bayesian estimation (MAP estimation).
}
\examples{
  # # Simulate data
  set.seed(123)
  N <- 500; J <- 10
  true_theta <- rnorm(N)
  true_b <- seq(-2, 2, length.out=J)
  true_a <- runif(J, 0.8, 1.2)
  data_mat <- matrix(NA, N, J)
  for(i in 1:N) {
    p <- 1 / (1 + exp(-true_a * (true_theta[i] - true_b)))
    data_mat[i,] <- rbinom(J, 1, p)
  }
  df <- as.data.frame(data_mat)
  names(df) <- paste0("Q", 1:J)

  # # Run Function without prior
  res <- binary_irt(df, model="2PL", method="EM")

\donttest{
  # # Run Function with prior (function-based)
  res_prior <- binary_irt(df, model="2PL", method="EM",
                          control=list(prior=list(
                            a = function(x) dlnorm(x, 0, 0.5, log=TRUE),
                            b = function(x) dnorm(x, 0, 2, log=TRUE)
                          )))

  # # Run Function with fixed value prior
  res_fixed <- binary_irt(df, model="2PL", method="EM",
                          control=list(prior=list(
                            a = 1,  # Fix all discrimination prior to 1
                            b = function(x) dnorm(x, 0, 2, log=TRUE)
                          )))

  # View Results
  head(res$item_params)
  head(res$person_params)
  print(res$model_fit)
  # --- Example 2: With Package Data ---
  data("ela1", package = "tirt")
  # Subset the first 30 columns (must use the object name 'data_binary')
  df <- ela1[, 1:30]
  # Run Function on package data
  real_res <- binary_irt(df, model="2PL", method="EM")
  head(real_res$item_params)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/item_fit.R
\name{item_fit}
\alias{item_fit}
\title{Item Fit Statistics (Infit and Outfit)}
\usage{
item_fit(
  data,
  item_params,
  theta = NULL,
  theta_range = c(-4, 4),
  n_points = 41,
  model = NULL,
  D = 1
)
}
\arguments{
\item{data}{A data frame of item responses (rows = persons, columns = items).
Dichotomous items are 0/1; polytomous items are consecutive integers
starting at 0. Item order must match \code{item_params}.}

\item{item_params}{A data frame of item parameters (the \code{item_params}
element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
\code{\link{mixed_irt}}).}

\item{theta}{Optional numeric vector of estimated abilities, or a person-
parameter data frame with an \code{ability}/\code{theta} column. If
\code{NULL} (default), abilities are estimated internally by EAP scoring.}

\item{theta_range}{Numeric vector of length 2 giving the ability grid bounds
used for internal EAP scoring (default \code{c(-4, 4)}).}

\item{n_points}{Integer. Number of quadrature points for internal EAP scoring
(default \code{41}).}

\item{model}{Optional model override (see \code{\link{item_info}}).}

\item{D}{Scaling constant for the dichotomous logistic metric (default
\code{1}).}
}
\value{
A data frame with one row per item and the columns:
\itemize{
\item \code{item}: item name.
\item \code{n}: number of responses used.
\item \code{outfit}: outfit mean square (unweighted).
\item \code{outfit_z}: standardized outfit (ZSTD).
\item \code{infit}: infit mean square (information-weighted).
\item \code{infit_z}: standardized infit (ZSTD).
}
Mean-square values near 1 indicate good fit; values above about 1.3-1.5
suggest underfit (noise), and values below about 0.7 suggest overfit
(dependency or redundancy).
}
\description{
Computes item-level fit statistics based on standardized response residuals:
the information-weighted mean square (infit) and the unweighted mean square
(outfit), together with their standardized (ZSTD) transformations. These
statistics flag items whose observed responses depart from the fitted model,
either because of noise in the tails of the ability distribution (outfit) or
near the item location (infit). The statistics are computed for dichotomous
and polytomous items alike.
}
\examples{
  set.seed(123)
  sim <- sim_irt(n_people = 400,
                 item_structure = list(list(model = "2PL", n_items = 8)))
  fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                    control = list(max_iter = 15, verbose = FALSE))

  item_fit(sim$resp, fit$item_params)
}
\references{
Wright, B. D., & Masters, G. N. (1982). \emph{Rating scale analysis}. MESA
Press.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/information.R
\name{test_info}
\alias{test_info}
\title{Test Information Function and Conditional Standard Error}
\usage{
test_info(item_params, theta = seq(-4, 4, by = 0.1), model = NULL, D = 1)
}
\arguments{
\item{item_params}{A data frame of item parameters (see \code{\link{item_info}}).}

\item{theta}{A numeric vector of ability values at which to evaluate the
test information (default = \code{seq(-4, 4, by = 0.1)}).}

\item{model}{Optional model override (see \code{\link{item_info}}).}

\item{D}{Scaling constant for the dichotomous logistic metric (default
\code{1}).}
}
\value{
A data frame with one row per theta point and the columns:
\itemize{
\item \code{theta}: the ability value.
\item \code{test_info}: the test information (sum of item information).
\item \code{sem}: the conditional standard error of measurement,
\eqn{1 / \sqrt{test\_info}}.
\item \code{reliability}: the marginal-style conditional reliability
\eqn{TIF / (TIF + 1)} for a standard-normal ability scale.
}
}
\description{
Computes the test information function (TIF) as the sum of all item
information values at each ability point, together with the conditional
standard error of measurement (SEM). The test information function
summarizes where along the ability scale the test measures most precisely,
and the conditional SEM (\eqn{1 / \sqrt{TIF}}) translates that precision back
onto the theta metric. This is a routine part of test evaluation and form
assembly in operational testing programs.
}
\examples{
  set.seed(123)
  sim <- sim_irt(n_people = 400,
                 item_structure = list(list(model = "2PL", n_items = 10)))
  fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                    control = list(max_iter = 15, verbose = FALSE))

  tif <- test_info(fit$item_params, theta = seq(-3, 3, by = 0.5))
  print(tif)

  # The ability where the test is most informative
  tif$theta[which.max(tif$test_info)]
}
\seealso{
\code{\link{item_info}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixture_irt.R
\name{mixture_irt}
\alias{mixture_irt}
\title{Mixture Item Response Theory Model (Latent-Class Rasch / 2PL)}
\usage{
mixture_irt(data, n_class = 2, model = "Rasch", control = list())
}
\arguments{
\item{data}{A data frame of dichotomous (0/1) item responses (rows = persons,
columns = items).}

\item{n_class}{Integer. Number of latent classes to estimate (default
\code{2}).}

\item{model}{String. \code{"Rasch"} (default) or \code{"2PL"}; controls
whether class-specific discriminations are estimated.}

\item{control}{A \code{list} of control parameters for the algorithm:
\itemize{
\item \code{max_iter}: Maximum number of EM iterations (default = 100).
\item \code{converge_tol}: Convergence criterion on the log-likelihood
change (default = 1e-4).
\item \code{quad_points}: Number of quadrature points for the within-class
ability distribution (default = 21).
\item \code{theta_range}: Ability grid bounds (default = c(-4, 4)).
\item \code{verbose}: Logical; print progress (default = TRUE).
}}
}
\value{
A list containing:
\itemize{
\item \code{item_params}: A data frame of class-specific item difficulties
(and discriminations for the 2PL model), with one column per class.
\item \code{class_params}: A data frame of estimated mixing proportions for
each latent class.
\item \code{person_params}: A data frame of posterior class-membership
probabilities and the modal (most likely) class for each person.
\item \code{model_fit}: A data frame with the log-likelihood, AIC, BIC,
number of classes, and classification entropy.
}
}
\description{
Fits a mixture item response theory model (Rost, 1990) in which the examinee
population is assumed to consist of a small number of unobserved latent
classes, each with its own set of item parameters. Mixture IRT models are used
to detect qualitatively different response strategies, unmodeled
subpopulations, or classes for which item difficulty ordering differs. The
model is estimated by the Expectation-Maximization algorithm with a fixed
standard-normal ability distribution within each class.
}
\examples{
  # Two classes with reversed difficulty ordering
  set.seed(2025)
  N <- 300; J <- 8
  b1 <- seq(-1.5, 1.5, length.out = J)
  b2 <- rev(b1)
  theta <- rnorm(N)
  cls <- rep(1:2, each = N / 2)
  resp <- matrix(0, N, J)
  for (i in 1:N) {
    b <- if (cls[i] == 1) b1 else b2
    resp[i, ] <- rbinom(J, 1, 1 / (1 + exp(-(theta[i] - b))))
  }
  df <- as.data.frame(resp); names(df) <- paste0("I", 1:J)

  fit <- mixture_irt(df, n_class = 2, model = "Rasch",
                     control = list(max_iter = 30, verbose = FALSE))
  fit$class_params
  head(fit$item_params)
}
\references{
Rost, J. (1990). Rasch models in latent classes: An integration of two
approaches to item analysis. \emph{Applied Psychological Measurement, 14}(3),
271-282.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/information.R
\name{item_info}
\alias{item_info}
\title{Item Information Function}
\usage{
item_info(item_params, theta = seq(-4, 4, by = 0.1), model = NULL, D = 1)
}
\arguments{
\item{item_params}{A data frame of item parameters, typically the
\code{item_params} element returned by \code{\link{binary_irt}},
\code{\link{polytomous_irt}}, or \code{\link{mixed_irt}}. The function
auto-detects the model of each item from the available columns
(discrimination, difficulty, guessing, thresholds/steps) or from a
\code{model} column if present. All models in the package are supported:
Rasch, 2PL, 3PL, GRM, GPCM, and PCM.}

\item{theta}{A numeric vector of ability values at which to evaluate the
information (default = \code{seq(-4, 4, by = 0.1)}).}

\item{model}{Optional character string (applied to all items) or character
vector of length equal to the number of items, forcing the model family
used for each item. Advanced use only; when \code{NULL} (default) the model
is detected automatically.}

\item{D}{Scaling constant for the dichotomous logistic metric (default
\code{1}, matching the estimation functions in this package). Advanced
users who calibrated on the normal-metric scale may set \code{D = 1.702}.}
}
\value{
A numeric matrix with one row per item and one column per theta point.
Row names are the item names and column names are the theta values.
}
\description{
Computes the Fisher item information function for every item at a set of
ability (theta) points. The item information function shows how precisely an
item measures ability across the latent trait continuum: high information at
a given theta means the item discriminates well among examinees located
there. It is the building block of the test information function
(\code{\link{test_info}}) and of the conditional standard error of
measurement.
}
\details{
For dichotomous models the information is
\eqn{I(\theta) = (Da)^2 P(\theta)(1 - P(\theta))} for the Rasch and 2PL
models, and the standard Birnbaum form
\eqn{I(\theta) = (Da)^2 (1 - c)(1 - P^*)P^{*2} / P} for the 3PL model,
where \eqn{P^*} is the 2PL probability and \eqn{P} the full 3PL probability
(this reduces to the 2PL expression when the guessing parameter \eqn{c = 0}).
For polytomous models the general expected-information formula
\eqn{I(\theta) = \sum_k (\partial P_k / \partial \theta)^2 / P_k} is used,
which for the GPCM and PCM equals \eqn{(Da)^2 \mathrm{Var}(X \mid \theta)}.
}
\examples{
  # --- Example 1: dichotomous items ---
  set.seed(123)
  sim <- sim_irt(n_people = 400,
                 item_structure = list(list(model = "2PL", n_items = 6)))
  fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                    control = list(max_iter = 15, verbose = FALSE))

  # Information for each item at a grid of abilities
  info <- item_info(fit$item_params, theta = seq(-3, 3, by = 0.5))
  round(info, 3)

  \donttest{
  # --- Example 2: polytomous (GRM) items ---
  simp <- sim_irt(n_people = 400,
                  item_structure = list(list(model = "GRM", n_items = 5,
                                             categories = 4)))
  fitp <- polytomous_irt(simp$resp, model = "GRM", method = "EM",
                         control = list(max_iter = 15, verbose = FALSE))
  item_info(fitp$item_params, theta = c(-2, -1, 0, 1, 2))
  }
}
\seealso{
\code{\link{test_info}}, \code{\link{binary_irt}},
\code{\link{polytomous_irt}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fixed_item.R
\name{fixed_item}
\alias{fixed_item}
\title{Fixed Item Calibration}
\usage{
fixed_item(response_df, item_params_df, control = list())
}
\arguments{
\item{response_df}{A data.frame of responses. Rows=Students, Cols=Items.
Data MUST be from 0-indexed (0, 1, 2...).}

\item{item_params_df}{A data.frame of known parameters.
Required: "item", "model".}

\item{control}{A \code{list} of control parameters for the estimation algorithm:
\itemize{
\item \code{max_iter}: Maximum number of EM iterations (default = 50).
\item \code{conv_crit}: Convergence criterion for parameter change (default = 0.005).
\item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
}}
}
\value{
A list containing:
\itemize{
\item \code{item_params}: Estimated parameters for unknown items.
\item \code{person_params}: Estimated person parameters.
\item \code{model_fit}: A data frame containing number off estimated parameters and fit statistics such as Akaike’s Information Criterion (AIC), the Bayesian Information Criterion (BIC), and Log-Likelihood.
}
}
\description{
Estimates unknown item parameters using Marginal Maximum Likelihood via Expectation-Maximization Algorithm.
Uses a custom Bounded Newton-Raphson solver.
Supports mixed-format data containing dichotomous and polytomous responses
}
\examples{
# 1. TOY EXAMPLE
# ===========================================================================
set.seed(123)
# Create a very small dataset (N=50, J=4)
N_toy <- 50
df_toy <- data.frame(
  I1 = rbinom(N_toy, 1, 0.5), I2 = rbinom(N_toy, 1, 0.6), # Known items
  U1 = rbinom(N_toy, 1, 0.5), U2 = rbinom(N_toy, 1, 0.4)  # Unknown items
)

# Define the "Known" parameters for I1 and I2
known_params <- data.frame(
  item = c("I1", "I2"),
  model = c("2PL", "2PL"),
  a = c(1.0, 1.2),
  b = c(-0.5, 0.5)
)

# Run Fixed Item Calibration with very low iterations
fit_toy <- fixed_item(df_toy, known_params, control=list(max_iter=2, verbose=FALSE))
print(head(fit_toy$item_params))
  \donttest{
  # --- Example 2: Simulation ---
  set.seed(123)
  N <- 500
  true_theta <- rnorm(N, 0, 1)

  # 1. Simulation Helpers
  sim_2pl <- function(theta, a, b) {
    p <- 1 / (1 + exp(-1.7 * a * (theta - b)))
    rbinom(N, 1, p)
  }
  sim_poly <- function(theta, a, steps) {
    n_cat <- length(steps) + 1
    probs <- matrix(0, length(theta), n_cat)
    for(k in 1:n_cat) {
      score <- k - 1
      if(score == 0) num <- 0
      else num <- a * (score * theta - sum(steps[1:score]))
      probs[, k] <- exp(num)
    }
    probs <- probs / rowSums(probs)
    apply(probs, 1, function(x) sample(0:(n_cat-1), 1, prob=x))
  }

  # 2. Generate Data (Mixed Known/Unknown Items)
  # Items 1-5: Known Binary (2PL)
  # Items 6-10: Unknown Binary (2PL)
  # Items 11-12: Known Poly (GPCM)
  # Items 13-15: Unknown Poly (GPCM)

  resp_mat <- matrix(NA, N, 15)
  colnames(resp_mat) <- paste0("Item_", 1:15)

  # Known Binary Parameters
  a_bin <- c(1.0, 1.2, 0.9, 1.1, 0.8)
  b_bin <- c(-1, -0.5, 0, 0.5, 1)

  for(i in 1:5) resp_mat[,i] <- sim_2pl(true_theta, a_bin[i], b_bin[i])
  for(i in 6:10) resp_mat[,i] <- sim_2pl(true_theta, runif(1,0.8,1.2), rnorm(1))

  # Known Poly Parameters
  a_poly <- c(1.0, 0.8)
  d_poly <- list(c(-1, 1), c(-0.5, 0.5))

  resp_mat[,11] <- sim_poly(true_theta, a_poly[1], d_poly[[1]])
  resp_mat[,12] <- sim_poly(true_theta, a_poly[2], d_poly[[2]])
  for(i in 13:15) resp_mat[,i] <- sim_poly(true_theta, 1.0, c(-0.5, 0.5))

  df_resp <- as.data.frame(resp_mat)

  # 3. Create 'Known Parameters' Dataframe
  # This tells the function: "Fix these, Estimate the rest"
  known_df <- data.frame(
    item = c(paste0("Item_", 1:5), "Item_11", "Item_12"),
    model = c(rep("2PL", 5), rep("GPCM", 2)),
    a = c(a_bin, a_poly),
    b = c(b_bin, NA, NA),     # Binary difficulty
    step_1 = c(rep(NA, 5), -1, -0.5), # Poly steps
    step_2 = c(rep(NA, 5),  1,  0.5),
    stringsAsFactors = FALSE
  )

  # 4. Run Estimation
  res <- fixed_item(df_resp, known_df, control=list(max_iter=20))

  # View Results
  # Notice Items 1-5 and 11-12 have Status "Fixed"
  head(res$item_params, 12)

  # --- Example 2: With Package Data ---
  data("ela1", package = "tirt")

  # Let's treat the first 5 items as "Known" with arbitrary parameters
  # just to demonstrate syntax.
  df_real <- ela1[, 1:20]

  known_real <- data.frame(
    item = paste0("Q", 1:5),
    model = "2PL",
    a = 1.0,
    b = seq(-1, 1, length.out=5)
  )

  # Ideally, column names in df_real should match 'item' column in known_real
  colnames(df_real)[1:5] <- paste0("Q", 1:5)

  real_res <- fixed_item(df_real, known_real, control=list(max_iter=10))
  head(real_res$item_params)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/score_table.R
\name{score_table}
\alias{score_table}
\title{Summed-Score to Theta Conversion Table}
\usage{
score_table(
  item_params,
  method = c("EAP", "WLE", "MLE"),
  prior_mean = 0,
  prior_sd = 1,
  theta_range = c(-4, 4),
  n_points = 81,
  model = NULL,
  D = 1
)
}
\arguments{
\item{item_params}{A data frame of item parameters (the \code{item_params}
element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
\code{\link{mixed_irt}}). All models in the package are supported.}

\item{method}{Character. Scoring method: \code{"EAP"} (default),
\code{"WLE"}, or \code{"MLE"}.}

\item{prior_mean, prior_sd}{Mean and standard deviation of the normal
population prior used for the EAP method and for the quadrature weighting
(defaults \code{0} and \code{1}).}

\item{theta_range}{Numeric vector of length 2 giving the ability grid bounds
(default \code{c(-4, 4)}).}

\item{n_points}{Integer. Number of ability grid points used to evaluate the
summed-score distributions (default \code{81}).}

\item{model}{Optional model override (see \code{\link{item_info}}).}

\item{D}{Scaling constant for the dichotomous logistic metric (default
\code{1}).}
}
\value{
A data frame with one row per possible summed score and the columns:
\itemize{
\item \code{summed_score}: the integer summed score (0 to the maximum
possible score).
\item \code{theta}: the ability estimate for that summed score.
\item \code{se}: the standard error of the ability estimate.
}
}
\description{
Generates a raw-score (summed-score) to scale-score conversion table, mapping
every possible integer summed score to its corresponding ability estimate and
standard error. Such tables are standard in operational testing programs
because they let each examinee's number-correct (or number-of-points) score
be translated directly into a theta estimate without needing the full
response pattern. Three scoring methods are supported: expected a posteriori
(\code{"EAP"}), weighted likelihood (\code{"WLE"}), and maximum likelihood
(\code{"MLE"}).
}
\details{
The distribution of the summed score at each ability grid point is obtained
with the Lord-Wingersky (1984) recursion, which works for any mixture of
dichotomous and polytomous items. The \code{"EAP"} estimate is the mean of
the posterior of theta given the summed score, with \code{se} equal to the
posterior standard deviation. The \code{"MLE"} estimate maximizes the
summed-score likelihood; the \code{"WLE"} estimate (Warm, 1989) maximizes the
summed-score likelihood weighted by the square root of the test information,
which reduces the bias of the MLE. At the two extreme summed scores the MLE is
not finite and is reported at the boundary of \code{theta_range} with a missing
standard error; the WLE remains finite there and is estimated normally.
}
\examples{
  set.seed(123)
  sim <- sim_irt(n_people = 400,
                 item_structure = list(list(model = "2PL", n_items = 10)))
  fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                    control = list(max_iter = 15, verbose = FALSE))

  # EAP conversion table (0 to 10 correct)
  score_table(fit$item_params, method = "EAP")

  \donttest{
  # Maximum-likelihood conversion table
  score_table(fit$item_params, method = "MLE")
  }
}
\references{
Lord, F. M., & Wingersky, M. S. (1984). Comparison of IRT true-score and
equipercentile observed-score equatings. \emph{Applied Psychological
Measurement, 8}(4), 453-461.

Warm, T. A. (1989). Weighted likelihood estimation of ability in item
response theory. \emph{Psychometrika, 54}(3), 427-450.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tcc.R
\name{tcc}
\alias{tcc}
\title{Test and Item Characteristic Curves (Expected Scores)}
\usage{
tcc(item_params, theta = seq(-4, 4, by = 0.1), model = NULL, D = 1)
}
\arguments{
\item{item_params}{A data frame of item parameters (the \code{item_params}
element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
\code{\link{mixed_irt}}). All models in the package are supported.}

\item{theta}{A numeric vector of ability values at which to evaluate the
curves (default = \code{seq(-4, 4, by = 0.1)}).}

\item{model}{Optional model override (see \code{\link{item_info}}).}

\item{D}{Scaling constant for the dichotomous logistic metric (default
\code{1}).}
}
\value{
A list with two elements:
\itemize{
\item \code{test_curve}: a data frame with columns \code{theta} and
\code{expected_score} (the test characteristic curve).
\item \code{item_curves}: a numeric matrix with one row per item and one
column per theta value, giving the expected score of each item.
}
}
\description{
Computes item characteristic curves (the expected score of each item as a
function of ability) and the test characteristic curve (the expected total
score as a function of ability). The test characteristic curve maps the latent
ability scale onto the number-correct (true-score) scale and underlies
true-score equating and score reporting; the item curves show how each item's
expected score rises across the trait.
}
\examples{
  set.seed(123)
  sim <- sim_irt(n_people = 400,
                 item_structure = list(list(model = "2PL", n_items = 10)))
  fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                    control = list(max_iter = 15, verbose = FALSE))

  curves <- tcc(fit$item_params, theta = seq(-3, 3, by = 0.5))
  curves$test_curve

  # Expected score on the first item across ability
  curves$item_curves[1, ]
}
\seealso{
\code{\link{item_info}}, \code{\link{score_table}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ela3}
\alias{ela3}
\title{Large-Scale Mixed-Format English Language Arts (ELA) Assessment Data (Form 3)}
\format{
A data frame with 2,434,185 observations and 5 variables:
\itemize{
\item{STUDENTID}: Unique student identifier.
\item{FORMID}: Unique test form identifier.
\item{ITEMID}: Unique item identifier, where _T1 indicates trait 1 and _T2 indicates trait 2.
\item{SEQ}: Position of the item in a form.
\item{SCORE}: Dichotomously and polytomously scored item responses (0,1,2...).
}
}
\source{
Tang, C., Xiong, J., & Engelhard, G. (2025). Identification of writing
strategies in educational assessments with an unsupervised learning
measurement framework. \emph{Education Sciences}, 15(7), 912.
\doi{10.3390/educsci15070912}
}
\usage{
ela3
}
\description{
A long format dataset containing binary and polytomous responses for demonstration.
}
\examples{
data(ela3)
head(ela3)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliability.R
\name{reliability}
\alias{reliability}
\title{Test Reliability Indices}
\usage{
reliability(
  person_params = NULL,
  data = NULL,
  item_params = NULL,
  prior_sd = 1,
  theta_range = c(-4, 4),
  n_points = 61,
  model = NULL,
  D = 1
)
}
\arguments{
\item{person_params}{Optional. A person-parameter data frame containing an
\code{ability}/\code{theta} column and a standard-error column (such as
\code{ability_se} or \code{se}); typically the \code{person_params} element
returned by the estimation functions. Used for the empirical reliability.}

\item{data}{Optional. A data frame of item responses used to compute
Cronbach's alpha (rows = persons, columns = items).}

\item{item_params}{Optional. A data frame of item parameters used to compute
the model-based marginal reliability from the test information function.}

\item{prior_sd}{Numeric. Standard deviation of the population ability
distribution used for the model-based marginal reliability (default
\code{1}).}

\item{theta_range}{Numeric vector of length 2 giving the ability grid bounds
for the marginal-reliability integration (default \code{c(-4, 4)}).}

\item{n_points}{Integer. Number of grid points for the marginal-reliability
integration (default \code{61}).}

\item{model}{Optional model override (see \code{\link{item_info}}).}

\item{D}{Scaling constant for the dichotomous logistic metric (default
\code{1}).}
}
\value{
A data frame with the columns \code{Index} and \code{Value},
reporting whichever reliability coefficients could be computed from the
supplied arguments:
\itemize{
\item \code{empirical_reliability}: from person estimates and their SEs.
\item \code{marginal_reliability}: from the test information function.
\item \code{cronbach_alpha}: classical internal-consistency reliability.
\item \code{n_items}, \code{n_persons}: sample descriptors when available.
}
}
\description{
Computes commonly reported reliability coefficients for a test. Depending on
the information supplied it returns the empirical (marginal) reliability from
latent-trait estimates and their standard errors, the model-based marginal
reliability from the test information function, and the classical Cronbach's
alpha from the raw responses. Reliability summarizes how consistently the test
orders examinees and is a standard entry in every technical report.
}
\details{
The empirical reliability is
\eqn{\mathrm{Var}(\hat\theta) / (\mathrm{Var}(\hat\theta) + \overline{SE^2})}.
The model-based marginal reliability is
\eqn{\sigma^2 / (\sigma^2 + \overline{1/I(\theta)})}, where the average error
variance is taken over the population ability distribution. Cronbach's alpha
is \eqn{\frac{J}{J-1}\left(1 - \frac{\sum_j s_j^2}{s_T^2}\right)}.
}
\examples{
  set.seed(123)
  sim <- sim_irt(n_people = 500,
                 item_structure = list(list(model = "2PL", n_items = 12)))
  fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                    control = list(max_iter = 15, verbose = FALSE))

  reliability(person_params = fit$person_params,
              data = sim$resp,
              item_params = fit$item_params)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/person_fit.R
\name{person_fit}
\alias{person_fit}
\title{Person Fit Statistics (Standardized Log-Likelihood, lz)}
\usage{
person_fit(data, item_params, theta, critical = -1.96, model = NULL, D = 1)
}
\arguments{
\item{data}{A data frame of item responses (rows = persons, columns = items).
Dichotomous items are 0/1; polytomous items are consecutive integers
starting at 0. Missing values (\code{NA}) are allowed and are skipped
person-by-person.}

\item{item_params}{A data frame of item parameters (the \code{item_params}
element from \code{\link{binary_irt}}, \code{\link{polytomous_irt}}, or
\code{\link{mixed_irt}}). Item order must match the columns of \code{data}.}

\item{theta}{A numeric vector of estimated person abilities (length equal to
the number of rows in \code{data}), or a person-parameter data frame
containing an \code{ability} or \code{theta} column (such as the
\code{person_params} element returned by the estimation functions).}

\item{critical}{Numeric. Persons with \eqn{l_z} below this value are flagged
as potentially misfitting (default = \code{-1.96}, the lower 2.5\\% point of
the standard normal distribution; \code{-2} is another common choice).}

\item{model}{Optional model override (see \code{\link{item_info}}).}

\item{D}{Scaling constant for the dichotomous logistic metric (default
\code{1}).}
}
\value{
A data frame with one row per person and the columns:
\itemize{
\item \code{person}: person index (row number).
\item \code{n_items}: number of items answered.
\item \code{theta}: the ability estimate used.
\item \code{loglik}: the observed log-likelihood of the response pattern.
\item \code{lz}: the standardized log-likelihood person-fit index.
\item \code{flag}: logical; \code{TRUE} if \code{lz < critical}.
}
}
\description{
Computes person-level fit statistics to detect examinees whose response
patterns are unlikely under the fitted item response model (for example,
careless responding, cheating, or unusual guessing). The function returns the
\eqn{l_z} standardized log-likelihood index of Drasgow, Levine, and Williams
(1985), which under model fit is approximately standard normal, and flags
persons whose \eqn{l_z} falls below a critical value.
}
\details{
For a person with ability \eqn{\theta}, let \eqn{l_0} be the log-likelihood of
the observed responses. The index is
\eqn{l_z = (l_0 - E[l_0]) / \sqrt{\mathrm{Var}(l_0)}}, where the expectation
and variance are taken over the model-implied category probabilities at
\eqn{\theta}. Large negative values indicate response patterns that are less
likely than the model predicts.
}
\examples{
  set.seed(123)
  sim <- sim_irt(n_people = 300,
                 item_structure = list(list(model = "2PL", n_items = 15)))
  fit <- binary_irt(sim$resp, model = "2PL", method = "EM",
                    control = list(max_iter = 15, verbose = FALSE))

  pf <- person_fit(sim$resp, fit$item_params, fit$person_params)
  head(pf)

  # How many examinees are flagged as misfitting?
  sum(pf$flag, na.rm = TRUE)
}
\references{
Drasgow, F., Levine, M. V., & Williams, E. A. (1985). Appropriateness
measurement with polychotomous item response models and standardized indices.
\emph{British Journal of Mathematical and Statistical Psychology, 38}(1),
67-86.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dif.R
\name{dif}
\alias{dif}
\title{Differential Item Functioning (Mantel-Haenszel and Logistic Regression)}
\usage{
dif(data, group, focal = NULL, match = NULL, purify = FALSE)
}
\arguments{
\item{data}{A data frame of dichotomous (0/1) item responses (rows = persons,
columns = items).}

\item{group}{A vector of length equal to the number of rows in \code{data}
giving the group membership of each person. It must have exactly two
distinct non-missing values.}

\item{focal}{Optional. The value of \code{group} that identifies the focal
group. If \code{NULL} (default), the second value (in sorted order) is used
as the focal group and the first as the reference group.}

\item{match}{Optional numeric vector of matching scores (length equal to the
number of rows in \code{data}). If \code{NULL} (default), the total number-
correct score across all items is used.}

\item{purify}{Logical. If \code{TRUE}, the matching total score is computed
as the rest score (total minus the studied item) separately for each item,
which avoids contaminating the match with the item under study (default
\code{FALSE}).}
}
\value{
A data frame with one row per item and the columns:
\itemize{
\item \code{item}: the item name.
\item \code{MH_chisq}, \code{MH_p}: the Mantel-Haenszel chi-square (with
continuity correction) and its p-value.
\item \code{MH_OR}: the Mantel-Haenszel common odds ratio.
\item \code{MH_delta}: the ETS delta effect size, \eqn{-2.35 \ln(OR)}.
\item \code{ETS_class}: the ETS DIF classification, \code{"A"} (negligible),
\code{"B"} (moderate), or \code{"C"} (large).
\item \code{LR_chisq}, \code{LR_p}: the 2-degree-of-freedom logistic
regression test for combined (uniform + non-uniform) DIF.
\item \code{LR_uniform_p}: p-value for uniform DIF (group main effect).
\item \code{LR_nonuniform_p}: p-value for non-uniform DIF (group-by-match
interaction).
}
}
\description{
Detects Differential Item Functioning (DIF) for dichotomous items, that is,
items that behave differently for two groups of examinees (for example, a
reference and a focal group) after matching on ability. Two widely used
approaches are provided: the Mantel-Haenszel (1959) procedure, with the ETS
delta effect size and A/B/C flagging scheme (Holland & Thayer, 1988), and the
logistic regression approach (Swaminathan & Rogers, 1990), which separates
uniform and non-uniform DIF.
}
\examples{
  set.seed(123)
  sim <- sim_irt(n_people = 600,
                 item_structure = list(list(model = "2PL", n_items = 10)))
  resp <- sim$resp

  # Create two groups and plant DIF in item 3 (harder for group "B")
  grp <- rep(c("A", "B"), each = 300)
  flip <- grp == "B" & resp[[3]] == 1
  resp[[3]][flip] <- rbinom(sum(flip), 1, 0.6)

  dif_res <- dif(resp, group = grp)
  dif_res
}
\references{
Holland, P. W., & Thayer, D. T. (1988). Differential item performance and the
Mantel-Haenszel procedure. In H. Wainer & H. I. Braun (Eds.), \emph{Test
validity} (pp. 129-145). Erlbaum.

Swaminathan, H., & Rogers, H. J. (1990). Detecting differential item
functioning using logistic regression procedures. \emph{Journal of
Educational Measurement, 27}(4), 361-370.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_tirt.R
\name{sim_tirt}
\alias{sim_tirt}
\title{Simulate Mixed IRT and Testlet (TRT) Data}
\usage{
sim_tirt(
  n_people = 1000,
  item_structure = list(),
  theta = NULL,
  theta_mean = 0,
  theta_sd = 1
)
}
\arguments{
\item{n_people}{Integer. Number of examinees.}

\item{item_structure}{List of lists defining item blocks (see Details).}

\item{theta}{Numeric vector (Optional). If provided, these exact ability
values are used.}

\item{theta_mean}{Numeric. Mean of the latent trait (used if \code{theta} is
\code{NULL}).}

\item{theta_sd}{Numeric. SD of the latent trait (used if \code{theta} is
\code{NULL}).}
}
\value{
A list containing:
\item{resp}{data.frame of responses (rows = people, cols = items).}
\item{true_item_params}{data.frame of true item parameters, including a
\code{model} column and a \code{testlet} column (\code{NA} for
independent items). Its \code{item_id}, \code{model}, and \code{testlet}
columns provide everything needed to build the \code{item_spec} argument
of \code{\link{irt_trt}} (rename \code{item_id} to \code{item}).}
\item{true_person_params}{data.frame of true person parameters: the primary
ability plus one gamma column per testlet.}
}
\description{
Simulates a data set that mixes ordinary independent item response theory
items with testlet (locally dependent) items in a single test form, the
simulation counterpart of \code{\link{irt_trt}}. Independent blocks depend
only on the primary trait (theta); testlet blocks additionally depend on a
testlet-specific nuisance effect (gamma). Both dichotomous and polytomous
formats are supported for the independent and the testlet parts, so the whole
family of models handled by \code{\link{irt_trt}} can be generated at once.
}
\details{
Each element of \code{item_structure} is a list describing one block. The
\code{model} name determines whether the block is independent or a testlet:
\itemize{
\item Independent (theta only): \code{"Rasch"}, \code{"2PL"}, \code{"3PL"},
\code{"PCM"}, \code{"GPCM"}, \code{"GRM"}.
\item Testlet (theta + gamma): \code{"RaschT"}, \code{"2PLT"},
\code{"3PLT"}, \code{"BiFT"}, \code{"PCT"}, \code{"GPCT"}, \code{"GRT"}.
}
Other recognized keys are \code{n_items}, \code{categories} (polytomous),
\code{a}, \code{b}, \code{c} (given as a fixed value or a \code{c(lo, hi)}
range to sample from), \code{s} (testlet loading for \code{"BiFT"}),
\code{testlet_id} (the testlet label, required for testlet blocks),
\code{testlet_var} (variance of the gamma effect), and \code{gamma_vector}
(a user-supplied gamma effect of length \code{n_people}).
}
\examples{
  # A form with independent items and two testlets
  set.seed(2025)
  design <- list(
    list(model = "2PL",  n_items = 8),                       # independent
    list(model = "GRM",  n_items = 4, categories = 3),       # independent poly
    list(model = "2PLT", n_items = 4, testlet_id = "P1",
         testlet_var = 0.6),                                 # testlet
    list(model = "GPCT", n_items = 3, categories = 3,
         testlet_id = "P2", testlet_var = 0.5)               # testlet poly
  )

  sim <- sim_tirt(n_people = 600, item_structure = design)

  head(sim$resp)
  sim$true_item_params[, c("item_id", "model", "testlet")]
  head(sim$true_person_params)

  \donttest{
  # Feed the true structure straight into the joint estimator
  spec <- data.frame(
    item = sim$true_item_params$item_id,
    model = sim$true_item_params$model,
    testlet = sim$true_item_params$testlet,
    stringsAsFactors = FALSE
  )
  fit <- irt_trt(sim$resp, spec, method = "EM",
                 control = list(max_iter = 15, verbose = FALSE))
  head(fit$item_params)
  }
}
\seealso{
\code{\link{irt_trt}}, \code{\link{sim_trt}}, \code{\link{sim_irt}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixirt.R
\name{mixed_irt}
\alias{mixed_irt}
\title{Mixed Item Response Model Estimation (Dichotomous & Polytomous) with Prior Support}
\usage{
mixed_irt(data, model = "2PL", method = "EM", control = list())
}
\arguments{
\item{data}{A N x J data.frame. Binary items must be 0/1. Polytomous items should be continuous integers (0, 1, 2...).}

\item{model}{A character vector of length J (one model per item).
Supported: "Rasch", "2PL" (2-Parameter Logistic), "3PL" (3-Parameter Logistic), "GRM" (Graded Response Model), "GPCM" (Generalized Partial Credit Model), "PCM" (Partial Credit Model).
If a single string is provided, it is applied to all the same type of items.}

\item{method}{String. "EM" (Marginal Maximum Likelihood via Expectation-Maximization) or "MLE" (Joint Maximum Likelihood).}

\item{control}{A \code{list} of control parameters for the estimation algorithm:
\itemize{
\item \code{max_iter}: Maximum number of EM iterations (default = 100).
\item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
\item \code{theta_range}: Numeric vector of length 2 specifying the integration
grid bounds (default = c(-4, 4)).
\item \code{quad_points}: Number of quadrature points (default = 21).
\item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
\item \code{prior}: A list of model-specific prior distributions. Default is NULL (no priors).
Format: \code{list("2PL" = list(a = function(x) ..., b = function(x) ...), "GRM" = list(...))}
Each model gets its own prior specification. All items of the same model share the same prior.
Example:
\code{prior = list(
        "2PL" = list(a = function(x) dlnorm(x, 0, 0.5, log=TRUE), b = function(x) dnorm(x, 0, 2, log=TRUE)),
        "GRM" = list(a = function(x) dlnorm(x, 0, 0.5, log=TRUE), d = function(x) dnorm(x, 0, 2, log=TRUE))
      )}
}}
}
\value{
A list containing:
\itemize{
\item \code{item_params}: Data frame of item parameters (discrimination, difficulty/thresholds, guessing).
\item \code{person_params}: A data frame of estimated person abilities (theta) and standard errors.
\item \code{model_fit}: A data frame containing fit statistics such as Akaike’s Information Criterion (AIC) and the Bayesian Information Criterion (BIC).
\item \code{settings}: A list of control parameters used in the estimation.
}
}
\description{
Provides a estimation framework for a broad class of different item response theory models.
This function can model different combinations of item categories.
Now supports flexible prior distributions for Bayesian estimation (MAP estimation).
}
\examples{
  # --- Example 1: Simulation (Mixed 2PL + GPCM) ---
  set.seed(2025)
  N <- 500
  n_bin <- 5
  n_poly <- 2
  J <- n_bin + n_poly

  # 1. Generate Theta (Wide range to match user request)
  true_theta <- rnorm(N, mean = 0, sd = 3)

  # 2. Simulation Helper: GPCM
  sim_gpcm <- function(theta, a, steps) {
    n_cat <- length(steps) + 1
    probs <- matrix(0, length(theta), n_cat)
    for(k in 1:n_cat) {
      score <- k - 1
      if(score == 0) numer <- rep(0, length(theta))
      else numer <- a * (score * theta - sum(steps[1:score]))
      probs[, k] <- exp(numer)
    }
    probs <- probs / rowSums(probs)
    apply(probs, 1, function(p) sample(0:(n_cat-1), 1, prob=p))
  }

  # 3. Create Data
  data_sim <- data.frame(matrix(NA, nrow = N, ncol = J))
  colnames(data_sim) <- paste0("Item_", 1:J)

  # Binary Items (2PL)
  a_bin <- runif(n_bin, 0.8, 1.5)
  b_bin <- seq(-3, 3, length.out = n_bin)
  for(j in 1:n_bin) {
    prob <- 1 / (1 + exp(-(a_bin[j] * (true_theta - b_bin[j]))))
    data_sim[, j] <- rbinom(N, 1, prob)
  }

  # Polytomous Items (GPCM)
  # Item 6: 2 steps (-2, 2)
  data_sim[, 6] <- sim_gpcm(true_theta, a=1.0, steps=c(-2, 2))
  # Item 7: 5 steps
  data_sim[, 7] <- sim_gpcm(true_theta, a=1.2, steps=c(-5, -2.5, 0, 2.5, 5))

  # 4. Run Estimation without prior
  # Note: Wide theta_range needed due to SD=3 in simulation
  my_models <- c(rep("2PL", n_bin), rep("GPCM", n_poly))

  res <- mixed_irt(data = data_sim, model = my_models, method = "EM",
                   control = list(max_iter = 20, theta_range = c(-6, 6)))

  head(res$item_params)
  print(res$model_fit)

  \donttest{
  # 5. Run Estimation with prior (MAP)
  res_prior <- mixed_irt(data = data_sim, model = my_models, method = "EM",
                         control = list(max_iter = 20, theta_range = c(-6, 6),
                                       prior = list(
                                         "2PL" = list(
                                           a = function(x) dlnorm(x, 0, 0.5, log=TRUE),
                                           b = function(x) dnorm(x, 0, 2, log=TRUE)
                                         ),
                                         "GPCM" = list(
                                           a = function(x) dlnorm(x, 0, 0.5, log=TRUE),
                                           d = function(x) dnorm(x, 0, 2, log=TRUE)
                                         )
                                       )))
  head(res_prior$item_params)
  print(res_prior$model_fit)
  # --- Example 2: With Package Data ---
  data("ela2", package = "tirt")

  # Define Models (7 Binary, 3 Poly)
  real_models <- c(rep("2PL", 7), rep("GRM", 3))

  # Run Estimation
  real_res <- mixed_irt(ela2, model = real_models, method = "EM",
                        control = list(max_iter = 10))

  head(real_res$item_params)
  print(real_res$model_fit)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/irt_trt.R
\name{irt_trt}
\alias{irt_trt}
\title{Joint Item Response Theory and Testlet Response Theory Estimation (Dichotomous & Polytomous)}
\usage{
irt_trt(data, item_spec, method = "EM", control = list())
}
\arguments{
\item{data}{A \code{data.frame} or \code{matrix} containing item responses.
Responses should be 0-indexed integers. Missing values should be coded as \code{NA}.}

\item{item_spec}{A \code{data.frame} providing item metadata. Must include
columns \code{"item"} (matching \code{colnames(data)}) and \code{"model"}.
Optionally includes a \code{"testlet"} column for TRT specifications.}

\item{method}{A character string specifying the estimation method. Currently
supports \code{"EM"} (Expectation-Maximization). Defaults to \code{"EM"}.}

\item{control}{A \code{list} of control parameters for the estimation algorithm:
\itemize{
\item \code{max_iter}: Maximum number of EM iterations (default = 100).
\item \code{converge_tol}: Convergence criterion for parameter change (default = 1e-4).
\item \code{theta_range}: Numeric vector of length 2 specifying the integration
grid bounds (default = c(-4, 4)).
\item \code{quad_points}: Number of quadrature points (default = 21).
\item \code{verbose}: Logical; if \code{TRUE}, prints progress to console.
\item \code{fix_discrimination}: Logical; default=FALSE
}}
}
\value{
A \code{list} containing three components:
\item{item_params}{A data frame of estimated item slopes (discrimination),
difficulties/thresholds, and guessing parameters with associated standard errors.}
\item{person_params}{A data frame of EAP-based ability estimates (\eqn{\theta})
and testlet effect estimates (\eqn{\gamma}).}
\item{model_fit}{A data frame containing Log-Likelihood, AIC, and BIC indices.}
}
\description{
Provides a unified marginal maximum likelihood estimation framework for
a broad class of item response theory and testlet response theory models. The function automatically detects data
structures to apply appropriate models, along with their testlet-effect extensions (Bradlow et al., 1999).
}
\details{
The estimation utilizes a robust Newton-Raphson update within the M-step. For
testlet models, dimension reduction is achieved through the integration of
the nuisance testlet effect (Li et al., 2006). The function automatically
corrects model specifications if the data levels (binary vs. polytomous)
do not align with the requested model string.
}
\examples{
  # --- Quick Example (small data) ---
  set.seed(1)
  n <- 50
  resp <- data.frame(
    I1 = rbinom(n, 1, 0.6),
    I2 = rbinom(n, 1, 0.5),
    I3 = rbinom(n, 1, 0.4),
    I4 = sample(0:2, n, replace = TRUE),
    I5 = rbinom(n, 1, 0.5),
    I6 = rbinom(n, 1, 0.6)
  )
  spec <- data.frame(
    item = colnames(resp),
    model = c("2PL", "2PL", "2PL", "GRM", "2PLT", "2PLT"),
    testlet = c(NA, NA, NA, NA, "T1", "T1"),
    stringsAsFactors = FALSE
  )
  res <- irt_trt(resp, spec, method = "EM",
                 control = list(max_iter = 5, verbose = FALSE))
  head(res$item_params)

  \donttest{
  # --- Full Example: Simulation (Binary + Poly + Testlets) ---
  set.seed(2025)
  N <- 100; J <- 20

  # 1. Generate Parameters
  theta <- rnorm(N, 0, 1)
  gamma_1 <- rnorm(N, 0, 0.5) # Testlet 1 effect
  gamma_2 <- rnorm(N, 0, 0.6) # Testlet 2 effect

  a_true <- runif(J, 0.8, 1.5)
  b_true <- seq(-1.5, 1.5, length.out = J)

  resp_matrix <- matrix(NA, N, J)
  colnames(resp_matrix) <- paste0("Item_", 1:J)

  # 2. Simulate Responses
  # Items 1-10: Binary Independent (Model: 2PL)
  for(j in 1:10) {
    p <- 1 / (1 + exp(-a_true[j] * (theta - b_true[j])))
    resp_matrix[,j] <- rbinom(N, 1, p)
  }

  # Items 11-15: Poly Independent (Model: GRM)
  for(j in 11:15) {
    thresh <- sort(c(b_true[j] - 0.7, b_true[j] + 0.7))
    p1 <- 1 / (1 + exp(-a_true[j] * (theta - thresh[1])))
    p2 <- 1 / (1 + exp(-a_true[j] * (theta - thresh[2])))
    probs <- cbind(1-p1, p1-p2, p2)
    resp_matrix[,j] <- apply(probs, 1, function(p) sample(0:2, 1, prob=p))
  }

  # Items 16-17: Binary Testlet 1 (Model: 2PLT)
  for(j in 16:17) {
    eff_theta <- theta + gamma_1
    p <- 1 / (1 + exp(-a_true[j] * (eff_theta - b_true[j])))
    resp_matrix[,j] <- rbinom(N, 1, p)
  }

  # Items 18-20: Poly Testlet 2 (Model: GRT)
  for(j in 18:20) {
    eff_theta <- theta + gamma_2
    thresh <- sort(c(b_true[j] - 0.5, b_true[j] + 0.5))
    p1 <- 1 / (1 + exp(-a_true[j] * (eff_theta - thresh[1])))
    p2 <- 1 / (1 + exp(-a_true[j] * (eff_theta - thresh[2])))
    probs <- cbind(1-p1, p1-p2, p2)
    resp_matrix[,j] <- apply(probs, 1, function(p) sample(0:2, 1, prob=p))
  }

  df_sim <- as.data.frame(resp_matrix)

  # 3. Create Item Specification
  # STRICT naming: Independent=2PL/GRM, Testlet=2PLT/GRT
  spec <- data.frame(
    item = colnames(df_sim),
    model = c(rep("2PL", 10), rep("GRM", 5), rep("2PLT", 2), rep("GRT", 3)),
    testlet = c(rep(NA, 15), rep("T1", 2), rep("T2", 3)),
    stringsAsFactors = FALSE
  )

  # 4. Run Estimation
  res <- irt_trt(df_sim, spec, method = "EM",
                 control = list(max_iter = 20, verbose = FALSE))

  head(res$item_params)
  head(res$person_params)
  }
}
\references{
Bradlow, E. T., Wainer, H., & Wang, X. (1999). A testlet response model for
multidimensionality in item response theory. \emph{Psychometrika, 64}(2), 147-168.

Li, Y., Bolt, D. M., & Fu, J. (2006). A comparison of methods for estimating
secondary dimensions in testlet-based data. \emph{Applied Psychological
Measurement, 30}(3), 203-223.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_irt.R
\name{sim_irt}
\alias{sim_irt}
\title{Simulate Item Response Theory Data}
\usage{
sim_irt(
  n_people = 1000,
  item_structure = list(),
  theta = NULL,
  theta_mean = 0,
  theta_sd = 1
)
}
\arguments{
\item{n_people}{Integer. Number of students.}

\item{item_structure}{List of lists defining item blocks.}

\item{theta}{Numeric vector (Optional). If provided, these exact ability values are used.}

\item{theta_mean}{Numeric. Mean of latent trait (used if theta is NULL).}

\item{theta_sd}{Numeric. SD of latent trait (used if theta is NULL).}
}
\value{
A list containing:
\item{resp}{data.frame of responses (rows=people, cols=items)}
\item{true_params}{data.frame of true item parameters}
\item{theta}{vector of true latent traits}
}
\description{
Simulate item responses data. Support both dichotomous and polytomous responses.
Provide an easy implementation with a few default settings.
}
\examples{
  # 1. Define the Test Blueprint
  # We want:
  # - 10 items using 2PL (medium difficulty)
  # - 5 items using 3PL (difficult, with guessing)
  # - 5 items using GPCM (4-point Likert scale)
  # - 5 items using GRM (5-point Likert scale)

  my_test_structure <- list(
    # Block 1: 2PL
    list(model = "2PL", n_items = 10, a = c(0.8, 1.2), b = c(-1, 1)),

    # Block 2: 3PL (Harder items, b from 1 to 2.5, fixing guessing at 0.2)
    list(model = "3PL", n_items = 5, a = c(1.0, 1.5), b = c(1.0, 2.5), c = 0.2),

    # Block 3: GPCM (Polytomous, 4 categories 0-3)
    list(model = "GPCM", n_items = 5, categories = 4, a = c(0.7, 1.3), b = c(-1, 1)),

    # Block 4: GRM (Polytomous, 5 categories 0-4)
    list(model = "GRM", n_items = 5, categories = 5, a = c(1.0, 2.0))
  )

  # 2. Run the Simulation
  # Define N and a specific Theta vector
  N <- 2000
  theta_vec <- rnorm(N, 0, 2)

  sim_data <- sim_irt(
    n_people = N,
    theta = theta_vec,
    item_structure = my_test_structure
  )

  # 3. Inspect the Output
  # The Response Matrix
  head(sim_data$resp)

  # The True Parameters (Useful for recovery studies)
  # Note how it aligns a, b, and threshold parameters (step_1, step_2...)
  head(sim_data$true_params)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ela2}
\alias{ela2}
\title{Mixed-Format English Language Arts (ELA) Assessment Data (Form 2)}
\format{
A data frame with columns representing item responses.
\itemize{
\item \code{ITEM1} - \code{ITEM7}: Binary responses (0 = Incorrect, 1 = Correct).
\item \code{ITEM8}: Polytomous response (scored 0-2).
\item \code{ITEM9}: Polytomous response (scored 0-5).
\item \code{ITEM10}: Polytomous response (scored 0-5).
}
}
\source{
Tang, C., Xiong, J., & Engelhard, G. (2025). Identification of writing
strategies in educational assessments with an unsupervised learning
measurement framework. \emph{Education Sciences}, 15(7), 912.
\doi{10.3390/educsci15070912}
}
\usage{
ela2
}
\description{
A smaller dataset containing item responses.
}
\examples{
data(ela2)
head(ela2)
}
\keyword{datasets}
